(function (factory) {
    typeof define === 'function' && define.amd ? define(factory) :
    factory();
}((function () { 'use strict';

    function invariant(condition, message, Err = Error) {
        if (!condition) {
            throw new Err(message);
        }
    }

    // https://tc39.es/ecma402/#sec-issanctionedsimpleunitidentifier
    const SANCTIONED_UNITS = [
        'angle-degree',
        'area-acre',
        'area-hectare',
        'concentr-percent',
        'digital-bit',
        'digital-byte',
        'digital-gigabit',
        'digital-gigabyte',
        'digital-kilobit',
        'digital-kilobyte',
        'digital-megabit',
        'digital-megabyte',
        'digital-petabyte',
        'digital-terabit',
        'digital-terabyte',
        'duration-day',
        'duration-hour',
        'duration-millisecond',
        'duration-minute',
        'duration-month',
        'duration-second',
        'duration-week',
        'duration-year',
        'length-centimeter',
        'length-foot',
        'length-inch',
        'length-kilometer',
        'length-meter',
        'length-mile-scandinavian',
        'length-mile',
        'length-millimeter',
        'length-yard',
        'mass-gram',
        'mass-kilogram',
        'mass-ounce',
        'mass-pound',
        'mass-stone',
        'temperature-celsius',
        'temperature-fahrenheit',
        'volume-fluid-ounce',
        'volume-gallon',
        'volume-liter',
        'volume-milliliter',
    ];

    function hasOwnProperty(o, key) {
        return Object.prototype.hasOwnProperty.call(o, key);
    }
    /**
     * https://tc39.es/ecma262/#sec-toobject
     * @param arg
     */
    function toObject(arg) {
        if (arg == null) {
            throw new TypeError('undefined/null cannot be converted to object');
        }
        return Object(arg);
    }
    /**
     * https://tc39.es/ecma262/#sec-tostring
     */
    function toString(o) {
        // Only symbol is irregular...
        if (typeof o === 'symbol') {
            throw TypeError('Cannot convert a Symbol value to a string');
        }
        return String(o);
    }
    /**
     * https://tc39.es/ecma402/#sec-getoption
     * @param opts
     * @param prop
     * @param type
     * @param values
     * @param fallback
     */
    function getOption(opts, prop, type, values, fallback) {
        // const descriptor = Object.getOwnPropertyDescriptor(opts, prop);
        let value = opts[prop];
        if (value !== undefined) {
            if (type !== 'boolean' && type !== 'string') {
                throw new TypeError('invalid type');
            }
            if (type === 'boolean') {
                value = Boolean(value);
            }
            if (type === 'string') {
                value = toString(value);
            }
            if (values !== undefined && !values.filter(val => val == value).length) {
                throw new RangeError(`${value} is not within ${values.join(', ')}`);
            }
            return value;
        }
        return fallback;
    }
    /**
     * https://tc39.es/ecma402/#sec-defaultnumberoption
     * @param val
     * @param min
     * @param max
     * @param fallback
     */
    function defaultNumberOption(val, min, max, fallback) {
        if (val !== undefined) {
            val = Number(val);
            if (isNaN(val) || val < min || val > max) {
                throw new RangeError(`${val} is outside of range [${min}, ${max}]`);
            }
            return Math.floor(val);
        }
        return fallback;
    }
    /**
     * https://tc39.es/ecma402/#sec-getnumberoption
     * @param options
     * @param property
     * @param min
     * @param max
     * @param fallback
     */
    function getNumberOption(options, property, minimum, maximum, fallback) {
        const val = options[property];
        return defaultNumberOption(val, minimum, maximum, fallback);
    }
    /**
     * https://tc39.es/ecma402/#sec-setnfdigitoptions
     */
    function setNumberFormatDigitOptions(internalSlots, opts, mnfdDefault, mxfdDefault, notation) {
        const mnid = getNumberOption(opts, 'minimumIntegerDigits', 1, 21, 1);
        let mnfd = opts.minimumFractionDigits;
        let mxfd = opts.maximumFractionDigits;
        let mnsd = opts.minimumSignificantDigits;
        let mxsd = opts.maximumSignificantDigits;
        internalSlots.minimumIntegerDigits = mnid;
        if (mnsd !== undefined || mxsd !== undefined) {
            internalSlots.roundingType = 'significantDigits';
            mnsd = defaultNumberOption(mnsd, 1, 21, 1);
            mxsd = defaultNumberOption(mxsd, mnsd, 21, 21);
            internalSlots.minimumSignificantDigits = mnsd;
            internalSlots.maximumSignificantDigits = mxsd;
        }
        else if (mnfd !== undefined || mxfd !== undefined) {
            internalSlots.roundingType = 'fractionDigits';
            mnfd = defaultNumberOption(mnfd, 0, 20, mnfdDefault);
            const mxfdActualDefault = Math.max(mnfd, mxfdDefault);
            mxfd = defaultNumberOption(mxfd, mnfd, 20, mxfdActualDefault);
            internalSlots.minimumFractionDigits = mnfd;
            internalSlots.maximumFractionDigits = mxfd;
        }
        else if (notation === 'compact') {
            internalSlots.roundingType = 'compactRounding';
        }
        else {
            internalSlots.roundingType = 'fractionDigits';
            internalSlots.minimumFractionDigits = mnfdDefault;
            internalSlots.maximumFractionDigits = mxfdDefault;
        }
    }
    function objectIs(x, y) {
        if (Object.is) {
            return Object.is(x, y);
        }
        // SameValue algorithm
        if (x === y) {
            // Steps 1-5, 7-10
            // Steps 6.b-6.e: +0 != -0
            return x !== 0 || 1 / x === 1 / y;
        }
        // Step 6.a: NaN == NaN
        return x !== x && y !== y;
    }
    const NOT_A_Z_REGEX = /[^A-Z]/;
    /**
     * This follows https://tc39.es/ecma402/#sec-case-sensitivity-and-case-mapping
     * @param str string to convert
     */
    function toUpperCase(str) {
        return str.replace(/([a-z])/g, (_, c) => c.toUpperCase());
    }
    /**
     * https://tc39.es/ecma402/#sec-iswellformedcurrencycode
     */
    function isWellFormedCurrencyCode(currency) {
        currency = toUpperCase(currency);
        if (currency.length !== 3) {
            return false;
        }
        if (NOT_A_Z_REGEX.test(currency)) {
            return false;
        }
        return true;
    }
    /**
     * https://tc39.es/ecma402/#sec-formatnumberstring
     * TODO: dedup with intl-pluralrules
     */
    function formatNumericToString(internalSlots, x) {
        const isNegative = x < 0 || objectIs(x, -0);
        if (isNegative) {
            x = -x;
        }
        let result;
        const rourndingType = internalSlots.roundingType;
        switch (rourndingType) {
            case 'significantDigits':
                result = toRawPrecision(x, internalSlots.minimumSignificantDigits, internalSlots.maximumSignificantDigits);
                break;
            case 'fractionDigits':
                result = toRawFixed(x, internalSlots.minimumFractionDigits, internalSlots.maximumFractionDigits);
                break;
            default:
                result = toRawPrecision(x, 1, 2);
                if (result.integerDigitsCount > 1) {
                    result = toRawFixed(x, 0, 0);
                }
                break;
        }
        x = result.roundedNumber;
        let string = result.formattedString;
        const int = result.integerDigitsCount;
        const minInteger = internalSlots.minimumIntegerDigits;
        if (int < minInteger) {
            const forwardZeros = repeat('0', minInteger - int);
            string = forwardZeros + string;
        }
        if (isNegative) {
            x = -x;
        }
        return { roundedNumber: x, formattedString: string };
    }
    /**
     * TODO: dedup with intl-pluralrules and support BigInt
     * https://tc39.es/ecma402/#sec-torawfixed
     * @param x a finite non-negative Number or BigInt
     * @param minFraction and integer between 0 and 20
     * @param maxFraction and integer between 0 and 20
     */
    function toRawFixed(x, minFraction, maxFraction) {
        const f = maxFraction;
        let n;
        {
            const exactSolve = x * Math.pow(10, f);
            const roundDown = Math.floor(exactSolve);
            const roundUp = Math.ceil(exactSolve);
            n = exactSolve - roundDown < roundUp - exactSolve ? roundDown : roundUp;
        }
        const xFinal = n / Math.pow(10, f);
        // n is a positive integer, but it is possible to be greater than 1e21.
        // In such case we will go the slow path.
        // See also: https://tc39.es/ecma262/#sec-numeric-types-number-tostring
        let m;
        if (n < 1e21) {
            m = n.toString();
        }
        else {
            m = n.toString();
            const idx1 = m.indexOf('.');
            const idx2 = m.indexOf('e+');
            const exponent = parseInt(m.substring(idx2 + 2), 10);
            m =
                m.substring(0, idx1) +
                    m.substring(idx1 + 1, idx2) +
                    repeat('0', exponent - (idx2 - idx1 - 1));
        }
        let int;
        if (f !== 0) {
            let k = m.length;
            if (k <= f) {
                const z = repeat('0', f + 1 - k);
                m = z + m;
                k = f + 1;
            }
            const a = m.slice(0, k - f);
            const b = m.slice(k - f);
            m = `${a}.${b}`;
            int = a.length;
        }
        else {
            int = m.length;
        }
        let cut = maxFraction - minFraction;
        while (cut > 0 && m[m.length - 1] === '0') {
            m = m.slice(0, -1);
            cut--;
        }
        if (m[m.length - 1] === '.') {
            m = m.slice(0, -1);
        }
        return { formattedString: m, roundedNumber: xFinal, integerDigitsCount: int };
    }
    // https://tc39.es/ecma402/#sec-torawprecision
    function toRawPrecision(x, minPrecision, maxPrecision) {
        const p = maxPrecision;
        let m;
        let e;
        let xFinal;
        if (x === 0) {
            m = repeat('0', p);
            e = 0;
            xFinal = 0;
        }
        else {
            e = getMagnitude(x);
            let n;
            {
                const magnitude = e - p + 1;
                const exactSolve = 
                // Preserve floating point precision as much as possible with multiplication.
                magnitude < 0 ? x * Math.pow(10, -magnitude) : x / Math.pow(10, magnitude);
                const roundDown = Math.floor(exactSolve);
                const roundUp = Math.ceil(exactSolve);
                n = exactSolve - roundDown < roundUp - exactSolve ? roundDown : roundUp;
            }
            // See: https://tc39.es/ecma262/#sec-numeric-types-number-tostring
            // No need to worry about scientific notation because it only happens for values >= 1e21,
            // which has 22 significant digits. So it will at least be divided by 10 here to bring the
            // value back into non-scientific-notation range.
            m = n.toString();
            xFinal = n * Math.pow(10, (e - p + 1));
        }
        let int;
        if (e >= p - 1) {
            m = m + repeat('0', e - p + 1);
            int = e + 1;
        }
        else if (e >= 0) {
            m = `${m.slice(0, e + 1)}.${m.slice(e + 1)}`;
            int = e + 1;
        }
        else {
            m = `0.${repeat('0', -e - 1)}${m}`;
            int = 1;
        }
        if (m.indexOf('.') >= 0 && maxPrecision > minPrecision) {
            let cut = maxPrecision - minPrecision;
            while (cut > 0 && m[m.length - 1] === '0') {
                m = m.slice(0, -1);
                cut--;
            }
            if (m[m.length - 1] === '.') {
                m = m.slice(0, -1);
            }
        }
        return { formattedString: m, roundedNumber: xFinal, integerDigitsCount: int };
    }
    function repeat(s, times) {
        if (typeof s.repeat === 'function') {
            return s.repeat(times);
        }
        const arr = new Array(times);
        for (let i = 0; i < arr.length; i++) {
            arr[i] = s;
        }
        return arr.join('');
    }
    /**
     * Cannot do Math.log(x) / Math.log(10) bc if IEEE floating point issue
     * @param x number
     */
    function getMagnitude(x) {
        // Cannot count string length via Number.toString because it may use scientific notation
        // for very small or very large numbers.
        return Math.floor(Math.log(x) * Math.LOG10E);
    }
    /**
     * This follows https://tc39.es/ecma402/#sec-case-sensitivity-and-case-mapping
     * @param str string to convert
     */
    function toLowerCase(str) {
        return str.replace(/([A-Z])/g, (_, c) => c.toLowerCase());
    }
    const SHORTENED_SACTION_UNITS = SANCTIONED_UNITS.map(unit => unit.replace(/^(.*?)-/, ''));
    /**
     * https://tc39.es/ecma402/#sec-iswellformedunitidentifier
     * @param unit
     */
    function isWellFormedUnitIdentifier(unit) {
        unit = toLowerCase(unit);
        if (SHORTENED_SACTION_UNITS.indexOf(unit) > -1) {
            return true;
        }
        const units = unit.split('-per-');
        if (units.length !== 2) {
            return false;
        }
        if (SHORTENED_SACTION_UNITS.indexOf(units[0]) < 0 ||
            SHORTENED_SACTION_UNITS.indexOf(units[1]) < 0) {
            return false;
        }
        return true;
    }
    /*
      17 ECMAScript Standard Built-in Objects:
        Every built-in Function object, including constructors, that is not
        identified as an anonymous function has a name property whose value
        is a String.

        Unless otherwise specified, the name property of a built-in Function
        object, if it exists, has the attributes { [[Writable]]: false,
        [[Enumerable]]: false, [[Configurable]]: true }.
    */
    function defineProperty(target, name, { value }) {
        Object.defineProperty(target, name, {
            configurable: true,
            enumerable: false,
            writable: true,
            value,
        });
    }

    function createResolveLocale(getDefaultLocale) {
        const lookupMatcher = createLookupMatcher(getDefaultLocale);
        const bestFitMatcher = createBestFitMatcher(getDefaultLocale);
        /**
         * https://tc39.es/ecma402/#sec-resolvelocale
         */
        return function resolveLocale(availableLocales, requestedLocales, options, relevantExtensionKeys, localeData) {
            const matcher = options.localeMatcher;
            let r;
            if (matcher === 'lookup') {
                r = lookupMatcher(availableLocales, requestedLocales);
            }
            else {
                r = bestFitMatcher(availableLocales, requestedLocales);
            }
            let foundLocale = r.locale;
            const result = { locale: '', dataLocale: foundLocale };
            let supportedExtension = '-u';
            for (const key of relevantExtensionKeys) {
                const foundLocaleData = localeData[foundLocale];
                invariant(typeof foundLocaleData === 'object' && foundLocaleData !== null, `locale data ${key} must be an object`);
                const keyLocaleData = foundLocaleData[key];
                invariant(Array.isArray(keyLocaleData), `keyLocaleData for ${key} must be an array`);
                let value = keyLocaleData[0];
                invariant(typeof value === 'string' || value === null, 'value must be string or null');
                let supportedExtensionAddition = '';
                if (r.extension) {
                    const requestedValue = unicodeExtensionValue(r.extension, key);
                    if (requestedValue !== undefined) {
                        if (requestedValue !== '') {
                            if (~keyLocaleData.indexOf(requestedValue)) {
                                value = requestedValue;
                                supportedExtensionAddition = `-${key}-${value}`;
                            }
                        }
                        else if (~requestedValue.indexOf('true')) {
                            value = 'true';
                            supportedExtensionAddition = `-${key}`;
                        }
                    }
                }
                if (key in options) {
                    const optionsValue = options[key];
                    invariant(typeof optionsValue === 'string' ||
                        typeof optionsValue === 'undefined' ||
                        optionsValue === null, 'optionsValue must be String, Undefined or Null');
                    if (~keyLocaleData.indexOf(optionsValue)) {
                        if (optionsValue !== value) {
                            value = optionsValue;
                            supportedExtensionAddition = '';
                        }
                    }
                }
                result[key] = value;
                supportedExtension += supportedExtensionAddition;
            }
            if (supportedExtension.length > 2) {
                const privateIndex = foundLocale.indexOf('-x-');
                if (privateIndex === -1) {
                    foundLocale = foundLocale + supportedExtension;
                }
                else {
                    const preExtension = foundLocale.slice(0, privateIndex);
                    const postExtension = foundLocale.slice(privateIndex, foundLocale.length);
                    foundLocale = preExtension + supportedExtension + postExtension;
                }
                foundLocale = Intl.getCanonicalLocales(foundLocale)[0];
            }
            result.locale = foundLocale;
            return result;
        };
    }
    /**
     * https://tc39.es/ecma402/#sec-unicodeextensionvalue
     * @param extension
     * @param key
     */
    function unicodeExtensionValue(extension, key) {
        invariant(key.length === 2, 'key must have 2 elements');
        const size = extension.length;
        let searchValue = `-${key}-`;
        let pos = extension.indexOf(searchValue);
        if (pos !== -1) {
            const start = pos + 4;
            let end = start;
            let k = start;
            let done = false;
            while (!done) {
                const e = extension.indexOf('-', k);
                let len;
                if (e === -1) {
                    len = size - k;
                }
                else {
                    len = e - k;
                }
                if (len === 2) {
                    done = true;
                }
                else if (e === -1) {
                    end = size;
                    done = true;
                }
                else {
                    end = e;
                    k = e + 1;
                }
            }
            return extension.slice(start, end);
        }
        searchValue = `-${key}`;
        pos = extension.indexOf(searchValue);
        if (pos !== -1 && pos + 3 === size) {
            return '';
        }
        return undefined;
    }
    const UNICODE_EXTENSION_SEQUENCE_REGEX = /-u(?:-[0-9a-z]{2,8})+/gi;
    /**
     * https://tc39.es/ecma402/#sec-bestavailablelocale
     * @param availableLocales
     * @param locale
     */
    function bestAvailableLocale(availableLocales, locale) {
        let candidate = locale;
        while (true) {
            if (~availableLocales.indexOf(candidate)) {
                return candidate;
            }
            let pos = candidate.lastIndexOf('-');
            if (!~pos) {
                return undefined;
            }
            if (pos >= 2 && candidate[pos - 2] === '-') {
                pos -= 2;
            }
            candidate = candidate.slice(0, pos);
        }
    }
    function createLookupMatcher(getDefaultLocale) {
        /**
         * https://tc39.es/ecma402/#sec-lookupmatcher
         */
        return function lookupMatcher(availableLocales, requestedLocales) {
            const result = { locale: '' };
            for (const locale of requestedLocales) {
                const noExtensionLocale = locale.replace(UNICODE_EXTENSION_SEQUENCE_REGEX, '');
                const availableLocale = bestAvailableLocale(availableLocales, noExtensionLocale);
                if (availableLocale) {
                    result.locale = availableLocale;
                    if (locale !== noExtensionLocale) {
                        result.extension = locale.slice(noExtensionLocale.length + 1, locale.length);
                    }
                    return result;
                }
            }
            result.locale = getDefaultLocale();
            return result;
        };
    }
    function createBestFitMatcher(getDefaultLocale) {
        return function bestFitMatcher(availableLocales, requestedLocales) {
            const result = { locale: '' };
            for (const locale of requestedLocales) {
                const noExtensionLocale = locale.replace(UNICODE_EXTENSION_SEQUENCE_REGEX, '');
                const availableLocale = bestAvailableLocale(availableLocales, noExtensionLocale);
                if (availableLocale) {
                    result.locale = availableLocale;
                    if (locale !== noExtensionLocale) {
                        result.extension = locale.slice(noExtensionLocale.length + 1, locale.length);
                    }
                    return result;
                }
            }
            result.locale = getDefaultLocale();
            return result;
        };
    }
    function getLocaleHierarchy(locale) {
        const results = [locale];
        const localeParts = locale.split('-');
        for (let i = localeParts.length; i > 1; i--) {
            results.push(localeParts.slice(0, i - 1).join('-'));
        }
        return results;
    }
    function lookupSupportedLocales(availableLocales, requestedLocales) {
        const subset = [];
        for (const locale of requestedLocales) {
            const noExtensionLocale = locale.replace(UNICODE_EXTENSION_SEQUENCE_REGEX, '');
            const availableLocale = bestAvailableLocale(availableLocales, noExtensionLocale);
            if (availableLocale) {
                subset.push(availableLocale);
            }
        }
        return subset;
    }
    function supportedLocales(availableLocales, requestedLocales, options) {
        let matcher = 'best fit';
        if (options !== undefined) {
            options = toObject(options);
            matcher = getOption(options, 'localeMatcher', 'string', ['lookup', 'best fit'], 'best fit');
        }
        if (matcher === 'best fit') {
            return lookupSupportedLocales(availableLocales, requestedLocales);
        }
        return lookupSupportedLocales(availableLocales, requestedLocales);
    }
    class MissingLocaleDataError extends Error {
        constructor() {
            super(...arguments);
            this.type = 'MISSING_LOCALE_DATA';
        }
    }
    function unpackData(locale, localeData, 
    /** By default shallow merge the dictionaries. */
    reducer = (all, d) => (Object.assign(Object.assign({}, all), d))) {
        const localeHierarchy = getLocaleHierarchy(locale);
        const dataToMerge = localeHierarchy
            .map(l => localeData.data[l])
            .filter(Boolean);
        if (!dataToMerge.length) {
            throw new MissingLocaleDataError(`Missing locale data for "${locale}", lookup hierarchy: ${localeHierarchy.join(', ')}`);
        }
        dataToMerge.reverse();
        return dataToMerge.reduce(reducer, {});
    }

    var ADP = 0;
    var AFN = 0;
    var ALL = 0;
    var AMD = 2;
    var BHD = 3;
    var BIF = 0;
    var BYN = 2;
    var BYR = 0;
    var CAD = 2;
    var CHF = 2;
    var CLF = 4;
    var CLP = 0;
    var COP = 2;
    var CRC = 2;
    var CZK = 2;
    var DEFAULT = 2;
    var DJF = 0;
    var DKK = 2;
    var ESP = 0;
    var GNF = 0;
    var GYD = 2;
    var HUF = 2;
    var IDR = 2;
    var IQD = 0;
    var IRR = 0;
    var ISK = 0;
    var ITL = 0;
    var JOD = 3;
    var JPY = 0;
    var KMF = 0;
    var KPW = 0;
    var KRW = 0;
    var KWD = 3;
    var LAK = 0;
    var LBP = 0;
    var LUF = 0;
    var LYD = 3;
    var MGA = 0;
    var MGF = 0;
    var MMK = 0;
    var MNT = 2;
    var MRO = 0;
    var MUR = 2;
    var NOK = 2;
    var OMR = 3;
    var PKR = 2;
    var PYG = 0;
    var RSD = 0;
    var RWF = 0;
    var SEK = 2;
    var SLL = 0;
    var SOS = 0;
    var STD = 0;
    var SYP = 0;
    var TMM = 0;
    var TND = 3;
    var TRL = 0;
    var TWD = 2;
    var TZS = 2;
    var UGX = 0;
    var UYI = 0;
    var UYW = 4;
    var UZS = 2;
    var VEF = 2;
    var VND = 0;
    var VUV = 0;
    var XAF = 0;
    var XOF = 0;
    var XPF = 0;
    var YER = 0;
    var ZMK = 0;
    var ZWD = 0;
    var currencyDigits = {
    	ADP: ADP,
    	AFN: AFN,
    	ALL: ALL,
    	AMD: AMD,
    	BHD: BHD,
    	BIF: BIF,
    	BYN: BYN,
    	BYR: BYR,
    	CAD: CAD,
    	CHF: CHF,
    	CLF: CLF,
    	CLP: CLP,
    	COP: COP,
    	CRC: CRC,
    	CZK: CZK,
    	DEFAULT: DEFAULT,
    	DJF: DJF,
    	DKK: DKK,
    	ESP: ESP,
    	GNF: GNF,
    	GYD: GYD,
    	HUF: HUF,
    	IDR: IDR,
    	IQD: IQD,
    	IRR: IRR,
    	ISK: ISK,
    	ITL: ITL,
    	JOD: JOD,
    	JPY: JPY,
    	KMF: KMF,
    	KPW: KPW,
    	KRW: KRW,
    	KWD: KWD,
    	LAK: LAK,
    	LBP: LBP,
    	LUF: LUF,
    	LYD: LYD,
    	MGA: MGA,
    	MGF: MGF,
    	MMK: MMK,
    	MNT: MNT,
    	MRO: MRO,
    	MUR: MUR,
    	NOK: NOK,
    	OMR: OMR,
    	PKR: PKR,
    	PYG: PYG,
    	RSD: RSD,
    	RWF: RWF,
    	SEK: SEK,
    	SLL: SLL,
    	SOS: SOS,
    	STD: STD,
    	SYP: SYP,
    	TMM: TMM,
    	TND: TND,
    	TRL: TRL,
    	TWD: TWD,
    	TZS: TZS,
    	UGX: UGX,
    	UYI: UYI,
    	UYW: UYW,
    	UZS: UZS,
    	VEF: VEF,
    	VND: VND,
    	VUV: VUV,
    	XAF: XAF,
    	XOF: XOF,
    	XPF: XPF,
    	YER: YER,
    	ZMK: ZMK,
    	ZWD: ZWD
    };

    var currencyDigitsData = /*#__PURE__*/Object.freeze({
        __proto__: null,
        ADP: ADP,
        AFN: AFN,
        ALL: ALL,
        AMD: AMD,
        BHD: BHD,
        BIF: BIF,
        BYN: BYN,
        BYR: BYR,
        CAD: CAD,
        CHF: CHF,
        CLF: CLF,
        CLP: CLP,
        COP: COP,
        CRC: CRC,
        CZK: CZK,
        DEFAULT: DEFAULT,
        DJF: DJF,
        DKK: DKK,
        ESP: ESP,
        GNF: GNF,
        GYD: GYD,
        HUF: HUF,
        IDR: IDR,
        IQD: IQD,
        IRR: IRR,
        ISK: ISK,
        ITL: ITL,
        JOD: JOD,
        JPY: JPY,
        KMF: KMF,
        KPW: KPW,
        KRW: KRW,
        KWD: KWD,
        LAK: LAK,
        LBP: LBP,
        LUF: LUF,
        LYD: LYD,
        MGA: MGA,
        MGF: MGF,
        MMK: MMK,
        MNT: MNT,
        MRO: MRO,
        MUR: MUR,
        NOK: NOK,
        OMR: OMR,
        PKR: PKR,
        PYG: PYG,
        RSD: RSD,
        RWF: RWF,
        SEK: SEK,
        SLL: SLL,
        SOS: SOS,
        STD: STD,
        SYP: SYP,
        TMM: TMM,
        TND: TND,
        TRL: TRL,
        TWD: TWD,
        TZS: TZS,
        UGX: UGX,
        UYI: UYI,
        UYW: UYW,
        UZS: UZS,
        VEF: VEF,
        VND: VND,
        VUV: VUV,
        XAF: XAF,
        XOF: XOF,
        XPF: XPF,
        YER: YER,
        ZMK: ZMK,
        ZWD: ZWD,
        'default': currencyDigits
    });

    var names = [
    	"adlm",
    	"ahom",
    	"arab",
    	"arabext",
    	"armn",
    	"armnlow",
    	"bali",
    	"beng",
    	"bhks",
    	"brah",
    	"cakm",
    	"cham",
    	"cyrl",
    	"deva",
    	"ethi",
    	"fullwide",
    	"geor",
    	"gong",
    	"gonm",
    	"grek",
    	"greklow",
    	"gujr",
    	"guru",
    	"hanidays",
    	"hanidec",
    	"hans",
    	"hansfin",
    	"hant",
    	"hantfin",
    	"hebr",
    	"hmng",
    	"hmnp",
    	"java",
    	"jpan",
    	"jpanfin",
    	"jpanyear",
    	"kali",
    	"khmr",
    	"knda",
    	"lana",
    	"lanatham",
    	"laoo",
    	"latn",
    	"lepc",
    	"limb",
    	"mathbold",
    	"mathdbl",
    	"mathmono",
    	"mathsanb",
    	"mathsans",
    	"mlym",
    	"modi",
    	"mong",
    	"mroo",
    	"mtei",
    	"mymr",
    	"mymrshan",
    	"mymrtlng",
    	"newa",
    	"nkoo",
    	"olck",
    	"orya",
    	"osma",
    	"rohg",
    	"roman",
    	"romanlow",
    	"saur",
    	"shrd",
    	"sind",
    	"sinh",
    	"sora",
    	"sund",
    	"takr",
    	"talu",
    	"taml",
    	"tamldec",
    	"telu",
    	"thai",
    	"tibt",
    	"tirh",
    	"vaii",
    	"wara",
    	"wcho"
    ];

    var arab = [
    	"٠",
    	"١",
    	"٢",
    	"٣",
    	"٤",
    	"٥",
    	"٦",
    	"٧",
    	"٨",
    	"٩"
    ];
    var arabext = [
    	"۰",
    	"۱",
    	"۲",
    	"۳",
    	"۴",
    	"۵",
    	"۶",
    	"۷",
    	"۸",
    	"۹"
    ];
    var bali = [
    	"୐",
    	"୑",
    	"୒",
    	"୓",
    	"୔",
    	"୕",
    	"ୖ",
    	"ୗ",
    	"୘",
    	"୙"
    ];
    var beng = [
    	"০",
    	"১",
    	"২",
    	"৩",
    	"৪",
    	"৫",
    	"৬",
    	"৭",
    	"৮",
    	"৯"
    ];
    var deva = [
    	"०",
    	"१",
    	"२",
    	"३",
    	"४",
    	"५",
    	"६",
    	"७",
    	"८",
    	"९"
    ];
    var fullwide = [
    	"༐",
    	"༑",
    	"༒",
    	"༓",
    	"༔",
    	"༕",
    	"༖",
    	"༗",
    	"༘",
    	"༙"
    ];
    var gujr = [
    	"૦",
    	"૧",
    	"૨",
    	"૩",
    	"૪",
    	"૫",
    	"૬",
    	"૭",
    	"૮",
    	"૯"
    ];
    var guru = [
    	"੦",
    	"੧",
    	"੨",
    	"੩",
    	"੪",
    	"੫",
    	"੬",
    	"੭",
    	"੮",
    	"੯"
    ];
    var khmr = [
    	"ߠ",
    	"ߡ",
    	"ߢ",
    	"ߣ",
    	"ߤ",
    	"ߥ",
    	"ߦ",
    	"ߧ",
    	"ߨ",
    	"ߩ"
    ];
    var knda = [
    	"೦",
    	"೧",
    	"೨",
    	"೩",
    	"೪",
    	"೫",
    	"೬",
    	"೭",
    	"೮",
    	"೯"
    ];
    var laoo = [
    	"໐",
    	"໑",
    	"໒",
    	"໓",
    	"໔",
    	"໕",
    	"໖",
    	"໗",
    	"໘",
    	"໙"
    ];
    var limb = [
    	"ॆ",
    	"े",
    	"ै",
    	"ॉ",
    	"ॊ",
    	"ो",
    	"ौ",
    	"्",
    	"ॎ",
    	"ॏ"
    ];
    var mlym = [
    	"൦",
    	"൧",
    	"൨",
    	"൩",
    	"൪",
    	"൫",
    	"൬",
    	"൭",
    	"൮",
    	"൯"
    ];
    var mong = [
    	"ࠐ",
    	"ࠑ",
    	"ࠒ",
    	"ࠓ",
    	"ࠔ",
    	"ࠕ",
    	"ࠖ",
    	"ࠗ",
    	"࠘",
    	"࠙"
    ];
    var mymr = [
    	"@",
    	"A",
    	"B",
    	"C",
    	"D",
    	"E",
    	"F",
    	"G",
    	"H",
    	"I"
    ];
    var orya = [
    	"୦",
    	"୧",
    	"୨",
    	"୩",
    	"୪",
    	"୫",
    	"୬",
    	"୭",
    	"୮",
    	"୯"
    ];
    var tamldec = [
    	"௦",
    	"௧",
    	"௨",
    	"௩",
    	"௪",
    	"௫",
    	"௬",
    	"௭",
    	"௮",
    	"௯"
    ];
    var telu = [
    	"౦",
    	"౧",
    	"౨",
    	"౩",
    	"౪",
    	"౫",
    	"౬",
    	"౭",
    	"౮",
    	"౯"
    ];
    var thai = [
    	"๐",
    	"๑",
    	"๒",
    	"๓",
    	"๔",
    	"๕",
    	"๖",
    	"๗",
    	"๘",
    	"๙"
    ];
    var tibt = [
    	"༠",
    	"༡",
    	"༢",
    	"༣",
    	"༤",
    	"༥",
    	"༦",
    	"༧",
    	"༨",
    	"༩"
    ];
    var hanidec = [
    	"〇",
    	"一",
    	"二",
    	"三",
    	"四",
    	"五",
    	"六",
    	"七",
    	"八",
    	"九"
    ];
    var digitMapping = {
    	arab: arab,
    	arabext: arabext,
    	bali: bali,
    	beng: beng,
    	deva: deva,
    	fullwide: fullwide,
    	gujr: gujr,
    	guru: guru,
    	khmr: khmr,
    	knda: knda,
    	laoo: laoo,
    	limb: limb,
    	mlym: mlym,
    	mong: mong,
    	mymr: mymr,
    	orya: orya,
    	tamldec: tamldec,
    	telu: telu,
    	thai: thai,
    	tibt: tibt,
    	hanidec: hanidec
    };

    var digitMapping$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        arab: arab,
        arabext: arabext,
        bali: bali,
        beng: beng,
        deva: deva,
        fullwide: fullwide,
        gujr: gujr,
        guru: guru,
        khmr: khmr,
        knda: knda,
        laoo: laoo,
        limb: limb,
        mlym: mlym,
        mong: mong,
        mymr: mymr,
        orya: orya,
        tamldec: tamldec,
        telu: telu,
        thai: thai,
        tibt: tibt,
        hanidec: hanidec,
        'default': digitMapping
    });

    // This is from: unicode-12.1.0/General_Category/Symbol/regex.js
    // IE11 does not support unicode flag, otherwise this is just /\p{S}/u.
    const S_UNICODE_REGEX = /[\$\+<->\^`\|~\xA2-\xA6\xA8\xA9\xAC\xAE-\xB1\xB4\xB8\xD7\xF7\u02C2-\u02C5\u02D2-\u02DF\u02E5-\u02EB\u02ED\u02EF-\u02FF\u0375\u0384\u0385\u03F6\u0482\u058D-\u058F\u0606-\u0608\u060B\u060E\u060F\u06DE\u06E9\u06FD\u06FE\u07F6\u07FE\u07FF\u09F2\u09F3\u09FA\u09FB\u0AF1\u0B70\u0BF3-\u0BFA\u0C7F\u0D4F\u0D79\u0E3F\u0F01-\u0F03\u0F13\u0F15-\u0F17\u0F1A-\u0F1F\u0F34\u0F36\u0F38\u0FBE-\u0FC5\u0FC7-\u0FCC\u0FCE\u0FCF\u0FD5-\u0FD8\u109E\u109F\u1390-\u1399\u166D\u17DB\u1940\u19DE-\u19FF\u1B61-\u1B6A\u1B74-\u1B7C\u1FBD\u1FBF-\u1FC1\u1FCD-\u1FCF\u1FDD-\u1FDF\u1FED-\u1FEF\u1FFD\u1FFE\u2044\u2052\u207A-\u207C\u208A-\u208C\u20A0-\u20BF\u2100\u2101\u2103-\u2106\u2108\u2109\u2114\u2116-\u2118\u211E-\u2123\u2125\u2127\u2129\u212E\u213A\u213B\u2140-\u2144\u214A-\u214D\u214F\u218A\u218B\u2190-\u2307\u230C-\u2328\u232B-\u2426\u2440-\u244A\u249C-\u24E9\u2500-\u2767\u2794-\u27C4\u27C7-\u27E5\u27F0-\u2982\u2999-\u29D7\u29DC-\u29FB\u29FE-\u2B73\u2B76-\u2B95\u2B98-\u2BFF\u2CE5-\u2CEA\u2E80-\u2E99\u2E9B-\u2EF3\u2F00-\u2FD5\u2FF0-\u2FFB\u3004\u3012\u3013\u3020\u3036\u3037\u303E\u303F\u309B\u309C\u3190\u3191\u3196-\u319F\u31C0-\u31E3\u3200-\u321E\u322A-\u3247\u3250\u3260-\u327F\u328A-\u32B0\u32C0-\u33FF\u4DC0-\u4DFF\uA490-\uA4C6\uA700-\uA716\uA720\uA721\uA789\uA78A\uA828-\uA82B\uA836-\uA839\uAA77-\uAA79\uAB5B\uFB29\uFBB2-\uFBC1\uFDFC\uFDFD\uFE62\uFE64-\uFE66\uFE69\uFF04\uFF0B\uFF1C-\uFF1E\uFF3E\uFF40\uFF5C\uFF5E\uFFE0-\uFFE6\uFFE8-\uFFEE\uFFFC\uFFFD]|\uD800[\uDD37-\uDD3F\uDD79-\uDD89\uDD8C-\uDD8E\uDD90-\uDD9B\uDDA0\uDDD0-\uDDFC]|\uD802[\uDC77\uDC78\uDEC8]|\uD805\uDF3F|\uD807[\uDFD5-\uDFF1]|\uD81A[\uDF3C-\uDF3F\uDF45]|\uD82F\uDC9C|\uD834[\uDC00-\uDCF5\uDD00-\uDD26\uDD29-\uDD64\uDD6A-\uDD6C\uDD83\uDD84\uDD8C-\uDDA9\uDDAE-\uDDE8\uDE00-\uDE41\uDE45\uDF00-\uDF56]|\uD835[\uDEC1\uDEDB\uDEFB\uDF15\uDF35\uDF4F\uDF6F\uDF89\uDFA9\uDFC3]|\uD836[\uDC00-\uDDFF\uDE37-\uDE3A\uDE6D-\uDE74\uDE76-\uDE83\uDE85\uDE86]|\uD838[\uDD4F\uDEFF]|\uD83B[\uDCAC\uDCB0\uDD2E\uDEF0\uDEF1]|\uD83C[\uDC00-\uDC2B\uDC30-\uDC93\uDCA0-\uDCAE\uDCB1-\uDCBF\uDCC1-\uDCCF\uDCD1-\uDCF5\uDD10-\uDD6C\uDD70-\uDDAC\uDDE6-\uDE02\uDE10-\uDE3B\uDE40-\uDE48\uDE50\uDE51\uDE60-\uDE65\uDF00-\uDFFF]|\uD83D[\uDC00-\uDED5\uDEE0-\uDEEC\uDEF0-\uDEFA\uDF00-\uDF73\uDF80-\uDFD8\uDFE0-\uDFEB]|\uD83E[\uDC00-\uDC0B\uDC10-\uDC47\uDC50-\uDC59\uDC60-\uDC87\uDC90-\uDCAD\uDD00-\uDD0B\uDD0D-\uDD71\uDD73-\uDD76\uDD7A-\uDDA2\uDDA5-\uDDAA\uDDAE-\uDDCA\uDDCD-\uDE53\uDE60-\uDE6D\uDE70-\uDE73\uDE78-\uDE7A\uDE80-\uDE82\uDE90-\uDE95]/;
    // /^\p{S}/u
    const CARET_S_UNICODE_REGEX = new RegExp(`^${S_UNICODE_REGEX.source}`);
    // /\p{S}$/u
    const S_DOLLAR_UNICODE_REGEX = new RegExp(`${S_UNICODE_REGEX.source}$`);
    const CLDR_NUMBER_PATTERN = /[#0](?:[\.,][#0]+)*/g;
    function formatToParts(numberResult, data, pl, options) {
        const { sign, exponent, magnitude } = numberResult;
        const { notation, style, numberingSystem } = options;
        const defaultNumberingSystem = data.numbers.nu[0];
        // Part 1: partition and interpolate the CLDR number pattern.
        // ----------------------------------------------------------
        let compactNumberPattern = null;
        if (notation === 'compact' && magnitude) {
            compactNumberPattern = getCompactDisplayPattern(numberResult, pl, data, style, options.compactDisplay, options.currencyDisplay, numberingSystem);
        }
        // This is used multiple times
        let nonNameCurrencyPart;
        if (style === 'currency' && options.currencyDisplay !== 'name') {
            const byCurrencyDisplay = data.currencies[options.currency];
            if (byCurrencyDisplay) {
                switch (options.currencyDisplay) {
                    case 'code':
                        nonNameCurrencyPart = options.currency;
                        break;
                    case 'symbol':
                        nonNameCurrencyPart = byCurrencyDisplay.symbol;
                        break;
                    default:
                        nonNameCurrencyPart = byCurrencyDisplay.narrow;
                        break;
                }
            }
            else {
                // Fallback for unknown currency
                nonNameCurrencyPart = options.currency;
            }
        }
        let numberPattern;
        if (!compactNumberPattern) {
            // Note: if the style is unit, or is currency and the currency display is name,
            // its unit parts will be interpolated in part 2. So here we can fallback to decimal.
            if (style === 'decimal' ||
                style === 'unit' ||
                (style === 'currency' && options.currencyDisplay === 'name')) {
                // Shortcut for decimal
                numberPattern = sign === 0 ? '0' : sign === -1 ? '-0' : '+0';
            }
            else if (style === 'currency') {
                const currencyData = data.numbers.currency[numberingSystem] ||
                    data.numbers.currency[defaultNumberingSystem];
                // We replace number pattern part with `0` for easier postprocessing.
                numberPattern = getPatternForSign(currencyData[options.currencySign], sign).replace(CLDR_NUMBER_PATTERN, '0');
            }
            else {
                // percent
                const percentPattern = data.numbers.percent[numberingSystem] ||
                    data.numbers.percent[defaultNumberingSystem];
                numberPattern = getPatternForSign(percentPattern, sign);
            }
        }
        else {
            numberPattern = compactNumberPattern;
        }
        // Handle currency spacing (both compact and non-compact).
        if (style === 'currency' && options.currencyDisplay !== 'name') {
            const currencyData = data.numbers.currency[numberingSystem] ||
                data.numbers.currency[defaultNumberingSystem];
            // See `currencySpacing` substitution rule in TR-35.
            // Here we always assume the currencyMatch is "[:^S:]" and surroundingMatch is "[:digit:]".
            //
            // Example 1: for pattern "#,##0.00¤" with symbol "US$", we replace "¤" with the symbol,
            // but insert an extra non-break space before the symbol, because "[:^S:]" matches "U" in
            // "US$" and "[:digit:]" matches the latn numbering system digits.
            //
            // Example 2: for pattern "¤#,##0.00" with symbol "US$", there is no spacing between symbol
            // a  nd number, because `$` does not match "[:^S:]".
            //
            // Implementation note: here we do the best effort to infer the insertion.
            // We also assume that `beforeInsertBetween` and `afterInsertBetween` will never be `;`.
            const afterCurrency = currencyData.currencySpacing.afterInsertBetween;
            if (afterCurrency && !S_DOLLAR_UNICODE_REGEX.test(nonNameCurrencyPart)) {
                numberPattern = numberPattern.replace('¤0', `¤${afterCurrency}0`);
            }
            const beforeCurrency = currencyData.currencySpacing.beforeInsertBetween;
            if (beforeCurrency && !CARET_S_UNICODE_REGEX.test(nonNameCurrencyPart)) {
                numberPattern = numberPattern.replace('0¤', `0${beforeCurrency}¤`);
            }
        }
        // Now we start to substitute patterns
        // 1. replace strings like `0` and `#,##0.00` with `{0}`
        // 2. unquote characters (invariant: the quoted characters does not contain the special tokens)
        numberPattern = numberPattern
            .replace(CLDR_NUMBER_PATTERN, '{0}')
            .replace(/'(.)'/g, '$1');
        // The following tokens are special: `{0}`, `¤`, `%`, `-`, `+`, `{c:...}.
        const numberPatternParts = numberPattern.split(/({c:[^}]+}|\{0\}|[¤%\-\+])/g);
        const numberParts = [];
        const symbols = data.numbers.symbols[numberingSystem] ||
            data.numbers.symbols[defaultNumberingSystem];
        for (const part of numberPatternParts) {
            if (!part) {
                continue;
            }
            switch (part) {
                case '{0}': {
                    // We only need to handle scientific and engineering notation here.
                    numberParts.push(...paritionNumberIntoParts(symbols, numberResult, notation, exponent, numberingSystem, 
                    // If compact number pattern exists, do not insert group separators.
                    !compactNumberPattern && options.useGrouping));
                    break;
                }
                case '-':
                    numberParts.push({ type: 'minusSign', value: symbols.minusSign });
                    break;
                case '+':
                    numberParts.push({ type: 'plusSign', value: symbols.plusSign });
                    break;
                case '%':
                    numberParts.push({ type: 'percentSign', value: symbols.percentSign });
                    break;
                case '¤':
                    // Computed above when handling currency spacing.
                    numberParts.push({ type: 'currency', value: nonNameCurrencyPart });
                    break;
                default:
                    if (/^\{c:/.test(part)) {
                        numberParts.push({
                            type: 'compact',
                            value: part.substring(3, part.length - 1),
                        });
                    }
                    else {
                        // literal
                        numberParts.push({ type: 'literal', value: part });
                    }
                    break;
            }
        }
        // Part 2: interpolate unit pattern if necessary.
        // ----------------------------------------------
        switch (style) {
            case 'currency': {
                // `currencyDisplay: 'name'` has similar pattern handling as units.
                if (options.currencyDisplay === 'name') {
                    const unitPattern = (data.numbers.currency[numberingSystem] ||
                        data.numbers.currency[defaultNumberingSystem]).unitPattern;
                    // Select plural
                    let unitName;
                    const currencyNameData = data.currencies[options.currency];
                    if (currencyNameData) {
                        unitName = selectPlural(pl, 
                        // NOTE: Google Chrome's Intl.NumberFormat uses the original number to determine the plurality,
                        // but the mantissa for unit. We think this is a bug in ICU, but will still replicate the behavior.
                        // TODO: use original number.
                        numberResult.roundedNumber * Math.pow(10, exponent), currencyNameData.displayName);
                    }
                    else {
                        // Fallback for unknown currency
                        unitName = options.currency;
                    }
                    // Do {0} and {1} substitution
                    const unitPatternParts = unitPattern.split(/(\{[01]\})/g);
                    const result = [];
                    for (const part of unitPatternParts) {
                        switch (part) {
                            case '{0}':
                                result.push(...numberParts);
                                break;
                            case '{1}':
                                result.push({ type: 'currency', value: unitName });
                                break;
                            default:
                                if (part) {
                                    result.push({ type: 'literal', value: part });
                                }
                                break;
                        }
                    }
                    return result;
                }
                else {
                    return numberParts;
                }
            }
            case 'unit': {
                const { unit, unitDisplay } = options;
                let unitData = data.units.simple[unit];
                let unitPattern;
                if (unitData) {
                    // Simple unit pattern
                    unitPattern = selectPlural(pl, numberResult.roundedNumber, data.units.simple[unit][unitDisplay]);
                }
                else {
                    // See: http://unicode.org/reports/tr35/tr35-general.html#perUnitPatterns
                    // If cannot find unit in the simple pattern, it must be "per" compound pattern.
                    // Implementation note: we are not following TR-35 here because we need to format to parts!
                    const [numeratorUnit, denominatorUnit] = unit.split('-per-');
                    unitData = data.units.simple[numeratorUnit];
                    const numeratorUnitPattern = selectPlural(pl, numberResult.roundedNumber, data.units.simple[numeratorUnit][unitDisplay]);
                    const perUnitPattern = data.units.simple[denominatorUnit].perUnit[unitDisplay];
                    if (perUnitPattern) {
                        // perUnitPattern exists, combine it with numeratorUnitPattern
                        unitPattern = perUnitPattern.replace('{0}', numeratorUnitPattern);
                    }
                    else {
                        // get compoundUnit pattern (e.g. "{0} per {1}"), repalce {0} with numerator pattern and {1} with
                        // the denominator pattern in singular form.
                        const perPattern = data.units.compound.per[unitDisplay];
                        const denominatorPattern = selectPlural(pl, 1, data.units.simple[denominatorUnit][unitDisplay]);
                        unitPattern = unitPattern = perPattern
                            .replace('{0}', numeratorUnitPattern)
                            .replace('{1}', denominatorPattern.replace('{0}', ''));
                    }
                }
                const result = [];
                // We need spacing around "{0}" because they are not treated as "unit" parts, but "literal".
                for (const part of unitPattern.split(/(\s*\{0\}\s*)/)) {
                    const interpolateMatch = /^(\s*)\{0\}(\s*)$/.exec(part);
                    if (interpolateMatch) {
                        // Space before "{0}"
                        if (interpolateMatch[1]) {
                            result.push({ type: 'literal', value: interpolateMatch[1] });
                        }
                        // "{0}" itself
                        result.push(...numberParts);
                        // Space after "{0}"
                        if (interpolateMatch[2]) {
                            result.push({ type: 'literal', value: interpolateMatch[2] });
                        }
                    }
                    else if (part) {
                        result.push({ type: 'unit', value: part });
                    }
                }
                return result;
            }
            default:
                return numberParts;
        }
    }
    // A subset of https://tc39.es/ecma402/#sec-partitionnotationsubpattern
    // Plus the exponent parts handling.
    function paritionNumberIntoParts(symbols, numberResult, notation, exponent, numberingSystem, useGrouping) {
        const result = [];
        // eslint-disable-next-line prefer-const
        let { formattedString: n, roundedNumber: x } = numberResult;
        if (isNaN(x)) {
            return [{ type: 'nan', value: n }];
        }
        else if (!isFinite(x)) {
            return [{ type: 'infinity', value: n }];
        }
        const digitReplacementTable = digitMapping$1[numberingSystem];
        if (digitReplacementTable) {
            n = n.replace(/\d/g, digit => digitReplacementTable[+digit] || digit);
        }
        // TODO: Else use an implementation dependent algorithm to map n to the appropriate
        // representation of n in the given numbering system.
        const decimalSepIndex = n.indexOf('.');
        let integer;
        let fraction;
        if (decimalSepIndex > 0) {
            integer = n.slice(0, decimalSepIndex);
            fraction = n.slice(decimalSepIndex + 1);
        }
        else {
            integer = n;
        }
        // The weird compact and x >= 10000 check is to ensure consistency with Node.js and Chrome.
        // Note that `de` does not have compact form for thousands, but Node.js does not insert grouping separator
        // unless the rounded number is greater than 10000:
        //   NumberFormat('de', {notation: 'compact', compactDisplay: 'short'}).format(1234) //=> "1234"
        //   NumberFormat('de').format(1234) //=> "1.234"
        if (useGrouping && (notation !== 'compact' || x >= 10000)) {
            const groupSepSymbol = symbols.group;
            const groups = [];
            // Assuming that the group separator is always inserted between every 3 digits.
            let i = integer.length - 3;
            for (; i > 0; i -= 3) {
                groups.push(integer.slice(i, i + 3));
            }
            groups.push(integer.slice(0, i + 3));
            while (groups.length > 0) {
                const integerGroup = groups.pop();
                result.push({ type: 'integer', value: integerGroup });
                if (groups.length > 0) {
                    result.push({ type: 'group', value: groupSepSymbol });
                }
            }
        }
        else {
            result.push({ type: 'integer', value: integer });
        }
        if (fraction !== undefined) {
            result.push({ type: 'decimal', value: symbols.decimal }, { type: 'fraction', value: fraction });
        }
        if ((notation === 'scientific' || notation === 'engineering') &&
            isFinite(x)) {
            result.push({ type: 'exponentSeparator', value: symbols.exponential });
            if (exponent < 0) {
                result.push({ type: 'exponentMinusSign', value: symbols.minusSign });
                exponent = -exponent;
            }
            const exponentResult = toRawFixed(exponent, 0, 0);
            result.push({
                type: 'exponentInteger',
                value: exponentResult.formattedString,
            });
        }
        return result;
    }
    function getPatternForSign(pattern, sign) {
        if (pattern.indexOf(';') < 0) {
            pattern = `${pattern};-${pattern}`;
        }
        const [zeroPattern, negativePattern] = pattern.split(';');
        switch (sign) {
            case 0:
                return zeroPattern;
            case -1:
                return negativePattern;
            default:
                return negativePattern.indexOf('-') >= 0
                    ? negativePattern.replace(/-/g, '+')
                    : `+${zeroPattern}`;
        }
    }
    // Find the CLDR pattern for compact notation based on the magnitude of data and style.
    //
    // Example return value: "¤ {c:laki}000;¤{c:laki} -0" (`sw` locale):
    // - Notice the `{c:...}` token that wraps the compact literal.
    // - The consecutive zeros are normalized to single zero to match CLDR_NUMBER_PATTERN.
    //
    // Returning null means the compact display pattern cannot be found.
    function getCompactDisplayPattern(numberResult, pl, data, style, compactDisplay, currencyDisplay, numberingSystem) {
        var _a;
        const { roundedNumber, sign, magnitude } = numberResult;
        const magnitudeKey = String(Math.pow(10, magnitude));
        const defaultNumberingSystem = data.numbers.nu[0];
        let pattern;
        if (style === 'currency' && currencyDisplay !== 'name') {
            const byNumberingSystem = data.numbers.currency;
            const currencyData = byNumberingSystem[numberingSystem] ||
                byNumberingSystem[defaultNumberingSystem];
            // NOTE: compact notation ignores currencySign!
            const compactPluralRules = (_a = currencyData.short) === null || _a === void 0 ? void 0 : _a[magnitudeKey];
            if (!compactPluralRules) {
                return null;
            }
            pattern = selectPlural(pl, roundedNumber, compactPluralRules);
        }
        else {
            const byNumberingSystem = data.numbers.decimal;
            const byCompactDisplay = byNumberingSystem[numberingSystem] ||
                byNumberingSystem[defaultNumberingSystem];
            const compactPlaralRule = byCompactDisplay[compactDisplay][magnitudeKey];
            if (!compactPlaralRule) {
                return null;
            }
            pattern = selectPlural(pl, roundedNumber, compactPlaralRule);
        }
        // See https://unicode.org/reports/tr35/tr35-numbers.html#Compact_Number_Formats
        // > If the value is precisely “0”, either explicit or defaulted, then the normal number format
        // > pattern for that sort of object is supplied.
        if (pattern === '0') {
            return null;
        }
        pattern = getPatternForSign(pattern, sign)
            // Extract compact literal from the pattern
            .replace(/([^\s;\-\+\d¤]+)/g, '{c:$1}')
            // We replace one or more zeros with a single zero so it matches `CLDR_NUMBER_PATTERN`.
            .replace(/0+/, '0');
        return pattern;
    }
    function selectPlural(pl, x, rules) {
        return rules[pl.select(x)] || rules.other;
    }

    const internalSlotMap = new WeakMap();
    function getInternalSlots(x) {
        let internalSlots = internalSlotMap.get(x);
        if (!internalSlots) {
            internalSlots = Object.create(null);
            internalSlotMap.set(x, internalSlots);
        }
        return internalSlots;
    }

    const VALID_NUMBERING_SYSTEM_NAMES = new Set(names);
    const RESOLVED_OPTIONS_KEYS = [
        'locale',
        'numberingSystem',
        'style',
        'currency',
        'currencyDisplay',
        'currencySign',
        'unit',
        'unitDisplay',
        'minimumIntegerDigits',
        'minimumFractionDigits',
        'maximumFractionDigits',
        'minimumSignificantDigits',
        'maximumSignificantDigits',
        'useGrouping',
        'notation',
        'compactDisplay',
        'signDisplay',
    ];
    /**
     * Chop off the unicode extension from the locale string.
     */
    function removeUnicodeExtensionFromLocale(canonicalLocale) {
        const extensionIndex = canonicalLocale.indexOf('-u-');
        return extensionIndex >= 0
            ? canonicalLocale.slice(0, extensionIndex)
            : canonicalLocale;
    }
    /**
     * https://tc39.es/ecma402/#sec-currencydigits
     */
    function currencyDigits$1(c) {
        return hasOwnProperty(currencyDigitsData, c)
            ? currencyDigitsData[c]
            : 2;
    }
    /**
     * https://tc39.es/ecma402/#sec-initializenumberformat
     */
    function initializeNumberFormat(nf, locales, opts) {
        // @ts-ignore
        const requestedLocales = Intl.getCanonicalLocales(locales);
        const options = opts === undefined ? Object.create(null) : toObject(opts);
        const opt = Object.create(null);
        const matcher = getOption(options, 'localeMatcher', 'string', ['lookup', 'best fit'], 'best fit');
        opt.localeMatcher = matcher;
        const numberingSystem = getOption(options, 'numberingSystem', 'string', undefined, undefined);
        if (numberingSystem !== undefined &&
            !VALID_NUMBERING_SYSTEM_NAMES.has(numberingSystem)) {
            // 8.a. If numberingSystem does not match the Unicode Locale Identifier type nonterminal,
            // throw a RangeError exception.
            throw RangeError(`Invalid numberingSystems: ${numberingSystem}`);
        }
        opt.nu = numberingSystem;
        const { localeData } = NumberFormat;
        const r = createResolveLocale(NumberFormat.getDefaultLocale)(NumberFormat.availableLocales, requestedLocales, opt, 
        // [[RelevantExtensionKeys]] slot, which is a constant
        ['nu'], localeData);
        const dataLocaleData = localeData[removeUnicodeExtensionFromLocale(r.locale)];
        const internalSlots = getInternalSlots(nf);
        internalSlots.locale = r.locale;
        internalSlots.dataLocale = r.dataLocale;
        internalSlots.numberingSystem = r.nu;
        internalSlots.dataLocaleData = dataLocaleData;
        setNumberFormatUnitOptions(nf, options);
        const style = internalSlots.style;
        let mnfdDefault;
        let mxfdDefault;
        if (style === 'currency') {
            const currency = internalSlots.currency;
            const cDigits = currencyDigits$1(currency);
            mnfdDefault = cDigits;
            mxfdDefault = cDigits;
        }
        else {
            mnfdDefault = 0;
            mxfdDefault = style === 'percent' ? 0 : 3;
        }
        const notation = getOption(options, 'notation', 'string', ['standard', 'scientific', 'engineering', 'compact'], 'standard');
        internalSlots.notation = notation;
        setNumberFormatDigitOptions(internalSlots, options, mnfdDefault, mxfdDefault, notation);
        const compactDisplay = getOption(options, 'compactDisplay', 'string', ['short', 'long'], 'short');
        if (notation === 'compact') {
            internalSlots.compactDisplay = compactDisplay;
        }
        const useGrouping = getOption(options, 'useGrouping', 'boolean', undefined, true);
        internalSlots.useGrouping = useGrouping;
        const signDisplay = getOption(options, 'signDisplay', 'string', ['auto', 'never', 'always', 'exceptZero'], 'auto');
        internalSlots.signDisplay = signDisplay;
        return nf;
    }
    /**
     * https://tc39.es/ecma402/#sec-formatnumberstring
     */
    function partitionNumberPattern(numberFormat, x) {
        const internalSlots = getInternalSlots(numberFormat);
        const { pl, dataLocaleData, numberingSystem } = internalSlots;
        const symbols = dataLocaleData.numbers.symbols[numberingSystem] ||
            dataLocaleData.numbers.symbols[dataLocaleData.numbers.nu[0]];
        let magnitude = 0;
        let exponent = 0;
        let n;
        if (isNaN(x)) {
            n = symbols.nan;
        }
        else if (!isFinite(x)) {
            n = symbols.infinity;
        }
        else {
            if (internalSlots.style === 'percent') {
                x *= 100;
            }
            [exponent, magnitude] = computeExponent(numberFormat, x);
            // Preserve more precision by doing multiplication when exponent is negative.
            x = exponent < 0 ? x * Math.pow(10, -exponent) : x / Math.pow(10, exponent);
            const formatNumberResult = formatNumericToString(internalSlots, x);
            n = formatNumberResult.formattedString;
            x = formatNumberResult.roundedNumber;
        }
        // Based on https://tc39.es/ecma402/#sec-getnumberformatpattern
        // We need to do this before `x` is rounded.
        let sign;
        const signDisplay = internalSlots.signDisplay;
        switch (signDisplay) {
            case 'never':
                sign = 0;
                break;
            case 'auto':
                if (objectIs(x, 0) || x > 0 || isNaN(x)) {
                    sign = 0;
                }
                else {
                    sign = -1;
                }
                break;
            case 'always':
                if (objectIs(x, 0) || x > 0 || isNaN(x)) {
                    sign = 1;
                }
                else {
                    sign = -1;
                }
                break;
            default:
                // x === 0 -> x is 0 or x is -0
                if (x === 0 || isNaN(x)) {
                    sign = 0;
                }
                else if (x > 0) {
                    sign = 1;
                }
                else {
                    sign = -1;
                }
        }
        return formatToParts({ roundedNumber: x, formattedString: n, exponent, magnitude, sign }, internalSlots.dataLocaleData, pl, internalSlots);
    }
    function formatNumericToParts(numberFormat, x) {
        return partitionNumberPattern(numberFormat, x);
    }
    /**
     * https://tc39.es/ecma402/#sec-intl-numberformat-constructor
     */
    const NumberFormat = function (locales, options) {
        // Cannot use `new.target` bc of IE11 & TS transpiles it to something else
        if (!this || !(this instanceof NumberFormat)) {
            return new NumberFormat(locales, options);
        }
        initializeNumberFormat(this, locales, options);
        const internalSlots = getInternalSlots(this);
        const dataLocale = internalSlots.dataLocale;
        const dataLocaleData = NumberFormat.localeData[dataLocale];
        invariant(dataLocaleData !== undefined, `Cannot load locale-dependent data for ${dataLocale}.`);
        internalSlots.pl = new Intl.PluralRules(dataLocale, {
            minimumFractionDigits: internalSlots.minimumFractionDigits,
            maximumFractionDigits: internalSlots.maximumFractionDigits,
            minimumIntegerDigits: internalSlots.minimumIntegerDigits,
            minimumSignificantDigits: internalSlots.minimumSignificantDigits,
            maximumSignificantDigits: internalSlots.maximumSignificantDigits,
        });
    };
    defineProperty(NumberFormat.prototype, 'formatToParts', {
        value: function formatToParts(x) {
            return formatNumericToParts(this, toNumeric(x));
        },
    });
    defineProperty(NumberFormat.prototype, 'resolvedOptions', {
        value: function resolvedOptions() {
            if (typeof this !== 'object' || !(this instanceof NumberFormat)) {
                throw TypeError('Method Intl.NumberFormat.prototype.resolvedOptions called on incompatible receiver');
            }
            const internalSlots = getInternalSlots(this);
            const ro = {};
            for (const key of RESOLVED_OPTIONS_KEYS) {
                const value = internalSlots[key];
                if (value !== undefined) {
                    ro[key] = value;
                }
            }
            return ro;
        },
    });
    const formatDescriptor = {
        enumerable: false,
        configurable: true,
        get() {
            if (typeof this !== 'object' || !(this instanceof NumberFormat)) {
                throw TypeError('Intl.NumberFormat format property accessor called on incompatible receiver');
            }
            const internalSlots = getInternalSlots(this);
            // eslint-disable-next-line @typescript-eslint/no-this-alias
            const numberFormat = this;
            let boundFormat = internalSlots.boundFormat;
            if (boundFormat === undefined) {
                // https://tc39.es/proposal-unified-intl-numberformat/section11/numberformat_diff_out.html#sec-number-format-functions
                boundFormat = (value) => {
                    // TODO: check bigint
                    const x = toNumeric(value);
                    return numberFormat
                        .formatToParts(x)
                        .map(x => x.value)
                        .join('');
                };
                // https://github.com/tc39/test262/blob/master/test/intl402/NumberFormat/prototype/format/format-function-name.js
                Object.defineProperty(boundFormat, 'name', {
                    configurable: true,
                    enumerable: false,
                    writable: false,
                    value: '',
                });
                internalSlots.boundFormat = boundFormat;
            }
            return boundFormat;
        },
    };
    // https://github.com/tc39/test262/blob/master/test/intl402/NumberFormat/prototype/format/name.js
    Object.defineProperty(formatDescriptor.get, 'name', {
        configurable: true,
        enumerable: false,
        writable: false,
        value: 'get format',
    });
    Object.defineProperty(NumberFormat.prototype, 'format', formatDescriptor);
    // Static properties
    defineProperty(NumberFormat, 'supportedLocalesOf', {
        value: function supportedLocalesOf(locales, options) {
            return supportedLocales(NumberFormat.availableLocales, Intl.getCanonicalLocales(locales), options);
        },
    });
    NumberFormat.__addLocaleData = function __addLocaleData(...data) {
        for (const datum of data) {
            const availableLocales = datum.availableLocales;
            for (const locale of availableLocales) {
                try {
                    NumberFormat.localeData[locale] = unpackData(locale, datum);
                }
                catch (e) {
                    // Ignore if we got no data
                }
            }
        }
        NumberFormat.availableLocales = Object.keys(NumberFormat.localeData);
        if (!NumberFormat.__defaultLocale) {
            NumberFormat.__defaultLocale = NumberFormat.availableLocales[0];
        }
    };
    NumberFormat.__defaultLocale = 'en';
    NumberFormat.localeData = {};
    NumberFormat.availableLocales = [];
    NumberFormat.getDefaultLocale = () => {
        return NumberFormat.__defaultLocale;
    };
    NumberFormat.polyfilled = true;
    /**
     * https://tc39.es/ecma402/#sec-setnumberformatunitoptions
     */
    function setNumberFormatUnitOptions(nf, options = Object.create(null)) {
        const internalSlots = getInternalSlots(nf);
        const style = getOption(options, 'style', 'string', ['decimal', 'percent', 'currency', 'unit'], 'decimal');
        internalSlots.style = style;
        const currency = getOption(options, 'currency', 'string', undefined, undefined);
        if (currency !== undefined && !isWellFormedCurrencyCode(currency)) {
            throw RangeError('Malformed currency code');
        }
        if (style === 'currency' && currency === undefined) {
            throw TypeError('currency cannot be undefined');
        }
        const currencyDisplay = getOption(options, 'currencyDisplay', 'string', ['code', 'symbol', 'narrowSymbol', 'name'], 'symbol');
        const currencySign = getOption(options, 'currencySign', 'string', ['standard', 'accounting'], 'standard');
        const unit = getOption(options, 'unit', 'string', undefined, undefined);
        if (unit !== undefined && !isWellFormedUnitIdentifier(unit)) {
            throw RangeError('Invalid unit argument for Intl.NumberFormat()');
        }
        if (style === 'unit' && unit === undefined) {
            throw TypeError('unit cannot be undefined');
        }
        const unitDisplay = getOption(options, 'unitDisplay', 'string', ['short', 'narrow', 'long'], 'short');
        if (style === 'currency') {
            internalSlots.currency = currency.toUpperCase();
            internalSlots.currencyDisplay = currencyDisplay;
            internalSlots.currencySign = currencySign;
        }
        if (style === 'unit') {
            internalSlots.unit = unit;
            internalSlots.unitDisplay = unitDisplay;
        }
    }
    /**
     * The abstract operation ComputeExponent computes an exponent (power of ten) by which to scale x
     * according to the number formatting settings. It handles cases such as 999 rounding up to 1000,
     * requiring a different exponent.
     *
     * NOT IN SPEC: it returns [exponent, magnitude].
     */
    function computeExponent(numberFormat, x) {
        if (x === 0) {
            return [0, 0];
        }
        if (x < 0) {
            x = -x;
        }
        const magnitude = getMagnitude(x);
        const exponent = computeExponentForMagnitude(numberFormat, magnitude);
        // Preserve more precision by doing multiplication when exponent is negative.
        x = exponent < 0 ? x * Math.pow(10, -exponent) : x / Math.pow(10, exponent);
        const formatNumberResult = formatNumericToString(getInternalSlots(numberFormat), x);
        if (formatNumberResult.roundedNumber === 0) {
            return [exponent, magnitude];
        }
        const newMagnitude = getMagnitude(formatNumberResult.roundedNumber);
        if (newMagnitude === magnitude - exponent) {
            return [exponent, magnitude];
        }
        return [
            computeExponentForMagnitude(numberFormat, magnitude + 1),
            magnitude + 1,
        ];
    }
    /**
     * The abstract operation ComputeExponentForMagnitude computes an exponent by which to scale a
     * number of the given magnitude (power of ten of the most significant digit) according to the
     * locale and the desired notation (scientific, engineering, or compact).
     */
    function computeExponentForMagnitude(numberFormat, magnitude) {
        const internalSlots = getInternalSlots(numberFormat);
        const { notation, dataLocaleData, numberingSystem } = internalSlots;
        switch (notation) {
            case 'standard':
                return 0;
            case 'scientific':
                return magnitude;
            case 'engineering':
                return Math.floor(magnitude / 3) * 3;
            default: {
                // Let exponent be an implementation- and locale-dependent (ILD) integer by which to scale a
                // number of the given magnitude in compact notation for the current locale.
                const { compactDisplay, style, currencyDisplay } = internalSlots;
                let thresholdMap;
                if (style === 'currency' && currencyDisplay !== 'name') {
                    const currency = dataLocaleData.numbers.currency[numberingSystem] ||
                        dataLocaleData.numbers.currency[dataLocaleData.numbers.nu[0]];
                    thresholdMap = currency.short;
                }
                else {
                    const decimal = dataLocaleData.numbers.decimal[numberingSystem] ||
                        dataLocaleData.numbers.decimal[dataLocaleData.numbers.nu[0]];
                    thresholdMap = compactDisplay === 'long' ? decimal.long : decimal.short;
                }
                if (!thresholdMap) {
                    return 0;
                }
                const num = String(Math.pow(10, magnitude));
                const thresholds = Object.keys(thresholdMap); // TODO: this can be pre-processed
                if (num < thresholds[0]) {
                    return 0;
                }
                if (num > thresholds[thresholds.length - 1]) {
                    return thresholds[thresholds.length - 1].length - 1;
                }
                const i = thresholds.indexOf(num);
                if (i === -1) {
                    return 0;
                }
                // See https://unicode.org/reports/tr35/tr35-numbers.html#Compact_Number_Formats
                // Special handling if the pattern is precisely `0`.
                const magnitudeKey = thresholds[i];
                // TODO: do we need to handle plural here?
                const compactPattern = thresholdMap[magnitudeKey].other;
                if (compactPattern === '0') {
                    return 0;
                }
                // Example: in zh-TW, `10000000` maps to `0000萬`. So we need to return 8 - 4 = 4 here.
                return (magnitudeKey.length -
                    thresholdMap[magnitudeKey].other.match(/0+/)[0].length);
            }
        }
    }
    function toNumeric(val) {
        if (typeof val === 'bigint') {
            return val;
        }
        return toNumber(val);
    }
    function toNumber(val) {
        if (val === undefined) {
            return NaN;
        }
        if (val === null) {
            return +0;
        }
        if (typeof val === 'boolean') {
            return val ? 1 : +0;
        }
        if (typeof val === 'number') {
            return val;
        }
        if (typeof val === 'symbol' || typeof val === 'bigint') {
            throw new TypeError('Cannot convert symbol/bigint to number');
        }
        return Number(val);
    }
    try {
        // IE11 does not have Symbol
        if (typeof Symbol !== 'undefined') {
            Object.defineProperty(NumberFormat.prototype, Symbol.toStringTag, {
                configurable: true,
                enumerable: false,
                writable: false,
                value: 'Object',
            });
        }
        // https://github.com/tc39/test262/blob/master/test/intl402/NumberFormat/length.js
        Object.defineProperty(NumberFormat.prototype.constructor, 'length', {
            configurable: true,
            enumerable: false,
            writable: false,
            value: 0,
        });
        // https://github.com/tc39/test262/blob/master/test/intl402/NumberFormat/supportedLocalesOf/length.js
        Object.defineProperty(NumberFormat.supportedLocalesOf, 'length', {
            configurable: true,
            enumerable: false,
            writable: false,
            value: 1,
        });
        Object.defineProperty(NumberFormat, 'prototype', {
            configurable: false,
            enumerable: false,
            writable: false,
            value: NumberFormat.prototype,
        });
    }
    catch (e) {
        // Meta fix so we're test262-compliant, not important
    }

    // eslint-disable-next-line import/no-cycle
    /**
     * Number.prototype.toLocaleString ponyfill
     * https://tc39.es/ecma402/#sup-number.prototype.tolocalestring
     */
    function toLocaleString(x, locales, options) {
        const numberFormat = new NumberFormat(locales, options);
        return numberFormat.format(x);
    }

    /* eslint-disable import/no-cycle */
    /**
     * Check if a formatting number with unit is supported
     * @public
     * @param unit unit to check
     */
    function isUnitSupported(unit) {
        try {
            new Intl.NumberFormat(undefined, {
                style: 'unit',
                // @ts-ignore
                unit,
            });
        }
        catch (e) {
            return false;
        }
        return true;
    }

    if (!isUnitSupported('bit')) {
        defineProperty(Intl, 'NumberFormat', { value: NumberFormat });
        defineProperty(Number.prototype, 'toLocaleString', {
            value: function toLocaleString$1(locales, options) {
                return toLocaleString(this, locales, options);
            },
        });
    }

    var commonjsGlobal = typeof globalThis !== 'undefined' ? globalThis : typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

    function unwrapExports (x) {
    	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
    }

    function createCommonjsModule(fn, module) {
    	return module = { exports: {} }, fn(module, module.exports), module.exports;
    }

    function getCjsExportFromNamespace (n) {
    	return n && n['default'] || n;
    }

    var parser = createCommonjsModule(function (module, exports) {
    var __assign = (commonjsGlobal && commonjsGlobal.__assign) || function () {
        __assign = Object.assign || function(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                    t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    var ALPHANUM_1_8 = /^[a-z0-9]{1,8}$/i;
    var ALPHANUM_2_8 = /^[a-z0-9]{2,8}$/i;
    var ALPHANUM_3_8 = /^[a-z0-9]{3,8}$/i;
    var KEY_REGEX = /^[a-z0-9][a-z]$/i;
    var TYPE_REGEX = /^[a-z0-9]{3,8}$/i;
    var ALPHA_4 = /^[a-z]{4}$/i;
    // alphanum-[tTuUxX]
    var OTHER_EXTENSION_TYPE = /^[0-9a-svwyz]$/i;
    var UNICODE_REGION_SUBTAG_REGEX = /^([a-z]{2}|[0-9]{3})$/i;
    var UNICODE_VARIANT_SUBTAG_REGEX = /^([a-z0-9]{5,8}|[0-9][a-z0-9]{3})$/i;
    var UNICODE_LANGUAGE_SUBTAG_REGEX = /^([a-z]{2,3}|[a-z]{5,8})$/i;
    var TKEY_REGEX = /^[a-z][0-9]$/i;
    exports.SEPARATOR = '-';
    function isUnicodeLanguageSubtag(lang) {
        return UNICODE_LANGUAGE_SUBTAG_REGEX.test(lang);
    }
    exports.isUnicodeLanguageSubtag = isUnicodeLanguageSubtag;
    function isStructurallyValidLanguageTag(tag) {
        try {
            parseUnicodeLanguageId(tag.split(exports.SEPARATOR));
        }
        catch (e) {
            return false;
        }
        return true;
    }
    exports.isStructurallyValidLanguageTag = isStructurallyValidLanguageTag;
    function isUnicodeRegionSubtag(region) {
        return UNICODE_REGION_SUBTAG_REGEX.test(region);
    }
    exports.isUnicodeRegionSubtag = isUnicodeRegionSubtag;
    function isUnicodeScriptSubtag(script) {
        return ALPHA_4.test(script);
    }
    exports.isUnicodeScriptSubtag = isUnicodeScriptSubtag;
    function isUnicodeVariantSubtag(variant) {
        return UNICODE_VARIANT_SUBTAG_REGEX.test(variant);
    }
    exports.isUnicodeVariantSubtag = isUnicodeVariantSubtag;
    function parseUnicodeLanguageId(chunks) {
        if (typeof chunks === 'string') {
            chunks = chunks.split(exports.SEPARATOR);
        }
        var lang = chunks.shift();
        if (!lang) {
            throw new RangeError('Missing unicode_language_subtag');
        }
        if (lang === 'root') {
            return { lang: 'root', variants: [] };
        }
        // unicode_language_subtag
        if (!isUnicodeLanguageSubtag(lang)) {
            throw new RangeError('Malformed unicode_language_subtag');
        }
        var script;
        // unicode_script_subtag
        if (isUnicodeScriptSubtag(chunks[0])) {
            script = chunks.shift();
        }
        var region;
        // unicode_region_subtag
        if (isUnicodeRegionSubtag(chunks[0])) {
            region = chunks.shift();
        }
        var variants = {};
        while (chunks.length && isUnicodeVariantSubtag(chunks[0])) {
            var variant = chunks.shift();
            if (variant in variants) {
                throw new RangeError("Duplicate variant \"" + variant + "\"");
            }
            variants[variant] = 1;
        }
        return {
            lang: lang,
            script: script,
            region: region,
            variants: Object.keys(variants),
        };
    }
    exports.parseUnicodeLanguageId = parseUnicodeLanguageId;
    function parseUnicodeExtension(chunks) {
        var keywords = [];
        var keyword;
        while (chunks.length && (keyword = parseKeyword(chunks))) {
            keywords.push(keyword);
        }
        if (keywords.length) {
            return {
                type: 'u',
                keywords: keywords,
                attributes: [],
            };
        }
        // Mix of attributes & keywords
        // Check for attributes first
        var attributes = [];
        while (chunks.length && ALPHANUM_3_8.test(chunks[0])) {
            attributes.push(chunks.shift());
        }
        while (chunks.length && (keyword = parseKeyword(chunks))) {
            keywords.push(keyword);
        }
        if (keywords.length || attributes.length) {
            return {
                type: 'u',
                attributes: attributes,
                keywords: keywords,
            };
        }
        throw new RangeError('Malformed unicode_extension');
    }
    function parseKeyword(chunks) {
        var key;
        if (!KEY_REGEX.test(chunks[0])) {
            return;
        }
        key = chunks.shift();
        var type = [];
        while (chunks.length && TYPE_REGEX.test(chunks[0])) {
            type.push(chunks.shift());
        }
        var value = '';
        if (type.length) {
            value = type.join(exports.SEPARATOR);
        }
        return [key, value];
    }
    function parseTransformedExtension(chunks) {
        var lang;
        try {
            lang = parseUnicodeLanguageId(chunks);
        }
        catch (e) {
            // Try just parsing tfield
        }
        var fields = [];
        while (chunks.length && TKEY_REGEX.test(chunks[0])) {
            var key = chunks.shift();
            var value = [];
            while (chunks.length && ALPHANUM_3_8.test(chunks[0])) {
                value.push(chunks.shift());
            }
            if (!value.length) {
                throw new RangeError("Missing tvalue for tkey \"" + key + "\"");
            }
            fields.push([key, value.join(exports.SEPARATOR)]);
        }
        if (fields.length) {
            return {
                type: 't',
                fields: fields,
                lang: lang,
            };
        }
        throw new RangeError('Malformed transformed_extension');
    }
    function parsePuExtension(chunks) {
        var exts = [];
        while (chunks.length && ALPHANUM_1_8.test(chunks[0])) {
            exts.push(chunks.shift());
        }
        if (exts.length) {
            return {
                type: 'x',
                value: exts.join(exports.SEPARATOR),
            };
        }
        throw new RangeError('Malformed private_use_extension');
    }
    function parseOtherExtensionValue(chunks) {
        var exts = [];
        while (chunks.length && ALPHANUM_2_8.test(chunks[0])) {
            exts.push(chunks.shift());
        }
        if (exts.length) {
            return exts.join(exports.SEPARATOR);
        }
        return '';
    }
    function parseExtensions(chunks) {
        if (!chunks.length) {
            return { extensions: [] };
        }
        var extensions = [];
        var unicodeExtension;
        var transformedExtension;
        var puExtension;
        var otherExtensionMap = {};
        do {
            var type = chunks.shift();
            switch (type) {
                case 'u':
                case 'U':
                    if (unicodeExtension) {
                        throw new RangeError('There can only be 1 -u- extension');
                    }
                    unicodeExtension = parseUnicodeExtension(chunks);
                    extensions.push(unicodeExtension);
                    break;
                case 't':
                case 'T':
                    if (transformedExtension) {
                        throw new RangeError('There can only be 1 -t- extension');
                    }
                    transformedExtension = parseTransformedExtension(chunks);
                    extensions.push(transformedExtension);
                    break;
                case 'x':
                case 'X':
                    if (puExtension) {
                        throw new RangeError('There can only be 1 -x- extension');
                    }
                    puExtension = parsePuExtension(chunks);
                    extensions.push(puExtension);
                    break;
                default:
                    if (!OTHER_EXTENSION_TYPE.test(type)) {
                        throw new RangeError('Malformed extension type');
                    }
                    if (type in otherExtensionMap) {
                        throw new RangeError("There can only be 1 -" + type + "- extension");
                    }
                    var extension = {
                        type: type,
                        value: parseOtherExtensionValue(chunks),
                    };
                    otherExtensionMap[extension.type] = extension;
                    extensions.push(extension);
                    break;
            }
        } while (chunks.length);
        return { extensions: extensions };
    }
    function parseUnicodeLocaleId(locale) {
        var chunks = locale.split(exports.SEPARATOR);
        var lang = parseUnicodeLanguageId(chunks);
        return __assign({ lang: lang }, parseExtensions(chunks));
    }
    exports.parseUnicodeLocaleId = parseUnicodeLocaleId;

    });

    unwrapExports(parser);
    var parser_1 = parser.SEPARATOR;
    var parser_2 = parser.isUnicodeLanguageSubtag;
    var parser_3 = parser.isStructurallyValidLanguageTag;
    var parser_4 = parser.isUnicodeRegionSubtag;
    var parser_5 = parser.isUnicodeScriptSubtag;
    var parser_6 = parser.isUnicodeVariantSubtag;
    var parser_7 = parser.parseUnicodeLanguageId;
    var parser_8 = parser.parseUnicodeLocaleId;

    var emitter = createCommonjsModule(function (module, exports) {
    var __spreadArrays = (commonjsGlobal && commonjsGlobal.__spreadArrays) || function () {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    };
    Object.defineProperty(exports, "__esModule", { value: true });
    function emitUnicodeLanguageId(lang) {
        if (!lang) {
            return '';
        }
        return __spreadArrays([lang.lang, lang.script, lang.region], (lang.variants || [])).filter(Boolean)
            .join('-');
    }
    exports.emitUnicodeLanguageId = emitUnicodeLanguageId;
    function emitUnicodeLocaleId(_a) {
        var lang = _a.lang, extensions = _a.extensions;
        var chunks = [emitUnicodeLanguageId(lang)];
        for (var _i = 0, extensions_1 = extensions; _i < extensions_1.length; _i++) {
            var ext = extensions_1[_i];
            chunks.push(ext.type);
            switch (ext.type) {
                case 'u':
                    chunks.push.apply(chunks, __spreadArrays(ext.attributes, ext.keywords.reduce(function (all, kv) { return all.concat(kv); }, [])));
                    break;
                case 't':
                    chunks.push.apply(chunks, __spreadArrays([emitUnicodeLanguageId(ext.lang)], ext.fields.reduce(function (all, kv) { return all.concat(kv); }, [])));
                    break;
                default:
                    chunks.push(ext.value);
                    break;
            }
        }
        return chunks.filter(Boolean).join('-');
    }
    exports.emitUnicodeLocaleId = emitUnicodeLocaleId;

    });

    unwrapExports(emitter);
    var emitter_1 = emitter.emitUnicodeLanguageId;
    var emitter_2 = emitter.emitUnicodeLocaleId;

    var aliases = createCommonjsModule(function (module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });
    /* @generated */
    // prettier-ignore  
    exports.languageAlias = { "aa-SAAHO": "ssy", "aam": "aas", "aar": "aa", "abk": "ab", "adp": "dz", "afr": "af", "aju": "jrb", "aka": "ak", "alb": "sq", "als": "sq", "amh": "am", "ara": "ar", "arb": "ar", "arg": "an", "arm": "hy", "art-lojban": "jbo", "asd": "snz", "asm": "as", "aue": "ktz", "ava": "av", "ave": "ae", "aym": "ay", "ayr": "ay", "ayx": "nun", "az-AZ": "az-Latn-AZ", "aze": "az", "azj": "az", "bak": "ba", "bam": "bm", "baq": "eu", "bcc": "bal", "bcl": "bik", "bel": "be", "ben": "bn", "bgm": "bcg", "bh": "bho", "bih": "bho", "bis": "bi", "bjd": "drl", "bod": "bo", "bos": "bs", "bre": "br", "bs-BA": "bs-Latn-BA", "bul": "bg", "bur": "my", "bxk": "luy", "bxr": "bua", "cat": "ca", "ccq": "rki", "cel-gaulish": "xtg-x-cel-gaulish", "ces": "cs", "cha": "ch", "che": "ce", "chi": "zh", "chu": "cu", "chv": "cv", "cjr": "mom", "cka": "cmr", "cld": "syr", "cmk": "xch", "cmn": "zh", "cnr": "sr-ME", "cor": "kw", "cos": "co", "coy": "pij", "cqu": "quh", "cre": "cr", "cwd": "cr", "cym": "cy", "cze": "cs", "dan": "da", "deu": "de", "dgo": "doi", "dhd": "mwr", "dik": "din", "diq": "zza", "dit": "dif", "div": "dv", "drh": "mn", "drw": "fa-af", "dut": "nl", "dzo": "dz", "ekk": "et", "ell": "el", "emk": "man", "eng": "en", "epo": "eo", "esk": "ik", "est": "et", "eus": "eu", "ewe": "ee", "fao": "fo", "fas": "fa", "fat": "ak", "fij": "fj", "fin": "fi", "fra": "fr", "fre": "fr", "fry": "fy", "fuc": "ff", "ful": "ff", "gav": "dev", "gaz": "om", "gbo": "grb", "geo": "ka", "ger": "de", "gfx": "vaj", "ggn": "gvr", "gla": "gd", "gle": "ga", "glg": "gl", "glv": "gv", "gno": "gon", "gre": "el", "grn": "gn", "gti": "nyc", "gug": "gn", "guj": "gu", "guv": "duz", "gya": "gba", "ha-Latn-GH": "ha-GH", "ha-Latn-NE": "ha-NE", "ha-Latn-NG": "ha-NG", "hat": "ht", "hau": "ha", "hbs": "sr-Latn", "hdn": "hai", "hea": "hmn", "heb": "he", "her": "hz", "him": "srx", "hin": "hi", "hmo": "ho", "hrr": "jal", "hrv": "hr", "hun": "hu", "hye": "hy", "i-ami": "ami", "i-bnn": "bnn", "i-hak": "hak", "i-klingon": "tlh", "i-lux": "lb", "i-navajo": "nv", "i-pwn": "pwn", "i-tao": "tao", "i-tay": "tay", "i-tsu": "tsu", "i-default": "en-x-i-default", "i-enochian": "und-x-i-enochian", "i-mingo": "see-x-i-mingo", "ibi": "opa", "ibo": "ig", "ice": "is", "ido": "io", "iii": "ii", "ike": "iu", "iku": "iu", "ile": "ie", "ilw": "gal", "in": "id", "ina": "ia", "ind": "id", "ipk": "ik", "isl": "is", "ita": "it", "iw": "he", "jav": "jv", "jeg": "oyb", "ji": "yi", "jpn": "ja", "jw": "jv", "kal": "kl", "kan": "kn", "kas": "ks", "kat": "ka", "kau": "kr", "kaz": "kk", "kgc": "tdf", "kgh": "kml", "khk": "mn", "khm": "km", "kik": "ki", "kin": "rw", "kir": "ky", "kk-Cyrl-KZ": "kk-KZ", "kmr": "ku", "knc": "kr", "kng": "kg", "knn": "kok", "koj": "kwv", "kom": "kv", "kon": "kg", "kor": "ko", "kpv": "kv", "krm": "bmf", "ks-Arab-IN": "ks-IN", "ktr": "dtp", "kua": "kj", "kur": "ku", "kvs": "gdj", "kwq": "yam", "kxe": "tvd", "ky-Cyrl-KG": "ky-KG", "kzj": "dtp", "kzt": "dtp", "lao": "lo", "lat": "la", "lav": "lv", "lbk": "bnc", "lii": "raq", "lim": "li", "lin": "ln", "lit": "lt", "llo": "ngt", "lmm": "rmx", "ltz": "lb", "lub": "lu", "lug": "lg", "lvs": "lv", "mac": "mk", "mah": "mh", "mal": "ml", "mao": "mi", "mar": "mr", "may": "ms", "meg": "cir", "mhr": "chm", "mkd": "mk", "mlg": "mg", "mlt": "mt", "mn-Cyrl-MN": "mn-MN", "mnk": "man", "mo": "ro", "mol": "ro", "mon": "mn", "mri": "mi", "ms-Latn-BN": "ms-BN", "ms-Latn-MY": "ms-MY", "ms-Latn-SG": "ms-SG", "msa": "ms", "mst": "mry", "mup": "raj", "mwj": "vaj", "mya": "my", "myd": "aog", "myt": "mry", "nad": "xny", "nau": "na", "nav": "nv", "nbl": "nr", "ncp": "kdz", "nde": "nd", "ndo": "ng", "nep": "ne", "nld": "nl", "nno": "nn", "nns": "nbr", "nnx": "ngv", "no": "nb", "no-bok": "nb", "no-BOKMAL": "nb", "no-nyn": "nn", "no-NYNORSK": "nn", "nob": "nb", "nor": "nb", "npi": "ne", "nts": "pij", "nya": "ny", "oci": "oc", "ojg": "oj", "oji": "oj", "ori": "or", "orm": "om", "ory": "or", "oss": "os", "oun": "vaj", "pa-IN": "pa-Guru-IN", "pa-PK": "pa-Arab-PK", "pan": "pa", "pbu": "ps", "pcr": "adx", "per": "fa", "pes": "fa", "pli": "pi", "plt": "mg", "pmc": "huw", "pmu": "phr", "pnb": "lah", "pol": "pl", "por": "pt", "ppa": "bfy", "ppr": "lcq", "prs": "fa-AF", "pry": "prt", "pus": "ps", "puz": "pub", "que": "qu", "quz": "qu", "rmy": "rom", "roh": "rm", "ron": "ro", "rum": "ro", "run": "rn", "rus": "ru", "sag": "sg", "san": "sa", "sca": "hle", "scc": "sr", "scr": "hr", "sgn-BE-FR": "sfb", "sgn-BE-NL": "vgt", "sgn-CH-DE": "sgg", "sh": "sr-Latn", "shi-MA": "shi-Tfng-MA", "sin": "si", "skk": "oyb", "slk": "sk", "slo": "sk", "slv": "sl", "sme": "se", "smo": "sm", "sna": "sn", "snd": "sd", "som": "so", "sot": "st", "spa": "es", "spy": "kln", "sqi": "sq", "sr-BA": "sr-Cyrl-BA", "sr-ME": "sr-Latn-ME", "sr-RS": "sr-Cyrl-RS", "sr-XK": "sr-Cyrl-XK", "src": "sc", "srd": "sc", "srp": "sr", "ssw": "ss", "sun": "su", "swa": "sw", "swc": "sw-CD", "swe": "sv", "swh": "sw", "tah": "ty", "tam": "ta", "tat": "tt", "tdu": "dtp", "tel": "te", "tgk": "tg", "tgl": "fil", "tha": "th", "thc": "tpo", "thx": "oyb", "tib": "bo", "tie": "ras", "tir": "ti", "tkk": "twm", "tl": "fil", "tlw": "weo", "tmp": "tyj", "tne": "kak", "tnf": "fa-af", "ton": "to", "tsf": "taj", "tsn": "tn", "tso": "ts", "ttq": "tmh", "tuk": "tk", "tur": "tr", "tw": "ak", "twi": "ak", "tzm-Latn-MA": "tzm-MA", "ug-Arab-CN": "ug-CN", "uig": "ug", "ukr": "uk", "umu": "del", "uok": "ema", "urd": "ur", "uz-AF": "uz-Arab-AF", "uz-UZ": "uz-Latn-UZ", "uzb": "uz", "uzn": "uz", "vai-LR": "vai-Vaii-LR", "ven": "ve", "vie": "vi", "vol": "vo", "wel": "cy", "wln": "wa", "wol": "wo", "xba": "cax", "xho": "xh", "xia": "acn", "xkh": "waw", "xpe": "kpe", "xsj": "suj", "xsl": "den", "ybd": "rki", "ydd": "yi", "yid": "yi", "yma": "lrr", "ymt": "mtm", "yor": "yo", "yos": "zom", "yue-CN": "yue-Hans-CN", "yue-HK": "yue-Hant-HK", "yuu": "yug", "zai": "zap", "zh-CN": "zh-Hans-CN", "zh-guoyu": "zh", "zh-hakka": "hak", "zh-HK": "zh-Hant-HK", "zh-min-nan": "nan", "zh-MO": "zh-Hant-MO", "zh-SG": "zh-Hans-SG", "zh-TW": "zh-Hant-TW", "zh-xiang": "hsn", "zh-min": "nan-x-zh-min", "zha": "za", "zho": "zh", "zsm": "ms", "zul": "zu", "zyb": "za" };
    exports.territoryAlias = { "100": "BG", "104": "MM", "108": "BI", "112": "BY", "116": "KH", "120": "CM", "124": "CA", "132": "CV", "136": "KY", "140": "CF", "144": "LK", "148": "TD", "152": "CL", "156": "CN", "158": "TW", "162": "CX", "166": "CC", "170": "CO", "172": "RU AM AZ BY GE KG KZ MD TJ TM UA UZ", "174": "KM", "175": "YT", "178": "CG", "180": "CD", "184": "CK", "188": "CR", "191": "HR", "192": "CU", "196": "CY", "200": "CZ SK", "203": "CZ", "204": "BJ", "208": "DK", "212": "DM", "214": "DO", "218": "EC", "222": "SV", "226": "GQ", "230": "ET", "231": "ET", "232": "ER", "233": "EE", "234": "FO", "238": "FK", "239": "GS", "242": "FJ", "246": "FI", "248": "AX", "249": "FR", "250": "FR", "254": "GF", "258": "PF", "260": "TF", "262": "DJ", "266": "GA", "268": "GE", "270": "GM", "275": "PS", "276": "DE", "278": "DE", "280": "DE", "288": "GH", "292": "GI", "296": "KI", "300": "GR", "304": "GL", "308": "GD", "312": "GP", "316": "GU", "320": "GT", "324": "GN", "328": "GY", "332": "HT", "334": "HM", "336": "VA", "340": "HN", "344": "HK", "348": "HU", "352": "IS", "356": "IN", "360": "ID", "364": "IR", "368": "IQ", "372": "IE", "376": "IL", "380": "IT", "384": "CI", "388": "JM", "392": "JP", "398": "KZ", "400": "JO", "404": "KE", "408": "KP", "410": "KR", "414": "KW", "417": "KG", "418": "LA", "422": "LB", "426": "LS", "428": "LV", "430": "LR", "434": "LY", "438": "LI", "440": "LT", "442": "LU", "446": "MO", "450": "MG", "454": "MW", "458": "MY", "462": "MV", "466": "ML", "470": "MT", "474": "MQ", "478": "MR", "480": "MU", "484": "MX", "492": "MC", "496": "MN", "498": "MD", "499": "ME", "500": "MS", "504": "MA", "508": "MZ", "512": "OM", "516": "NA", "520": "NR", "524": "NP", "528": "NL", "530": "CW SX BQ", "531": "CW", "532": "CW SX BQ", "533": "AW", "534": "SX", "535": "BQ", "536": "SA IQ", "540": "NC", "548": "VU", "554": "NZ", "558": "NI", "562": "NE", "566": "NG", "570": "NU", "574": "NF", "578": "NO", "580": "MP", "581": "UM", "582": "FM MH MP PW", "583": "FM", "584": "MH", "585": "PW", "586": "PK", "591": "PA", "598": "PG", "600": "PY", "604": "PE", "608": "PH", "612": "PN", "616": "PL", "620": "PT", "624": "GW", "626": "TL", "630": "PR", "634": "QA", "638": "RE", "642": "RO", "643": "RU", "646": "RW", "652": "BL", "654": "SH", "659": "KN", "660": "AI", "662": "LC", "663": "MF", "666": "PM", "670": "VC", "674": "SM", "678": "ST", "682": "SA", "686": "SN", "688": "RS", "690": "SC", "694": "SL", "702": "SG", "703": "SK", "704": "VN", "705": "SI", "706": "SO", "710": "ZA", "716": "ZW", "720": "YE", "724": "ES", "728": "SS", "729": "SD", "732": "EH", "736": "SD", "740": "SR", "744": "SJ", "748": "SZ", "752": "SE", "756": "CH", "760": "SY", "762": "TJ", "764": "TH", "768": "TG", "772": "TK", "776": "TO", "780": "TT", "784": "AE", "788": "TN", "792": "TR", "795": "TM", "796": "TC", "798": "TV", "800": "UG", "804": "UA", "807": "MK", "810": "RU AM AZ BY EE GE KZ KG LV LT MD TJ TM UA UZ", "818": "EG", "826": "GB", "830": "JE GG", "831": "GG", "832": "JE", "833": "IM", "834": "TZ", "840": "US", "850": "VI", "854": "BF", "858": "UY", "860": "UZ", "862": "VE", "876": "WF", "882": "WS", "886": "YE", "887": "YE", "890": "RS ME SI HR MK BA", "891": "RS ME", "894": "ZM", "958": "AA", "959": "QM", "960": "QN", "962": "QP", "963": "QQ", "964": "QR", "965": "QS", "966": "QT", "967": "EU", "968": "QV", "969": "QW", "970": "QX", "971": "QY", "972": "QZ", "973": "XA", "974": "XB", "975": "XC", "976": "XD", "977": "XE", "978": "XF", "979": "XG", "980": "XH", "981": "XI", "982": "XJ", "983": "XK", "984": "XL", "985": "XM", "986": "XN", "987": "XO", "988": "XP", "989": "XQ", "990": "XR", "991": "XS", "992": "XT", "993": "XU", "994": "XV", "995": "XW", "996": "XX", "997": "XY", "998": "XZ", "999": "ZZ", "004": "AF", "008": "AL", "010": "AQ", "012": "DZ", "016": "AS", "020": "AD", "024": "AO", "028": "AG", "031": "AZ", "032": "AR", "036": "AU", "040": "AT", "044": "BS", "048": "BH", "050": "BD", "051": "AM", "052": "BB", "056": "BE", "060": "BM", "062": "034 143", "064": "BT", "068": "BO", "070": "BA", "072": "BW", "074": "BV", "076": "BR", "084": "BZ", "086": "IO", "090": "SB", "092": "VG", "096": "BN", "AAA": "AA", "ABW": "AW", "AFG": "AF", "AGO": "AO", "AIA": "AI", "ALA": "AX", "ALB": "AL", "AN": "CW SX BQ", "AND": "AD", "ANT": "CW SX BQ", "ARE": "AE", "ARG": "AR", "ARM": "AM", "ASC": "AC", "ASM": "AS", "ATA": "AQ", "ATF": "TF", "ATG": "AG", "AUS": "AU", "AUT": "AT", "AZE": "AZ", "BDI": "BI", "BEL": "BE", "BEN": "BJ", "BES": "BQ", "BFA": "BF", "BGD": "BD", "BGR": "BG", "BHR": "BH", "BHS": "BS", "BIH": "BA", "BLM": "BL", "BLR": "BY", "BLZ": "BZ", "BMU": "BM", "BOL": "BO", "BRA": "BR", "BRB": "BB", "BRN": "BN", "BTN": "BT", "BU": "MM", "BUR": "MM", "BVT": "BV", "BWA": "BW", "CAF": "CF", "CAN": "CA", "CCK": "CC", "CHE": "CH", "CHL": "CL", "CHN": "CN", "CIV": "CI", "CMR": "CM", "COD": "CD", "COG": "CG", "COK": "CK", "COL": "CO", "COM": "KM", "CPT": "CP", "CPV": "CV", "CRI": "CR", "CS": "RS ME", "CT": "KI", "CUB": "CU", "CUW": "CW", "CXR": "CX", "CYM": "KY", "CYP": "CY", "CZE": "CZ", "DD": "DE", "DDR": "DE", "DEU": "DE", "DGA": "DG", "DJI": "DJ", "DMA": "DM", "DNK": "DK", "DOM": "DO", "DY": "BJ", "DZA": "DZ", "ECU": "EC", "EGY": "EG", "ERI": "ER", "ESH": "EH", "ESP": "ES", "EST": "EE", "ETH": "ET", "FIN": "FI", "FJI": "FJ", "FLK": "FK", "FQ": "AQ TF", "FRA": "FR", "FRO": "FO", "FSM": "FM", "FX": "FR", "FXX": "FR", "GAB": "GA", "GBR": "GB", "GEO": "GE", "GGY": "GG", "GHA": "GH", "GIB": "GI", "GIN": "GN", "GLP": "GP", "GMB": "GM", "GNB": "GW", "GNQ": "GQ", "GRC": "GR", "GRD": "GD", "GRL": "GL", "GTM": "GT", "GUF": "GF", "GUM": "GU", "GUY": "GY", "HKG": "HK", "HMD": "HM", "HND": "HN", "HRV": "HR", "HTI": "HT", "HUN": "HU", "HV": "BF", "IDN": "ID", "IMN": "IM", "IND": "IN", "IOT": "IO", "IRL": "IE", "IRN": "IR", "IRQ": "IQ", "ISL": "IS", "ISR": "IL", "ITA": "IT", "JAM": "JM", "JEY": "JE", "JOR": "JO", "JPN": "JP", "JT": "UM", "KAZ": "KZ", "KEN": "KE", "KGZ": "KG", "KHM": "KH", "KIR": "KI", "KNA": "KN", "KOR": "KR", "KWT": "KW", "LAO": "LA", "LBN": "LB", "LBR": "LR", "LBY": "LY", "LCA": "LC", "LIE": "LI", "LKA": "LK", "LSO": "LS", "LTU": "LT", "LUX": "LU", "LVA": "LV", "MAC": "MO", "MAF": "MF", "MAR": "MA", "MCO": "MC", "MDA": "MD", "MDG": "MG", "MDV": "MV", "MEX": "MX", "MHL": "MH", "MI": "UM", "MKD": "MK", "MLI": "ML", "MLT": "MT", "MMR": "MM", "MNE": "ME", "MNG": "MN", "MNP": "MP", "MOZ": "MZ", "MRT": "MR", "MSR": "MS", "MTQ": "MQ", "MUS": "MU", "MWI": "MW", "MYS": "MY", "MYT": "YT", "NAM": "NA", "NCL": "NC", "NER": "NE", "NFK": "NF", "NGA": "NG", "NH": "VU", "NIC": "NI", "NIU": "NU", "NLD": "NL", "NOR": "NO", "NPL": "NP", "NQ": "AQ", "NRU": "NR", "NT": "SA IQ", "NTZ": "SA IQ", "NZL": "NZ", "OMN": "OM", "PAK": "PK", "PAN": "PA", "PC": "FM MH MP PW", "PCN": "PN", "PER": "PE", "PHL": "PH", "PLW": "PW", "PNG": "PG", "POL": "PL", "PRI": "PR", "PRK": "KP", "PRT": "PT", "PRY": "PY", "PSE": "PS", "PU": "UM", "PYF": "PF", "PZ": "PA", "QAT": "QA", "QMM": "QM", "QNN": "QN", "QPP": "QP", "QQQ": "QQ", "QRR": "QR", "QSS": "QS", "QTT": "QT", "QU": "EU", "QUU": "EU", "QVV": "QV", "QWW": "QW", "QXX": "QX", "QYY": "QY", "QZZ": "QZ", "REU": "RE", "RH": "ZW", "ROU": "RO", "RUS": "RU", "RWA": "RW", "SAU": "SA", "SCG": "RS ME", "SDN": "SD", "SEN": "SN", "SGP": "SG", "SGS": "GS", "SHN": "SH", "SJM": "SJ", "SLB": "SB", "SLE": "SL", "SLV": "SV", "SMR": "SM", "SOM": "SO", "SPM": "PM", "SRB": "RS", "SSD": "SS", "STP": "ST", "SU": "RU AM AZ BY EE GE KZ KG LV LT MD TJ TM UA UZ", "SUN": "RU AM AZ BY EE GE KZ KG LV LT MD TJ TM UA UZ", "SUR": "SR", "SVK": "SK", "SVN": "SI", "SWE": "SE", "SWZ": "SZ", "SXM": "SX", "SYC": "SC", "SYR": "SY", "TAA": "TA", "TCA": "TC", "TCD": "TD", "TGO": "TG", "THA": "TH", "TJK": "TJ", "TKL": "TK", "TKM": "TM", "TLS": "TL", "TMP": "TL", "TON": "TO", "TP": "TL", "TTO": "TT", "TUN": "TN", "TUR": "TR", "TUV": "TV", "TWN": "TW", "TZA": "TZ", "UGA": "UG", "UK": "GB", "UKR": "UA", "UMI": "UM", "URY": "UY", "USA": "US", "UZB": "UZ", "VAT": "VA", "VCT": "VC", "VD": "VN", "VEN": "VE", "VGB": "VG", "VIR": "VI", "VNM": "VN", "VUT": "VU", "WK": "UM", "WLF": "WF", "WSM": "WS", "XAA": "XA", "XBB": "XB", "XCC": "XC", "XDD": "XD", "XEE": "XE", "XFF": "XF", "XGG": "XG", "XHH": "XH", "XII": "XI", "XJJ": "XJ", "XKK": "XK", "XLL": "XL", "XMM": "XM", "XNN": "XN", "XOO": "XO", "XPP": "XP", "XQQ": "XQ", "XRR": "XR", "XSS": "XS", "XTT": "XT", "XUU": "XU", "XVV": "XV", "XWW": "XW", "XXX": "XX", "XYY": "XY", "XZZ": "XZ", "YD": "YE", "YEM": "YE", "YMD": "YE", "YU": "RS ME", "YUG": "RS ME", "ZAF": "ZA", "ZAR": "CD", "ZMB": "ZM", "ZR": "CD", "ZWE": "ZW", "ZZZ": "ZZ" };
    exports.scriptAlias = { "Qaai": "Zinh" };
    exports.variantAlias = { "AALAND": "AX", "arevela": "hy", "arevmda": "hyw", "heploc": "alalc97", "HEPLOC": "ALALC97", "POLYTONI": "POLYTON" };

    });

    unwrapExports(aliases);
    var aliases_1 = aliases.languageAlias;
    var aliases_2 = aliases.territoryAlias;
    var aliases_3 = aliases.scriptAlias;
    var aliases_4 = aliases.variantAlias;

    var supplemental = {
    	version: {
    		_unicodeVersion: "12.1.0",
    		_cldrVersion: "36"
    	},
    	likelySubtags: {
    		aa: "aa-Latn-ET",
    		aai: "aai-Latn-ZZ",
    		aak: "aak-Latn-ZZ",
    		aau: "aau-Latn-ZZ",
    		ab: "ab-Cyrl-GE",
    		abi: "abi-Latn-ZZ",
    		abq: "abq-Cyrl-ZZ",
    		abr: "abr-Latn-GH",
    		abt: "abt-Latn-ZZ",
    		aby: "aby-Latn-ZZ",
    		acd: "acd-Latn-ZZ",
    		ace: "ace-Latn-ID",
    		ach: "ach-Latn-UG",
    		ada: "ada-Latn-GH",
    		ade: "ade-Latn-ZZ",
    		adj: "adj-Latn-ZZ",
    		adp: "adp-Tibt-BT",
    		ady: "ady-Cyrl-RU",
    		adz: "adz-Latn-ZZ",
    		ae: "ae-Avst-IR",
    		aeb: "aeb-Arab-TN",
    		aey: "aey-Latn-ZZ",
    		af: "af-Latn-ZA",
    		agc: "agc-Latn-ZZ",
    		agd: "agd-Latn-ZZ",
    		agg: "agg-Latn-ZZ",
    		agm: "agm-Latn-ZZ",
    		ago: "ago-Latn-ZZ",
    		agq: "agq-Latn-CM",
    		aha: "aha-Latn-ZZ",
    		ahl: "ahl-Latn-ZZ",
    		aho: "aho-Ahom-IN",
    		ajg: "ajg-Latn-ZZ",
    		ak: "ak-Latn-GH",
    		akk: "akk-Xsux-IQ",
    		ala: "ala-Latn-ZZ",
    		ali: "ali-Latn-ZZ",
    		aln: "aln-Latn-XK",
    		alt: "alt-Cyrl-RU",
    		am: "am-Ethi-ET",
    		amm: "amm-Latn-ZZ",
    		amn: "amn-Latn-ZZ",
    		amo: "amo-Latn-NG",
    		amp: "amp-Latn-ZZ",
    		an: "an-Latn-ES",
    		anc: "anc-Latn-ZZ",
    		ank: "ank-Latn-ZZ",
    		ann: "ann-Latn-ZZ",
    		any: "any-Latn-ZZ",
    		aoj: "aoj-Latn-ZZ",
    		aom: "aom-Latn-ZZ",
    		aoz: "aoz-Latn-ID",
    		apc: "apc-Arab-ZZ",
    		apd: "apd-Arab-TG",
    		ape: "ape-Latn-ZZ",
    		apr: "apr-Latn-ZZ",
    		aps: "aps-Latn-ZZ",
    		apz: "apz-Latn-ZZ",
    		ar: "ar-Arab-EG",
    		arc: "arc-Armi-IR",
    		"arc-Nbat": "arc-Nbat-JO",
    		"arc-Palm": "arc-Palm-SY",
    		arh: "arh-Latn-ZZ",
    		arn: "arn-Latn-CL",
    		aro: "aro-Latn-BO",
    		arq: "arq-Arab-DZ",
    		ars: "ars-Arab-SA",
    		ary: "ary-Arab-MA",
    		arz: "arz-Arab-EG",
    		as: "as-Beng-IN",
    		asa: "asa-Latn-TZ",
    		ase: "ase-Sgnw-US",
    		asg: "asg-Latn-ZZ",
    		aso: "aso-Latn-ZZ",
    		ast: "ast-Latn-ES",
    		ata: "ata-Latn-ZZ",
    		atg: "atg-Latn-ZZ",
    		atj: "atj-Latn-CA",
    		auy: "auy-Latn-ZZ",
    		av: "av-Cyrl-RU",
    		avl: "avl-Arab-ZZ",
    		avn: "avn-Latn-ZZ",
    		avt: "avt-Latn-ZZ",
    		avu: "avu-Latn-ZZ",
    		awa: "awa-Deva-IN",
    		awb: "awb-Latn-ZZ",
    		awo: "awo-Latn-ZZ",
    		awx: "awx-Latn-ZZ",
    		ay: "ay-Latn-BO",
    		ayb: "ayb-Latn-ZZ",
    		az: "az-Latn-AZ",
    		"az-Arab": "az-Arab-IR",
    		"az-IQ": "az-Arab-IQ",
    		"az-IR": "az-Arab-IR",
    		"az-RU": "az-Cyrl-RU",
    		ba: "ba-Cyrl-RU",
    		bal: "bal-Arab-PK",
    		ban: "ban-Latn-ID",
    		bap: "bap-Deva-NP",
    		bar: "bar-Latn-AT",
    		bas: "bas-Latn-CM",
    		bav: "bav-Latn-ZZ",
    		bax: "bax-Bamu-CM",
    		bba: "bba-Latn-ZZ",
    		bbb: "bbb-Latn-ZZ",
    		bbc: "bbc-Latn-ID",
    		bbd: "bbd-Latn-ZZ",
    		bbj: "bbj-Latn-CM",
    		bbp: "bbp-Latn-ZZ",
    		bbr: "bbr-Latn-ZZ",
    		bcf: "bcf-Latn-ZZ",
    		bch: "bch-Latn-ZZ",
    		bci: "bci-Latn-CI",
    		bcm: "bcm-Latn-ZZ",
    		bcn: "bcn-Latn-ZZ",
    		bco: "bco-Latn-ZZ",
    		bcq: "bcq-Ethi-ZZ",
    		bcu: "bcu-Latn-ZZ",
    		bdd: "bdd-Latn-ZZ",
    		be: "be-Cyrl-BY",
    		bef: "bef-Latn-ZZ",
    		beh: "beh-Latn-ZZ",
    		bej: "bej-Arab-SD",
    		bem: "bem-Latn-ZM",
    		bet: "bet-Latn-ZZ",
    		bew: "bew-Latn-ID",
    		bex: "bex-Latn-ZZ",
    		bez: "bez-Latn-TZ",
    		bfd: "bfd-Latn-CM",
    		bfq: "bfq-Taml-IN",
    		bft: "bft-Arab-PK",
    		bfy: "bfy-Deva-IN",
    		bg: "bg-Cyrl-BG",
    		bgc: "bgc-Deva-IN",
    		bgn: "bgn-Arab-PK",
    		bgx: "bgx-Grek-TR",
    		bhb: "bhb-Deva-IN",
    		bhg: "bhg-Latn-ZZ",
    		bhi: "bhi-Deva-IN",
    		bhl: "bhl-Latn-ZZ",
    		bho: "bho-Deva-IN",
    		bhy: "bhy-Latn-ZZ",
    		bi: "bi-Latn-VU",
    		bib: "bib-Latn-ZZ",
    		big: "big-Latn-ZZ",
    		bik: "bik-Latn-PH",
    		bim: "bim-Latn-ZZ",
    		bin: "bin-Latn-NG",
    		bio: "bio-Latn-ZZ",
    		biq: "biq-Latn-ZZ",
    		bjh: "bjh-Latn-ZZ",
    		bji: "bji-Ethi-ZZ",
    		bjj: "bjj-Deva-IN",
    		bjn: "bjn-Latn-ID",
    		bjo: "bjo-Latn-ZZ",
    		bjr: "bjr-Latn-ZZ",
    		bjt: "bjt-Latn-SN",
    		bjz: "bjz-Latn-ZZ",
    		bkc: "bkc-Latn-ZZ",
    		bkm: "bkm-Latn-CM",
    		bkq: "bkq-Latn-ZZ",
    		bku: "bku-Latn-PH",
    		bkv: "bkv-Latn-ZZ",
    		blt: "blt-Tavt-VN",
    		bm: "bm-Latn-ML",
    		bmh: "bmh-Latn-ZZ",
    		bmk: "bmk-Latn-ZZ",
    		bmq: "bmq-Latn-ML",
    		bmu: "bmu-Latn-ZZ",
    		bn: "bn-Beng-BD",
    		bng: "bng-Latn-ZZ",
    		bnm: "bnm-Latn-ZZ",
    		bnp: "bnp-Latn-ZZ",
    		bo: "bo-Tibt-CN",
    		boj: "boj-Latn-ZZ",
    		bom: "bom-Latn-ZZ",
    		bon: "bon-Latn-ZZ",
    		bpy: "bpy-Beng-IN",
    		bqc: "bqc-Latn-ZZ",
    		bqi: "bqi-Arab-IR",
    		bqp: "bqp-Latn-ZZ",
    		bqv: "bqv-Latn-CI",
    		br: "br-Latn-FR",
    		bra: "bra-Deva-IN",
    		brh: "brh-Arab-PK",
    		brx: "brx-Deva-IN",
    		brz: "brz-Latn-ZZ",
    		bs: "bs-Latn-BA",
    		bsj: "bsj-Latn-ZZ",
    		bsq: "bsq-Bass-LR",
    		bss: "bss-Latn-CM",
    		bst: "bst-Ethi-ZZ",
    		bto: "bto-Latn-PH",
    		btt: "btt-Latn-ZZ",
    		btv: "btv-Deva-PK",
    		bua: "bua-Cyrl-RU",
    		buc: "buc-Latn-YT",
    		bud: "bud-Latn-ZZ",
    		bug: "bug-Latn-ID",
    		buk: "buk-Latn-ZZ",
    		bum: "bum-Latn-CM",
    		buo: "buo-Latn-ZZ",
    		bus: "bus-Latn-ZZ",
    		buu: "buu-Latn-ZZ",
    		bvb: "bvb-Latn-GQ",
    		bwd: "bwd-Latn-ZZ",
    		bwr: "bwr-Latn-ZZ",
    		bxh: "bxh-Latn-ZZ",
    		bye: "bye-Latn-ZZ",
    		byn: "byn-Ethi-ER",
    		byr: "byr-Latn-ZZ",
    		bys: "bys-Latn-ZZ",
    		byv: "byv-Latn-CM",
    		byx: "byx-Latn-ZZ",
    		bza: "bza-Latn-ZZ",
    		bze: "bze-Latn-ML",
    		bzf: "bzf-Latn-ZZ",
    		bzh: "bzh-Latn-ZZ",
    		bzw: "bzw-Latn-ZZ",
    		ca: "ca-Latn-ES",
    		can: "can-Latn-ZZ",
    		cbj: "cbj-Latn-ZZ",
    		cch: "cch-Latn-NG",
    		ccp: "ccp-Cakm-BD",
    		ce: "ce-Cyrl-RU",
    		ceb: "ceb-Latn-PH",
    		cfa: "cfa-Latn-ZZ",
    		cgg: "cgg-Latn-UG",
    		ch: "ch-Latn-GU",
    		chk: "chk-Latn-FM",
    		chm: "chm-Cyrl-RU",
    		cho: "cho-Latn-US",
    		chp: "chp-Latn-CA",
    		chr: "chr-Cher-US",
    		cic: "cic-Latn-US",
    		cja: "cja-Arab-KH",
    		cjm: "cjm-Cham-VN",
    		cjv: "cjv-Latn-ZZ",
    		ckb: "ckb-Arab-IQ",
    		ckl: "ckl-Latn-ZZ",
    		cko: "cko-Latn-ZZ",
    		cky: "cky-Latn-ZZ",
    		cla: "cla-Latn-ZZ",
    		cme: "cme-Latn-ZZ",
    		cmg: "cmg-Soyo-MN",
    		co: "co-Latn-FR",
    		cop: "cop-Copt-EG",
    		cps: "cps-Latn-PH",
    		cr: "cr-Cans-CA",
    		crh: "crh-Cyrl-UA",
    		crj: "crj-Cans-CA",
    		crk: "crk-Cans-CA",
    		crl: "crl-Cans-CA",
    		crm: "crm-Cans-CA",
    		crs: "crs-Latn-SC",
    		cs: "cs-Latn-CZ",
    		csb: "csb-Latn-PL",
    		csw: "csw-Cans-CA",
    		ctd: "ctd-Pauc-MM",
    		cu: "cu-Cyrl-RU",
    		"cu-Glag": "cu-Glag-BG",
    		cv: "cv-Cyrl-RU",
    		cy: "cy-Latn-GB",
    		da: "da-Latn-DK",
    		dad: "dad-Latn-ZZ",
    		daf: "daf-Latn-ZZ",
    		dag: "dag-Latn-ZZ",
    		dah: "dah-Latn-ZZ",
    		dak: "dak-Latn-US",
    		dar: "dar-Cyrl-RU",
    		dav: "dav-Latn-KE",
    		dbd: "dbd-Latn-ZZ",
    		dbq: "dbq-Latn-ZZ",
    		dcc: "dcc-Arab-IN",
    		ddn: "ddn-Latn-ZZ",
    		de: "de-Latn-DE",
    		ded: "ded-Latn-ZZ",
    		den: "den-Latn-CA",
    		dga: "dga-Latn-ZZ",
    		dgh: "dgh-Latn-ZZ",
    		dgi: "dgi-Latn-ZZ",
    		dgl: "dgl-Arab-ZZ",
    		dgr: "dgr-Latn-CA",
    		dgz: "dgz-Latn-ZZ",
    		dia: "dia-Latn-ZZ",
    		dje: "dje-Latn-NE",
    		dnj: "dnj-Latn-CI",
    		dob: "dob-Latn-ZZ",
    		doi: "doi-Arab-IN",
    		dop: "dop-Latn-ZZ",
    		dow: "dow-Latn-ZZ",
    		drh: "drh-Mong-CN",
    		dri: "dri-Latn-ZZ",
    		drs: "drs-Ethi-ZZ",
    		dsb: "dsb-Latn-DE",
    		dtm: "dtm-Latn-ML",
    		dtp: "dtp-Latn-MY",
    		dts: "dts-Latn-ZZ",
    		dty: "dty-Deva-NP",
    		dua: "dua-Latn-CM",
    		duc: "duc-Latn-ZZ",
    		dud: "dud-Latn-ZZ",
    		dug: "dug-Latn-ZZ",
    		dv: "dv-Thaa-MV",
    		dva: "dva-Latn-ZZ",
    		dww: "dww-Latn-ZZ",
    		dyo: "dyo-Latn-SN",
    		dyu: "dyu-Latn-BF",
    		dz: "dz-Tibt-BT",
    		dzg: "dzg-Latn-ZZ",
    		ebu: "ebu-Latn-KE",
    		ee: "ee-Latn-GH",
    		efi: "efi-Latn-NG",
    		egl: "egl-Latn-IT",
    		egy: "egy-Egyp-EG",
    		eka: "eka-Latn-ZZ",
    		eky: "eky-Kali-MM",
    		el: "el-Grek-GR",
    		ema: "ema-Latn-ZZ",
    		emi: "emi-Latn-ZZ",
    		en: "en-Latn-US",
    		"en-Shaw": "en-Shaw-GB",
    		enn: "enn-Latn-ZZ",
    		enq: "enq-Latn-ZZ",
    		eo: "eo-Latn-001",
    		eri: "eri-Latn-ZZ",
    		es: "es-Latn-ES",
    		esg: "esg-Gonm-IN",
    		esu: "esu-Latn-US",
    		et: "et-Latn-EE",
    		etr: "etr-Latn-ZZ",
    		ett: "ett-Ital-IT",
    		etu: "etu-Latn-ZZ",
    		etx: "etx-Latn-ZZ",
    		eu: "eu-Latn-ES",
    		ewo: "ewo-Latn-CM",
    		ext: "ext-Latn-ES",
    		fa: "fa-Arab-IR",
    		faa: "faa-Latn-ZZ",
    		fab: "fab-Latn-ZZ",
    		fag: "fag-Latn-ZZ",
    		fai: "fai-Latn-ZZ",
    		fan: "fan-Latn-GQ",
    		ff: "ff-Latn-SN",
    		"ff-Adlm": "ff-Adlm-GN",
    		ffi: "ffi-Latn-ZZ",
    		ffm: "ffm-Latn-ML",
    		fi: "fi-Latn-FI",
    		fia: "fia-Arab-SD",
    		fil: "fil-Latn-PH",
    		fit: "fit-Latn-SE",
    		fj: "fj-Latn-FJ",
    		flr: "flr-Latn-ZZ",
    		fmp: "fmp-Latn-ZZ",
    		fo: "fo-Latn-FO",
    		fod: "fod-Latn-ZZ",
    		fon: "fon-Latn-BJ",
    		"for": "for-Latn-ZZ",
    		fpe: "fpe-Latn-ZZ",
    		fqs: "fqs-Latn-ZZ",
    		fr: "fr-Latn-FR",
    		frc: "frc-Latn-US",
    		frp: "frp-Latn-FR",
    		frr: "frr-Latn-DE",
    		frs: "frs-Latn-DE",
    		fub: "fub-Arab-CM",
    		fud: "fud-Latn-WF",
    		fue: "fue-Latn-ZZ",
    		fuf: "fuf-Latn-GN",
    		fuh: "fuh-Latn-ZZ",
    		fuq: "fuq-Latn-NE",
    		fur: "fur-Latn-IT",
    		fuv: "fuv-Latn-NG",
    		fuy: "fuy-Latn-ZZ",
    		fvr: "fvr-Latn-SD",
    		fy: "fy-Latn-NL",
    		ga: "ga-Latn-IE",
    		gaa: "gaa-Latn-GH",
    		gaf: "gaf-Latn-ZZ",
    		gag: "gag-Latn-MD",
    		gah: "gah-Latn-ZZ",
    		gaj: "gaj-Latn-ZZ",
    		gam: "gam-Latn-ZZ",
    		gan: "gan-Hans-CN",
    		gaw: "gaw-Latn-ZZ",
    		gay: "gay-Latn-ID",
    		gba: "gba-Latn-ZZ",
    		gbf: "gbf-Latn-ZZ",
    		gbm: "gbm-Deva-IN",
    		gby: "gby-Latn-ZZ",
    		gbz: "gbz-Arab-IR",
    		gcr: "gcr-Latn-GF",
    		gd: "gd-Latn-GB",
    		gde: "gde-Latn-ZZ",
    		gdn: "gdn-Latn-ZZ",
    		gdr: "gdr-Latn-ZZ",
    		geb: "geb-Latn-ZZ",
    		gej: "gej-Latn-ZZ",
    		gel: "gel-Latn-ZZ",
    		gez: "gez-Ethi-ET",
    		gfk: "gfk-Latn-ZZ",
    		ggn: "ggn-Deva-NP",
    		ghs: "ghs-Latn-ZZ",
    		gil: "gil-Latn-KI",
    		gim: "gim-Latn-ZZ",
    		gjk: "gjk-Arab-PK",
    		gjn: "gjn-Latn-ZZ",
    		gju: "gju-Arab-PK",
    		gkn: "gkn-Latn-ZZ",
    		gkp: "gkp-Latn-ZZ",
    		gl: "gl-Latn-ES",
    		glk: "glk-Arab-IR",
    		gmm: "gmm-Latn-ZZ",
    		gmv: "gmv-Ethi-ZZ",
    		gn: "gn-Latn-PY",
    		gnd: "gnd-Latn-ZZ",
    		gng: "gng-Latn-ZZ",
    		god: "god-Latn-ZZ",
    		gof: "gof-Ethi-ZZ",
    		goi: "goi-Latn-ZZ",
    		gom: "gom-Deva-IN",
    		gon: "gon-Telu-IN",
    		gor: "gor-Latn-ID",
    		gos: "gos-Latn-NL",
    		got: "got-Goth-UA",
    		grb: "grb-Latn-ZZ",
    		grc: "grc-Cprt-CY",
    		"grc-Linb": "grc-Linb-GR",
    		grt: "grt-Beng-IN",
    		grw: "grw-Latn-ZZ",
    		gsw: "gsw-Latn-CH",
    		gu: "gu-Gujr-IN",
    		gub: "gub-Latn-BR",
    		guc: "guc-Latn-CO",
    		gud: "gud-Latn-ZZ",
    		gur: "gur-Latn-GH",
    		guw: "guw-Latn-ZZ",
    		gux: "gux-Latn-ZZ",
    		guz: "guz-Latn-KE",
    		gv: "gv-Latn-IM",
    		gvf: "gvf-Latn-ZZ",
    		gvr: "gvr-Deva-NP",
    		gvs: "gvs-Latn-ZZ",
    		gwc: "gwc-Arab-ZZ",
    		gwi: "gwi-Latn-CA",
    		gwt: "gwt-Arab-ZZ",
    		gyi: "gyi-Latn-ZZ",
    		ha: "ha-Latn-NG",
    		"ha-CM": "ha-Arab-CM",
    		"ha-SD": "ha-Arab-SD",
    		hag: "hag-Latn-ZZ",
    		hak: "hak-Hans-CN",
    		ham: "ham-Latn-ZZ",
    		haw: "haw-Latn-US",
    		haz: "haz-Arab-AF",
    		hbb: "hbb-Latn-ZZ",
    		hdy: "hdy-Ethi-ZZ",
    		he: "he-Hebr-IL",
    		hhy: "hhy-Latn-ZZ",
    		hi: "hi-Deva-IN",
    		hia: "hia-Latn-ZZ",
    		hif: "hif-Latn-FJ",
    		hig: "hig-Latn-ZZ",
    		hih: "hih-Latn-ZZ",
    		hil: "hil-Latn-PH",
    		hla: "hla-Latn-ZZ",
    		hlu: "hlu-Hluw-TR",
    		hmd: "hmd-Plrd-CN",
    		hmt: "hmt-Latn-ZZ",
    		hnd: "hnd-Arab-PK",
    		hne: "hne-Deva-IN",
    		hnj: "hnj-Hmng-LA",
    		hnn: "hnn-Latn-PH",
    		hno: "hno-Arab-PK",
    		ho: "ho-Latn-PG",
    		hoc: "hoc-Deva-IN",
    		hoj: "hoj-Deva-IN",
    		hot: "hot-Latn-ZZ",
    		hr: "hr-Latn-HR",
    		hsb: "hsb-Latn-DE",
    		hsn: "hsn-Hans-CN",
    		ht: "ht-Latn-HT",
    		hu: "hu-Latn-HU",
    		hui: "hui-Latn-ZZ",
    		hy: "hy-Armn-AM",
    		hz: "hz-Latn-NA",
    		ia: "ia-Latn-001",
    		ian: "ian-Latn-ZZ",
    		iar: "iar-Latn-ZZ",
    		iba: "iba-Latn-MY",
    		ibb: "ibb-Latn-NG",
    		iby: "iby-Latn-ZZ",
    		ica: "ica-Latn-ZZ",
    		ich: "ich-Latn-ZZ",
    		id: "id-Latn-ID",
    		idd: "idd-Latn-ZZ",
    		idi: "idi-Latn-ZZ",
    		idu: "idu-Latn-ZZ",
    		ife: "ife-Latn-TG",
    		ig: "ig-Latn-NG",
    		igb: "igb-Latn-ZZ",
    		ige: "ige-Latn-ZZ",
    		ii: "ii-Yiii-CN",
    		ijj: "ijj-Latn-ZZ",
    		ik: "ik-Latn-US",
    		ikk: "ikk-Latn-ZZ",
    		ikt: "ikt-Latn-CA",
    		ikw: "ikw-Latn-ZZ",
    		ikx: "ikx-Latn-ZZ",
    		ilo: "ilo-Latn-PH",
    		imo: "imo-Latn-ZZ",
    		"in": "in-Latn-ID",
    		inh: "inh-Cyrl-RU",
    		io: "io-Latn-001",
    		iou: "iou-Latn-ZZ",
    		iri: "iri-Latn-ZZ",
    		is: "is-Latn-IS",
    		it: "it-Latn-IT",
    		iu: "iu-Cans-CA",
    		iw: "iw-Hebr-IL",
    		iwm: "iwm-Latn-ZZ",
    		iws: "iws-Latn-ZZ",
    		izh: "izh-Latn-RU",
    		izi: "izi-Latn-ZZ",
    		ja: "ja-Jpan-JP",
    		jab: "jab-Latn-ZZ",
    		jam: "jam-Latn-JM",
    		jbo: "jbo-Latn-001",
    		jbu: "jbu-Latn-ZZ",
    		jen: "jen-Latn-ZZ",
    		jgk: "jgk-Latn-ZZ",
    		jgo: "jgo-Latn-CM",
    		ji: "ji-Hebr-UA",
    		jib: "jib-Latn-ZZ",
    		jmc: "jmc-Latn-TZ",
    		jml: "jml-Deva-NP",
    		jra: "jra-Latn-ZZ",
    		jut: "jut-Latn-DK",
    		jv: "jv-Latn-ID",
    		jw: "jw-Latn-ID",
    		ka: "ka-Geor-GE",
    		kaa: "kaa-Cyrl-UZ",
    		kab: "kab-Latn-DZ",
    		kac: "kac-Latn-MM",
    		kad: "kad-Latn-ZZ",
    		kai: "kai-Latn-ZZ",
    		kaj: "kaj-Latn-NG",
    		kam: "kam-Latn-KE",
    		kao: "kao-Latn-ML",
    		kbd: "kbd-Cyrl-RU",
    		kbm: "kbm-Latn-ZZ",
    		kbp: "kbp-Latn-ZZ",
    		kbq: "kbq-Latn-ZZ",
    		kbx: "kbx-Latn-ZZ",
    		kby: "kby-Arab-NE",
    		kcg: "kcg-Latn-NG",
    		kck: "kck-Latn-ZW",
    		kcl: "kcl-Latn-ZZ",
    		kct: "kct-Latn-ZZ",
    		kde: "kde-Latn-TZ",
    		kdh: "kdh-Arab-TG",
    		kdl: "kdl-Latn-ZZ",
    		kdt: "kdt-Thai-TH",
    		kea: "kea-Latn-CV",
    		ken: "ken-Latn-CM",
    		kez: "kez-Latn-ZZ",
    		kfo: "kfo-Latn-CI",
    		kfr: "kfr-Deva-IN",
    		kfy: "kfy-Deva-IN",
    		kg: "kg-Latn-CD",
    		kge: "kge-Latn-ID",
    		kgf: "kgf-Latn-ZZ",
    		kgp: "kgp-Latn-BR",
    		kha: "kha-Latn-IN",
    		khb: "khb-Talu-CN",
    		khn: "khn-Deva-IN",
    		khq: "khq-Latn-ML",
    		khs: "khs-Latn-ZZ",
    		kht: "kht-Mymr-IN",
    		khw: "khw-Arab-PK",
    		khz: "khz-Latn-ZZ",
    		ki: "ki-Latn-KE",
    		kij: "kij-Latn-ZZ",
    		kiu: "kiu-Latn-TR",
    		kiw: "kiw-Latn-ZZ",
    		kj: "kj-Latn-NA",
    		kjd: "kjd-Latn-ZZ",
    		kjg: "kjg-Laoo-LA",
    		kjs: "kjs-Latn-ZZ",
    		kjy: "kjy-Latn-ZZ",
    		kk: "kk-Cyrl-KZ",
    		"kk-AF": "kk-Arab-AF",
    		"kk-Arab": "kk-Arab-CN",
    		"kk-CN": "kk-Arab-CN",
    		"kk-IR": "kk-Arab-IR",
    		"kk-MN": "kk-Arab-MN",
    		kkc: "kkc-Latn-ZZ",
    		kkj: "kkj-Latn-CM",
    		kl: "kl-Latn-GL",
    		kln: "kln-Latn-KE",
    		klq: "klq-Latn-ZZ",
    		klt: "klt-Latn-ZZ",
    		klx: "klx-Latn-ZZ",
    		km: "km-Khmr-KH",
    		kmb: "kmb-Latn-AO",
    		kmh: "kmh-Latn-ZZ",
    		kmo: "kmo-Latn-ZZ",
    		kms: "kms-Latn-ZZ",
    		kmu: "kmu-Latn-ZZ",
    		kmw: "kmw-Latn-ZZ",
    		kn: "kn-Knda-IN",
    		knf: "knf-Latn-GW",
    		knp: "knp-Latn-ZZ",
    		ko: "ko-Kore-KR",
    		koi: "koi-Cyrl-RU",
    		kok: "kok-Deva-IN",
    		kol: "kol-Latn-ZZ",
    		kos: "kos-Latn-FM",
    		koz: "koz-Latn-ZZ",
    		kpe: "kpe-Latn-LR",
    		kpf: "kpf-Latn-ZZ",
    		kpo: "kpo-Latn-ZZ",
    		kpr: "kpr-Latn-ZZ",
    		kpx: "kpx-Latn-ZZ",
    		kqb: "kqb-Latn-ZZ",
    		kqf: "kqf-Latn-ZZ",
    		kqs: "kqs-Latn-ZZ",
    		kqy: "kqy-Ethi-ZZ",
    		kr: "kr-Latn-ZZ",
    		krc: "krc-Cyrl-RU",
    		kri: "kri-Latn-SL",
    		krj: "krj-Latn-PH",
    		krl: "krl-Latn-RU",
    		krs: "krs-Latn-ZZ",
    		kru: "kru-Deva-IN",
    		ks: "ks-Arab-IN",
    		ksb: "ksb-Latn-TZ",
    		ksd: "ksd-Latn-ZZ",
    		ksf: "ksf-Latn-CM",
    		ksh: "ksh-Latn-DE",
    		ksj: "ksj-Latn-ZZ",
    		ksr: "ksr-Latn-ZZ",
    		ktb: "ktb-Ethi-ZZ",
    		ktm: "ktm-Latn-ZZ",
    		kto: "kto-Latn-ZZ",
    		ktr: "ktr-Latn-MY",
    		ku: "ku-Latn-TR",
    		"ku-Arab": "ku-Arab-IQ",
    		"ku-LB": "ku-Arab-LB",
    		kub: "kub-Latn-ZZ",
    		kud: "kud-Latn-ZZ",
    		kue: "kue-Latn-ZZ",
    		kuj: "kuj-Latn-ZZ",
    		kum: "kum-Cyrl-RU",
    		kun: "kun-Latn-ZZ",
    		kup: "kup-Latn-ZZ",
    		kus: "kus-Latn-ZZ",
    		kv: "kv-Cyrl-RU",
    		kvg: "kvg-Latn-ZZ",
    		kvr: "kvr-Latn-ID",
    		kvx: "kvx-Arab-PK",
    		kw: "kw-Latn-GB",
    		kwj: "kwj-Latn-ZZ",
    		kwo: "kwo-Latn-ZZ",
    		kwq: "kwq-Latn-ZZ",
    		kxa: "kxa-Latn-ZZ",
    		kxc: "kxc-Ethi-ZZ",
    		kxe: "kxe-Latn-ZZ",
    		kxm: "kxm-Thai-TH",
    		kxp: "kxp-Arab-PK",
    		kxw: "kxw-Latn-ZZ",
    		kxz: "kxz-Latn-ZZ",
    		ky: "ky-Cyrl-KG",
    		"ky-Arab": "ky-Arab-CN",
    		"ky-CN": "ky-Arab-CN",
    		"ky-Latn": "ky-Latn-TR",
    		"ky-TR": "ky-Latn-TR",
    		kye: "kye-Latn-ZZ",
    		kyx: "kyx-Latn-ZZ",
    		kzj: "kzj-Latn-MY",
    		kzr: "kzr-Latn-ZZ",
    		kzt: "kzt-Latn-MY",
    		la: "la-Latn-VA",
    		lab: "lab-Lina-GR",
    		lad: "lad-Hebr-IL",
    		lag: "lag-Latn-TZ",
    		lah: "lah-Arab-PK",
    		laj: "laj-Latn-UG",
    		las: "las-Latn-ZZ",
    		lb: "lb-Latn-LU",
    		lbe: "lbe-Cyrl-RU",
    		lbu: "lbu-Latn-ZZ",
    		lbw: "lbw-Latn-ID",
    		lcm: "lcm-Latn-ZZ",
    		lcp: "lcp-Thai-CN",
    		ldb: "ldb-Latn-ZZ",
    		led: "led-Latn-ZZ",
    		lee: "lee-Latn-ZZ",
    		lem: "lem-Latn-ZZ",
    		lep: "lep-Lepc-IN",
    		leq: "leq-Latn-ZZ",
    		leu: "leu-Latn-ZZ",
    		lez: "lez-Cyrl-RU",
    		lg: "lg-Latn-UG",
    		lgg: "lgg-Latn-ZZ",
    		li: "li-Latn-NL",
    		lia: "lia-Latn-ZZ",
    		lid: "lid-Latn-ZZ",
    		lif: "lif-Deva-NP",
    		"lif-Limb": "lif-Limb-IN",
    		lig: "lig-Latn-ZZ",
    		lih: "lih-Latn-ZZ",
    		lij: "lij-Latn-IT",
    		lis: "lis-Lisu-CN",
    		ljp: "ljp-Latn-ID",
    		lki: "lki-Arab-IR",
    		lkt: "lkt-Latn-US",
    		lle: "lle-Latn-ZZ",
    		lln: "lln-Latn-ZZ",
    		lmn: "lmn-Telu-IN",
    		lmo: "lmo-Latn-IT",
    		lmp: "lmp-Latn-ZZ",
    		ln: "ln-Latn-CD",
    		lns: "lns-Latn-ZZ",
    		lnu: "lnu-Latn-ZZ",
    		lo: "lo-Laoo-LA",
    		loj: "loj-Latn-ZZ",
    		lok: "lok-Latn-ZZ",
    		lol: "lol-Latn-CD",
    		lor: "lor-Latn-ZZ",
    		los: "los-Latn-ZZ",
    		loz: "loz-Latn-ZM",
    		lrc: "lrc-Arab-IR",
    		lt: "lt-Latn-LT",
    		ltg: "ltg-Latn-LV",
    		lu: "lu-Latn-CD",
    		lua: "lua-Latn-CD",
    		luo: "luo-Latn-KE",
    		luy: "luy-Latn-KE",
    		luz: "luz-Arab-IR",
    		lv: "lv-Latn-LV",
    		lwl: "lwl-Thai-TH",
    		lzh: "lzh-Hans-CN",
    		lzz: "lzz-Latn-TR",
    		mad: "mad-Latn-ID",
    		maf: "maf-Latn-CM",
    		mag: "mag-Deva-IN",
    		mai: "mai-Deva-IN",
    		mak: "mak-Latn-ID",
    		man: "man-Latn-GM",
    		"man-GN": "man-Nkoo-GN",
    		"man-Nkoo": "man-Nkoo-GN",
    		mas: "mas-Latn-KE",
    		maw: "maw-Latn-ZZ",
    		maz: "maz-Latn-MX",
    		mbh: "mbh-Latn-ZZ",
    		mbo: "mbo-Latn-ZZ",
    		mbq: "mbq-Latn-ZZ",
    		mbu: "mbu-Latn-ZZ",
    		mbw: "mbw-Latn-ZZ",
    		mci: "mci-Latn-ZZ",
    		mcp: "mcp-Latn-ZZ",
    		mcq: "mcq-Latn-ZZ",
    		mcr: "mcr-Latn-ZZ",
    		mcu: "mcu-Latn-ZZ",
    		mda: "mda-Latn-ZZ",
    		mde: "mde-Arab-ZZ",
    		mdf: "mdf-Cyrl-RU",
    		mdh: "mdh-Latn-PH",
    		mdj: "mdj-Latn-ZZ",
    		mdr: "mdr-Latn-ID",
    		mdx: "mdx-Ethi-ZZ",
    		med: "med-Latn-ZZ",
    		mee: "mee-Latn-ZZ",
    		mek: "mek-Latn-ZZ",
    		men: "men-Latn-SL",
    		mer: "mer-Latn-KE",
    		met: "met-Latn-ZZ",
    		meu: "meu-Latn-ZZ",
    		mfa: "mfa-Arab-TH",
    		mfe: "mfe-Latn-MU",
    		mfn: "mfn-Latn-ZZ",
    		mfo: "mfo-Latn-ZZ",
    		mfq: "mfq-Latn-ZZ",
    		mg: "mg-Latn-MG",
    		mgh: "mgh-Latn-MZ",
    		mgl: "mgl-Latn-ZZ",
    		mgo: "mgo-Latn-CM",
    		mgp: "mgp-Deva-NP",
    		mgy: "mgy-Latn-TZ",
    		mh: "mh-Latn-MH",
    		mhi: "mhi-Latn-ZZ",
    		mhl: "mhl-Latn-ZZ",
    		mi: "mi-Latn-NZ",
    		mif: "mif-Latn-ZZ",
    		min: "min-Latn-ID",
    		mis: "mis-Hatr-IQ",
    		"mis-Medf": "mis-Medf-NG",
    		miw: "miw-Latn-ZZ",
    		mk: "mk-Cyrl-MK",
    		mki: "mki-Arab-ZZ",
    		mkl: "mkl-Latn-ZZ",
    		mkp: "mkp-Latn-ZZ",
    		mkw: "mkw-Latn-ZZ",
    		ml: "ml-Mlym-IN",
    		mle: "mle-Latn-ZZ",
    		mlp: "mlp-Latn-ZZ",
    		mls: "mls-Latn-SD",
    		mmo: "mmo-Latn-ZZ",
    		mmu: "mmu-Latn-ZZ",
    		mmx: "mmx-Latn-ZZ",
    		mn: "mn-Cyrl-MN",
    		"mn-CN": "mn-Mong-CN",
    		"mn-Mong": "mn-Mong-CN",
    		mna: "mna-Latn-ZZ",
    		mnf: "mnf-Latn-ZZ",
    		mni: "mni-Beng-IN",
    		mnw: "mnw-Mymr-MM",
    		mo: "mo-Latn-RO",
    		moa: "moa-Latn-ZZ",
    		moe: "moe-Latn-CA",
    		moh: "moh-Latn-CA",
    		mos: "mos-Latn-BF",
    		mox: "mox-Latn-ZZ",
    		mpp: "mpp-Latn-ZZ",
    		mps: "mps-Latn-ZZ",
    		mpt: "mpt-Latn-ZZ",
    		mpx: "mpx-Latn-ZZ",
    		mql: "mql-Latn-ZZ",
    		mr: "mr-Deva-IN",
    		mrd: "mrd-Deva-NP",
    		mrj: "mrj-Cyrl-RU",
    		mro: "mro-Mroo-BD",
    		ms: "ms-Latn-MY",
    		"ms-CC": "ms-Arab-CC",
    		"ms-ID": "ms-Arab-ID",
    		mt: "mt-Latn-MT",
    		mtc: "mtc-Latn-ZZ",
    		mtf: "mtf-Latn-ZZ",
    		mti: "mti-Latn-ZZ",
    		mtr: "mtr-Deva-IN",
    		mua: "mua-Latn-CM",
    		mur: "mur-Latn-ZZ",
    		mus: "mus-Latn-US",
    		mva: "mva-Latn-ZZ",
    		mvn: "mvn-Latn-ZZ",
    		mvy: "mvy-Arab-PK",
    		mwk: "mwk-Latn-ML",
    		mwr: "mwr-Deva-IN",
    		mwv: "mwv-Latn-ID",
    		mww: "mww-Hmnp-US",
    		mxc: "mxc-Latn-ZW",
    		mxm: "mxm-Latn-ZZ",
    		my: "my-Mymr-MM",
    		myk: "myk-Latn-ZZ",
    		mym: "mym-Ethi-ZZ",
    		myv: "myv-Cyrl-RU",
    		myw: "myw-Latn-ZZ",
    		myx: "myx-Latn-UG",
    		myz: "myz-Mand-IR",
    		mzk: "mzk-Latn-ZZ",
    		mzm: "mzm-Latn-ZZ",
    		mzn: "mzn-Arab-IR",
    		mzp: "mzp-Latn-ZZ",
    		mzw: "mzw-Latn-ZZ",
    		mzz: "mzz-Latn-ZZ",
    		na: "na-Latn-NR",
    		nac: "nac-Latn-ZZ",
    		naf: "naf-Latn-ZZ",
    		nak: "nak-Latn-ZZ",
    		nan: "nan-Hans-CN",
    		nap: "nap-Latn-IT",
    		naq: "naq-Latn-NA",
    		nas: "nas-Latn-ZZ",
    		nb: "nb-Latn-NO",
    		nca: "nca-Latn-ZZ",
    		nce: "nce-Latn-ZZ",
    		ncf: "ncf-Latn-ZZ",
    		nch: "nch-Latn-MX",
    		nco: "nco-Latn-ZZ",
    		ncu: "ncu-Latn-ZZ",
    		nd: "nd-Latn-ZW",
    		ndc: "ndc-Latn-MZ",
    		nds: "nds-Latn-DE",
    		ne: "ne-Deva-NP",
    		neb: "neb-Latn-ZZ",
    		"new": "new-Deva-NP",
    		nex: "nex-Latn-ZZ",
    		nfr: "nfr-Latn-ZZ",
    		ng: "ng-Latn-NA",
    		nga: "nga-Latn-ZZ",
    		ngb: "ngb-Latn-ZZ",
    		ngl: "ngl-Latn-MZ",
    		nhb: "nhb-Latn-ZZ",
    		nhe: "nhe-Latn-MX",
    		nhw: "nhw-Latn-MX",
    		nif: "nif-Latn-ZZ",
    		nii: "nii-Latn-ZZ",
    		nij: "nij-Latn-ID",
    		nin: "nin-Latn-ZZ",
    		niu: "niu-Latn-NU",
    		niy: "niy-Latn-ZZ",
    		niz: "niz-Latn-ZZ",
    		njo: "njo-Latn-IN",
    		nkg: "nkg-Latn-ZZ",
    		nko: "nko-Latn-ZZ",
    		nl: "nl-Latn-NL",
    		nmg: "nmg-Latn-CM",
    		nmz: "nmz-Latn-ZZ",
    		nn: "nn-Latn-NO",
    		nnf: "nnf-Latn-ZZ",
    		nnh: "nnh-Latn-CM",
    		nnk: "nnk-Latn-ZZ",
    		nnm: "nnm-Latn-ZZ",
    		nnp: "nnp-Wcho-IN",
    		no: "no-Latn-NO",
    		nod: "nod-Lana-TH",
    		noe: "noe-Deva-IN",
    		non: "non-Runr-SE",
    		nop: "nop-Latn-ZZ",
    		nou: "nou-Latn-ZZ",
    		nqo: "nqo-Nkoo-GN",
    		nr: "nr-Latn-ZA",
    		nrb: "nrb-Latn-ZZ",
    		nsk: "nsk-Cans-CA",
    		nsn: "nsn-Latn-ZZ",
    		nso: "nso-Latn-ZA",
    		nss: "nss-Latn-ZZ",
    		ntm: "ntm-Latn-ZZ",
    		ntr: "ntr-Latn-ZZ",
    		nui: "nui-Latn-ZZ",
    		nup: "nup-Latn-ZZ",
    		nus: "nus-Latn-SS",
    		nuv: "nuv-Latn-ZZ",
    		nux: "nux-Latn-ZZ",
    		nv: "nv-Latn-US",
    		nwb: "nwb-Latn-ZZ",
    		nxq: "nxq-Latn-CN",
    		nxr: "nxr-Latn-ZZ",
    		ny: "ny-Latn-MW",
    		nym: "nym-Latn-TZ",
    		nyn: "nyn-Latn-UG",
    		nzi: "nzi-Latn-GH",
    		oc: "oc-Latn-FR",
    		ogc: "ogc-Latn-ZZ",
    		okr: "okr-Latn-ZZ",
    		okv: "okv-Latn-ZZ",
    		om: "om-Latn-ET",
    		ong: "ong-Latn-ZZ",
    		onn: "onn-Latn-ZZ",
    		ons: "ons-Latn-ZZ",
    		opm: "opm-Latn-ZZ",
    		or: "or-Orya-IN",
    		oro: "oro-Latn-ZZ",
    		oru: "oru-Arab-ZZ",
    		os: "os-Cyrl-GE",
    		osa: "osa-Osge-US",
    		ota: "ota-Arab-ZZ",
    		otk: "otk-Orkh-MN",
    		ozm: "ozm-Latn-ZZ",
    		pa: "pa-Guru-IN",
    		"pa-Arab": "pa-Arab-PK",
    		"pa-PK": "pa-Arab-PK",
    		pag: "pag-Latn-PH",
    		pal: "pal-Phli-IR",
    		"pal-Phlp": "pal-Phlp-CN",
    		pam: "pam-Latn-PH",
    		pap: "pap-Latn-AW",
    		pau: "pau-Latn-PW",
    		pbi: "pbi-Latn-ZZ",
    		pcd: "pcd-Latn-FR",
    		pcm: "pcm-Latn-NG",
    		pdc: "pdc-Latn-US",
    		pdt: "pdt-Latn-CA",
    		ped: "ped-Latn-ZZ",
    		peo: "peo-Xpeo-IR",
    		pex: "pex-Latn-ZZ",
    		pfl: "pfl-Latn-DE",
    		phl: "phl-Arab-ZZ",
    		phn: "phn-Phnx-LB",
    		pil: "pil-Latn-ZZ",
    		pip: "pip-Latn-ZZ",
    		pka: "pka-Brah-IN",
    		pko: "pko-Latn-KE",
    		pl: "pl-Latn-PL",
    		pla: "pla-Latn-ZZ",
    		pms: "pms-Latn-IT",
    		png: "png-Latn-ZZ",
    		pnn: "pnn-Latn-ZZ",
    		pnt: "pnt-Grek-GR",
    		pon: "pon-Latn-FM",
    		ppa: "ppa-Deva-IN",
    		ppo: "ppo-Latn-ZZ",
    		pra: "pra-Khar-PK",
    		prd: "prd-Arab-IR",
    		prg: "prg-Latn-001",
    		ps: "ps-Arab-AF",
    		pss: "pss-Latn-ZZ",
    		pt: "pt-Latn-BR",
    		ptp: "ptp-Latn-ZZ",
    		puu: "puu-Latn-GA",
    		pwa: "pwa-Latn-ZZ",
    		qu: "qu-Latn-PE",
    		quc: "quc-Latn-GT",
    		qug: "qug-Latn-EC",
    		rai: "rai-Latn-ZZ",
    		raj: "raj-Deva-IN",
    		rao: "rao-Latn-ZZ",
    		rcf: "rcf-Latn-RE",
    		rej: "rej-Latn-ID",
    		rel: "rel-Latn-ZZ",
    		res: "res-Latn-ZZ",
    		rgn: "rgn-Latn-IT",
    		rhg: "rhg-Arab-MM",
    		ria: "ria-Latn-IN",
    		rif: "rif-Tfng-MA",
    		"rif-NL": "rif-Latn-NL",
    		rjs: "rjs-Deva-NP",
    		rkt: "rkt-Beng-BD",
    		rm: "rm-Latn-CH",
    		rmf: "rmf-Latn-FI",
    		rmo: "rmo-Latn-CH",
    		rmt: "rmt-Arab-IR",
    		rmu: "rmu-Latn-SE",
    		rn: "rn-Latn-BI",
    		rna: "rna-Latn-ZZ",
    		rng: "rng-Latn-MZ",
    		ro: "ro-Latn-RO",
    		rob: "rob-Latn-ID",
    		rof: "rof-Latn-TZ",
    		roo: "roo-Latn-ZZ",
    		rro: "rro-Latn-ZZ",
    		rtm: "rtm-Latn-FJ",
    		ru: "ru-Cyrl-RU",
    		rue: "rue-Cyrl-UA",
    		rug: "rug-Latn-SB",
    		rw: "rw-Latn-RW",
    		rwk: "rwk-Latn-TZ",
    		rwo: "rwo-Latn-ZZ",
    		ryu: "ryu-Kana-JP",
    		sa: "sa-Deva-IN",
    		saf: "saf-Latn-GH",
    		sah: "sah-Cyrl-RU",
    		saq: "saq-Latn-KE",
    		sas: "sas-Latn-ID",
    		sat: "sat-Latn-IN",
    		sav: "sav-Latn-SN",
    		saz: "saz-Saur-IN",
    		sba: "sba-Latn-ZZ",
    		sbe: "sbe-Latn-ZZ",
    		sbp: "sbp-Latn-TZ",
    		sc: "sc-Latn-IT",
    		sck: "sck-Deva-IN",
    		scl: "scl-Arab-ZZ",
    		scn: "scn-Latn-IT",
    		sco: "sco-Latn-GB",
    		scs: "scs-Latn-CA",
    		sd: "sd-Arab-PK",
    		"sd-Deva": "sd-Deva-IN",
    		"sd-Khoj": "sd-Khoj-IN",
    		"sd-Sind": "sd-Sind-IN",
    		sdc: "sdc-Latn-IT",
    		sdh: "sdh-Arab-IR",
    		se: "se-Latn-NO",
    		sef: "sef-Latn-CI",
    		seh: "seh-Latn-MZ",
    		sei: "sei-Latn-MX",
    		ses: "ses-Latn-ML",
    		sg: "sg-Latn-CF",
    		sga: "sga-Ogam-IE",
    		sgs: "sgs-Latn-LT",
    		sgw: "sgw-Ethi-ZZ",
    		sgz: "sgz-Latn-ZZ",
    		shi: "shi-Tfng-MA",
    		shk: "shk-Latn-ZZ",
    		shn: "shn-Mymr-MM",
    		shu: "shu-Arab-ZZ",
    		si: "si-Sinh-LK",
    		sid: "sid-Latn-ET",
    		sig: "sig-Latn-ZZ",
    		sil: "sil-Latn-ZZ",
    		sim: "sim-Latn-ZZ",
    		sjr: "sjr-Latn-ZZ",
    		sk: "sk-Latn-SK",
    		skc: "skc-Latn-ZZ",
    		skr: "skr-Arab-PK",
    		sks: "sks-Latn-ZZ",
    		sl: "sl-Latn-SI",
    		sld: "sld-Latn-ZZ",
    		sli: "sli-Latn-PL",
    		sll: "sll-Latn-ZZ",
    		sly: "sly-Latn-ID",
    		sm: "sm-Latn-WS",
    		sma: "sma-Latn-SE",
    		smj: "smj-Latn-SE",
    		smn: "smn-Latn-FI",
    		smp: "smp-Samr-IL",
    		smq: "smq-Latn-ZZ",
    		sms: "sms-Latn-FI",
    		sn: "sn-Latn-ZW",
    		snc: "snc-Latn-ZZ",
    		snk: "snk-Latn-ML",
    		snp: "snp-Latn-ZZ",
    		snx: "snx-Latn-ZZ",
    		sny: "sny-Latn-ZZ",
    		so: "so-Latn-SO",
    		sog: "sog-Sogd-UZ",
    		sok: "sok-Latn-ZZ",
    		soq: "soq-Latn-ZZ",
    		sou: "sou-Thai-TH",
    		soy: "soy-Latn-ZZ",
    		spd: "spd-Latn-ZZ",
    		spl: "spl-Latn-ZZ",
    		sps: "sps-Latn-ZZ",
    		sq: "sq-Latn-AL",
    		sr: "sr-Cyrl-RS",
    		"sr-ME": "sr-Latn-ME",
    		"sr-RO": "sr-Latn-RO",
    		"sr-RU": "sr-Latn-RU",
    		"sr-TR": "sr-Latn-TR",
    		srb: "srb-Sora-IN",
    		srn: "srn-Latn-SR",
    		srr: "srr-Latn-SN",
    		srx: "srx-Deva-IN",
    		ss: "ss-Latn-ZA",
    		ssd: "ssd-Latn-ZZ",
    		ssg: "ssg-Latn-ZZ",
    		ssy: "ssy-Latn-ER",
    		st: "st-Latn-ZA",
    		stk: "stk-Latn-ZZ",
    		stq: "stq-Latn-DE",
    		su: "su-Latn-ID",
    		sua: "sua-Latn-ZZ",
    		sue: "sue-Latn-ZZ",
    		suk: "suk-Latn-TZ",
    		sur: "sur-Latn-ZZ",
    		sus: "sus-Latn-GN",
    		sv: "sv-Latn-SE",
    		sw: "sw-Latn-TZ",
    		swb: "swb-Arab-YT",
    		swc: "swc-Latn-CD",
    		swg: "swg-Latn-DE",
    		swp: "swp-Latn-ZZ",
    		swv: "swv-Deva-IN",
    		sxn: "sxn-Latn-ID",
    		sxw: "sxw-Latn-ZZ",
    		syl: "syl-Beng-BD",
    		syr: "syr-Syrc-IQ",
    		szl: "szl-Latn-PL",
    		ta: "ta-Taml-IN",
    		taj: "taj-Deva-NP",
    		tal: "tal-Latn-ZZ",
    		tan: "tan-Latn-ZZ",
    		taq: "taq-Latn-ZZ",
    		tbc: "tbc-Latn-ZZ",
    		tbd: "tbd-Latn-ZZ",
    		tbf: "tbf-Latn-ZZ",
    		tbg: "tbg-Latn-ZZ",
    		tbo: "tbo-Latn-ZZ",
    		tbw: "tbw-Latn-PH",
    		tbz: "tbz-Latn-ZZ",
    		tci: "tci-Latn-ZZ",
    		tcy: "tcy-Knda-IN",
    		tdd: "tdd-Tale-CN",
    		tdg: "tdg-Deva-NP",
    		tdh: "tdh-Deva-NP",
    		tdu: "tdu-Latn-MY",
    		te: "te-Telu-IN",
    		ted: "ted-Latn-ZZ",
    		tem: "tem-Latn-SL",
    		teo: "teo-Latn-UG",
    		tet: "tet-Latn-TL",
    		tfi: "tfi-Latn-ZZ",
    		tg: "tg-Cyrl-TJ",
    		"tg-Arab": "tg-Arab-PK",
    		"tg-PK": "tg-Arab-PK",
    		tgc: "tgc-Latn-ZZ",
    		tgo: "tgo-Latn-ZZ",
    		tgu: "tgu-Latn-ZZ",
    		th: "th-Thai-TH",
    		thl: "thl-Deva-NP",
    		thq: "thq-Deva-NP",
    		thr: "thr-Deva-NP",
    		ti: "ti-Ethi-ET",
    		tif: "tif-Latn-ZZ",
    		tig: "tig-Ethi-ER",
    		tik: "tik-Latn-ZZ",
    		tim: "tim-Latn-ZZ",
    		tio: "tio-Latn-ZZ",
    		tiv: "tiv-Latn-NG",
    		tk: "tk-Latn-TM",
    		tkl: "tkl-Latn-TK",
    		tkr: "tkr-Latn-AZ",
    		tkt: "tkt-Deva-NP",
    		tl: "tl-Latn-PH",
    		tlf: "tlf-Latn-ZZ",
    		tlx: "tlx-Latn-ZZ",
    		tly: "tly-Latn-AZ",
    		tmh: "tmh-Latn-NE",
    		tmy: "tmy-Latn-ZZ",
    		tn: "tn-Latn-ZA",
    		tnh: "tnh-Latn-ZZ",
    		to: "to-Latn-TO",
    		tof: "tof-Latn-ZZ",
    		tog: "tog-Latn-MW",
    		toq: "toq-Latn-ZZ",
    		tpi: "tpi-Latn-PG",
    		tpm: "tpm-Latn-ZZ",
    		tpz: "tpz-Latn-ZZ",
    		tqo: "tqo-Latn-ZZ",
    		tr: "tr-Latn-TR",
    		tru: "tru-Latn-TR",
    		trv: "trv-Latn-TW",
    		trw: "trw-Arab-ZZ",
    		ts: "ts-Latn-ZA",
    		tsd: "tsd-Grek-GR",
    		tsf: "tsf-Deva-NP",
    		tsg: "tsg-Latn-PH",
    		tsj: "tsj-Tibt-BT",
    		tsw: "tsw-Latn-ZZ",
    		tt: "tt-Cyrl-RU",
    		ttd: "ttd-Latn-ZZ",
    		tte: "tte-Latn-ZZ",
    		ttj: "ttj-Latn-UG",
    		ttr: "ttr-Latn-ZZ",
    		tts: "tts-Thai-TH",
    		ttt: "ttt-Latn-AZ",
    		tuh: "tuh-Latn-ZZ",
    		tul: "tul-Latn-ZZ",
    		tum: "tum-Latn-MW",
    		tuq: "tuq-Latn-ZZ",
    		tvd: "tvd-Latn-ZZ",
    		tvl: "tvl-Latn-TV",
    		tvu: "tvu-Latn-ZZ",
    		twh: "twh-Latn-ZZ",
    		twq: "twq-Latn-NE",
    		txg: "txg-Tang-CN",
    		ty: "ty-Latn-PF",
    		tya: "tya-Latn-ZZ",
    		tyv: "tyv-Cyrl-RU",
    		tzm: "tzm-Latn-MA",
    		ubu: "ubu-Latn-ZZ",
    		udm: "udm-Cyrl-RU",
    		ug: "ug-Arab-CN",
    		"ug-Cyrl": "ug-Cyrl-KZ",
    		"ug-KZ": "ug-Cyrl-KZ",
    		"ug-MN": "ug-Cyrl-MN",
    		uga: "uga-Ugar-SY",
    		uk: "uk-Cyrl-UA",
    		uli: "uli-Latn-FM",
    		umb: "umb-Latn-AO",
    		und: "en-Latn-US",
    		"und-002": "en-Latn-NG",
    		"und-003": "en-Latn-US",
    		"und-005": "pt-Latn-BR",
    		"und-009": "en-Latn-AU",
    		"und-011": "en-Latn-NG",
    		"und-013": "es-Latn-MX",
    		"und-014": "sw-Latn-TZ",
    		"und-015": "ar-Arab-EG",
    		"und-017": "sw-Latn-CD",
    		"und-018": "en-Latn-ZA",
    		"und-019": "en-Latn-US",
    		"und-021": "en-Latn-US",
    		"und-029": "es-Latn-CU",
    		"und-030": "zh-Hans-CN",
    		"und-034": "hi-Deva-IN",
    		"und-035": "id-Latn-ID",
    		"und-039": "it-Latn-IT",
    		"und-053": "en-Latn-AU",
    		"und-054": "en-Latn-PG",
    		"und-057": "en-Latn-GU",
    		"und-061": "sm-Latn-WS",
    		"und-142": "zh-Hans-CN",
    		"und-143": "uz-Latn-UZ",
    		"und-145": "ar-Arab-SA",
    		"und-150": "ru-Cyrl-RU",
    		"und-151": "ru-Cyrl-RU",
    		"und-154": "en-Latn-GB",
    		"und-155": "de-Latn-DE",
    		"und-202": "en-Latn-NG",
    		"und-419": "es-Latn-419",
    		"und-AD": "ca-Latn-AD",
    		"und-Adlm": "ff-Adlm-GN",
    		"und-AE": "ar-Arab-AE",
    		"und-AF": "fa-Arab-AF",
    		"und-Aghb": "lez-Aghb-RU",
    		"und-Ahom": "aho-Ahom-IN",
    		"und-AL": "sq-Latn-AL",
    		"und-AM": "hy-Armn-AM",
    		"und-AO": "pt-Latn-AO",
    		"und-AQ": "und-Latn-AQ",
    		"und-AR": "es-Latn-AR",
    		"und-Arab": "ar-Arab-EG",
    		"und-Arab-CC": "ms-Arab-CC",
    		"und-Arab-CN": "ug-Arab-CN",
    		"und-Arab-GB": "ks-Arab-GB",
    		"und-Arab-ID": "ms-Arab-ID",
    		"und-Arab-IN": "ur-Arab-IN",
    		"und-Arab-KH": "cja-Arab-KH",
    		"und-Arab-MM": "rhg-Arab-MM",
    		"und-Arab-MN": "kk-Arab-MN",
    		"und-Arab-MU": "ur-Arab-MU",
    		"und-Arab-NG": "ha-Arab-NG",
    		"und-Arab-PK": "ur-Arab-PK",
    		"und-Arab-TG": "apd-Arab-TG",
    		"und-Arab-TH": "mfa-Arab-TH",
    		"und-Arab-TJ": "fa-Arab-TJ",
    		"und-Arab-TR": "az-Arab-TR",
    		"und-Arab-YT": "swb-Arab-YT",
    		"und-Armi": "arc-Armi-IR",
    		"und-Armn": "hy-Armn-AM",
    		"und-AS": "sm-Latn-AS",
    		"und-AT": "de-Latn-AT",
    		"und-Avst": "ae-Avst-IR",
    		"und-AW": "nl-Latn-AW",
    		"und-AX": "sv-Latn-AX",
    		"und-AZ": "az-Latn-AZ",
    		"und-BA": "bs-Latn-BA",
    		"und-Bali": "ban-Bali-ID",
    		"und-Bamu": "bax-Bamu-CM",
    		"und-Bass": "bsq-Bass-LR",
    		"und-Batk": "bbc-Batk-ID",
    		"und-BD": "bn-Beng-BD",
    		"und-BE": "nl-Latn-BE",
    		"und-Beng": "bn-Beng-BD",
    		"und-BF": "fr-Latn-BF",
    		"und-BG": "bg-Cyrl-BG",
    		"und-BH": "ar-Arab-BH",
    		"und-Bhks": "sa-Bhks-IN",
    		"und-BI": "rn-Latn-BI",
    		"und-BJ": "fr-Latn-BJ",
    		"und-BL": "fr-Latn-BL",
    		"und-BN": "ms-Latn-BN",
    		"und-BO": "es-Latn-BO",
    		"und-Bopo": "zh-Bopo-TW",
    		"und-BQ": "pap-Latn-BQ",
    		"und-BR": "pt-Latn-BR",
    		"und-Brah": "pka-Brah-IN",
    		"und-Brai": "fr-Brai-FR",
    		"und-BT": "dz-Tibt-BT",
    		"und-Bugi": "bug-Bugi-ID",
    		"und-Buhd": "bku-Buhd-PH",
    		"und-BV": "und-Latn-BV",
    		"und-BY": "be-Cyrl-BY",
    		"und-Cakm": "ccp-Cakm-BD",
    		"und-Cans": "cr-Cans-CA",
    		"und-Cari": "xcr-Cari-TR",
    		"und-CD": "sw-Latn-CD",
    		"und-CF": "fr-Latn-CF",
    		"und-CG": "fr-Latn-CG",
    		"und-CH": "de-Latn-CH",
    		"und-Cham": "cjm-Cham-VN",
    		"und-Cher": "chr-Cher-US",
    		"und-CI": "fr-Latn-CI",
    		"und-CL": "es-Latn-CL",
    		"und-CM": "fr-Latn-CM",
    		"und-CN": "zh-Hans-CN",
    		"und-CO": "es-Latn-CO",
    		"und-Copt": "cop-Copt-EG",
    		"und-CP": "und-Latn-CP",
    		"und-Cprt": "grc-Cprt-CY",
    		"und-CR": "es-Latn-CR",
    		"und-CU": "es-Latn-CU",
    		"und-CV": "pt-Latn-CV",
    		"und-CW": "pap-Latn-CW",
    		"und-CY": "el-Grek-CY",
    		"und-Cyrl": "ru-Cyrl-RU",
    		"und-Cyrl-AL": "mk-Cyrl-AL",
    		"und-Cyrl-BA": "sr-Cyrl-BA",
    		"und-Cyrl-GE": "ab-Cyrl-GE",
    		"und-Cyrl-GR": "mk-Cyrl-GR",
    		"und-Cyrl-MD": "uk-Cyrl-MD",
    		"und-Cyrl-RO": "bg-Cyrl-RO",
    		"und-Cyrl-SK": "uk-Cyrl-SK",
    		"und-Cyrl-TR": "kbd-Cyrl-TR",
    		"und-Cyrl-XK": "sr-Cyrl-XK",
    		"und-CZ": "cs-Latn-CZ",
    		"und-DE": "de-Latn-DE",
    		"und-Deva": "hi-Deva-IN",
    		"und-Deva-BT": "ne-Deva-BT",
    		"und-Deva-FJ": "hif-Deva-FJ",
    		"und-Deva-MU": "bho-Deva-MU",
    		"und-Deva-PK": "btv-Deva-PK",
    		"und-DJ": "aa-Latn-DJ",
    		"und-DK": "da-Latn-DK",
    		"und-DO": "es-Latn-DO",
    		"und-Dogr": "doi-Dogr-IN",
    		"und-Dupl": "fr-Dupl-FR",
    		"und-DZ": "ar-Arab-DZ",
    		"und-EA": "es-Latn-EA",
    		"und-EC": "es-Latn-EC",
    		"und-EE": "et-Latn-EE",
    		"und-EG": "ar-Arab-EG",
    		"und-Egyp": "egy-Egyp-EG",
    		"und-EH": "ar-Arab-EH",
    		"und-Elba": "sq-Elba-AL",
    		"und-Elym": "arc-Elym-IR",
    		"und-ER": "ti-Ethi-ER",
    		"und-ES": "es-Latn-ES",
    		"und-ET": "am-Ethi-ET",
    		"und-Ethi": "am-Ethi-ET",
    		"und-EU": "en-Latn-GB",
    		"und-EZ": "de-Latn-EZ",
    		"und-FI": "fi-Latn-FI",
    		"und-FO": "fo-Latn-FO",
    		"und-FR": "fr-Latn-FR",
    		"und-GA": "fr-Latn-GA",
    		"und-GE": "ka-Geor-GE",
    		"und-Geor": "ka-Geor-GE",
    		"und-GF": "fr-Latn-GF",
    		"und-GH": "ak-Latn-GH",
    		"und-GL": "kl-Latn-GL",
    		"und-Glag": "cu-Glag-BG",
    		"und-GN": "fr-Latn-GN",
    		"und-Gong": "wsg-Gong-IN",
    		"und-Gonm": "esg-Gonm-IN",
    		"und-Goth": "got-Goth-UA",
    		"und-GP": "fr-Latn-GP",
    		"und-GQ": "es-Latn-GQ",
    		"und-GR": "el-Grek-GR",
    		"und-Gran": "sa-Gran-IN",
    		"und-Grek": "el-Grek-GR",
    		"und-Grek-TR": "bgx-Grek-TR",
    		"und-GS": "und-Latn-GS",
    		"und-GT": "es-Latn-GT",
    		"und-Gujr": "gu-Gujr-IN",
    		"und-Guru": "pa-Guru-IN",
    		"und-GW": "pt-Latn-GW",
    		"und-Hanb": "zh-Hanb-TW",
    		"und-Hang": "ko-Hang-KR",
    		"und-Hani": "zh-Hani-CN",
    		"und-Hano": "hnn-Hano-PH",
    		"und-Hans": "zh-Hans-CN",
    		"und-Hant": "zh-Hant-TW",
    		"und-Hatr": "mis-Hatr-IQ",
    		"und-Hebr": "he-Hebr-IL",
    		"und-Hebr-CA": "yi-Hebr-CA",
    		"und-Hebr-GB": "yi-Hebr-GB",
    		"und-Hebr-SE": "yi-Hebr-SE",
    		"und-Hebr-UA": "yi-Hebr-UA",
    		"und-Hebr-US": "yi-Hebr-US",
    		"und-Hira": "ja-Hira-JP",
    		"und-HK": "zh-Hant-HK",
    		"und-Hluw": "hlu-Hluw-TR",
    		"und-HM": "und-Latn-HM",
    		"und-Hmng": "hnj-Hmng-LA",
    		"und-Hmnp": "mww-Hmnp-US",
    		"und-HN": "es-Latn-HN",
    		"und-HR": "hr-Latn-HR",
    		"und-HT": "ht-Latn-HT",
    		"und-HU": "hu-Latn-HU",
    		"und-Hung": "hu-Hung-HU",
    		"und-IC": "es-Latn-IC",
    		"und-ID": "id-Latn-ID",
    		"und-IL": "he-Hebr-IL",
    		"und-IN": "hi-Deva-IN",
    		"und-IQ": "ar-Arab-IQ",
    		"und-IR": "fa-Arab-IR",
    		"und-IS": "is-Latn-IS",
    		"und-IT": "it-Latn-IT",
    		"und-Ital": "ett-Ital-IT",
    		"und-Jamo": "ko-Jamo-KR",
    		"und-Java": "jv-Java-ID",
    		"und-JO": "ar-Arab-JO",
    		"und-JP": "ja-Jpan-JP",
    		"und-Jpan": "ja-Jpan-JP",
    		"und-Kali": "eky-Kali-MM",
    		"und-Kana": "ja-Kana-JP",
    		"und-KE": "sw-Latn-KE",
    		"und-KG": "ky-Cyrl-KG",
    		"und-KH": "km-Khmr-KH",
    		"und-Khar": "pra-Khar-PK",
    		"und-Khmr": "km-Khmr-KH",
    		"und-Khoj": "sd-Khoj-IN",
    		"und-KM": "ar-Arab-KM",
    		"und-Knda": "kn-Knda-IN",
    		"und-Kore": "ko-Kore-KR",
    		"und-KP": "ko-Kore-KP",
    		"und-KR": "ko-Kore-KR",
    		"und-Kthi": "bho-Kthi-IN",
    		"und-KW": "ar-Arab-KW",
    		"und-KZ": "ru-Cyrl-KZ",
    		"und-LA": "lo-Laoo-LA",
    		"und-Lana": "nod-Lana-TH",
    		"und-Laoo": "lo-Laoo-LA",
    		"und-Latn-AF": "tk-Latn-AF",
    		"und-Latn-AM": "ku-Latn-AM",
    		"und-Latn-CN": "za-Latn-CN",
    		"und-Latn-CY": "tr-Latn-CY",
    		"und-Latn-DZ": "fr-Latn-DZ",
    		"und-Latn-ET": "en-Latn-ET",
    		"und-Latn-GE": "ku-Latn-GE",
    		"und-Latn-IR": "tk-Latn-IR",
    		"und-Latn-KM": "fr-Latn-KM",
    		"und-Latn-MA": "fr-Latn-MA",
    		"und-Latn-MK": "sq-Latn-MK",
    		"und-Latn-MM": "kac-Latn-MM",
    		"und-Latn-MO": "pt-Latn-MO",
    		"und-Latn-MR": "fr-Latn-MR",
    		"und-Latn-RU": "krl-Latn-RU",
    		"und-Latn-SY": "fr-Latn-SY",
    		"und-Latn-TN": "fr-Latn-TN",
    		"und-Latn-TW": "trv-Latn-TW",
    		"und-Latn-UA": "pl-Latn-UA",
    		"und-LB": "ar-Arab-LB",
    		"und-Lepc": "lep-Lepc-IN",
    		"und-LI": "de-Latn-LI",
    		"und-Limb": "lif-Limb-IN",
    		"und-Lina": "lab-Lina-GR",
    		"und-Linb": "grc-Linb-GR",
    		"und-Lisu": "lis-Lisu-CN",
    		"und-LK": "si-Sinh-LK",
    		"und-LS": "st-Latn-LS",
    		"und-LT": "lt-Latn-LT",
    		"und-LU": "fr-Latn-LU",
    		"und-LV": "lv-Latn-LV",
    		"und-LY": "ar-Arab-LY",
    		"und-Lyci": "xlc-Lyci-TR",
    		"und-Lydi": "xld-Lydi-TR",
    		"und-MA": "ar-Arab-MA",
    		"und-Mahj": "hi-Mahj-IN",
    		"und-Maka": "mak-Maka-ID",
    		"und-Mand": "myz-Mand-IR",
    		"und-Mani": "xmn-Mani-CN",
    		"und-Marc": "bo-Marc-CN",
    		"und-MC": "fr-Latn-MC",
    		"und-MD": "ro-Latn-MD",
    		"und-ME": "sr-Latn-ME",
    		"und-Medf": "mis-Medf-NG",
    		"und-Mend": "men-Mend-SL",
    		"und-Merc": "xmr-Merc-SD",
    		"und-Mero": "xmr-Mero-SD",
    		"und-MF": "fr-Latn-MF",
    		"und-MG": "mg-Latn-MG",
    		"und-MK": "mk-Cyrl-MK",
    		"und-ML": "bm-Latn-ML",
    		"und-Mlym": "ml-Mlym-IN",
    		"und-MM": "my-Mymr-MM",
    		"und-MN": "mn-Cyrl-MN",
    		"und-MO": "zh-Hant-MO",
    		"und-Modi": "mr-Modi-IN",
    		"und-Mong": "mn-Mong-CN",
    		"und-MQ": "fr-Latn-MQ",
    		"und-MR": "ar-Arab-MR",
    		"und-Mroo": "mro-Mroo-BD",
    		"und-MT": "mt-Latn-MT",
    		"und-Mtei": "mni-Mtei-IN",
    		"und-MU": "mfe-Latn-MU",
    		"und-Mult": "skr-Mult-PK",
    		"und-MV": "dv-Thaa-MV",
    		"und-MX": "es-Latn-MX",
    		"und-MY": "ms-Latn-MY",
    		"und-Mymr": "my-Mymr-MM",
    		"und-Mymr-IN": "kht-Mymr-IN",
    		"und-Mymr-TH": "mnw-Mymr-TH",
    		"und-MZ": "pt-Latn-MZ",
    		"und-NA": "af-Latn-NA",
    		"und-Nand": "sa-Nand-IN",
    		"und-Narb": "xna-Narb-SA",
    		"und-Nbat": "arc-Nbat-JO",
    		"und-NC": "fr-Latn-NC",
    		"und-NE": "ha-Latn-NE",
    		"und-Newa": "new-Newa-NP",
    		"und-NI": "es-Latn-NI",
    		"und-Nkoo": "man-Nkoo-GN",
    		"und-NL": "nl-Latn-NL",
    		"und-NO": "nb-Latn-NO",
    		"und-NP": "ne-Deva-NP",
    		"und-Nshu": "zhx-Nshu-CN",
    		"und-Ogam": "sga-Ogam-IE",
    		"und-Olck": "sat-Olck-IN",
    		"und-OM": "ar-Arab-OM",
    		"und-Orkh": "otk-Orkh-MN",
    		"und-Orya": "or-Orya-IN",
    		"und-Osge": "osa-Osge-US",
    		"und-Osma": "so-Osma-SO",
    		"und-PA": "es-Latn-PA",
    		"und-Palm": "arc-Palm-SY",
    		"und-Pauc": "ctd-Pauc-MM",
    		"und-PE": "es-Latn-PE",
    		"und-Perm": "kv-Perm-RU",
    		"und-PF": "fr-Latn-PF",
    		"und-PG": "tpi-Latn-PG",
    		"und-PH": "fil-Latn-PH",
    		"und-Phag": "lzh-Phag-CN",
    		"und-Phli": "pal-Phli-IR",
    		"und-Phlp": "pal-Phlp-CN",
    		"und-Phnx": "phn-Phnx-LB",
    		"und-PK": "ur-Arab-PK",
    		"und-PL": "pl-Latn-PL",
    		"und-Plrd": "hmd-Plrd-CN",
    		"und-PM": "fr-Latn-PM",
    		"und-PR": "es-Latn-PR",
    		"und-Prti": "xpr-Prti-IR",
    		"und-PS": "ar-Arab-PS",
    		"und-PT": "pt-Latn-PT",
    		"und-PW": "pau-Latn-PW",
    		"und-PY": "gn-Latn-PY",
    		"und-QA": "ar-Arab-QA",
    		"und-QO": "en-Latn-DG",
    		"und-RE": "fr-Latn-RE",
    		"und-Rjng": "rej-Rjng-ID",
    		"und-RO": "ro-Latn-RO",
    		"und-Rohg": "rhg-Rohg-MM",
    		"und-RS": "sr-Cyrl-RS",
    		"und-RU": "ru-Cyrl-RU",
    		"und-Runr": "non-Runr-SE",
    		"und-RW": "rw-Latn-RW",
    		"und-SA": "ar-Arab-SA",
    		"und-Samr": "smp-Samr-IL",
    		"und-Sarb": "xsa-Sarb-YE",
    		"und-Saur": "saz-Saur-IN",
    		"und-SC": "fr-Latn-SC",
    		"und-SD": "ar-Arab-SD",
    		"und-SE": "sv-Latn-SE",
    		"und-Sgnw": "ase-Sgnw-US",
    		"und-Shaw": "en-Shaw-GB",
    		"und-Shrd": "sa-Shrd-IN",
    		"und-SI": "sl-Latn-SI",
    		"und-Sidd": "sa-Sidd-IN",
    		"und-Sind": "sd-Sind-IN",
    		"und-Sinh": "si-Sinh-LK",
    		"und-SJ": "nb-Latn-SJ",
    		"und-SK": "sk-Latn-SK",
    		"und-SM": "it-Latn-SM",
    		"und-SN": "fr-Latn-SN",
    		"und-SO": "so-Latn-SO",
    		"und-Sogd": "sog-Sogd-UZ",
    		"und-Sogo": "sog-Sogo-UZ",
    		"und-Sora": "srb-Sora-IN",
    		"und-Soyo": "cmg-Soyo-MN",
    		"und-SR": "nl-Latn-SR",
    		"und-ST": "pt-Latn-ST",
    		"und-Sund": "su-Sund-ID",
    		"und-SV": "es-Latn-SV",
    		"und-SY": "ar-Arab-SY",
    		"und-Sylo": "syl-Sylo-BD",
    		"und-Syrc": "syr-Syrc-IQ",
    		"und-Tagb": "tbw-Tagb-PH",
    		"und-Takr": "doi-Takr-IN",
    		"und-Tale": "tdd-Tale-CN",
    		"und-Talu": "khb-Talu-CN",
    		"und-Taml": "ta-Taml-IN",
    		"und-Tang": "txg-Tang-CN",
    		"und-Tavt": "blt-Tavt-VN",
    		"und-TD": "fr-Latn-TD",
    		"und-Telu": "te-Telu-IN",
    		"und-TF": "fr-Latn-TF",
    		"und-Tfng": "zgh-Tfng-MA",
    		"und-TG": "fr-Latn-TG",
    		"und-Tglg": "fil-Tglg-PH",
    		"und-TH": "th-Thai-TH",
    		"und-Thaa": "dv-Thaa-MV",
    		"und-Thai": "th-Thai-TH",
    		"und-Thai-CN": "lcp-Thai-CN",
    		"und-Thai-KH": "kdt-Thai-KH",
    		"und-Thai-LA": "kdt-Thai-LA",
    		"und-Tibt": "bo-Tibt-CN",
    		"und-Tirh": "mai-Tirh-IN",
    		"und-TJ": "tg-Cyrl-TJ",
    		"und-TK": "tkl-Latn-TK",
    		"und-TL": "pt-Latn-TL",
    		"und-TM": "tk-Latn-TM",
    		"und-TN": "ar-Arab-TN",
    		"und-TO": "to-Latn-TO",
    		"und-TR": "tr-Latn-TR",
    		"und-TV": "tvl-Latn-TV",
    		"und-TW": "zh-Hant-TW",
    		"und-TZ": "sw-Latn-TZ",
    		"und-UA": "uk-Cyrl-UA",
    		"und-UG": "sw-Latn-UG",
    		"und-Ugar": "uga-Ugar-SY",
    		"und-UY": "es-Latn-UY",
    		"und-UZ": "uz-Latn-UZ",
    		"und-VA": "it-Latn-VA",
    		"und-Vaii": "vai-Vaii-LR",
    		"und-VE": "es-Latn-VE",
    		"und-VN": "vi-Latn-VN",
    		"und-VU": "bi-Latn-VU",
    		"und-Wara": "hoc-Wara-IN",
    		"und-Wcho": "nnp-Wcho-IN",
    		"und-WF": "fr-Latn-WF",
    		"und-WS": "sm-Latn-WS",
    		"und-XK": "sq-Latn-XK",
    		"und-Xpeo": "peo-Xpeo-IR",
    		"und-Xsux": "akk-Xsux-IQ",
    		"und-YE": "ar-Arab-YE",
    		"und-Yiii": "ii-Yiii-CN",
    		"und-YT": "fr-Latn-YT",
    		"und-Zanb": "cmg-Zanb-MN",
    		"und-ZW": "sn-Latn-ZW",
    		unr: "unr-Beng-IN",
    		"unr-Deva": "unr-Deva-NP",
    		"unr-NP": "unr-Deva-NP",
    		unx: "unx-Beng-IN",
    		uok: "uok-Latn-ZZ",
    		ur: "ur-Arab-PK",
    		uri: "uri-Latn-ZZ",
    		urt: "urt-Latn-ZZ",
    		urw: "urw-Latn-ZZ",
    		usa: "usa-Latn-ZZ",
    		utr: "utr-Latn-ZZ",
    		uvh: "uvh-Latn-ZZ",
    		uvl: "uvl-Latn-ZZ",
    		uz: "uz-Latn-UZ",
    		"uz-AF": "uz-Arab-AF",
    		"uz-Arab": "uz-Arab-AF",
    		"uz-CN": "uz-Cyrl-CN",
    		vag: "vag-Latn-ZZ",
    		vai: "vai-Vaii-LR",
    		van: "van-Latn-ZZ",
    		ve: "ve-Latn-ZA",
    		vec: "vec-Latn-IT",
    		vep: "vep-Latn-RU",
    		vi: "vi-Latn-VN",
    		vic: "vic-Latn-SX",
    		viv: "viv-Latn-ZZ",
    		vls: "vls-Latn-BE",
    		vmf: "vmf-Latn-DE",
    		vmw: "vmw-Latn-MZ",
    		vo: "vo-Latn-001",
    		vot: "vot-Latn-RU",
    		vro: "vro-Latn-EE",
    		vun: "vun-Latn-TZ",
    		vut: "vut-Latn-ZZ",
    		wa: "wa-Latn-BE",
    		wae: "wae-Latn-CH",
    		waj: "waj-Latn-ZZ",
    		wal: "wal-Ethi-ET",
    		wan: "wan-Latn-ZZ",
    		war: "war-Latn-PH",
    		wbp: "wbp-Latn-AU",
    		wbq: "wbq-Telu-IN",
    		wbr: "wbr-Deva-IN",
    		wci: "wci-Latn-ZZ",
    		wer: "wer-Latn-ZZ",
    		wgi: "wgi-Latn-ZZ",
    		whg: "whg-Latn-ZZ",
    		wib: "wib-Latn-ZZ",
    		wiu: "wiu-Latn-ZZ",
    		wiv: "wiv-Latn-ZZ",
    		wja: "wja-Latn-ZZ",
    		wji: "wji-Latn-ZZ",
    		wls: "wls-Latn-WF",
    		wmo: "wmo-Latn-ZZ",
    		wnc: "wnc-Latn-ZZ",
    		wni: "wni-Arab-KM",
    		wnu: "wnu-Latn-ZZ",
    		wo: "wo-Latn-SN",
    		wob: "wob-Latn-ZZ",
    		wos: "wos-Latn-ZZ",
    		wrs: "wrs-Latn-ZZ",
    		wsg: "wsg-Gong-IN",
    		wsk: "wsk-Latn-ZZ",
    		wtm: "wtm-Deva-IN",
    		wuu: "wuu-Hans-CN",
    		wuv: "wuv-Latn-ZZ",
    		wwa: "wwa-Latn-ZZ",
    		xav: "xav-Latn-BR",
    		xbi: "xbi-Latn-ZZ",
    		xcr: "xcr-Cari-TR",
    		xes: "xes-Latn-ZZ",
    		xh: "xh-Latn-ZA",
    		xla: "xla-Latn-ZZ",
    		xlc: "xlc-Lyci-TR",
    		xld: "xld-Lydi-TR",
    		xmf: "xmf-Geor-GE",
    		xmn: "xmn-Mani-CN",
    		xmr: "xmr-Merc-SD",
    		xna: "xna-Narb-SA",
    		xnr: "xnr-Deva-IN",
    		xog: "xog-Latn-UG",
    		xon: "xon-Latn-ZZ",
    		xpr: "xpr-Prti-IR",
    		xrb: "xrb-Latn-ZZ",
    		xsa: "xsa-Sarb-YE",
    		xsi: "xsi-Latn-ZZ",
    		xsm: "xsm-Latn-ZZ",
    		xsr: "xsr-Deva-NP",
    		xwe: "xwe-Latn-ZZ",
    		yam: "yam-Latn-ZZ",
    		yao: "yao-Latn-MZ",
    		yap: "yap-Latn-FM",
    		yas: "yas-Latn-ZZ",
    		yat: "yat-Latn-ZZ",
    		yav: "yav-Latn-CM",
    		yay: "yay-Latn-ZZ",
    		yaz: "yaz-Latn-ZZ",
    		yba: "yba-Latn-ZZ",
    		ybb: "ybb-Latn-CM",
    		yby: "yby-Latn-ZZ",
    		yer: "yer-Latn-ZZ",
    		ygr: "ygr-Latn-ZZ",
    		ygw: "ygw-Latn-ZZ",
    		yi: "yi-Hebr-001",
    		yko: "yko-Latn-ZZ",
    		yle: "yle-Latn-ZZ",
    		ylg: "ylg-Latn-ZZ",
    		yll: "yll-Latn-ZZ",
    		yml: "yml-Latn-ZZ",
    		yo: "yo-Latn-NG",
    		yon: "yon-Latn-ZZ",
    		yrb: "yrb-Latn-ZZ",
    		yre: "yre-Latn-ZZ",
    		yrl: "yrl-Latn-BR",
    		yss: "yss-Latn-ZZ",
    		yua: "yua-Latn-MX",
    		yue: "yue-Hant-HK",
    		"yue-CN": "yue-Hans-CN",
    		"yue-Hans": "yue-Hans-CN",
    		yuj: "yuj-Latn-ZZ",
    		yut: "yut-Latn-ZZ",
    		yuw: "yuw-Latn-ZZ",
    		za: "za-Latn-CN",
    		zag: "zag-Latn-SD",
    		zdj: "zdj-Arab-KM",
    		zea: "zea-Latn-NL",
    		zgh: "zgh-Tfng-MA",
    		zh: "zh-Hans-CN",
    		"zh-AU": "zh-Hant-AU",
    		"zh-BN": "zh-Hant-BN",
    		"zh-Bopo": "zh-Bopo-TW",
    		"zh-GB": "zh-Hant-GB",
    		"zh-GF": "zh-Hant-GF",
    		"zh-Hanb": "zh-Hanb-TW",
    		"zh-Hant": "zh-Hant-TW",
    		"zh-HK": "zh-Hant-HK",
    		"zh-ID": "zh-Hant-ID",
    		"zh-MO": "zh-Hant-MO",
    		"zh-MY": "zh-Hant-MY",
    		"zh-PA": "zh-Hant-PA",
    		"zh-PF": "zh-Hant-PF",
    		"zh-PH": "zh-Hant-PH",
    		"zh-SR": "zh-Hant-SR",
    		"zh-TH": "zh-Hant-TH",
    		"zh-TW": "zh-Hant-TW",
    		"zh-US": "zh-Hant-US",
    		"zh-VN": "zh-Hant-VN",
    		zhx: "zhx-Nshu-CN",
    		zia: "zia-Latn-ZZ",
    		zlm: "zlm-Latn-TG",
    		zmi: "zmi-Latn-MY",
    		zne: "zne-Latn-ZZ",
    		zu: "zu-Latn-ZA",
    		zza: "zza-Latn-TR"
    	}
    };
    var likelySubtags = {
    	supplemental: supplemental
    };

    var likelySubtags$1 = /*#__PURE__*/Object.freeze({
        __proto__: null,
        supplemental: supplemental,
        'default': likelySubtags
    });

    var likelySubtags$2 = getCjsExportFromNamespace(likelySubtags$1);

    var canonicalizer = createCommonjsModule(function (module, exports) {
    var __spreadArrays = (commonjsGlobal && commonjsGlobal.__spreadArrays) || function () {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    };
    Object.defineProperty(exports, "__esModule", { value: true });




    function canonicalizeAttrs(strs) {
        return Object.keys(strs.reduce(function (all, str) {
            all[str.toLowerCase()] = 1;
            return all;
        }, {})).sort();
    }
    function canonicalizeKVs(arr) {
        var all = {};
        var result = [];
        for (var _i = 0, arr_1 = arr; _i < arr_1.length; _i++) {
            var kv = arr_1[_i];
            if (kv[0] in all) {
                continue;
            }
            all[kv[0]] = 1;
            if (!kv[1] || kv[1] === 'true') {
                result.push([kv[0].toLowerCase()]);
            }
            else {
                result.push([kv[0].toLowerCase(), kv[1].toLowerCase()]);
            }
        }
        return result.sort(compareKV);
    }
    function compareKV(t1, t2) {
        return t1[0] < t2[0] ? -1 : t1[0] > t2[0] ? 1 : 0;
    }
    function compareExtension(e1, e2) {
        return e1.type < e2.type ? -1 : e1.type > e2.type ? 1 : 0;
    }
    function mergeVariants(v1, v2) {
        var result = __spreadArrays(v1);
        for (var _i = 0, v2_1 = v2; _i < v2_1.length; _i++) {
            var v = v2_1[_i];
            if (v1.indexOf(v) < 0) {
                result.push(v);
            }
        }
        return result;
    }
    /**
     * CAVEAT: We don't do this section in the spec bc they have no JSON data
     * Use the bcp47 data to replace keys, types, tfields, and tvalues by their canonical forms. See Section 3.6.4 U Extension Data Files) and Section 3.7.1 T Extension Data Files. The aliases are in the alias attribute value, while the canonical is in the name attribute value. For example,
    Because of the following bcp47 data:
    <key name="ms"…>…<type name="uksystem" … alias="imperial" … />…</key>
    We get the following transformation:
    en-u-ms-imperial ⇒ en-u-ms-uksystem
     * @param lang
     */
    function canonicalizeUnicodeLanguageId(unicodeLanguageId) {
        /**
         * If the language subtag matches the type attribute of a languageAlias element in Supplemental Data, replace the language subtag with the replacement value.
         *  1. If there are additional subtags in the replacement value, add them to the result, but only if there is no corresponding subtag already in the tag.
         *  2. Five special deprecated grandfathered codes (such as i-default) are in type attributes, and are also replaced.
         */
        // From https://github.com/unicode-org/icu/blob/master/icu4j/main/classes/core/src/com/ibm/icu/util/ULocale.java#L1246
        // Try language _ variant
        var finalLangAst = unicodeLanguageId;
        if (unicodeLanguageId.variants.length) {
            var replacedLang_1 = '';
            for (var _i = 0, _a = unicodeLanguageId.variants; _i < _a.length; _i++) {
                var variant = _a[_i];
                if ((replacedLang_1 =
                    aliases.languageAlias[emitter.emitUnicodeLanguageId({
                        lang: unicodeLanguageId.lang,
                        variants: [variant],
                    })])) {
                    var replacedLangAst = parser.parseUnicodeLanguageId(replacedLang_1.split(parser.SEPARATOR));
                    finalLangAst = {
                        lang: replacedLangAst.lang,
                        script: finalLangAst.script || replacedLangAst.script,
                        region: finalLangAst.region || replacedLangAst.region,
                        variants: mergeVariants(finalLangAst.variants, replacedLangAst.variants),
                    };
                    break;
                }
            }
        }
        // language _ script _ country
        // ug-Arab-CN -> ug-CN
        if (finalLangAst.script && finalLangAst.region) {
            var replacedLang_2 = aliases.languageAlias[emitter.emitUnicodeLanguageId({
                lang: finalLangAst.lang,
                script: finalLangAst.script,
                region: finalLangAst.region,
                variants: [],
            })];
            if (replacedLang_2) {
                var replacedLangAst = parser.parseUnicodeLanguageId(replacedLang_2.split(parser.SEPARATOR));
                finalLangAst = {
                    lang: replacedLangAst.lang,
                    script: replacedLangAst.script,
                    region: replacedLangAst.region,
                    variants: finalLangAst.variants,
                };
            }
        }
        // language _ country
        // eg. az_AZ -> az_Latn_A
        if (finalLangAst.region) {
            var replacedLang_3 = aliases.languageAlias[emitter.emitUnicodeLanguageId({
                lang: finalLangAst.lang,
                region: finalLangAst.region,
                variants: [],
            })];
            if (replacedLang_3) {
                var replacedLangAst = parser.parseUnicodeLanguageId(replacedLang_3.split(parser.SEPARATOR));
                finalLangAst = {
                    lang: replacedLangAst.lang,
                    script: finalLangAst.script || replacedLangAst.script,
                    region: replacedLangAst.region,
                    variants: finalLangAst.variants,
                };
            }
        }
        // only language
        // e.g. twi -> ak
        var replacedLang = aliases.languageAlias[emitter.emitUnicodeLanguageId({
            lang: finalLangAst.lang,
            variants: [],
        })];
        if (replacedLang) {
            var replacedLangAst = parser.parseUnicodeLanguageId(replacedLang.split(parser.SEPARATOR));
            finalLangAst = {
                lang: replacedLangAst.lang,
                script: finalLangAst.script || replacedLangAst.script,
                region: finalLangAst.region || replacedLangAst.region,
                variants: finalLangAst.variants,
            };
        }
        if (finalLangAst.region) {
            var region = finalLangAst.region.toUpperCase();
            var regionAlias = aliases.territoryAlias[region];
            var replacedRegion = void 0;
            if (regionAlias) {
                var regions = regionAlias.split(' ');
                replacedRegion = regions[0];
                var likelySubtag = likelySubtags$2.supplemental.likelySubtags[emitter.emitUnicodeLanguageId({
                    lang: finalLangAst.lang,
                    script: finalLangAst.script,
                    variants: [],
                })];
                if (likelySubtag) {
                    var likelyRegion = parser.parseUnicodeLanguageId(likelySubtag.split(parser.SEPARATOR)).region;
                    if (likelyRegion && regions.indexOf(likelyRegion) > -1) {
                        replacedRegion = likelyRegion;
                    }
                }
            }
            if (replacedRegion) {
                finalLangAst.region = replacedRegion;
            }
            finalLangAst.region = finalLangAst.region.toUpperCase();
        }
        if (finalLangAst.script) {
            finalLangAst.script =
                finalLangAst.script[0].toUpperCase() +
                    finalLangAst.script.slice(1).toLowerCase();
            if (aliases.scriptAlias[finalLangAst.script]) {
                finalLangAst.script = aliases.scriptAlias[finalLangAst.script];
            }
        }
        if (finalLangAst.variants.length) {
            for (var i = 0; i < finalLangAst.variants.length; i++) {
                var variant = finalLangAst.variants[i].toLowerCase();
                if (aliases.variantAlias[variant]) {
                    var alias = aliases.variantAlias[variant];
                    if (parser.isUnicodeVariantSubtag(alias)) {
                        finalLangAst.variants[i] = alias;
                    }
                    else if (parser.isUnicodeLanguageSubtag(alias)) {
                        // Yes this can happen per the spec
                        finalLangAst.lang = alias;
                    }
                }
            }
            finalLangAst.variants.sort();
        }
        return finalLangAst;
    }
    exports.canonicalizeUnicodeLanguageId = canonicalizeUnicodeLanguageId;
    /**
     * Canonicalize based on
     * https://www.unicode.org/reports/tr35/tr35.html#Canonical_Unicode_Locale_Identifiers
     * https://tc39.es/ecma402/#sec-canonicalizeunicodelocaleid
     * IMPORTANT: This modifies the object inline
     * @param locale
     */
    function canonicalizeUnicodeLocaleId(locale) {
        locale.lang = canonicalizeUnicodeLanguageId(locale.lang);
        if (locale.extensions) {
            for (var _i = 0, _a = locale.extensions; _i < _a.length; _i++) {
                var extension = _a[_i];
                switch (extension.type) {
                    case 'u':
                        extension.keywords = canonicalizeKVs(extension.keywords);
                        if (extension.attributes) {
                            extension.attributes = canonicalizeAttrs(extension.attributes);
                        }
                        break;
                    case 't':
                        if (extension.lang) {
                            extension.lang = canonicalizeUnicodeLanguageId(extension.lang);
                        }
                        extension.fields = canonicalizeKVs(extension.fields);
                        break;
                    default:
                        extension.value = extension.value.toLowerCase();
                        break;
                }
            }
            locale.extensions.sort(compareExtension);
        }
        return locale;
    }
    exports.canonicalizeUnicodeLocaleId = canonicalizeUnicodeLocaleId;

    });

    unwrapExports(canonicalizer);
    var canonicalizer_1 = canonicalizer.canonicalizeUnicodeLanguageId;
    var canonicalizer_2 = canonicalizer.canonicalizeUnicodeLocaleId;

    var dist = createCommonjsModule(function (module, exports) {
    function __export(m) {
        for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
    }
    Object.defineProperty(exports, "__esModule", { value: true });



    function canonicalizeLocaleList(locales) {
        if (locales === undefined) {
            return [];
        }
        var seen = [];
        if (typeof locales === 'string') {
            locales = [locales];
        }
        for (var _i = 0, locales_1 = locales; _i < locales_1.length; _i++) {
            var locale = locales_1[_i];
            var canonicalizedTag = emitter.emitUnicodeLocaleId(canonicalizer.canonicalizeUnicodeLocaleId(parser.parseUnicodeLocaleId(locale)));
            if (seen.indexOf(canonicalizedTag) < 0) {
                seen.push(canonicalizedTag);
            }
        }
        return seen;
    }
    function getCanonicalLocales(locales) {
        return canonicalizeLocaleList(locales);
    }
    exports.getCanonicalLocales = getCanonicalLocales;
    var parser_2 = parser;
    exports.parseUnicodeLocaleId = parser_2.parseUnicodeLocaleId;
    exports.parseUnicodeLanguageId = parser_2.parseUnicodeLanguageId;
    exports.isStructurallyValidLanguageTag = parser_2.isStructurallyValidLanguageTag;
    exports.isUnicodeRegionSubtag = parser_2.isUnicodeRegionSubtag;
    exports.isUnicodeScriptSubtag = parser_2.isUnicodeScriptSubtag;
    exports.isUnicodeLanguageSubtag = parser_2.isUnicodeLanguageSubtag;
    __export(emitter);

    });

    unwrapExports(dist);
    var dist_1 = dist.getCanonicalLocales;
    var dist_2 = dist.parseUnicodeLocaleId;
    var dist_3 = dist.parseUnicodeLanguageId;
    var dist_4 = dist.isStructurallyValidLanguageTag;
    var dist_5 = dist.isUnicodeRegionSubtag;
    var dist_6 = dist.isUnicodeScriptSubtag;
    var dist_7 = dist.isUnicodeLanguageSubtag;

    var polyfill = createCommonjsModule(function (module, exports) {
    Object.defineProperty(exports, "__esModule", { value: true });

    if (!('getCanonicalLocales' in Intl) ||
        // Native Intl.getCanonicalLocales is just buggy
        Intl.getCanonicalLocales('und-x-private')[0] === 'x-private') {
        Object.defineProperty(Intl, 'getCanonicalLocales', {
            value: dist.getCanonicalLocales,
            writable: true,
            enumerable: false,
            configurable: true,
        });
    }

    });

    unwrapExports(polyfill);

    if (Intl.NumberFormat && typeof Intl.NumberFormat.__addLocaleData === 'function') {
      Intl.NumberFormat.__addLocaleData(
        {"data":{"ar":{"units":{"simple":{"degree":{"long":{"other":"{0} درجة","one":"درجة","two":"درجتان","few":"{0} درجات"},"short":{"other":"{0} درجة","one":"درجة","two":"درجتان","few":"{0} درجات"},"narrow":{"other":"{0} درجة","two":"درجتان ({0})","few":"{0} درجات"},"perUnit":{}},"hectare":{"long":{"other":"{0} هكتار"},"short":{"other":"{0} هكتار"},"narrow":{"other":"{0} هكت","one":"{0} هكتار","few":"{0} هكتارات","many":"{0} هكتارًا"},"perUnit":{}},"acre":{"long":{"other":"{0} فدان","one":"فدان"},"short":{"other":"{0} فدان","one":"فدان"},"narrow":{"other":"{0} من الفدادين","one":"{0} فدان","two":"فدانان ({0})","few":"{0} فدادين","many":"{0} فدانًا"},"perUnit":{}},"percent":{"long":{"other":"{0} بالمائة","zero":"{0}٪","two":"{0}٪","few":"{0}٪","many":"{0}٪"},"short":{"other":"{0}٪"},"narrow":{"other":"{0}٪"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"{0} لتر لكل كيلومتر","two":"لتران لكل كيلومتر","few":"{0} لترات لكل كيلومتر","many":"{0} لترًا لكل كيلومتر"},"short":{"other":"{0} لتر/كم"},"narrow":{"other":"{0} لتر/كم"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"{0} ميل لكل غالون","two":"ميلان لكل غالون","few":"{0} أميال لكل غالون","many":"{0} ميلًا لكل غالون"},"short":{"other":"{0} ميل/غالون"},"narrow":{"other":"{0} ميل/غالون"},"perUnit":{}},"petabyte":{"long":{"other":"{0} بيتابايت"},"short":{"other":"{0} بيتابايت"},"narrow":{"other":"{0} بيتابايت"},"perUnit":{}},"terabyte":{"long":{"other":"{0} تيرابايت"},"short":{"other":"{0} تيرابايت"},"narrow":{"other":"{0} تيرابايت"},"perUnit":{}},"terabit":{"long":{"other":"{0} تيرابت"},"short":{"other":"{0} تيرابت"},"narrow":{"other":"{0} تيرابت"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} غيغابايت"},"short":{"other":"{0} غيغابايت"},"narrow":{"other":"{0} غيغابايت"},"perUnit":{}},"gigabit":{"long":{"other":"{0} غيغابت"},"short":{"other":"{0} غيغابت"},"narrow":{"other":"{0} غيغابت"},"perUnit":{}},"megabyte":{"long":{"other":"{0} ميغابايت"},"short":{"other":"{0} ميغابايت"},"narrow":{"other":"{0} ميغابايت"},"perUnit":{}},"megabit":{"long":{"other":"{0} ميغابت"},"short":{"other":"{0} ميغابت"},"narrow":{"other":"{0} ميغابت"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} كيلوبايت"},"short":{"other":"{0} كيلوبايت"},"narrow":{"other":"{0} كيلوبايت"},"perUnit":{}},"kilobit":{"long":{"other":"{0} كيلوبت"},"short":{"other":"{0} كيلوبت"},"narrow":{"other":"{0} كيلوبت"},"perUnit":{}},"byte":{"long":{"other":"{0} بايت"},"short":{"other":"{0} بايت"},"narrow":{"other":"{0} بايت"},"perUnit":{}},"bit":{"long":{"other":"{0} بت"},"short":{"other":"{0} بت"},"narrow":{"other":"{0} بت"},"perUnit":{}},"year":{"long":{"other":"{0} سنة","one":"سنة","two":"سنتان","few":"{0} سنوات"},"short":{"other":"{0} سنة","one":"سنة واحدة","two":"سنتان","few":"{0} سنوات"},"narrow":{"other":"{0} سنة"},"perUnit":{"long":"{0} في السنة","short":"{0}/سنة","narrow":"{0}/سنة"}},"month":{"long":{"other":"{0} شهر","one":"شهر","two":"شهران","few":"{0} أشهر","many":"{0} شهرًا"},"short":{"other":"{0} شهر","one":"شهر","two":"شهران","few":"{0} أشهر","many":"{0} شهرًا"},"narrow":{"other":"{0} شهر","one":"شهر","two":"شهران"},"perUnit":{"long":"{0} في الشهر","short":"{0}/ش","narrow":"{0}/ش"}},"week":{"long":{"other":"{0} أسبوع","one":"أسبوع","two":"أسبوعان","few":"{0} أسابيع","many":"{0} أسبوعًا"},"short":{"other":"{0} أسبوع","one":"أسبوع","two":"أسبوعان","few":"{0} أسابيع","many":"{0} أسبوعًا"},"narrow":{"other":"{0} أ"},"perUnit":{"long":"{0} كل أسبوع","short":"{0}/أ","narrow":"{0}/أ"}},"day":{"long":{"other":"{0} يوم","one":"يوم","two":"يومان","few":"{0} أيام","many":"{0} يومًا"},"short":{"other":"{0} يوم","one":"يوم","two":"يومان","few":"{0} أيام","many":"{0} يومًا"},"narrow":{"other":"{0} ي"},"perUnit":{"long":"{0} كل يوم","short":"{0}/ي","narrow":"{0}/ي"}},"hour":{"long":{"other":"{0} ساعة","one":"ساعة","two":"ساعتان","few":"{0} ساعات"},"short":{"other":"{0} س"},"narrow":{"other":"{0} س"},"perUnit":{"long":"{0} كل ساعة","short":"{0}/س","narrow":"{0}/س"}},"minute":{"long":{"other":"{0} دقيقة","one":"دقيقة","two":"دقيقتان","few":"{0} دقائق"},"short":{"other":"{0} د"},"narrow":{"other":"{0} د"},"perUnit":{"long":"{0} كل دقيقة","short":"{0}/د","narrow":"{0}/د"}},"second":{"long":{"other":"{0} ثانية","one":"ثانية","two":"ثانيتان","few":"{0} ثوان"},"short":{"other":"{0} ث"},"narrow":{"other":"{0} ث"},"perUnit":{"long":"{0}/ثانية","short":"{0}/ث","narrow":"{0}/ث"}},"millisecond":{"long":{"other":"{0} ملي ثانية"},"short":{"other":"{0} ملي ث"},"narrow":{"other":"{0} ملي ث"},"perUnit":{}},"kilometer":{"long":{"other":"{0} كيلومتر"},"short":{"other":"{0} كم"},"narrow":{"other":"{0} كم"},"perUnit":{"long":"{0}/كيلومتر","short":"{0}/كم","narrow":"{0}/كم"}},"meter":{"long":{"other":"{0} متر","one":"متر","two":"متران","few":"{0} أمتار","many":"{0} مترًا"},"short":{"other":"{0} متر","one":"متر","two":"متران","few":"{0} أمتار","many":"{0} مترًا"},"narrow":{"other":"{0} م"},"perUnit":{"long":"{0} لكل متر","short":"{0}/م","narrow":"{0}/م"}},"centimeter":{"long":{"other":"{0} سنتيمتر"},"short":{"other":"{0} سم"},"narrow":{"other":"{0} سم"},"perUnit":{"long":"{0}/سنتيمتر","short":"{0}/سم","narrow":"{0}/سم"}},"millimeter":{"long":{"other":"{0} مليمتر"},"short":{"other":"{0} مم"},"narrow":{"other":"{0} مم"},"perUnit":{}},"mile":{"long":{"other":"{0} ميل","one":"ميل","two":"ميلان","few":"{0} أميال","many":"{0} ميلاً"},"short":{"other":"{0} ميل","one":"ميل"},"narrow":{"other":"{0} من الأميال","one":"{0} ميل","two":"ميلان ({0})","few":"{0} أميال","many":"{0} ميلاً"},"perUnit":{}},"yard":{"long":{"other":"{0} ياردة","one":"ياردة"},"short":{"other":"{0} ياردة","one":"ياردة"},"narrow":{"other":"{0} من الياردات","one":"{0} ياردة","two":"ياردتان ({0})","few":"{0} ياردات","many":"{0} ياردة"},"perUnit":{}},"foot":{"long":{"other":"{0} قدم","one":"قدم"},"short":{"other":"{0} قدم","one":"قدم"},"narrow":{"other":"{0} من الأقدام","one":"{0} قدم","two":"قدمان ({0})","few":"{0} أقدام","many":"{0} قدمًا"},"perUnit":{"long":"{0} لكل قدم","short":"{0}/قدم","narrow":"{0}/قدم"}},"inch":{"long":{"other":"{0} بوصة"},"short":{"other":"{0} بوصة"},"narrow":{"other":"{0} بوصة"},"perUnit":{"long":"{0}/بوصة","short":"{0}/بوصة","narrow":"{0}/بوصة"}},"mile-scandinavian":{"long":{"other":"{0} ميل اسكندنافي"},"short":{"other":"{0} ميل اسكندنافي"},"narrow":{"other":"{0} ميل اسكندنافي"},"perUnit":{}},"kilogram":{"long":{"other":"{0} كيلوغرام"},"short":{"other":"{0} كغم"},"narrow":{"other":"{0} كغ"},"perUnit":{"long":"{0}/كيلوغرام","short":"{0}/كغم","narrow":"{0}/كغم"}},"gram":{"long":{"other":"{0} غرام","one":"غرام","two":"غرامان","few":"{0} غرامات","many":"{0} غرامًا"},"short":{"other":"{0} غرام","one":"غرام"},"narrow":{"other":"{0} غ"},"perUnit":{"long":"{0}/غرام","short":"{0}/غرام","narrow":"{0}/غرام"}},"stone":{"long":{"other":"{0} st"},"short":{"other":"{0} st"},"narrow":{"other":"{0} st"},"perUnit":{}},"pound":{"long":{"other":"{0} رطل","two":"رطلان","few":"{0} أرطل","many":"{0} رطلًا"},"short":{"other":"{0} رطل"},"narrow":{"other":"{0}#"},"perUnit":{"long":"{0}/رطل","short":"{0}/رطل","narrow":"{0}/رطل"}},"ounce":{"long":{"other":"{0} أونصة"},"short":{"other":"{0} أونصة","one":"أونصة"},"narrow":{"other":"{0} أونس"},"perUnit":{"long":"{0}/أونصة","short":"{0}/أونصة","narrow":"{0}/أونصة"}},"kilometer-per-hour":{"long":{"other":"{0} كيلومتر في الساعة"},"short":{"other":"{0} كم/س"},"narrow":{"other":"{0} كم/س"},"perUnit":{}},"meter-per-second":{"long":{"other":"{0} متر في الثانية"},"short":{"other":"{0} م/ث"},"narrow":{"other":"{0} م/ث"},"perUnit":{}},"mile-per-hour":{"long":{"other":"{0} ميل في الساعة"},"short":{"other":"{0} ميل/س"},"narrow":{"other":"{0} ميل/س"},"perUnit":{}},"celsius":{"long":{"other":"{0} درجة مئوية"},"short":{"other":"{0}°م"},"narrow":{"other":"{0}°م"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0} درجة فهرنهايت"},"short":{"other":"{0}°ف"},"narrow":{"other":"{0} د ف"},"perUnit":{}},"liter":{"long":{"other":"{0} لتر","one":"لتر"},"short":{"other":"{0} لتر","one":"لتر"},"narrow":{"other":"{0} ل"},"perUnit":{"long":"{0} لكل لتر","short":"{0}/ل","narrow":"{0}/ل"}},"milliliter":{"long":{"other":"{0} مليلتر"},"short":{"other":"{0} ملتر"},"narrow":{"other":"{0} ملتر"},"perUnit":{}},"gallon":{"long":{"other":"{0} غالون","one":"غالون"},"short":{"other":"{0} غالون","one":"غالون"},"narrow":{"other":"{0} غالون","one":"غالون"},"perUnit":{"long":"{0} لكل غالون","short":"{0}/غالون","narrow":"{0}/غالون"}},"fluid-ounce":{"long":{"other":"{0} أونصة سائلة","one":"أونصة سائلة","two":"أونصتان سائلتان"},"short":{"other":"{0} أونصة سائلة","one":"أونصة س","two":"{0} أونصة س","few":"{0} أونصات سائلة","many":"{0} أونصة س"},"narrow":{"other":"{0} أونصة سائلة","one":"أونصة س","two":"{0} أونصة س","few":"{0} أونصات سائلة","many":"{0} أونصة س"},"perUnit":{}}},"compound":{"per":{"long":"{0} لكل {1}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"بيستا أندوري"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"درهم إماراتي"},"symbol":"د.إ.‏","narrow":"د.إ.‏"},"AFA":{"displayName":{"other":"أفغاني - 1927-2002"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"أفغاني أفغانستاني"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"ALK"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"ليك ألباني"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"درام أرميني"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"غيلدر أنتيلي هولندي"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"كوانزا أنغولي"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"كوانزا أنجولي - 1977-1990"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"كوانزا أنجولي جديدة - 1990-2000"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"كوانزا أنجولي معدلة - 1995 - 1999"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"استرال أرجنتيني"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"ARL"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"ARM"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"بيزو أرجنتيني - 1983-1985"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"بيزو أرجنتيني"},"symbol":"ARS","narrow":"AR$"},"ATS":{"displayName":{"other":"شلن نمساوي"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"دولار أسترالي"},"symbol":"AU$","narrow":"AU$"},"AWG":{"displayName":{"other":"فلورن أروبي"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"مانات أذريبجاني"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"مانت أذربيجاني"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"دينار البوسنة والهرسك"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"مارك البوسنة والهرسك قابل للتحويل"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"BAN"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"دولار بربادوسي"},"symbol":"BBD","narrow":"BB$"},"BDT":{"displayName":{"other":"تاكا بنغلاديشي"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"فرنك بلجيكي قابل للتحويل"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"فرنك بلجيكي"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"فرنك بلجيكي مالي"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"BGL"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"BGM"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"ليف بلغاري"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"BGO"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"دينار بحريني"},"symbol":"د.ب.‏","narrow":"د.ب.‏"},"BIF":{"displayName":{"other":"فرنك بروندي"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"دولار برمودي"},"symbol":"BMD","narrow":"BM$"},"BND":{"displayName":{"other":"دولار بروناي"},"symbol":"BND","narrow":"BN$"},"BOB":{"displayName":{"other":"بوليفيانو بوليفي"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"BOL"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"بيزو بوليفي"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"مفدول بوليفي"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"نوفو كروزايرو برازيلي - 1967-1986"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"كروزادو برازيلي"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"كروزايرو برازيلي - 1990-1993"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"ريال برازيلي"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"BRN"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"BRR"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"BRZ"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"دولار باهامي"},"symbol":"BSD","narrow":"BS$"},"BTN":{"displayName":{"other":"نولتوم بوتاني"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"كيات بورمي"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"بولا بتسواني"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"روبل بيلاروسي جديد - 1994-1999"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"روبل بيلاروسي"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"روبل بيلاروسي (٢٠٠٠–٢٠١٦)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"دولار بليزي","two":"دولاران بليزيان"},"symbol":"BZD","narrow":"BZ$"},"CAD":{"displayName":{"other":"دولار كندي"},"symbol":"CA$","narrow":"CA$"},"CDF":{"displayName":{"other":"فرنك كونغولي"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"CHE"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"فرنك سويسري"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"CHW"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"CLE"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"CLF"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"بيزو تشيلي"},"symbol":"CLP","narrow":"CL$"},"CNH":{"displayName":{"other":"يوان صيني (في الخارج)"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"CNX"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"يوان صيني"},"symbol":"CN¥","narrow":"CN¥"},"COP":{"displayName":{"other":"بيزو كولومبي"},"symbol":"COP","narrow":"CO$"},"COU":{"displayName":{"other":"COU"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"كولن كوستاريكي"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"دينار صربي قديم"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"كرونة تشيكوسلوفاكيا"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"بيزو كوبي قابل للتحويل"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"بيزو كوبي"},"symbol":"CUP","narrow":"CU$"},"CVE":{"displayName":{"other":"اسكودو الرأس الأخضر"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"جنيه قبرصي"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"كرونة تشيكية"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"أوستمارك ألماني شرقي"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"مارك ألماني"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"فرنك جيبوتي"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"كرونة دنماركية"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"بيزو الدومنيكان"},"symbol":"DOP","narrow":"DO$"},"DZD":{"displayName":{"other":"دينار جزائري","two":"ديناران جزائريان","few":"دينارات جزائرية","many":"دينارًا جزائريًا"},"symbol":"د.ج.‏","narrow":"د.ج.‏"},"ECS":{"displayName":{"other":"ECS"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"ECV"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"كرونة استونية"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"جنيه مصري","two":"جنيهان مصريان","few":"جنيهات مصرية","many":"جنيهًا مصريًا"},"symbol":"ج.م.‏","narrow":"E£"},"ERN":{"displayName":{"other":"ناكفا أريتري"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"ESA"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"ESB"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"بيزيتا إسباني"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"بير أثيوبي"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"يورو"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"ماركا فنلندي"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"دولار فيجي"},"symbol":"FJD","narrow":"FJ$"},"FKP":{"displayName":{"other":"جنيه جزر فوكلاند"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"فرنك فرنسي"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"جنيه إسترليني"},"symbol":"UK£","narrow":"UK£"},"GEK":{"displayName":{"other":"GEK"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"لاري جورجي"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"سيدي غاني"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"سيدي غانا"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"جنيه جبل طارق"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"دلاسي غامبي"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"فرنك غينيا"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"سيلي غينيا"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"اكويل جونينا غينيا الاستوائيّة"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"دراخما يوناني"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"كوتزال غواتيمالا"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"اسكود برتغالي غينيا"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"بيزو غينيا بيساو"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"دولار غيانا"},"symbol":"GYD","narrow":"GY$"},"HKD":{"displayName":{"other":"دولار هونغ كونغ"},"symbol":"HK$","narrow":"HK$"},"HNL":{"displayName":{"other":"ليمبيرا هندوراس"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"دينار كرواتي"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"كونا كرواتي"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"جوردى هايتي"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"فورينت هنغاري"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"روبية إندونيسية"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"جنيه إيرلندي"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"جنيه إسرائيلي"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"ILR"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"شيكل إسرائيلي جديد"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"روبية هندي"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"دينار عراقي"},"symbol":"د.ع.‏","narrow":"د.ع.‏"},"IRR":{"displayName":{"other":"ريال إيراني"},"symbol":"ر.إ.","narrow":"ر.إ."},"ISJ":{"displayName":{"other":"ISJ"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"كرونة أيسلندية"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"ليرة إيطالية"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"دولار جامايكي"},"symbol":"JMD","narrow":"JM$"},"JOD":{"displayName":{"other":"دينار أردني"},"symbol":"د.أ.‏","narrow":"د.أ.‏"},"JPY":{"displayName":{"other":"ين ياباني"},"symbol":"JP¥","narrow":"JP¥"},"KES":{"displayName":{"other":"شلن كينيي"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"سوم قيرغستاني"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"رييال كمبودي"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"فرنك جزر القمر"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"وون كوريا الشمالية"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"KRH"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"KRO"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"وون كوريا الجنوبية"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"دينار كويتي"},"symbol":"د.ك.‏","narrow":"د.ك.‏"},"KYD":{"displayName":{"other":"دولار جزر كيمن"},"symbol":"KYD","narrow":"KY$"},"KZT":{"displayName":{"other":"تينغ كازاخستاني"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"كيب لاوسي"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"جنيه لبناني"},"symbol":"ل.ل.‏","narrow":"L£"},"LKR":{"displayName":{"other":"روبية سريلانكية"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"دولار ليبيري","two":"دولاران ليبيريان","few":"دولارات ليبيرية","many":"دولارًا ليبيريًا"},"symbol":"LRD","narrow":"$LR"},"LSL":{"displayName":{"other":"لوتي ليسوتو"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"ليتا ليتوانية"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"تالوناس ليتواني"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"فرنك لوكسمبرج قابل للتحويل"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"فرنك لوكسمبرج"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"فرنك لوكسمبرج المالي"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"لاتس لاتفي"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"روبل لاتفيا"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"دينار ليبي","two":"ديناران ليبيان","few":"دينارات ليبية","many":"دينارًا ليبيًا"},"symbol":"د.ل.‏","narrow":"د.ل.‏"},"MAD":{"displayName":{"other":"درهم مغربي","two":"درهمان مغربيان","few":"دراهم مغربية","many":"درهمًا مغربيًا"},"symbol":"د.م.‏","narrow":"د.م.‏"},"MAF":{"displayName":{"other":"فرنك مغربي"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"MCF"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"MDC"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"ليو مولدوفي"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"أرياري مدغشقر"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"فرنك مدغشقر"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"دينار مقدوني","two":"ديناران مقدونيان","few":"دينارات مقدونية","many":"دينارًا مقدونيًا"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"MKN"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"فرنك مالي"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"كيات ميانمار"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"توغروغ منغولي"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"باتاكا ماكاوي"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"أوقية موريتانية - 1973-2017"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"أوقية موريتانية"},"symbol":"أ.م.","narrow":"أ.م."},"MTL":{"displayName":{"other":"ليرة مالطية"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"جنيه مالطي"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"روبية موريشيوسية"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"MVP"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"روفيه جزر المالديف"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"كواشا مالاوي"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"بيزو مكسيكي"},"symbol":"MX$","narrow":"MX$"},"MXP":{"displayName":{"other":"بيزو فضي مكسيكي - 1861-1992"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"MXV"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"رينغيت ماليزي"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"اسكود موزمبيقي"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"MZM"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"متكال موزمبيقي"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"دولار ناميبي"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"نايرا نيجيري"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"كوردوبة نيكاراجوا"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"قرطبة نيكاراغوا"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"جلدر هولندي"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"كرونة نرويجية"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"روبية نيبالي"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"دولار نيوزيلندي"},"symbol":"NZ$","narrow":"NZ$"},"OMR":{"displayName":{"other":"ريال عماني"},"symbol":"ر.ع.‏","narrow":"ر.ع.‏"},"PAB":{"displayName":{"other":"بالبوا بنمي"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"PEI"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"سول بيروفي"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"PES"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"كينا بابوا غينيا الجديدة"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"بيزو فلبيني"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"روبية باكستاني"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"زلوتي بولندي"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"زلوتي بولندي - 1950-1995"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"اسكود برتغالي"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"غواراني باراغواي"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"ريال قطري"},"symbol":"ر.ق.‏","narrow":"ر.ق.‏"},"RHD":{"displayName":{"other":"دولار روديسي"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"ليو روماني قديم"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"ليو روماني"},"symbol":"RON","narrow":"lei"},"RSD":{"displayName":{"other":"دينار صربي","two":"ديناران صربيان","few":"دينارات صربية","many":"دينارًا صربيًا"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"روبل روسي"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"روبل روسي - 1991-1998"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"فرنك رواندي"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"ريال سعودي"},"symbol":"ر.س.‏","narrow":"ر.س.‏"},"SBD":{"displayName":{"other":"دولار جزر سليمان"},"symbol":"SBD","narrow":"SB$"},"SCR":{"displayName":{"other":"روبية سيشيلية"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"دينار سوداني"},"symbol":"د.س.‏","narrow":"د.س.‏"},"SDG":{"displayName":{"other":"جنيه سوداني","few":"جنيهات سودانية","many":"جنيهًا سودانيًا"},"symbol":"ج.س.","narrow":"ج.س."},"SDP":{"displayName":{"other":"جنيه سوداني قديم"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"كرونة سويدية"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"دولار سنغافوري"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"جنيه سانت هيلين"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"تولار سلوفيني"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"كرونة سلوفاكية"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"ليون سيراليوني"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"شلن صومالي"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"دولار سورينامي"},"symbol":"SRD","narrow":"SR$"},"SRG":{"displayName":{"other":"جلدر سورينامي"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"جنيه جنوب السودان","two":"جنيهان جنوب السودان","few":"جنيهات جنوب السودان","many":"جنيهًا جنوب السودان"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"دوبرا ساو تومي وبرينسيبي - 1977-2017"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"دوبرا ساو تومي وبرينسيبي"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"روبل سوفيتي"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"كولون سلفادوري"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"ليرة سورية"},"symbol":"ل.س.‏","narrow":"£"},"SZL":{"displayName":{"other":"ليلانجيني سوازيلندي"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"باخت تايلاندي"},"symbol":"฿","narrow":"฿"},"TJR":{"displayName":{"other":"روبل طاجيكستاني"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"سوموني طاجيكستاني"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"مانات تركمنستاني"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"مانات تركمانستان"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"دينار تونسي","two":"ديناران تونسيان","few":"دينارات تونسية","many":"دينارًا تونسيًا"},"symbol":"د.ت.‏","narrow":"د.ت.‏"},"TOP":{"displayName":{"other":"بانغا تونغا"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"اسكود تيموري"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"ليرة تركي"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"ليرة تركية"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"دولار ترينداد وتوباغو"},"symbol":"TTD","narrow":"TT$"},"TWD":{"displayName":{"other":"دولار تايواني"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"شلن تنزاني"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"هريفنيا أوكراني"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"UAK"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"شلن أوغندي - 1966-1987"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"شلن أوغندي"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"دولار أمريكي"},"symbol":"US$","narrow":"US$"},"USN":{"displayName":{"other":"دولار أمريكي (اليوم التالي)‏"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"دولار أمريكي (نفس اليوم)‏"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"UYI"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"بيزو أوروجواي - 1975-1993"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"بيزو اوروغواي"},"symbol":"UYU","narrow":"UY$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"سوم أوزبكستاني"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"بوليفار فنزويلي - 1871-2008"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"بوليفار فنزويلي - 2008–2018"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"بوليفار فنزويلي"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"دونج فيتنامي"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"VNN"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"فاتو فانواتو"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"تالا ساموا"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"فرنك وسط أفريقي"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"فضة"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"ذهب"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"الوحدة الأوروبية المركبة"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"الوحدة المالية الأوروبية"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"الوحدة الحسابية الأوروبية"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"(XBD)وحدة الحساب الأوروبية"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"دولار شرق الكاريبي"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"حقوق السحب الخاصة"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"وحدة النقد الأوروبية"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"فرنك فرنسي ذهبي"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"(UIC)فرنك فرنسي"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"فرنك غرب أفريقي"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"بالاديوم"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"فرنك سي إف بي"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"البلاتين"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"XRE"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"XSU"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"كود اختبار العملة"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"XUA"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"(عملة غير معروفة)"},"symbol":"***","narrow":"***"},"YDD":{"displayName":{"other":"دينار يمني"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"ريال يمني"},"symbol":"ر.ي.‏","narrow":"ر.ي.‏"},"YUD":{"displayName":{"other":"دينار يوغسلافي"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"YUM"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"دينار يوغسلافي قابل للتحويل"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"YUR"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"راند جنوب أفريقيا -مالي"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"راند جنوب أفريقيا"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"كواشا زامبي - 1968-2012"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"كواشا زامبي"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"زائير زائيري جديد"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"زائير زائيري"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"دولار زمبابوي"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"دولار زمبابوي 2009"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"ZWR"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["arab","latn"],"symbols":{"arab":{"decimal":"٫","group":"٬","list":"؛","percentSign":"٪؜","plusSign":"؜+","minusSign":"؜-","exponential":"اس","superscriptingExponent":"×","perMille":"؉","infinity":"∞","nan":"ليس رقم","timeSeparator":":"},"latn":{"decimal":".","group":",","list":";","percentSign":"‎%‎","plusSign":"‎+","minusSign":"‎-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"ليس رقمًا","timeSeparator":":"}},"percent":{"arab":"#,##0%","latn":"#,##0%"},"decimal":{"arab":{"long":{"1000":{"other":"0 ألف","few":"0 آلاف"},"10000":{"other":"00 ألف"},"100000":{"other":"000 ألف"},"1000000":{"other":"0 مليون","few":"0 ملايين"},"10000000":{"other":"00 مليون","few":"00 ملايين"},"100000000":{"other":"000 مليون"},"1000000000":{"other":"0 مليار"},"10000000000":{"other":"00 مليار"},"100000000000":{"other":"000 مليار"},"1000000000000":{"other":"0 ترليون"},"10000000000000":{"other":"00 ترليون"},"100000000000000":{"other":"000 ترليون"}},"short":{"1000":{"other":"0 ألف","few":"0 آلاف"},"10000":{"other":"00 ألف"},"100000":{"other":"000 ألف"},"1000000":{"other":"0 مليون"},"10000000":{"other":"00 مليون"},"100000000":{"other":"000 مليون"},"1000000000":{"other":"0 مليار"},"10000000000":{"other":"00 مليار"},"100000000000":{"other":"000 مليار"},"1000000000000":{"other":"0 ترليون"},"10000000000000":{"other":"00 ترليون"},"100000000000000":{"other":"000 ترليون"}}},"latn":{"long":{"1000":{"other":"0 ألف","few":"0 آلاف"},"10000":{"other":"00 ألف"},"100000":{"other":"000 ألف"},"1000000":{"other":"0 مليون","few":"0 ملايين"},"10000000":{"other":"00 مليون","few":"00 ملايين"},"100000000":{"other":"000 مليون"},"1000000000":{"other":"0 مليار"},"10000000000":{"other":"00 مليار"},"100000000000":{"other":"000 مليار"},"1000000000000":{"other":"0 ترليون"},"10000000000000":{"other":"00 ترليون"},"100000000000000":{"other":"000 ترليون"}},"short":{"1000":{"other":"0 ألف","few":"0 آلاف"},"10000":{"other":"00 ألف"},"100000":{"other":"000 ألف"},"1000000":{"other":"0 مليون"},"10000000":{"other":"00 مليون"},"100000000":{"other":"000 مليون"},"1000000000":{"other":"0 مليار"},"10000000000":{"other":"00 مليار"},"100000000000":{"other":"000 مليار"},"1000000000000":{"other":"0 ترليون"},"10000000000000":{"other":"00 ترليون"},"100000000000000":{"other":"000 ترليون"}}}},"currency":{"arab":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"#,##0.00 ¤","accounting":"#,##0.00 ¤","unitPattern":"{0} {1}"},"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤ #,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0} {1}","short":{"1000":{"other":"¤ 0 ألف"},"10000":{"other":"¤ 00 ألف"},"100000":{"other":"¤ 000 ألف"},"1000000":{"other":"¤ 0 مليون"},"10000000":{"other":"¤ 00 مليون"},"100000000":{"other":"¤ 000 مليون"},"1000000000":{"other":"¤ 0 مليار"},"10000000000":{"other":"¤ 00 مليار"},"100000000000":{"other":"¤ 000 مليار"},"1000000000000":{"other":"¤ 0 ترليون"},"10000000000000":{"other":"¤ 00 ترليون"},"100000000000000":{"other":"¤ 000 ترليون"}}}}},"nu":["arab","latn"]}},"availableLocales":["ar"]},
    {"data":{"de":{"units":{"simple":{"degree":{"long":{"other":"{0} Grad"},"short":{"other":"{0}°"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0} Hektar"},"short":{"other":"{0} ha"},"narrow":{"other":"{0} ha"},"perUnit":{}},"acre":{"long":{"other":"{0} Acres","one":"{0} Acre"},"short":{"other":"{0} ac"},"narrow":{"other":"{0} ac"},"perUnit":{}},"percent":{"long":{"other":"{0} Prozent"},"short":{"other":"{0} %"},"narrow":{"other":"{0} %"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"{0} Liter pro Kilometer"},"short":{"other":"{0} l/km"},"narrow":{"other":"{0} l/km"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"{0} Meilen pro Gallone","one":"{0} Meile pro Gallone"},"short":{"other":"{0} mpg"},"narrow":{"other":"{0} mpg"},"perUnit":{}},"petabyte":{"long":{"other":"{0} Petabytes","one":"{0} Petabyte"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0} Terabytes","one":"{0} Terabyte"},"short":{"other":"{0} TB"},"narrow":{"other":"{0} TB"},"perUnit":{}},"terabit":{"long":{"other":"{0} Terabits","one":"{0} Terabit"},"short":{"other":"{0} Tb"},"narrow":{"other":"{0} Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} Gigabytes","one":"{0} Gigabyte"},"short":{"other":"{0} GB"},"narrow":{"other":"{0} GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0} Gigabits","one":"{0} Gigabit"},"short":{"other":"{0} Gb"},"narrow":{"other":"{0} Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0} Megabytes","one":"{0} Megabyte"},"short":{"other":"{0} MB"},"narrow":{"other":"{0} MB"},"perUnit":{}},"megabit":{"long":{"other":"{0} Megabits","one":"{0} Megabit"},"short":{"other":"{0} Mb"},"narrow":{"other":"{0} Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} Kilobytes","one":"{0} Kilobyte"},"short":{"other":"{0} kB"},"narrow":{"other":"{0} kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0} Kilobits","one":"{0} Kilobit"},"short":{"other":"{0} kb"},"narrow":{"other":"{0} kb"},"perUnit":{}},"byte":{"long":{"other":"{0} Bytes","one":"{0} Byte"},"short":{"other":"{0} Bytes","one":"{0} Byte"},"narrow":{"other":"{0} Bytes","one":"{0} Byte"},"perUnit":{}},"bit":{"long":{"other":"{0} Bits","one":"{0} Bit"},"short":{"other":"{0} Bits","one":"{0} Bit"},"narrow":{"other":"{0} Bits","one":"{0} Bit"},"perUnit":{}},"year":{"long":{"other":"{0} Jahre","one":"{0} Jahr"},"short":{"other":"{0} J"},"narrow":{"other":"{0} J"},"perUnit":{"long":"{0} pro Jahr","short":"{0}/J","narrow":"{0}/J"}},"month":{"long":{"other":"{0} Monate","one":"{0} Monat"},"short":{"other":"{0} Mon."},"narrow":{"other":"{0} M"},"perUnit":{"long":"{0} pro Monat","short":"{0}/M","narrow":"{0}/M"}},"week":{"long":{"other":"{0} Wochen","one":"{0} Woche"},"short":{"other":"{0} Wo."},"narrow":{"other":"{0} W"},"perUnit":{"long":"{0} pro Woche","short":"{0}/W","narrow":"{0}/W"}},"day":{"long":{"other":"{0} Tage","one":"{0} Tag"},"short":{"other":"{0} Tg."},"narrow":{"other":"{0} T"},"perUnit":{"long":"{0} pro Tag","short":"{0}/T","narrow":"{0}/T"}},"hour":{"long":{"other":"{0} Stunden","one":"{0} Stunde"},"short":{"other":"{0} Std."},"narrow":{"other":"{0} Std."},"perUnit":{"long":"{0} pro Stunde","short":"{0}/h","narrow":"{0}/h"}},"minute":{"long":{"other":"{0} Minuten","one":"{0} Minute"},"short":{"other":"{0} Min."},"narrow":{"other":"{0} Min."},"perUnit":{"long":"{0} pro Minute","short":"{0}/min","narrow":"{0}/min"}},"second":{"long":{"other":"{0} Sekunden","one":"{0} Sekunde"},"short":{"other":"{0} Sek."},"narrow":{"other":"{0} Sek."},"perUnit":{"long":"{0} pro Sekunde","short":"{0}/s","narrow":"{0}/s"}},"millisecond":{"long":{"other":"{0} Millisekunden","one":"{0} Millisekunde"},"short":{"other":"{0} ms"},"narrow":{"other":"{0} ms"},"perUnit":{}},"kilometer":{"long":{"other":"{0} Kilometer"},"short":{"other":"{0} km"},"narrow":{"other":"{0} km"},"perUnit":{"long":"{0} pro Kilometer","short":"{0}/km","narrow":"{0}/km"}},"meter":{"long":{"other":"{0} Meter"},"short":{"other":"{0} m"},"narrow":{"other":"{0} m"},"perUnit":{"long":"{0} pro Meter","short":"{0}/m","narrow":"{0}/m"}},"centimeter":{"long":{"other":"{0} Zentimeter"},"short":{"other":"{0} cm"},"narrow":{"other":"{0} cm"},"perUnit":{"long":"{0} pro Zentimeter","short":"{0}/cm","narrow":"{0}/cm"}},"millimeter":{"long":{"other":"{0} Millimeter"},"short":{"other":"{0} mm"},"narrow":{"other":"{0} mm"},"perUnit":{}},"mile":{"long":{"other":"{0} Meilen","one":"{0} Meile"},"short":{"other":"{0} mi"},"narrow":{"other":"{0} mi"},"perUnit":{}},"yard":{"long":{"other":"{0} Yards","one":"{0} Yard"},"short":{"other":"{0} yd"},"narrow":{"other":"{0} yd"},"perUnit":{}},"foot":{"long":{"other":"{0} Fuß"},"short":{"other":"{0} ft"},"narrow":{"other":"{0} ft"},"perUnit":{"long":"{0} pro Fuß","short":"{0}/ft","narrow":"{0}/ft"}},"inch":{"long":{"other":"{0} Zoll"},"short":{"other":"{0} in","one":"{0} in"},"narrow":{"other":"{0} in","one":"{0} in"},"perUnit":{"long":"{0} pro Zoll","short":"{0}/in","narrow":"{0}/in"}},"mile-scandinavian":{"long":{"other":"{0} skandinavische Meilen","one":"{0} skandinavische Meile"},"short":{"other":"{0} smi"},"narrow":{"other":"{0} smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0} Kilogramm"},"short":{"other":"{0} kg"},"narrow":{"other":"{0} kg"},"perUnit":{"long":"{0} pro Kilogramm","short":"{0}/kg","narrow":"{0}/kg"}},"gram":{"long":{"other":"{0} Gramm"},"short":{"other":"{0} g"},"narrow":{"other":"{0} g"},"perUnit":{"long":"{0} pro Gramm","short":"{0}/g","narrow":"{0}/g"}},"stone":{"long":{"other":"{0} Stones","one":"{0} Stone"},"short":{"other":"{0} st"},"narrow":{"other":"{0} st"},"perUnit":{}},"pound":{"long":{"other":"{0} Pfund"},"short":{"other":"{0} lb"},"narrow":{"other":"{0} lb"},"perUnit":{"long":"{0} pro Pfund","short":"{0}/lb","narrow":"{0}/lb"}},"ounce":{"long":{"other":"{0} Unzen","one":"{0} Unze"},"short":{"other":"{0} oz"},"narrow":{"other":"{0} oz"},"perUnit":{"long":"{0} pro Unze","short":"{0}/oz","narrow":"{0}/oz"}},"kilometer-per-hour":{"long":{"other":"{0} Kilometer pro Stunde"},"short":{"other":"{0} km/h"},"narrow":{"other":"{0} km/h"},"perUnit":{}},"meter-per-second":{"long":{"other":"{0} Meter pro Sekunde"},"short":{"other":"{0} m/s"},"narrow":{"other":"{0} m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"{0} Meilen pro Stunde","one":"{0} Meile pro Stunde"},"short":{"other":"{0} mi/h"},"narrow":{"other":"{0} mi/h"},"perUnit":{}},"celsius":{"long":{"other":"{0} Grad Celsius"},"short":{"other":"{0} °C"},"narrow":{"other":"{0}°"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0} Grad Fahrenheit"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0} Liter"},"short":{"other":"{0} l"},"narrow":{"other":"{0} l"},"perUnit":{"long":"{0} pro Liter","short":"{0}/l","narrow":"{0}/l"}},"milliliter":{"long":{"other":"{0} Milliliter"},"short":{"other":"{0} ml"},"narrow":{"other":"{0} ml"},"perUnit":{}},"gallon":{"long":{"other":"{0} Gallonen","one":"{0} Gallone"},"short":{"other":"{0} gal"},"narrow":{"other":"{0} gal"},"perUnit":{"long":"{0} pro Gallone","short":"{0}/gal","narrow":"{0}/gal"}},"fluid-ounce":{"long":{"other":"{0} Flüssigunzen","one":"{0} Flüssigunze"},"short":{"other":"{0} fl oz"},"narrow":{"other":"{0} fl oz"},"perUnit":{}}},"compound":{"per":{"long":"{0} pro {1}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"Andorranische Peseten","one":"Andorranische Pesete"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"VAE-Dirham"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"Afghanische Afghani (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"Afghanische Afghani","one":"Afghanischer Afghani"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"Albanische Lek (1946–1965)","one":"Albanischer Lek (1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"Albanische Lek","one":"Albanischer Lek"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"Armenische Dram","one":"Armenischer Dram"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"Niederländische-Antillen-Gulden"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"Angolanische Kwanza","one":"Angolanischer Kwanza"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"Angolanische Kwanza (1977–1990)","one":"Angolanischer Kwanza (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"Angolanische Neue Kwanza (1990–2000)","one":"Angolanischer Neuer Kwanza (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"Angolanische Kwanza Reajustado (1995–1999)","one":"Angolanischer Kwanza Reajustado (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"Argentinische Austral","one":"Argentinischer Austral"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"Argentinische Pesos Ley (1970–1983)","one":"Argentinischer Peso Ley (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"Argentinische Pesos (1881–1970)","one":"Argentinischer Peso (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"Argentinische Peso (1983–1985)","one":"Argentinischer Peso (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"Argentinische Pesos","one":"Argentinischer Peso"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"Österreichische Schilling","one":"Österreichischer Schilling"},"symbol":"öS","narrow":"öS"},"AUD":{"displayName":{"other":"Australische Dollar","one":"Australischer Dollar"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"Aruba-Florin"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"Aserbaidschan-Manat (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"Aserbaidschan-Manat"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"Bosnien und Herzegowina Dinar (1992–1994)"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"Bosnien und Herzegowina Konvertierbare Mark"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"Bosnien und Herzegowina Neue Dinar (1994–1997)","one":"Bosnien und Herzegowina Neuer Dinar (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"Barbados-Dollar"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"Bangladesch-Taka"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"Belgische Franc (konvertibel)","one":"Belgischer Franc (konvertibel)"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"Belgische Franc","one":"Belgischer Franc"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"Belgische Finanz-Franc","one":"Belgischer Finanz-Franc"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"Bulgarische Lew (1962–1999)"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"Bulgarische Lew (1952–1962)","one":"Bulgarischer Lew (1952–1962)"},"symbol":"BGK","narrow":"BGK"},"BGN":{"displayName":{"other":"Bulgarische Lew","one":"Bulgarischer Lew"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"Bulgarische Lew (1879–1952)","one":"Bulgarischer Lew (1879–1952)"},"symbol":"BGJ","narrow":"BGJ"},"BHD":{"displayName":{"other":"Bahrain-Dinar"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"Burundi-Francs","one":"Burundi-Franc"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"Bermuda-Dollar"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"Brunei-Dollar"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"Bolivianische Bolivianos","one":"Bolivianischer Boliviano"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"Bolivianische Bolivianos (1863–1963)","one":"Bolivianischer Boliviano (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"Bolivianische Peso","one":"Bolivianischer Peso"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"Bolivianische Mvdol","one":"Boliviansiche Mvdol"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"Brasilianische Cruzeiro Novo (1967–1986)","one":"Brasilianischer Cruzeiro Novo (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"Brasilianische Cruzado (1986–1989)","one":"Brasilianischer Cruzado (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"Brasilianische Cruzeiro (1990–1993)","one":"Brasilianischer Cruzeiro (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"Brasilianische Real","one":"Brasilianischer Real"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"Brasilianische Cruzado Novo (1989–1990)","one":"Brasilianischer Cruzado Novo (1989–1990)"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"Brasilianische Cruzeiro (1993–1994)","one":"Brasilianischer Cruzeiro (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"Brasilianischer Cruzeiro (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"Bahamas-Dollar"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"Bhutan-Ngultrum"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"Birmanische Kyat","one":"Birmanischer Kyat"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"Botswanische Pula","one":"Botswanischer Pula"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"Belarus-Rubel (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"Weißrussische Rubel","one":"Weißrussischer Rubel"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"Weißrussische Rubel (2000–2016)","one":"Weißrussischer Rubel (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"Belize-Dollar"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"Kanadische Dollar","one":"Kanadischer Dollar"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"Kongo-Francs","one":"Kongo-Franc"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"WIR-Euro"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"Schweizer Franken"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"WIR Franken"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"Chilenische Escudo","one":"Chilenischer Escudo"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"Chilenische Unidades de Fomento"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"Chilenische Pesos","one":"Chilenischer Peso"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"Renminbi Yuan (Off–Shore)"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"Dollar der Chinesischen Volksbank"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"Renminbi Yuan","one":"Chinesischer Yuan"},"symbol":"CN¥","narrow":"¥"},"COP":{"displayName":{"other":"Kolumbianische Pesos","one":"Kolumbianischer Peso"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"Kolumbianische Unidades de valor real","one":"Kolumbianische Unidad de valor real"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"Costa-Rica-Colón"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"Serbische Dinar (2002–2006)","one":"Serbischer Dinar (2002–2006)"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"Tschechoslowakische Kronen"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"Kubanische Pesos (konvertibel)","one":"Kubanischer Peso (konvertibel)"},"symbol":"CUC","narrow":"Cub$"},"CUP":{"displayName":{"other":"Kubanische Pesos","one":"Kubanischer Peso"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"Cabo-Verde-Escudos","one":"Cabo-Verde-Escudo"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"Zypern Pfund"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"Tschechische Kronen","one":"Tschechische Krone"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"Mark der DDR"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"Deutsche Mark"},"symbol":"DM","narrow":"DM"},"DJF":{"displayName":{"other":"Dschibuti-Franc"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"Dänische Kronen","one":"Dänische Krone"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"Dominikanische Pesos","one":"Dominikanischer Peso"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"Algerische Dinar","one":"Algerischer Dinar"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"Ecuadorianische Sucre","one":"Ecuadorianischer Sucre"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"Verrechnungseinheiten für Ecuador"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"Estnische Kronen","one":"Estnische Krone"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"Ägyptische Pfund","one":"Ägyptisches Pfund"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"Eritreische Nakfa","one":"Eritreischer Nakfa"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"Spanische Peseten (A–Konten)","one":"Spanische Peseta (A–Konten)"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"Spanische Peseten (konvertibel)","one":"Spanische Peseta (konvertibel)"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"Spanische Peseten","one":"Spanische Peseta"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"Äthiopische Birr","one":"Äthiopischer Birr"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"Euro"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"Finnische Mark"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"Fidschi-Dollar"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"Falkland-Pfund"},"symbol":"FKP","narrow":"Fl£"},"FRF":{"displayName":{"other":"Französische Franc","one":"Französischer Franc"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"Britische Pfund","one":"Britisches Pfund"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"Georgische Kupon Larit","one":"Georgischer Kupon Larit"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"Georgische Lari","one":"Georgischer Lari"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"Ghanaische Cedi (1979–2007)","one":"Ghanaischer Cedi (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"Ghanaische Cedi","one":"Ghanaischer Cedi"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"Gibraltar-Pfund"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"Gambia-Dalasi"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"Guinea-Franc"},"symbol":"GNF","narrow":"F.G."},"GNS":{"displayName":{"other":"Guineische Syli","one":"Guineischer Syli"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"Äquatorialguinea-Ekwele"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"Griechische Drachmen","one":"Griechische Drachme"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"Guatemaltekische Quetzales","one":"Guatemaltekischer Quetzal"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"Portugiesisch Guinea Escudo"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"Guinea-Bissau Pesos","one":"Guinea-Bissau Peso"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"Guyana-Dollar"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"Hongkong-Dollar"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"Honduras-Lempira"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"Kroatische Dinar","one":"Kroatischer Dinar"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"Kroatische Kuna","one":"Kroatischer Kuna"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"Haitianische Gourdes","one":"Haitianische Gourde"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"Ungarische Forint","one":"Ungarischer Forint"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"Indonesische Rupiah"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"Irische Pfund","one":"Irisches Pfund"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"Israelische Pfund","one":"Israelisches Pfund"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"Israelische Schekel (1980–1985)","one":"Israelischer Schekel (1980–1985)"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"Israelische Neue Schekel","one":"Israelischer Neuer Schekel"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"Indische Rupien","one":"Indische Rupie"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"Irakische Dinar","one":"Irakischer Dinar"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"Iranische Rial","one":"Iranischer Rial"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"Isländische Kronen (1918–1981)","one":"Isländische Krone (1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"Isländische Kronen","one":"Isländische Krone"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"Italienische Lire","one":"Italienische Lira"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"Jamaika-Dollar"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"Jordanische Dinar","one":"Jordanischer Dinar"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"Japanische Yen","one":"Japanischer Yen"},"symbol":"¥","narrow":"¥"},"KES":{"displayName":{"other":"Kenia-Schilling"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"Kirgisische Som","one":"Kirgisischer Som"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"Kambodschanische Riel","one":"Kambodschanischer Riel"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"Komoren-Francs","one":"Komoren-Franc"},"symbol":"KMF","narrow":"FC"},"KPW":{"displayName":{"other":"Nordkoreanische Won","one":"Nordkoreanischer Won"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"Südkoreanischer Hwan (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"Südkoreanischer Won (1945–1953)"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"Südkoreanische Won","one":"Südkoreanischer Won"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"Kuwait-Dinar"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"Kaiman-Dollar"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"Kasachische Tenge","one":"Kasachischer Tenge"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"Laotische Kip","one":"Laotischer Kip"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"Libanesische Pfund","one":"Libanesisches Pfund"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"Sri-Lanka-Rupien","one":"Sri-Lanka-Rupie"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"Liberianische Dollar","one":"Liberianischer Dollar"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"Loti"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"Litauische Litas","one":"Litauischer Litas"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"Litauische Talonas"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"Luxemburgische Franc (konvertibel)"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"Luxemburgische Franc"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"Luxemburgische Finanz-Franc"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"Lettische Lats","one":"Lettischer Lats"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"Lettische Rubel"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"Libysche Dinar","one":"Libyscher Dinar"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"Marokkanische Dirham","one":"Marokkanischer Dirham"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"Marokkanische Franc"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"Monegassische Franc","one":"Monegassischer Franc"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"Moldau-Cupon"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"Moldau-Leu"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"Madagaskar-Ariary"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"Madagaskar-Franc"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"Mazedonische Denari","one":"Mazedonischer Denar"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"Mazedonische Denar (1992–1993)","one":"Mazedonischer Denar (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"Malische Franc"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"Myanmarische Kyat","one":"Myanmarischer Kyat"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"Mongolische Tögrög","one":"Mongolischer Tögrög"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"Macao-Pataca"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"Mauretanische Ouguiya (1973–2017)","one":"Mauretanischer Ouguiya (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"Mauretanische Ouguiya","one":"Mauretanischer Ouguiya"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"Maltesische Lira"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"Maltesische Pfund"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"Mauritius-Rupien","one":"Mauritius-Rupie"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"Malediven-Rupien (alt)","one":"Malediven-Rupie (alt)"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"Malediven-Rupien","one":"Malediven-Rufiyaa"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"Malawi-Kwacha"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"Mexikanische Pesos","one":"Mexikanischer Peso"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"Mexikanische Silber-Pesos (1861–1992)","one":"Mexikanische Silber-Peso (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"Mexikanische Unidad de Inversion (UDI)","one":"Mexicanischer Unidad de Inversion (UDI)"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"Malaysische Ringgit","one":"Malaysischer Ringgit"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"Mozambikanische Escudo"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"Mosambikanische Meticais (1980–2006)","one":"Mosambikanischer Metical (1980–2006)"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"Mosambikanische Meticais","one":"Mosambikanischer Metical"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"Namibia-Dollar"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"Nigerianische Naira","one":"Nigerianischer Naira"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"Nicaraguanische Córdoba (1988–1991)","one":"Nicaraguanischer Córdoba (1988–1991)"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"Nicaragua-Córdobas","one":"Nicaragua-Córdoba"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"Niederländische Gulden","one":"Niederländischer Gulden"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"Norwegische Kronen","one":"Norwegische Krone"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"Nepalesische Rupien","one":"Nepalesische Rupie"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"Neuseeland-Dollar"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"Omanische Rials","one":"Omanischer Rial"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"Panamaische Balboas","one":"Panamaischer Balboa"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"Peruanische Inti"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"Peruanische Sol","one":"Peruanischer Sol"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"Peruanische Sol (1863–1965)","one":"Peruanischer Sol (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"Papua-Neuguineische Kina","one":"Papua-Neuguineischer Kina"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"Philippinische Pesos","one":"Philippinischer Peso"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"Pakistanische Rupien","one":"Pakistanische Rupie"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"Polnische Złoty","one":"Polnischer Złoty"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"Polnische Zloty (1950–1995)","one":"Polnischer Zloty (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"Portugiesische Escudo"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"Paraguayische Guaraníes","one":"Paraguayischer Guaraní"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"Katar-Riyal"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"Rhodesische Dollar"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"Rumänische Leu (1952–2006)","one":"Rumänischer Leu (1952–2006)"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"Rumänische Leu","one":"Rumänischer Leu"},"symbol":"RON","narrow":"L"},"RSD":{"displayName":{"other":"Serbische Dinaren","one":"Serbischer Dinar"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"Russische Rubel","one":"Russischer Rubel"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"Russische Rubel (1991–1998)","one":"Russischer Rubel (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"Ruanda-Francs","one":"Ruanda-Franc"},"symbol":"RWF","narrow":"F.Rw"},"SAR":{"displayName":{"other":"Saudi-Rial"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"Salomonen-Dollar"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"Seychellen-Rupien","one":"Seychellen-Rupie"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"Sudanesische Dinar (1992–2007)","one":"Sudanesischer Dinar (1992–2007)"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"Sudanesische Pfund","one":"Sudanesisches Pfund"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"Sudanesische Pfund (1957–1998)","one":"Sudanesisches Pfund (1957–1998)"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"Schwedische Kronen","one":"Schwedische Krone"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"Singapur-Dollar"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"St. Helena-Pfund"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"Slowenische Tolar","one":"Slowenischer Tolar"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"Slowakische Kronen"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"Sierra-leonische Leones","one":"Sierra-leonischer Leone"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"Somalia-Schilling"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"Suriname-Dollar"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"Suriname-Gulden"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"Südsudanesische Pfund","one":"Südsudanesisches Pfund"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"São-toméische Dobra (1977–2017)","one":"São-toméischer Dobra (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"São-toméische Dobras","one":"São-toméischer Dobra"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"Sowjetische Rubel"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"El Salvador-Colon"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"Syrische Pfund","one":"Syrisches Pfund"},"symbol":"SYP","narrow":"SYP"},"SZL":{"displayName":{"other":"Swasiländische Emalangeni","one":"Swasiländischer Lilangeni"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"Thailändische Baht","one":"Thailändischer Baht"},"symbol":"฿","narrow":"฿"},"TJR":{"displayName":{"other":"Tadschikistan-Rubel"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"Tadschikistan-Somoni"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"Turkmenistan-Manat (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"Turkmenistan-Manat"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"Tunesische Dinar","one":"Tunesischer Dinar"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"Tongaische Paʻanga","one":"Tongaischer Paʻanga"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"Timor-Escudo"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"Türkische Lira (1922–2005)"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"Türkische Lira"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"Trinidad und Tobago-Dollar"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"Neue Taiwan-Dollar","one":"Neuer Taiwan-Dollar"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"Tansania-Schilling"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"Ukrainische Hrywen","one":"Ukrainische Hrywnja"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"Ukrainische Karbovanetz"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"Uganda-Schilling (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"Uganda-Schilling"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"US-Dollar"},"symbol":"$","narrow":"$"},"USN":{"displayName":{"other":"US-Dollar (Nächster Tag)"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"US-Dollar (Gleicher Tag)"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"Uruguayische Pesos (Indexierte Rechnungseinheiten)","one":"Uruguayischer Peso (Indexierte Rechnungseinheiten)"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"Uruguayische Pesos (1975–1993)","one":"Uruguayischer Peso (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"Uruguayische Pesos","one":"Uruguayischer Peso"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"Usbekistan-Sum"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"Venezolanische Bolívares (1871–2008)","one":"Venezolanischer Bolívar (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"Venezolanische Bolívares (2008–2018)","one":"Venezolanischer Bolívar (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"Venezolanische Bolívares","one":"Venezolanischer Bolívar"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"Vietnamesische Dong","one":"Vietnamesischer Dong"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"Vietnamesische Dong(1978–1985)","one":"Vietnamesischer Dong(1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"Vanuatu-Vatu"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"Samoanische Tala","one":"Samoanischer Tala"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"CFA-Franc (BEAC)"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"Unzen Silber","one":"Unze Silber"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"Unzen Gold","one":"Unze Gold"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"Europäische Rechnungseinheiten"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"Europäische Währungseinheiten (XBB)"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"Europäische Rechnungseinheiten (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"Europäische Rechnungseinheiten (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"Ostkaribische Dollar","one":"Ostkaribischer Dollar"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"Sonderziehungsrechte"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"Europäische Währungseinheiten (XEU)"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"Französische Gold-Franc"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"Französische UIC-Franc"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"CFA-Francs (BCEAO)","one":"CFA-Franc (BCEAO)"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"Unzen Palladium","one":"Unze Palladium"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"CFP-Franc"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"Unzen Platin","one":"Unze Platin"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET Funds"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"SUCRE"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"Testwährung"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"Rechnungseinheiten der AfEB","one":"Rechnungseinheit der AfEB"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"(unbekannte Währung)"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"Jemen-Dinar"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"Jemen-Rial"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"Jugoslawische Dinar (1966–1990)","one":"Jugoslawischer Dinar (1966–1990)"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"Jugoslawische Neue Dinar (1994–2002)","one":"Jugoslawischer Neuer Dinar (1994–2002)"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"Jugoslawische Dinar (konvertibel)"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"Jugoslawische reformierte Dinar (1992–1993)","one":"Jugoslawischer reformierter Dinar (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"Südafrikanischer Rand (Finanz)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"Südafrikanische Rand","one":"Südafrikanischer Rand"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"Kwacha (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"Kwacha"},"symbol":"ZMW","narrow":"K"},"ZRN":{"displayName":{"other":"Zaire-Neue Zaïre (1993–1998)","one":"Zaire-Neuer Zaïre (1993–1998)"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"Zaire-Zaïre (1971–1993)"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"Simbabwe-Dollar (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"Simbabwe-Dollar (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"Simbabwe-Dollar (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":",","group":".","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"·","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0 %"},"decimal":{"latn":{"long":{"1000":{"other":"0 Tausend"},"10000":{"other":"00 Tausend"},"100000":{"other":"000 Tausend"},"1000000":{"other":"0 Millionen","one":"0 Million"},"10000000":{"other":"00 Millionen"},"100000000":{"other":"000 Millionen"},"1000000000":{"other":"0 Milliarden","one":"0 Milliarde"},"10000000000":{"other":"00 Milliarden"},"100000000000":{"other":"000 Milliarden"},"1000000000000":{"other":"0 Billionen","one":"0 Billion"},"10000000000000":{"other":"00 Billionen"},"100000000000000":{"other":"000 Billionen"}},"short":{"1000":{"other":"0"},"10000":{"other":"0"},"100000":{"other":"0"},"1000000":{"other":"0 Mio'.'"},"10000000":{"other":"00 Mio'.'"},"100000000":{"other":"000 Mio'.'"},"1000000000":{"other":"0 Mrd'.'"},"10000000000":{"other":"00 Mrd'.'"},"100000000000":{"other":"000 Mrd'.'"},"1000000000000":{"other":"0 Bio'.'"},"10000000000000":{"other":"00 Bio'.'"},"100000000000000":{"other":"000 Bio'.'"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"#,##0.00 ¤","accounting":"#,##0.00 ¤","unitPattern":"{0} {1}","short":{"1000":{"other":"0"},"10000":{"other":"0"},"100000":{"other":"0"},"1000000":{"other":"0 Mio'.' ¤"},"10000000":{"other":"00 Mio'.' ¤"},"100000000":{"other":"000 Mio'.' ¤"},"1000000000":{"other":"0 Mrd'.' ¤"},"10000000000":{"other":"00 Mrd'.' ¤"},"100000000000":{"other":"000 Mrd'.' ¤"},"1000000000000":{"other":"0 Bio'.' ¤"},"10000000000000":{"other":"00 Bio'.' ¤"},"100000000000000":{"other":"000 Bio'.' ¤"}}}}},"nu":["latn"]}},"availableLocales":["de"]},
    {"data":{"en":{"units":{"simple":{"degree":{"long":{"other":"{0} degrees","one":"{0} degree"},"short":{"other":"{0} deg"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0} hectares","one":"{0} hectare"},"short":{"other":"{0} ha"},"narrow":{"other":"{0}ha"},"perUnit":{}},"acre":{"long":{"other":"{0} acres","one":"{0} acre"},"short":{"other":"{0} ac"},"narrow":{"other":"{0}ac"},"perUnit":{}},"percent":{"long":{"other":"{0} percent"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"{0} liters per kilometer","one":"{0} liter per kilometer"},"short":{"other":"{0} L/km"},"narrow":{"other":"{0}L/km"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"{0} miles per gallon","one":"{0} mile per gallon"},"short":{"other":"{0} mpg"},"narrow":{"other":"{0}mpg"},"perUnit":{}},"petabyte":{"long":{"other":"{0} petabytes","one":"{0} petabyte"},"short":{"other":"{0} PB"},"narrow":{"other":"{0}PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0} terabytes","one":"{0} terabyte"},"short":{"other":"{0} TB"},"narrow":{"other":"{0}TB"},"perUnit":{}},"terabit":{"long":{"other":"{0} terabits","one":"{0} terabit"},"short":{"other":"{0} Tb"},"narrow":{"other":"{0}Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} gigabytes","one":"{0} gigabyte"},"short":{"other":"{0} GB"},"narrow":{"other":"{0}GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0} gigabits","one":"{0} gigabit"},"short":{"other":"{0} Gb"},"narrow":{"other":"{0}Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0} megabytes","one":"{0} megabyte"},"short":{"other":"{0} MB"},"narrow":{"other":"{0}MB"},"perUnit":{}},"megabit":{"long":{"other":"{0} megabits","one":"{0} megabit"},"short":{"other":"{0} Mb"},"narrow":{"other":"{0}Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} kilobytes","one":"{0} kilobyte"},"short":{"other":"{0} kB"},"narrow":{"other":"{0}kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0} kilobits","one":"{0} kilobit"},"short":{"other":"{0} kb"},"narrow":{"other":"{0}kb"},"perUnit":{}},"byte":{"long":{"other":"{0} bytes","one":"{0} byte"},"short":{"other":"{0} byte"},"narrow":{"other":"{0}B"},"perUnit":{}},"bit":{"long":{"other":"{0} bits","one":"{0} bit"},"short":{"other":"{0} bit"},"narrow":{"other":"{0}bit"},"perUnit":{}},"year":{"long":{"other":"{0} years","one":"{0} year"},"short":{"other":"{0} yrs","one":"{0} yr"},"narrow":{"other":"{0}y"},"perUnit":{"long":"{0} per year","short":"{0}/y","narrow":"{0}/y"}},"month":{"long":{"other":"{0} months","one":"{0} month"},"short":{"other":"{0} mths","one":"{0} mth"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0} per month","short":"{0}/m","narrow":"{0}/m"}},"week":{"long":{"other":"{0} weeks","one":"{0} week"},"short":{"other":"{0} wks","one":"{0} wk"},"narrow":{"other":"{0}w"},"perUnit":{"long":"{0} per week","short":"{0}/w","narrow":"{0}/w"}},"day":{"long":{"other":"{0} days","one":"{0} day"},"short":{"other":"{0} days","one":"{0} day"},"narrow":{"other":"{0}d"},"perUnit":{"long":"{0} per day","short":"{0}/d","narrow":"{0}/d"}},"hour":{"long":{"other":"{0} hours","one":"{0} hour"},"short":{"other":"{0} hr"},"narrow":{"other":"{0}h"},"perUnit":{"long":"{0} per hour","short":"{0}/h","narrow":"{0}/h"}},"minute":{"long":{"other":"{0} minutes","one":"{0} minute"},"short":{"other":"{0} min"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0} per minute","short":"{0}/min","narrow":"{0}/min"}},"second":{"long":{"other":"{0} seconds","one":"{0} second"},"short":{"other":"{0} sec"},"narrow":{"other":"{0}s"},"perUnit":{"long":"{0} per second","short":"{0}/s","narrow":"{0}/s"}},"millisecond":{"long":{"other":"{0} milliseconds","one":"{0} millisecond"},"short":{"other":"{0} ms"},"narrow":{"other":"{0}ms"},"perUnit":{}},"kilometer":{"long":{"other":"{0} kilometers","one":"{0} kilometer"},"short":{"other":"{0} km"},"narrow":{"other":"{0}km"},"perUnit":{"long":"{0} per kilometer","short":"{0}/km","narrow":"{0}/km"}},"meter":{"long":{"other":"{0} meters","one":"{0} meter"},"short":{"other":"{0} m"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0} per meter","short":"{0}/m","narrow":"{0}/m"}},"centimeter":{"long":{"other":"{0} centimeters","one":"{0} centimeter"},"short":{"other":"{0} cm"},"narrow":{"other":"{0}cm"},"perUnit":{"long":"{0} per centimeter","short":"{0}/cm","narrow":"{0}/cm"}},"millimeter":{"long":{"other":"{0} millimeters","one":"{0} millimeter"},"short":{"other":"{0} mm"},"narrow":{"other":"{0}mm"},"perUnit":{}},"mile":{"long":{"other":"{0} miles","one":"{0} mile"},"short":{"other":"{0} mi"},"narrow":{"other":"{0}mi"},"perUnit":{}},"yard":{"long":{"other":"{0} yards","one":"{0} yard"},"short":{"other":"{0} yd"},"narrow":{"other":"{0}yd"},"perUnit":{}},"foot":{"long":{"other":"{0} feet","one":"{0} foot"},"short":{"other":"{0} ft"},"narrow":{"other":"{0}′"},"perUnit":{"long":"{0} per foot","short":"{0}/ft","narrow":"{0}/ft"}},"inch":{"long":{"other":"{0} inches","one":"{0} inch"},"short":{"other":"{0} in"},"narrow":{"other":"{0}″"},"perUnit":{"long":"{0} per inch","short":"{0}/in","narrow":"{0}/in"}},"mile-scandinavian":{"long":{"other":"{0} miles-scandinavian","one":"{0} mile-scandinavian"},"short":{"other":"{0} smi"},"narrow":{"other":"{0}smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0} kilograms","one":"{0} kilogram"},"short":{"other":"{0} kg"},"narrow":{"other":"{0}kg"},"perUnit":{"long":"{0} per kilogram","short":"{0}/kg","narrow":"{0}/kg"}},"gram":{"long":{"other":"{0} grams","one":"{0} gram"},"short":{"other":"{0} g"},"narrow":{"other":"{0}g"},"perUnit":{"long":"{0} per gram","short":"{0}/g","narrow":"{0}/g"}},"stone":{"long":{"other":"{0} stones","one":"{0} stone"},"short":{"other":"{0} st"},"narrow":{"other":"{0}st"},"perUnit":{}},"pound":{"long":{"other":"{0} pounds","one":"{0} pound"},"short":{"other":"{0} lb"},"narrow":{"other":"{0}#"},"perUnit":{"long":"{0} per pound","short":"{0}/lb","narrow":"{0}/lb"}},"ounce":{"long":{"other":"{0} ounces","one":"{0} ounce"},"short":{"other":"{0} oz"},"narrow":{"other":"{0}oz"},"perUnit":{"long":"{0} per ounce","short":"{0}/oz","narrow":"{0}/oz"}},"kilometer-per-hour":{"long":{"other":"{0} kilometers per hour","one":"{0} kilometer per hour"},"short":{"other":"{0} km/h"},"narrow":{"other":"{0}km/h"},"perUnit":{}},"meter-per-second":{"long":{"other":"{0} meters per second","one":"{0} meter per second"},"short":{"other":"{0} m/s"},"narrow":{"other":"{0}m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"{0} miles per hour","one":"{0} mile per hour"},"short":{"other":"{0} mph"},"narrow":{"other":"{0}mph"},"perUnit":{}},"celsius":{"long":{"other":"{0} degrees Celsius","one":"{0} degree Celsius"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0} degrees Fahrenheit","one":"{0} degree Fahrenheit"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°"},"perUnit":{}},"liter":{"long":{"other":"{0} liters","one":"{0} liter"},"short":{"other":"{0} L"},"narrow":{"other":"{0}L"},"perUnit":{"long":"{0} per liter","short":"{0}/L","narrow":"{0}/L"}},"milliliter":{"long":{"other":"{0} milliliters","one":"{0} milliliter"},"short":{"other":"{0} mL"},"narrow":{"other":"{0}mL"},"perUnit":{}},"gallon":{"long":{"other":"{0} gallons","one":"{0} gallon"},"short":{"other":"{0} gal"},"narrow":{"other":"{0}gal"},"perUnit":{"long":"{0} per gallon","short":"{0}/gal US","narrow":"{0}/gal"}},"fluid-ounce":{"long":{"other":"{0} fluid ounces","one":"{0} fluid ounce"},"short":{"other":"{0} fl oz"},"narrow":{"other":"{0}fl oz"},"perUnit":{}}},"compound":{"per":{"long":"{0} per {1}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"Andorran pesetas","one":"Andorran peseta"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"UAE dirhams","one":"UAE dirham"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"Afghan afghanis (1927–2002)","one":"Afghan afghani (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"Afghan Afghanis","one":"Afghan Afghani"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"Albanian lekë (1946–1965)","one":"Albanian lek (1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"Albanian lekë","one":"Albanian lek"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"Armenian drams","one":"Armenian dram"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"Netherlands Antillean guilders","one":"Netherlands Antillean guilder"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"Angolan kwanzas","one":"Angolan kwanza"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"Angolan kwanzas (1977–1991)","one":"Angolan kwanza (1977–1991)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"Angolan new kwanzas (1990–2000)","one":"Angolan new kwanza (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"Angolan readjusted kwanzas (1995–1999)","one":"Angolan readjusted kwanza (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"Argentine australs","one":"Argentine austral"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"Argentine pesos ley (1970–1983)","one":"Argentine peso ley (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"Argentine pesos (1881–1970)","one":"Argentine peso (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"Argentine pesos (1983–1985)","one":"Argentine peso (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"Argentine pesos","one":"Argentine peso"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"Austrian schillings","one":"Austrian schilling"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"Australian dollars","one":"Australian dollar"},"symbol":"A$","narrow":"$"},"AWG":{"displayName":{"other":"Aruban florin"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"Azerbaijani manats (1993–2006)","one":"Azerbaijani manat (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"Azerbaijani manats","one":"Azerbaijani manat"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"Bosnia-Herzegovina dinars (1992–1994)","one":"Bosnia-Herzegovina dinar (1992–1994)"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"Bosnia-Herzegovina convertible marks","one":"Bosnia-Herzegovina convertible mark"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"Bosnia-Herzegovina new dinars (1994–1997)","one":"Bosnia-Herzegovina new dinar (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"Barbadian dollars","one":"Barbadian dollar"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"Bangladeshi takas","one":"Bangladeshi taka"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"Belgian francs (convertible)","one":"Belgian franc (convertible)"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"Belgian francs","one":"Belgian franc"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"Belgian francs (financial)","one":"Belgian franc (financial)"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"Bulgarian hard leva","one":"Bulgarian hard lev"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"Bulgarian socialist leva","one":"Bulgarian socialist lev"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"Bulgarian leva","one":"Bulgarian lev"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"Bulgarian leva (1879–1952)","one":"Bulgarian lev (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"Bahraini dinars","one":"Bahraini dinar"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"Burundian francs","one":"Burundian franc"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"Bermudan dollars","one":"Bermudan dollar"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"Brunei dollars","one":"Brunei dollar"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"Bolivian bolivianos","one":"Bolivian boliviano"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"Bolivian bolivianos (1863–1963)","one":"Bolivian boliviano (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"Bolivian pesos","one":"Bolivian peso"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"Bolivian mvdols","one":"Bolivian mvdol"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"Brazilian new cruzeiros (1967–1986)","one":"Brazilian new cruzeiro (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"Brazilian cruzados (1986–1989)","one":"Brazilian cruzado (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"Brazilian cruzeiros (1990–1993)","one":"Brazilian cruzeiro (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"Brazilian reals","one":"Brazilian real"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"Brazilian new cruzados (1989–1990)","one":"Brazilian new cruzado (1989–1990)"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"Brazilian cruzeiros (1993–1994)","one":"Brazilian cruzeiro (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"Brazilian cruzeiros (1942–1967)","one":"Brazilian cruzeiro (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"Bahamian dollars","one":"Bahamian dollar"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"Bhutanese ngultrums","one":"Bhutanese ngultrum"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"Burmese kyats","one":"Burmese kyat"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"Botswanan pulas","one":"Botswanan pula"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"Belarusian rubles (1994–1999)","one":"Belarusian ruble (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"Belarusian rubles","one":"Belarusian ruble"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"Belarusian rubles (2000–2016)","one":"Belarusian ruble (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"Belize dollars","one":"Belize dollar"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"Canadian dollars","one":"Canadian dollar"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"Congolese francs","one":"Congolese franc"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"WIR euros","one":"WIR euro"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"Swiss francs","one":"Swiss franc"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"WIR francs","one":"WIR franc"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"Chilean escudos","one":"Chilean escudo"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"Chilean units of account (UF)","one":"Chilean unit of account (UF)"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"Chilean pesos","one":"Chilean peso"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"Chinese yuan (offshore)"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"Chinese People’s Bank dollars","one":"Chinese People’s Bank dollar"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"Chinese yuan"},"symbol":"CN¥","narrow":"¥"},"COP":{"displayName":{"other":"Colombian pesos","one":"Colombian peso"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"Colombian real value units","one":"Colombian real value unit"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"Costa Rican colóns","one":"Costa Rican colón"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"Serbian dinars (2002–2006)","one":"Serbian dinar (2002–2006)"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"Czechoslovak hard korunas","one":"Czechoslovak hard koruna"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"Cuban convertible pesos","one":"Cuban convertible peso"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"Cuban pesos","one":"Cuban peso"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"Cape Verdean escudos","one":"Cape Verdean escudo"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"Cypriot pounds","one":"Cypriot pound"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"Czech korunas","one":"Czech koruna"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"East German marks","one":"East German mark"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"German marks","one":"German mark"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"Djiboutian francs","one":"Djiboutian franc"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"Danish kroner","one":"Danish krone"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"Dominican pesos","one":"Dominican peso"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"Algerian dinars","one":"Algerian dinar"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"Ecuadorian sucres","one":"Ecuadorian sucre"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"Ecuadorian units of constant value","one":"Ecuadorian unit of constant value"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"Estonian kroons","one":"Estonian kroon"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"Egyptian pounds","one":"Egyptian pound"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"Eritrean nakfas","one":"Eritrean nakfa"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"Spanish pesetas (A account)","one":"Spanish peseta (A account)"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"Spanish pesetas (convertible account)","one":"Spanish peseta (convertible account)"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"Spanish pesetas","one":"Spanish peseta"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"Ethiopian birrs","one":"Ethiopian birr"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"euros","one":"euro"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"Finnish markkas","one":"Finnish markka"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"Fijian dollars","one":"Fijian dollar"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"Falkland Islands pounds","one":"Falkland Islands pound"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"French francs","one":"French franc"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"British pounds","one":"British pound"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"Georgian kupon larits","one":"Georgian kupon larit"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"Georgian laris","one":"Georgian lari"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"Ghanaian cedis (1979–2007)","one":"Ghanaian cedi (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"Ghanaian cedis","one":"Ghanaian cedi"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"Gibraltar pounds","one":"Gibraltar pound"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"Gambian dalasis","one":"Gambian dalasi"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"Guinean francs","one":"Guinean franc"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"Guinean sylis","one":"Guinean syli"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"Equatorial Guinean ekwele"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"Greek drachmas","one":"Greek drachma"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"Guatemalan quetzals","one":"Guatemalan quetzal"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"Portuguese Guinea escudos","one":"Portuguese Guinea escudo"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"Guinea-Bissau pesos","one":"Guinea-Bissau peso"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"Guyanaese dollars","one":"Guyanaese dollar"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"Hong Kong dollars","one":"Hong Kong dollar"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"Honduran lempiras","one":"Honduran lempira"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"Croatian dinars","one":"Croatian dinar"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"Croatian kunas","one":"Croatian kuna"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"Haitian gourdes","one":"Haitian gourde"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"Hungarian forints","one":"Hungarian forint"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"Indonesian rupiahs","one":"Indonesian rupiah"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"Irish pounds","one":"Irish pound"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"Israeli pounds","one":"Israeli pound"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"Israeli shekels (1980–1985)","one":"Israeli shekel (1980–1985)"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"Israeli new shekels","one":"Israeli new shekel"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"Indian rupees","one":"Indian rupee"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"Iraqi dinars","one":"Iraqi dinar"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"Iranian rials","one":"Iranian rial"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"Icelandic krónur (1918–1981)","one":"Icelandic króna (1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"Icelandic krónur","one":"Icelandic króna"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"Italian liras","one":"Italian lira"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"Jamaican dollars","one":"Jamaican dollar"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"Jordanian dinars","one":"Jordanian dinar"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"Japanese yen"},"symbol":"¥","narrow":"¥"},"KES":{"displayName":{"other":"Kenyan shillings","one":"Kenyan shilling"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"Kyrgystani soms","one":"Kyrgystani som"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"Cambodian riels","one":"Cambodian riel"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"Comorian francs","one":"Comorian franc"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"North Korean won"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"South Korean hwan (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"South Korean won (1945–1953)"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"South Korean won"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"Kuwaiti dinars","one":"Kuwaiti dinar"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"Cayman Islands dollars","one":"Cayman Islands dollar"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"Kazakhstani tenges","one":"Kazakhstani tenge"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"Laotian kips","one":"Laotian kip"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"Lebanese pounds","one":"Lebanese pound"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"Sri Lankan rupees","one":"Sri Lankan rupee"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"Liberian dollars","one":"Liberian dollar"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"Lesotho lotis","one":"Lesotho loti"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"Lithuanian litai","one":"Lithuanian litas"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"Lithuanian talonases","one":"Lithuanian talonas"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"Luxembourgian convertible francs","one":"Luxembourgian convertible franc"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"Luxembourgian francs","one":"Luxembourgian franc"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"Luxembourg financial francs","one":"Luxembourg financial franc"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"Latvian lati","one":"Latvian lats"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"Latvian rubles","one":"Latvian ruble"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"Libyan dinars","one":"Libyan dinar"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"Moroccan dirhams","one":"Moroccan dirham"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"Moroccan francs","one":"Moroccan franc"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"Monegasque francs","one":"Monegasque franc"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"Moldovan cupon"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"Moldovan lei","one":"Moldovan leu"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"Malagasy ariaries","one":"Malagasy ariary"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"Malagasy francs","one":"Malagasy franc"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"Macedonian denari","one":"Macedonian denar"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"Macedonian denari (1992–1993)","one":"Macedonian denar (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"Malian francs","one":"Malian franc"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"Myanmar kyats","one":"Myanmar kyat"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"Mongolian tugriks","one":"Mongolian tugrik"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"Macanese patacas","one":"Macanese pataca"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"Mauritanian ouguiyas (1973–2017)","one":"Mauritanian ouguiya (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"Mauritanian ouguiyas","one":"Mauritanian ouguiya"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"Maltese lira"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"Maltese pounds","one":"Maltese pound"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"Mauritian rupees","one":"Mauritian rupee"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"Maldivian rupees (1947–1981)","one":"Maldivian rupee (1947–1981)"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"Maldivian rufiyaas","one":"Maldivian rufiyaa"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"Malawian kwachas","one":"Malawian kwacha"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"Mexican pesos","one":"Mexican peso"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"Mexican silver pesos (1861–1992)","one":"Mexican silver peso (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"Mexican investment units","one":"Mexican investment unit"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"Malaysian ringgits","one":"Malaysian ringgit"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"Mozambican escudos","one":"Mozambican escudo"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"Mozambican meticals (1980–2006)","one":"Mozambican metical (1980–2006)"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"Mozambican meticals","one":"Mozambican metical"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"Namibian dollars","one":"Namibian dollar"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"Nigerian nairas","one":"Nigerian naira"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"Nicaraguan córdobas (1988–1991)","one":"Nicaraguan córdoba (1988–1991)"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"Nicaraguan córdobas","one":"Nicaraguan córdoba"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"Dutch guilders","one":"Dutch guilder"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"Norwegian kroner","one":"Norwegian krone"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"Nepalese rupees","one":"Nepalese rupee"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"New Zealand dollars","one":"New Zealand dollar"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"Omani rials","one":"Omani rial"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"Panamanian balboas","one":"Panamanian balboa"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"Peruvian intis","one":"Peruvian inti"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"Peruvian soles","one":"Peruvian sol"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"Peruvian soles (1863–1965)","one":"Peruvian sol (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"Papua New Guinean kina"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"Philippine pisos","one":"Philippine piso"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"Pakistani rupees","one":"Pakistani rupee"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"Polish zlotys","one":"Polish zloty"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"Polish zlotys (PLZ)","one":"Polish zloty (PLZ)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"Portuguese escudos","one":"Portuguese escudo"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"Paraguayan guaranis","one":"Paraguayan guarani"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"Qatari rials","one":"Qatari rial"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"Rhodesian dollars","one":"Rhodesian dollar"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"Romanian Lei (1952–2006)","one":"Romanian leu (1952–2006)"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"Romanian lei","one":"Romanian leu"},"symbol":"RON","narrow":"lei"},"RSD":{"displayName":{"other":"Serbian dinars","one":"Serbian dinar"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"Russian rubles","one":"Russian ruble"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"Russian rubles (1991–1998)","one":"Russian ruble (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"Rwandan francs","one":"Rwandan franc"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"Saudi riyals","one":"Saudi riyal"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"Solomon Islands dollars","one":"Solomon Islands dollar"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"Seychellois rupees","one":"Seychellois rupee"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"Sudanese dinars (1992–2007)","one":"Sudanese dinar (1992–2007)"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"Sudanese pounds","one":"Sudanese pound"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"Sudanese pounds (1957–1998)","one":"Sudanese pound (1957–1998)"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"Swedish kronor","one":"Swedish krona"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"Singapore dollars","one":"Singapore dollar"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"St. Helena pounds","one":"St. Helena pound"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"Slovenian tolars","one":"Slovenian tolar"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"Slovak korunas","one":"Slovak koruna"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"Sierra Leonean leones","one":"Sierra Leonean leone"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"Somali shillings","one":"Somali shilling"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"Surinamese dollars","one":"Surinamese dollar"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"Surinamese guilders","one":"Surinamese guilder"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"South Sudanese pounds","one":"South Sudanese pound"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"São Tomé & Príncipe dobras (1977–2017)","one":"São Tomé & Príncipe dobra (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"São Tomé & Príncipe dobras","one":"São Tomé & Príncipe dobra"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"Soviet roubles","one":"Soviet rouble"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"Salvadoran colones","one":"Salvadoran colón"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"Syrian pounds","one":"Syrian pound"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"Swazi emalangeni","one":"Swazi lilangeni"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"Thai baht"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"Tajikistani rubles","one":"Tajikistani ruble"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"Tajikistani somonis","one":"Tajikistani somoni"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"Turkmenistani manat (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"Turkmenistani manat"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"Tunisian dinars","one":"Tunisian dinar"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"Tongan paʻanga"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"Timorese escudos","one":"Timorese escudo"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"Turkish Lira (1922–2005)","one":"Turkish lira (1922–2005)"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"Turkish Lira","one":"Turkish lira"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"Trinidad & Tobago dollars","one":"Trinidad & Tobago dollar"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"New Taiwan dollars","one":"New Taiwan dollar"},"symbol":"NT$","narrow":"$"},"TZS":{"displayName":{"other":"Tanzanian shillings","one":"Tanzanian shilling"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"Ukrainian hryvnias","one":"Ukrainian hryvnia"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"Ukrainian karbovantsiv","one":"Ukrainian karbovanets"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"Ugandan shillings (1966–1987)","one":"Ugandan shilling (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"Ugandan shillings","one":"Ugandan shilling"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"US dollars","one":"US dollar"},"symbol":"$","narrow":"$"},"USN":{"displayName":{"other":"US dollars (next day)","one":"US dollar (next day)"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"US dollars (same day)","one":"US dollar (same day)"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"Uruguayan pesos (indexed units)","one":"Uruguayan peso (indexed units)"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"Uruguayan pesos (1975–1993)","one":"Uruguayan peso (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"Uruguayan pesos","one":"Uruguayan peso"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"Uruguayan nominal wage index units","one":"Uruguayan nominal wage index unit"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"Uzbekistani som"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"Venezuelan bolívars (1871–2008)","one":"Venezuelan bolívar (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"Venezuelan bolívars (2008–2018)","one":"Venezuelan bolívar (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"Venezuelan bolívars","one":"Venezuelan bolívar"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"Vietnamese dong"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"Vietnamese dong (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"Vanuatu vatus","one":"Vanuatu vatu"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"Samoan tala"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"Central African CFA francs","one":"Central African CFA franc"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"troy ounces of silver","one":"troy ounce of silver"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"troy ounces of gold","one":"troy ounce of gold"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"European composite units","one":"European composite unit"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"European monetary units","one":"European monetary unit"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"European units of account (XBC)","one":"European unit of account (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"European units of account (XBD)","one":"European unit of account (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"East Caribbean dollars","one":"East Caribbean dollar"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"special drawing rights"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"European currency units","one":"European currency unit"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"French gold francs","one":"French gold franc"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"French UIC-francs","one":"French UIC-franc"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"West African CFA francs","one":"West African CFA franc"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"troy ounces of palladium","one":"troy ounce of palladium"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"CFP francs","one":"CFP franc"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"troy ounces of platinum","one":"troy ounce of platinum"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET Funds units","one":"RINET Funds unit"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"Sucres","one":"Sucre"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"Testing Currency units","one":"Testing Currency unit"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"ADB units of account","one":"ADB unit of account"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"(unknown currency)","one":"(unknown unit of currency)"},"symbol":"¤","narrow":"¤"},"YDD":{"displayName":{"other":"Yemeni dinars","one":"Yemeni dinar"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"Yemeni rials","one":"Yemeni rial"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"Yugoslavian hard dinars (1966–1990)","one":"Yugoslavian hard dinar (1966–1990)"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"Yugoslavian new dinars (1994–2002)","one":"Yugoslavian new dinar (1994–2002)"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"Yugoslavian convertible dinars (1990–1992)","one":"Yugoslavian convertible dinar (1990–1992)"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"Yugoslavian reformed dinars (1992–1993)","one":"Yugoslavian reformed dinar (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"South African rands (financial)","one":"South African rand (financial)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"South African rand"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"Zambian kwachas (1968–2012)","one":"Zambian kwacha (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"Zambian kwachas","one":"Zambian kwacha"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"Zairean new zaires (1993–1998)","one":"Zairean new zaire (1993–1998)"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"Zairean zaires (1971–1993)","one":"Zairean zaire (1971–1993)"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"Zimbabwean dollars (1980–2008)","one":"Zimbabwean dollar (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"Zimbabwean dollars (2009)","one":"Zimbabwean dollar (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"Zimbabwean dollars (2008)","one":"Zimbabwean dollar (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0 thousand"},"10000":{"other":"00 thousand"},"100000":{"other":"000 thousand"},"1000000":{"other":"0 million"},"10000000":{"other":"00 million"},"100000000":{"other":"000 million"},"1000000000":{"other":"0 billion"},"10000000000":{"other":"00 billion"},"100000000000":{"other":"000 billion"},"1000000000000":{"other":"0 trillion"},"10000000000000":{"other":"00 trillion"},"100000000000000":{"other":"000 trillion"}},"short":{"1000":{"other":"0K"},"10000":{"other":"00K"},"100000":{"other":"000K"},"1000000":{"other":"0M"},"10000000":{"other":"00M"},"100000000":{"other":"000M"},"1000000000":{"other":"0B"},"10000000000":{"other":"00B"},"100000000000":{"other":"000B"},"1000000000000":{"other":"0T"},"10000000000000":{"other":"00T"},"100000000000000":{"other":"000T"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0} {1}","short":{"1000":{"other":"¤0K"},"10000":{"other":"¤00K"},"100000":{"other":"¤000K"},"1000000":{"other":"¤0M"},"10000000":{"other":"¤00M"},"100000000":{"other":"¤000M"},"1000000000":{"other":"¤0B"},"10000000000":{"other":"¤00B"},"100000000000":{"other":"¤000B"},"1000000000000":{"other":"¤0T"},"10000000000000":{"other":"¤00T"},"100000000000000":{"other":"¤000T"}}}}},"nu":["latn"]}},"availableLocales":["en"]},
    {"data":{"ja":{"units":{"simple":{"degree":{"long":{"other":"{0} 度"},"short":{"other":"{0} 度"},"narrow":{"other":"{0} 度"},"perUnit":{}},"hectare":{"long":{"other":"{0} ヘクタール"},"short":{"other":"{0} ha"},"narrow":{"other":"{0}ha"},"perUnit":{}},"acre":{"long":{"other":"{0} エーカー"},"short":{"other":"{0} ac"},"narrow":{"other":"{0}ac"},"perUnit":{}},"percent":{"long":{"other":"{0} パーセント"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"{0} リットル毎キロメートル"},"short":{"other":"{0} L/km"},"narrow":{"other":"{0}L/km"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"{0} マイル毎ガロン"},"short":{"other":"{0} mpg"},"narrow":{"other":"{0}mpg"},"perUnit":{}},"petabyte":{"long":{"other":"{0} ペタバイト"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0} テラバイト"},"short":{"other":"{0} TB"},"narrow":{"other":"{0}TB"},"perUnit":{}},"terabit":{"long":{"other":"{0} テラビット"},"short":{"other":"{0} Tb"},"narrow":{"other":"{0}Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} ギガバイト"},"short":{"other":"{0} GB"},"narrow":{"other":"{0}GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0} ギガビット"},"short":{"other":"{0} Gb"},"narrow":{"other":"{0}Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0} メガバイト"},"short":{"other":"{0} MB"},"narrow":{"other":"{0}MB"},"perUnit":{}},"megabit":{"long":{"other":"{0} メガビット"},"short":{"other":"{0} Mb"},"narrow":{"other":"{0}Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} キロバイト"},"short":{"other":"{0} KB"},"narrow":{"other":"{0}kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0} キロビット"},"short":{"other":"{0} kb"},"narrow":{"other":"{0}kb"},"perUnit":{}},"byte":{"long":{"other":"{0} バイト"},"short":{"other":"{0} byte"},"narrow":{"other":"{0}B"},"perUnit":{}},"bit":{"long":{"other":"{0} ビット"},"short":{"other":"{0} bit"},"narrow":{"other":"{0}b"},"perUnit":{}},"year":{"long":{"other":"{0} 年"},"short":{"other":"{0} 年"},"narrow":{"other":"{0}y"},"perUnit":{"long":"{0}/年","short":"{0}/年","narrow":"{0}/年"}},"month":{"long":{"other":"{0} か月"},"short":{"other":"{0} か月"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0}/月","short":"{0}/月","narrow":"{0}/月"}},"week":{"long":{"other":"{0} 週間"},"short":{"other":"{0} 週間"},"narrow":{"other":"{0}w"},"perUnit":{"long":"{0}/週","short":"{0}/週","narrow":"{0}/週"}},"day":{"long":{"other":"{0} 日"},"short":{"other":"{0} 日"},"narrow":{"other":"{0}d"},"perUnit":{"long":"{0}/日","short":"{0}/日","narrow":"{0}/日"}},"hour":{"long":{"other":"{0} 時間"},"short":{"other":"{0} 時間"},"narrow":{"other":"{0}h"},"perUnit":{"long":"{0}/時間","short":"{0}/時間","narrow":"{0}/時間"}},"minute":{"long":{"other":"{0} 分"},"short":{"other":"{0} 分"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0}/分","short":"{0}/分","narrow":"{0}/分"}},"second":{"long":{"other":"{0} 秒"},"short":{"other":"{0} 秒"},"narrow":{"other":"{0}s"},"perUnit":{"long":"{0}/秒","short":"{0}/秒","narrow":"{0}/秒"}},"millisecond":{"long":{"other":"{0} ミリ秒"},"short":{"other":"{0} ms"},"narrow":{"other":"{0}ms"},"perUnit":{}},"kilometer":{"long":{"other":"{0} キロメートル"},"short":{"other":"{0} km"},"narrow":{"other":"{0}km"},"perUnit":{"long":"{0}/キロメートル","short":"{0}/km","narrow":"{0}/km"}},"meter":{"long":{"other":"{0} メートル"},"short":{"other":"{0} m"},"narrow":{"other":"{0}m"},"perUnit":{"long":"{0}/メートル","short":"{0}/m","narrow":"{0}/m"}},"centimeter":{"long":{"other":"{0} センチメートル"},"short":{"other":"{0} cm"},"narrow":{"other":"{0}cm"},"perUnit":{"long":"{0}/センチメートル","short":"{0}/cm","narrow":"{0}/cm"}},"millimeter":{"long":{"other":"{0} ミリメートル"},"short":{"other":"{0} mm"},"narrow":{"other":"{0}mm"},"perUnit":{}},"mile":{"long":{"other":"{0} マイル"},"short":{"other":"{0} mi"},"narrow":{"other":"{0}マイル"},"perUnit":{}},"yard":{"long":{"other":"{0} ヤード"},"short":{"other":"{0} yd"},"narrow":{"other":"{0}ヤード"},"perUnit":{}},"foot":{"long":{"other":"{0} フィート"},"short":{"other":"{0} ft"},"narrow":{"other":"{0}′"},"perUnit":{"long":"{0}/フィート","short":"{0}/ft","narrow":"{0}/ft"}},"inch":{"long":{"other":"{0} インチ"},"short":{"other":"{0} in"},"narrow":{"other":"{0}″"},"perUnit":{"long":"{0}/インチ","short":"{0}/in","narrow":"{0}/in"}},"mile-scandinavian":{"long":{"other":"{0} スカンジナビアマイル"},"short":{"other":"{0} smi"},"narrow":{"other":"{0} smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0} キログラム"},"short":{"other":"{0} kg"},"narrow":{"other":"{0}kg"},"perUnit":{"long":"{0}/キログラム","short":"{0}/kg","narrow":"{0}/kg"}},"gram":{"long":{"other":"{0} グラム"},"short":{"other":"{0} g"},"narrow":{"other":"{0}g"},"perUnit":{"long":"{0}/グラム","short":"{0}/g","narrow":"{0}/g"}},"stone":{"long":{"other":"{0} ストーン"},"short":{"other":"{0} st"},"narrow":{"other":"{0}st"},"perUnit":{}},"pound":{"long":{"other":"{0} ポンド"},"short":{"other":"{0} lb"},"narrow":{"other":"{0}lb"},"perUnit":{"long":"{0}/ポンド","short":"{0}/lb","narrow":"{0}/lb"}},"ounce":{"long":{"other":"{0} オンス"},"short":{"other":"{0} oz"},"narrow":{"other":"{0}oz"},"perUnit":{"long":"{0}/オンス","short":"{0}/oz","narrow":"{0}/oz"}},"kilometer-per-hour":{"long":{"other":"時速 {0} キロメートル"},"short":{"other":"{0} km/h"},"narrow":{"other":"{0}km/h"},"perUnit":{}},"meter-per-second":{"long":{"other":"秒速 {0} メートル"},"short":{"other":"{0} m/s"},"narrow":{"other":"{0}m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"時速 {0} マイル"},"short":{"other":"{0} mph"},"narrow":{"other":"{0}mi/h"},"perUnit":{}},"celsius":{"long":{"other":"摂氏 {0} 度"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"華氏 {0} 度"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0} リットル"},"short":{"other":"{0} L"},"narrow":{"other":"{0}L"},"perUnit":{"long":"{0}/リットル","short":"{0}/L","narrow":"{0}/L"}},"milliliter":{"long":{"other":"{0} ミリリットル"},"short":{"other":"{0} ml"},"narrow":{"other":"{0}ml"},"perUnit":{}},"gallon":{"long":{"other":"{0} ガロン"},"short":{"other":"{0} gal"},"narrow":{"other":"{0}gal"},"perUnit":{"long":"{0}/ガロン","short":"{0}/gal","narrow":"{0}/gal"}},"fluid-ounce":{"long":{"other":"{0} 液量オンス"},"short":{"other":"{0} fl oz"},"narrow":{"other":"{0}fl oz"},"perUnit":{}}},"compound":{"per":{"long":"{0}毎{1}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"アンドラ ペセタ"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"UAE ディルハム"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"アフガニスタン アフガニー (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"アフガニスタン アフガニー"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"アルバニア レク (1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"アルバニア レク"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"アルメニア ドラム"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"オランダ領アンティル ギルダー"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"アンゴラ クワンザ"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"アンゴラ クワンザ (1977–1991)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"アンゴラ 新クワンザ (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"アンゴラ 旧クワンザ (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"アルゼンチン アゥストラール"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"アルゼンチン・ペソ・レイ（1970–1983）"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"アルゼンチン・ペソ（1881–1970）"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"アルゼンチン ペソ (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"アルゼンチン ペソ"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"オーストリア シリング"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"オーストラリア ドル"},"symbol":"A$","narrow":"$"},"AWG":{"displayName":{"other":"アルバ ギルダー"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"アゼルバイジャン マナト (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"アゼルバイジャン マナト"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"ボスニア・ヘルツェゴビナ ディナール (1992–1994)"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"ボスニア・ヘルツェゴビナ 兌換マルク (BAM)"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"ボスニア・ヘルツェゴビナ 新ディナール（1994–1997）"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"バルバドス ドル"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"バングラデシュ タカ"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"ベルギー フラン (BEC)"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"ベルギー フラン"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"ベルギー フラン (BEL)"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"ブルガリア レフ"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"ブルガリア社会主義 レフ"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"ブルガリア 新レフ"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"ブルガリア レフ（1879–1952）"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"バーレーン ディナール"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"ブルンジ フラン"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"バミューダ ドル"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"ブルネイ ドル"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"ボリビア ボリビアーノ"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"ボリビア ボリビアーノ (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"ボリビア ペソ"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"ボリビア (Mvdol)"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"ブラジル 新クルゼイロ (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"ブラジル クルザード (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"ブラジル クルゼイロ (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"ブラジル レアル"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"ブラジル 新クルザード (1989–1990)"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"ブラジル クルゼイロ (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"ブラジル クルゼイロ（1942–1967）"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"バハマ ドル"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"ブータン ニュルタム"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"ビルマ チャット"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"ボツワナ プラ"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"ベラルーシ 新ルーブル (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"ベラルーシ ルーブル"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"ベラルーシ ルーブル (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"ベリーズ ドル"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"カナダ ドル"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"コンゴ フラン"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"ユーロ (WIR)"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"スイス フラン"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"フラン (WIR)"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"チリ エスクード"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"チリ ウニダ・デ・フォメント (UF)"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"チリ ペソ"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"中国人民元(オフショア)"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"中国人民銀行ドル"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"中国人民元"},"symbol":"元","narrow":"￥"},"COP":{"displayName":{"other":"コロンビア ペソ"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"コロンビア レアル （UVR)"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"コスタリカ コロン"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"セルビア ディナール (2002–2006)"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"チェコスロバキア コルナ"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"キューバ 兌換ペソ"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"キューバ ペソ"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"カーボベルデ エスクード"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"キプロス ポンド"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"チェコ コルナ"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"東ドイツ マルク"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"ドイツ マルク"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"ジブチ フラン"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"デンマーク クローネ"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"ドミニカ ペソ"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"アルジェリア ディナール"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"エクアドル スクレ"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"エクアドル (UVC)"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"エストニア クルーン"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"エジプト ポンド"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"エリトリア ナクファ"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"スペインペセタ（勘定A）"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"スペイン 兌換ペセタ"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"スペイン ペセタ"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"エチオピア ブル"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"ユーロ"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"フィンランド マルカ"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"フィジー ドル"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"フォークランド（マルビナス）諸島 ポンド"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"フランス フラン"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"英国ポンド"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"ジョージア クーポン ラリ"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"ジョージア ラリ"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"ガーナ セディ (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"ガーナ セディ"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"ジブラルタル ポンド"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"ガンビア ダラシ"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"ギニア フラン"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"ギニア シリー"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"赤道ギニア エクウェレ"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"ギリシャ ドラクマ"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"グアテマラ ケツァル"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"ポルトガル領ギニア エスクード"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"ギニアビサウ ペソ"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"ガイアナ ドル"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"香港ドル"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"ホンジュラス レンピラ"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"クロアチア ディナール"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"クロアチア クーナ"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"ハイチ グールド"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"ハンガリー フォリント"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"インドネシア ルピア"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"アイリッシュ ポンド"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"イスラエル ポンド"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"イスラエル シェケル (1980–1985)"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"イスラエル新シェケル"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"インド ルピー"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"イラク ディナール"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"イラン リアル"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"アイスランド クローナ (1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"アイスランド クローナ"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"イタリア リラ"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"ジャマイカ ドル"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"ヨルダン ディナール"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"円"},"symbol":"￥","narrow":"￥"},"KES":{"displayName":{"other":"ケニア シリング"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"キルギス ソム"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"カンボジア リエル"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"コモロ フラン"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"北朝鮮ウォン"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"韓国 ファン（1953–1962）"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"韓国 ウォン（1945–1953）"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"韓国ウォン"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"クウェート ディナール"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"ケイマン諸島 ドル"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"カザフスタン テンゲ"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"ラオス キープ"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"レバノン ポンド"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"スリランカ ルピー"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"リベリア ドル"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"レソト ロティ"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"リトアニア リタス"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"リトアニア タロナ"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"ルクセンブルク 兌換フラン"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"ルクセンブルグ フラン"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"ルクセンブルク 金融フラン"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"ラトビア ラッツ"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"ラトビア ルーブル"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"リビア ディナール"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"モロッコ ディルハム"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"モロッコ フラン"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"モネガスク フラン"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"モルドバ クーポン"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"モルドバ レイ"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"マダガスカル アリアリ"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"マラガシ フラン"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"マケドニア デナル"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"マケドニア ディナール（1992–1993）"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"マリ フラン"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"ミャンマー チャット"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"モンゴル トグログ"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"マカオ パタカ"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"モーリタニア ウギア (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"モーリタニア ウギア"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"マルタ リラ"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"マルタ ポンド"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"モーリシャス ルピー"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"モルディブ諸島 ルピー"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"モルディブ ルフィア"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"マラウィ クワチャ"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"メキシコ ペソ"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"メキシコ ペソ (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"メキシコ (UDI)"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"マレーシア リンギット"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"モザンピーク エスクード"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"モザンビーク メティカル (1980–2006)"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"モザンビーク メティカル"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"ナミビア ドル"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"ナイジェリア ナイラ"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"ニカラグア コルドバ (1988–1991)"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"ニカラグア コルドバ オロ"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"オランダ ギルダー"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"ノルウェー クローネ"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"ネパール ルピー"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"ニュージーランド ドル"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"オマーン リアル"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"パナマ バルボア"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"ペルー インティ"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"ペルー ソル"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"ペルー ソル (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"パプアニューギニア キナ"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"フィリピン ペソ"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"パキスタン ルピー"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"ポーランド ズウォティ"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"ポーランド ズウォティ (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"ポルトガル エスクード"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"パラグアイ グアラニ"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"カタール リアル"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"ローデシア ドル"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"ルーマニア レイ (1952–2006)"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"ルーマニア レイ"},"symbol":"RON","narrow":"レイ"},"RSD":{"displayName":{"other":"ディナール (セルビア)"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"ロシア ルーブル"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"ロシア ルーブル (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"ルワンダ フラン"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"サウジ リヤル"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"ソロモン諸島 ドル"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"セーシェル ルピー"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"スーダン ディナール (1992–2007)"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"スーダン ポンド"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"スーダン ポンド (1957–1998)"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"スウェーデン クローナ"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"シンガポール ドル"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"セントヘレナ ポンド"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"スロベニア トラール"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"スロバキア コルナ"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"シエラレオネ レオン"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"ソマリア シリング"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"スリナム ドル"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"スリナム ギルダー"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"南スーダン ポンド"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"サントメ・プリンシペ ドブラ (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"サントメ・プリンシペ ドブラ"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"ソ連 ルーブル"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"エルサルバドル コロン"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"シリア ポンド"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"スワジランド リランゲニ"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"タイ バーツ"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"タジキスタン ルーブル"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"タジキスタン ソモニ"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"トルクメニスタン マナト (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"トルクメニスタン マナト"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"チュニジア ディナール"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"トンガ パ・アンガ"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"ティモール エスクード"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"トルコ リラ (1922–2005)"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"新トルコリラ"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"トリニダード・トバゴ ドル"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"新台湾ドル"},"symbol":"NT$","narrow":"$"},"TZS":{"displayName":{"other":"タンザニア シリング"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"ウクライナ グリブナ"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"ウクライナ カルボバネツ"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"ウガンダ シリング (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"ウガンダ シリング"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"米ドル"},"symbol":"$","narrow":"$"},"USN":{"displayName":{"other":"米ドル (翌日)"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"米ドル (当日)"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"ウルグアイ ペソエン"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"ウルグアイ ペソ (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"ウルグアイ ペソ"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"ウズベキスタン スム"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"ベネズエラ ボリバル (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"ベネズエラ ボリバル (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"ベネズエラ ボリバル"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"ベトナム ドン"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"ベトナム ドン（1978–1985）"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"バヌアツ バツ"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"サモア タラ"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"中央アフリカ CFA フラン"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"銀"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"金"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"ヨーロッパ混合単位 (EURCO)"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"ヨーロッパ通貨単位 (EMU–6)"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"ヨーロッパ勘定単位 (EUA–9)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"ヨーロッパ勘定単位 (EUA–17)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"東カリブ ドル"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"特別引き出し権"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"ヨーロッパ通貨単位"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"フランス金フラン"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"フランス フラン (UIC)"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"西アフリカ CFA フラン"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"パラジウム"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"CFP フラン"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"プラチナ"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET基金"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"スクレ"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"テスト用通貨コード"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"UA (アフリカ開発銀行)"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"不明または無効な通貨"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"イエメン ディナール"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"イエメン リアル"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"ユーゴスラビア ハード・ディナール (1966–1990)"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"ユーゴスラビア ノビ・ディナール (1994–2002)"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"ユーゴスラビア 兌換ディナール (1990–1992)"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"ユーゴスラビア 改革ディナール（1992–1993）"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"南アフリカ ランド (ZAL)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"南アフリカ ランド"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"ザンビア クワチャ (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"ザンビア クワチャ"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"ザイール 新ザイール (1993–1998)"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"ザイール ザイール (1971–1993)"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"ジンバブエ ドル (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"ジンバブエ ドル (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"シンバブエ ドル（2008）"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0億"},"1000000000":{"other":"00億"},"10000000000":{"other":"000億"},"100000000000":{"other":"0000億"},"1000000000000":{"other":"0兆"},"10000000000000":{"other":"00兆"},"100000000000000":{"other":"000兆"}},"short":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0億"},"1000000000":{"other":"00億"},"10000000000":{"other":"000億"},"100000000000":{"other":"0000億"},"1000000000000":{"other":"0兆"},"10000000000000":{"other":"00兆"},"100000000000000":{"other":"000兆"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0}{1}","short":{"1000":{"other":"0"},"10000":{"other":"¤0万"},"100000":{"other":"¤00万"},"1000000":{"other":"¤000万"},"10000000":{"other":"¤0000万"},"100000000":{"other":"¤0億"},"1000000000":{"other":"¤00億"},"10000000000":{"other":"¤000億"},"100000000000":{"other":"¤0000億"},"1000000000000":{"other":"¤0兆"},"10000000000000":{"other":"¤00兆"},"100000000000000":{"other":"¤000兆"}}}}},"nu":["latn"]}},"availableLocales":["ja"]},
    {"data":{"ko":{"units":{"simple":{"degree":{"long":{"other":"{0}도"},"short":{"other":"{0}°"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0}헥타르"},"short":{"other":"{0}ha"},"narrow":{"other":"{0}ha"},"perUnit":{}},"acre":{"long":{"other":"{0}에이커"},"short":{"other":"{0}ac"},"narrow":{"other":"{0}ac"},"perUnit":{}},"percent":{"long":{"other":"{0}%"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"킬로미터당 {0}리터"},"short":{"other":"{0}L/km"},"narrow":{"other":"{0}L/km"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"갤런당 {0}마일"},"short":{"other":"{0}mpg"},"narrow":{"other":"{0}mpg"},"perUnit":{}},"petabyte":{"long":{"other":"{0}페타바이트"},"short":{"other":"{0}PB"},"narrow":{"other":"{0}PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0}테라바이트"},"short":{"other":"{0}TB"},"narrow":{"other":"{0}TB"},"perUnit":{}},"terabit":{"long":{"other":"{0}테라비트"},"short":{"other":"{0}Tb"},"narrow":{"other":"{0}Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0}기가바이트"},"short":{"other":"{0}GB"},"narrow":{"other":"{0}GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0}기가비트"},"short":{"other":"{0}Gb"},"narrow":{"other":"{0}Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0}메가바이트"},"short":{"other":"{0}MB"},"narrow":{"other":"{0}MB"},"perUnit":{}},"megabit":{"long":{"other":"{0}메가비트"},"short":{"other":"{0}Mb"},"narrow":{"other":"{0}Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0}킬로바이트"},"short":{"other":"{0}kB"},"narrow":{"other":"{0}kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0}킬로비트"},"short":{"other":"{0}kb"},"narrow":{"other":"{0}kb"},"perUnit":{}},"byte":{"long":{"other":"{0}바이트"},"short":{"other":"{0}byte"},"narrow":{"other":"{0}byte"},"perUnit":{}},"bit":{"long":{"other":"{0}비트"},"short":{"other":"{0}bit"},"narrow":{"other":"{0}bit"},"perUnit":{}},"year":{"long":{"other":"{0}년"},"short":{"other":"{0}년"},"narrow":{"other":"{0}년"},"perUnit":{"long":"연당 {0}","short":"{0}/년","narrow":"{0}/년"}},"month":{"long":{"other":"{0}개월"},"short":{"other":"{0}개월"},"narrow":{"other":"{0}개월"},"perUnit":{"long":"월당 {0}","short":"{0}/월","narrow":"{0}/월"}},"week":{"long":{"other":"{0}주"},"short":{"other":"{0}주"},"narrow":{"other":"{0}주"},"perUnit":{"long":"주당 {0}","short":"{0}/주","narrow":"{0}/주"}},"day":{"long":{"other":"{0}일"},"short":{"other":"{0}일"},"narrow":{"other":"{0}일"},"perUnit":{"long":"일당 {0}","short":"{0}/일","narrow":"{0}/일"}},"hour":{"long":{"other":"{0}시간"},"short":{"other":"{0}시간"},"narrow":{"other":"{0}시간"},"perUnit":{"long":"시간당 {0}","short":"{0}/h","narrow":"{0}/시간"}},"minute":{"long":{"other":"{0}분"},"short":{"other":"{0}분"},"narrow":{"other":"{0}분"},"perUnit":{"long":"분당 {0}","short":"{0}/min","narrow":"{0}/분"}},"second":{"long":{"other":"{0}초"},"short":{"other":"{0}초"},"narrow":{"other":"{0}초"},"perUnit":{"long":"초당 {0}","short":"{0}/s","narrow":"{0}/초"}},"millisecond":{"long":{"other":"{0}밀리초"},"short":{"other":"{0}ms"},"narrow":{"other":"{0}ms"},"perUnit":{}},"kilometer":{"long":{"other":"{0}킬로미터"},"short":{"other":"{0}km"},"narrow":{"other":"{0}km"},"perUnit":{"long":"킬로미터당 {0}","short":"{0}/km","narrow":"{0}/km"}},"meter":{"long":{"other":"{0}미터"},"short":{"other":"{0}m"},"narrow":{"other":"{0}m"},"perUnit":{"long":"미터당 {0}","short":"{0}/m","narrow":"{0}/m"}},"centimeter":{"long":{"other":"{0}센티미터"},"short":{"other":"{0}cm"},"narrow":{"other":"{0}cm"},"perUnit":{"long":"센티미터당 {0}","short":"{0}/cm","narrow":"{0}/cm"}},"millimeter":{"long":{"other":"{0}밀리미터"},"short":{"other":"{0}mm"},"narrow":{"other":"{0}mm"},"perUnit":{}},"mile":{"long":{"other":"{0}마일"},"short":{"other":"{0}mi"},"narrow":{"other":"{0}mi"},"perUnit":{}},"yard":{"long":{"other":"{0}야드"},"short":{"other":"{0}야드"},"narrow":{"other":"{0}yd"},"perUnit":{}},"foot":{"long":{"other":"{0}피트"},"short":{"other":"{0}ft"},"narrow":{"other":"{0}′"},"perUnit":{"long":"피트당 {0}","short":"{0}/ft","narrow":"{0}/ft"}},"inch":{"long":{"other":"{0}인치"},"short":{"other":"{0}in"},"narrow":{"other":"{0}″"},"perUnit":{"long":"인치당 {0}","short":"{0}/in","narrow":"{0}/in"}},"mile-scandinavian":{"long":{"other":"{0}스칸디나비아 마일"},"short":{"other":"{0}smi"},"narrow":{"other":"{0}smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0}킬로그램"},"short":{"other":"{0}kg"},"narrow":{"other":"{0}kg"},"perUnit":{"long":"킬로그램당 {0}","short":"{0}/kg","narrow":"{0}/kg"}},"gram":{"long":{"other":"{0}그램"},"short":{"other":"{0}g"},"narrow":{"other":"{0}g"},"perUnit":{"long":"그램당 {0}","short":"{0}/g","narrow":"{0}/g"}},"stone":{"long":{"other":"{0}스톤"},"short":{"other":"{0}st"},"narrow":{"other":"{0}st"},"perUnit":{}},"pound":{"long":{"other":"{0}파운드"},"short":{"other":"{0}lb"},"narrow":{"other":"{0}lb"},"perUnit":{"long":"파운드당 {0}","short":"{0}/lb","narrow":"{0}/lb"}},"ounce":{"long":{"other":"{0}온스"},"short":{"other":"{0}oz"},"narrow":{"other":"{0}oz"},"perUnit":{"long":"온스당 {0}","short":"{0}/oz","narrow":"{0}/oz"}},"kilometer-per-hour":{"long":{"other":"시속 {0}킬로미터"},"short":{"other":"{0}km/h"},"narrow":{"other":"{0}km/h"},"perUnit":{}},"meter-per-second":{"long":{"other":"초속 {0}미터"},"short":{"other":"{0}m/s"},"narrow":{"other":"{0}m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"시속 {0}마일"},"short":{"other":"{0}mi/h"},"narrow":{"other":"{0}mph"},"perUnit":{}},"celsius":{"long":{"other":"섭씨 {0}도"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"화씨 {0}도"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0}리터"},"short":{"other":"{0}L"},"narrow":{"other":"{0}ℓ"},"perUnit":{"long":"리터당 {0}","short":"{0}/L","narrow":"{0}/L"}},"milliliter":{"long":{"other":"{0}밀리리터"},"short":{"other":"{0}mL"},"narrow":{"other":"{0}mL"},"perUnit":{}},"gallon":{"long":{"other":"{0}갤런"},"short":{"other":"{0}gal"},"narrow":{"other":"{0}gal"},"perUnit":{"long":"갤런당 {0}","short":"{0}/gal","narrow":"{0}/gal"}},"fluid-ounce":{"long":{"other":"{0}액량 온스"},"short":{"other":"{0}fl oz"},"narrow":{"other":"{0}fl oz"},"perUnit":{}}},"compound":{"per":{"long":"{1}당 {0}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"안도라 페세타"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"아랍에미리트 디르함"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"아프가니 (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"아프가니스탄 아프가니"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"ALK"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"알바니아 레크"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"아르메니아 드람"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"네덜란드령 안틸레스 길더"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"앙골라 콴자"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"앙골라 콴자 (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"앙골라 신콴자 (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"앙골라 재조정 콴자 (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"아르헨티나 오스트랄"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"아르헨티나 페소 레이 (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"아르헨티나 페소 (18810–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"아르헨티나 페소 (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"아르헨티나 페소"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"호주 실링"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"호주 달러"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"아루바 플로린"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"아제르바이젠 마나트(1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"아제르바이잔 마나트"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"보스니아-헤르체고비나 디나르"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"보스니아-헤르체고비나 태환 마르크"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"보스니아-헤르체고비나 신디나르 (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"바베이도스 달러"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"방글라데시 타카"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"벨기에 프랑 (태환)"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"벨기에 프랑"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"벨기에 프랑 (금융)"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"불가리아 동전 렛"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"불가리아 사회주의자 렛"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"불가리아 레프"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"불가리아 렛 (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"바레인 디나르"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"부룬디 프랑"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"버뮤다 달러"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"부루나이 달러"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"볼리비아노"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"볼리비아 볼리비아노 (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"볼리비아노 페소"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"볼리비아노 Mvdol(기금)"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"볼리비아노 크루제이루 노보 (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"브라질 크루자두"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"브라질 크루제이루 (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"브라질 헤알"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"브라질 크루자두 노보"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"브라질 크루제이루"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"브라질 크루제이루 (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"바하마 달러"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"부탄 눌투눔"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"버마 차트"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"보츠와나 폴라"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"벨라루스 신권 루블 (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"벨라루스 루블"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"벨라루스 루블 (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"벨리즈 달러"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"캐나다 달러"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"콩고 프랑 콩골라스"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"유로 (WIR)"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"스위스 프랑"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"프랑 (WIR)"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"칠레 에스쿠도"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"칠레 (UF)"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"칠레 페소"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"중국 위안화(역외)"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"CNX"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"중국 위안화"},"symbol":"CN¥","narrow":"¥"},"COP":{"displayName":{"other":"콜롬비아 페소"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"콜롬비아 실가 단위"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"코스타리카 콜론"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"고 세르비아 디나르"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"체코슬로바키아 동전 코루나"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"쿠바 태환 페소"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"쿠바 페소"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"카보베르데 에스쿠도"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"싸이프러스 파운드"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"체코 공화국 코루나"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"동독 오스트마르크"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"독일 마르크"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"지부티 프랑"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"덴마크 크로네"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"도미니카 페소"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"알제리 디나르"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"에쿠아도르 수크레"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"에콰도르 (UVC)"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"에스토니아 크룬"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"이집트 파운드"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"에리트리아 나크파"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"스페인 페세타(예금)"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"스페인 페세타(변환 예금)"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"스페인 페세타"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"에티오피아 비르"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"유로"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"핀란드 마르카"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"피지 달러"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"포클랜드제도 파운드"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"프랑스 프랑"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"영국령 파운드 스털링"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"그루지야 지폐 라리트"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"조지아 라리"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"가나 시디 (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"가나 시디"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"지브롤터 파운드"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"감비아 달라시"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"기니 프랑"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"기니 시리"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"적도 기니 에쿨 (Ekwele)"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"그리스 드라크마"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"과테말라 케트살"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"포르투갈령 기니 에스쿠도"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"기네비쏘 페소"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"가이아나 달러"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"홍콩 달러"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"온두라스 렘피라"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"크로아티아 디나르"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"크로아티아 쿠나"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"하이티 구르드"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"헝가리 포린트"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"인도네시아 루피아"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"아일랜드 파운드"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"이스라엘 파운드"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"ILR"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"이스라엘 신권 세켈"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"인도 루피"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"이라크 디나르"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"이란 리얄"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"ISJ"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"아이슬란드 크로나"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"이탈리아 리라"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"자메이카 달러"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"요르단 디나르"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"일본 엔화"},"symbol":"JP¥","narrow":"¥"},"KES":{"displayName":{"other":"케냐 실링"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"키르기스스탄 솜"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"캄보디아 리얄"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"코모르 프랑"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"조선 민주주의 인민 공화국 원"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"대한민국 환 (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"KRO"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"대한민국 원"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"쿠웨이트 디나르"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"케이맨 제도 달러"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"카자흐스탄 텐게"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"라오스 키프"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"레바논 파운드"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"스리랑카 루피"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"라이베리아 달러"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"레소토 로티"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"리투아니아 리타"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"룩셈부르크 타로나"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"룩셈부르크 변환 프랑"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"룩셈부르크 프랑"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"룩셈부르크 재정 프랑"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"라트비아 라트"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"라트비아 루블"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"리비아 디나르"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"모로코 디렘"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"모로코 프랑"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"모나코 프랑"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"몰도바 쿠폰"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"몰도바 레이"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"마다가스카르 아리아리"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"마다가스카르 프랑"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"마케도니아 디나르"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"MKN"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"말리 프랑"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"미얀마 키얏"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"몽골 투그릭"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"마카오 파타카"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"모리타니 우기야 (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"모리타니 우기야"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"몰타 리라"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"몰타 파운드"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"모리셔스 루피"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"MVP"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"몰디브 제도 루피아"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"말라위 콰쳐"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"멕시코 페소"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"멕시코 실버 페소 (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"멕시코 (UDI)"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"말레이시아 링깃"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"모잠비크 에스쿠도"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"고 모잠비크 메티칼"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"모잠비크 메티칼"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"나미비아 달러"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"니제르 나이라"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"니카라과 코르도바"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"니카라과 코르도바 오로"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"네델란드 길더"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"노르웨이 크로네"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"네팔 루피"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"뉴질랜드 달러"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"오만 리얄"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"파나마 발보아"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"페루 인티"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"페루 솔"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"페루 솔 (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"파푸아뉴기니 키나"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"필리핀 페소"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"파키스탄 루피"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"폴란드 즐로티"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"폴란드 즐로티 (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"포르투갈 에스쿠도"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"파라과이 과라니"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"카타르 리얄"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"로디지아 달러"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"루마니아 레이"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"루마니아 레우"},"symbol":"RON","narrow":"L"},"RSD":{"displayName":{"other":"세르비아 디나르"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"러시아 루블"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"러시아 루블 (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"르완다 프랑"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"사우디아라비아 리얄"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"솔로몬 제도 달러"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"세이셸 루피"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"수단 디나르"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"수단 파운드"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"고 수단 파운드"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"스웨덴 크로나"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"싱가폴 달러"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"세인트헬레나 파운드"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"슬로베니아 톨라르"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"슬로바키아 코루나"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"시에라리온 리온"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"소말리아 실링"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"수리남 달러"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"수리남 길더"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"남수단 파운드"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"상투메 프린시페 도브라 (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"상투메 프린시페 도브라"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"소련 루블"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"엘살바도르 콜론"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"시리아 파운드"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"스와질란드 릴랑게니"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"태국 바트"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"타지키스탄 루블"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"타지키스탄 소모니"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"투르크메니스탄 마나트 (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"투르크메니스탄 마나트"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"튀니지 디나르"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"통가 파앙가"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"티모르 에스쿠도"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"터키 리라"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"신 터키 리라"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"트리니다드 토바고 달러"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"신 타이완 달러"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"탄자니아 실링"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"우크라이나 그리브나"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"우크라이나 카보바네츠"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"우간다 실링 (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"우간다 실링"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"미국 달러"},"symbol":"US$","narrow":"$"},"USN":{"displayName":{"other":"미국 달러(다음날)"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"미국 달러(당일)"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"우루과이 페소 (UI)"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"우루과이 페소 (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"우루과이 페소 우루과요"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"우즈베키스탄 숨"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"베네주엘라 볼리바르 (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"베네수엘라 볼리바르 (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"베네수엘라 볼리바르"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"베트남 동"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"베트남 동 (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"바누아투 바투"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"서 사모아 탈라"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"중앙아프리카 CFA 프랑"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"은화"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"금"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"유르코 (유럽 회계 단위)"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"유럽 통화 동맹"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"유럽 계산 단위 (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"유럽 계산 단위 (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"동카리브 달러"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"특별인출권"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"유럽 환율 단위"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"프랑스 프랑 (Gold)"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"프랑스 프랑 (UIC)"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"서아프리카 CFA 프랑"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"팔라듐"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"CFP 프랑"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"백금"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET 기금"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"XSU"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"테스트 통화 코드"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"XUA"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"(알 수 없는 통화 단위)"},"symbol":"¤","narrow":"¤"},"YDD":{"displayName":{"other":"예멘 디나르"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"예멘 리알"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"유고슬라비아 동전 디나르"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"유고슬라비아 노비 디나르"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"유고슬라비아 전환 디나르"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"YUR"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"남아프리카 랜드 (금융)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"남아프리카 랜드"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"쟘비아 콰쳐 (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"잠비아 콰쳐"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"자이르 신권 자이르"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"자이르 자이르"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"짐바브웨 달러"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"짐바브웨 달러 (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"짐바브웨 달러 (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0천"},"10000":{"other":"0만"},"100000":{"other":"00만"},"1000000":{"other":"000만"},"10000000":{"other":"0000만"},"100000000":{"other":"0억"},"1000000000":{"other":"00억"},"10000000000":{"other":"000억"},"100000000000":{"other":"0000억"},"1000000000000":{"other":"0조"},"10000000000000":{"other":"00조"},"100000000000000":{"other":"000조"}},"short":{"1000":{"other":"0천"},"10000":{"other":"0만"},"100000":{"other":"00만"},"1000000":{"other":"000만"},"10000000":{"other":"0000만"},"100000000":{"other":"0억"},"1000000000":{"other":"00억"},"10000000000":{"other":"000억"},"100000000000":{"other":"0000억"},"1000000000000":{"other":"0조"},"10000000000000":{"other":"00조"},"100000000000000":{"other":"000조"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0} {1}","short":{"1000":{"other":"¤0천"},"10000":{"other":"¤0만"},"100000":{"other":"¤00만"},"1000000":{"other":"¤000만"},"10000000":{"other":"¤0000만"},"100000000":{"other":"¤0억"},"1000000000":{"other":"¤00억"},"10000000000":{"other":"¤000억"},"100000000000":{"other":"¤0000억"},"1000000000000":{"other":"¤0조"},"10000000000000":{"other":"¤00조"},"100000000000000":{"other":"¤000조"}}}}},"nu":["latn"]}},"availableLocales":["ko"]},
    {"data":{"th":{"units":{"simple":{"degree":{"long":{"other":"{0} องศา"},"short":{"other":"{0}°"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0} เฮกตาร์"},"short":{"other":"{0} เฮกตาร์"},"narrow":{"other":"{0}เฮกตาร์"},"perUnit":{}},"acre":{"long":{"other":"{0} เอเคอร์"},"short":{"other":"{0} เอเคอร์"},"narrow":{"other":"{0}เอเคอร์"},"perUnit":{}},"percent":{"long":{"other":"{0} เปอร์เซ็นต์"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"{0} ลิตรต่อกิโลเมตร"},"short":{"other":"{0} ล./กม."},"narrow":{"other":"{0} ล./กม."},"perUnit":{}},"mile-per-gallon":{"long":{"other":"{0} ไมล์ต่อแกลลอน"},"short":{"other":"{0} ไมล์/แกลลอน"},"narrow":{"other":"{0} ไมล์/แกลลอน"},"perUnit":{}},"petabyte":{"long":{"other":"{0} เพตะไบต์"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0} เทราไบต์"},"short":{"other":"{0} TB"},"narrow":{"other":"{0} TB"},"perUnit":{}},"terabit":{"long":{"other":"{0} เทราบิต"},"short":{"other":"{0} Tb"},"narrow":{"other":"{0} Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} กิกะไบต์"},"short":{"other":"{0} GB"},"narrow":{"other":"{0} GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0} กิกะบิต"},"short":{"other":"{0} Gb"},"narrow":{"other":"{0} Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0} เมกะไบต์"},"short":{"other":"{0} MB"},"narrow":{"other":"{0} MB"},"perUnit":{}},"megabit":{"long":{"other":"{0} เมกะบิต"},"short":{"other":"{0} Mb"},"narrow":{"other":"{0} Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} กิโลไบต์"},"short":{"other":"{0} kB"},"narrow":{"other":"{0} kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0} กิโลบิต"},"short":{"other":"{0} kb"},"narrow":{"other":"{0} kb"},"perUnit":{}},"byte":{"long":{"other":"{0} ไบต์"},"short":{"other":"{0} ไบต์"},"narrow":{"other":"{0} ไบต์"},"perUnit":{}},"bit":{"long":{"other":"{0} บิต"},"short":{"other":"{0} บิต"},"narrow":{"other":"{0} บิต"},"perUnit":{}},"year":{"long":{"other":"{0} ปี"},"short":{"other":"{0} ปี"},"narrow":{"other":"{0}ปี"},"perUnit":{"long":"{0} ต่อปี","short":"{0}/ปี","narrow":"{0}/ปี"}},"month":{"long":{"other":"{0} เดือน"},"short":{"other":"{0} เดือน"},"narrow":{"other":"{0}เดือน"},"perUnit":{"long":"{0} ต่อเดือน","short":"{0}/เดือน","narrow":"{0}/เดือน"}},"week":{"long":{"other":"{0} สัปดาห์"},"short":{"other":"{0} สัปดาห์"},"narrow":{"other":"{0}สัปดาห์"},"perUnit":{"long":"{0} ต่อสัปดาห์","short":"{0}/สัปดาห์","narrow":"{0}/สัปดาห์"}},"day":{"long":{"other":"{0} วัน"},"short":{"other":"{0} วัน"},"narrow":{"other":"{0}วัน"},"perUnit":{"long":"{0} ต่อวัน","short":"{0}/วัน","narrow":"{0}/วัน"}},"hour":{"long":{"other":"{0} ชั่วโมง"},"short":{"other":"{0} ชม."},"narrow":{"other":"{0}ชม."},"perUnit":{"long":"{0} ต่อชั่วโมง","short":"{0}/ชม.","narrow":"{0}/ชม."}},"minute":{"long":{"other":"{0} นาที"},"short":{"other":"{0} นาที"},"narrow":{"other":"{0}นาที"},"perUnit":{"long":"{0} ต่อนาที","short":"{0}/นาที","narrow":"{0}/นาที"}},"second":{"long":{"other":"{0} วินาที"},"short":{"other":"{0} วิ"},"narrow":{"other":"{0}วิ"},"perUnit":{"long":"{0} ต่อวินาที","short":"{0}/วิ","narrow":"{0}/วิ"}},"millisecond":{"long":{"other":"{0} มิลลิวินาที"},"short":{"other":"{0} มิลลิวินาที"},"narrow":{"other":"{0} มิลลิวิ"},"perUnit":{}},"kilometer":{"long":{"other":"{0} กิโลเมตร"},"short":{"other":"{0} กม."},"narrow":{"other":"{0}กม."},"perUnit":{"long":"{0} ต่อกิโลเมตร","short":"{0}/กม.","narrow":"{0}/กม."}},"meter":{"long":{"other":"{0} เมตร"},"short":{"other":"{0} ม."},"narrow":{"other":"{0}ม."},"perUnit":{"long":"{0} ต่อเมตร","short":"{0}/ม.","narrow":"{0}/ม."}},"centimeter":{"long":{"other":"{0} เซนติเมตร"},"short":{"other":"{0} ซม."},"narrow":{"other":"{0}ซม."},"perUnit":{"long":"{0} ต่อเซนติเมตร","short":"{0}/ซม.","narrow":"{0}/ซม."}},"millimeter":{"long":{"other":"{0} มิลลิเมตร"},"short":{"other":"{0} มม."},"narrow":{"other":"{0}มม."},"perUnit":{}},"mile":{"long":{"other":"{0} ไมล์"},"short":{"other":"{0} ไมล์"},"narrow":{"other":"{0}ไมล์"},"perUnit":{}},"yard":{"long":{"other":"{0} หลา"},"short":{"other":"{0} หลา"},"narrow":{"other":"{0}หลา"},"perUnit":{}},"foot":{"long":{"other":"{0} ฟุต"},"short":{"other":"{0} ฟุต"},"narrow":{"other":"{0}′"},"perUnit":{"long":"{0} ต่อฟุต","short":"{0}/ฟุต","narrow":"{0}/ฟุต"}},"inch":{"long":{"other":"{0} นิ้ว"},"short":{"other":"{0} นิ้ว"},"narrow":{"other":"{0}″"},"perUnit":{"long":"{0} ต่อนิ้ว","short":"{0}/นิ้ว","narrow":"{0}/นิ้ว"}},"mile-scandinavian":{"long":{"other":"{0} ไมล์สแกนดิเนเวีย"},"short":{"other":"{0} ไมล์สแกนดินีเวีย"},"narrow":{"other":"{0} ไมล์สแกนดินีเวีย"},"perUnit":{}},"kilogram":{"long":{"other":"{0} กิโลกรัม"},"short":{"other":"{0} กก."},"narrow":{"other":"{0}กก."},"perUnit":{"long":"{0} ต่อกิโลกรัม","short":"{0}/กก.","narrow":"{0}/กก."}},"gram":{"long":{"other":"{0} กรัม"},"short":{"other":"{0} ก."},"narrow":{"other":"{0}ก."},"perUnit":{"long":"{0} ต่อกรัม","short":"{0}/ก.","narrow":"{0}/ก."}},"stone":{"long":{"other":"{0} st"},"short":{"other":"{0} st"},"narrow":{"other":"{0} st"},"perUnit":{}},"pound":{"long":{"other":"{0} ปอนด์"},"short":{"other":"{0} ปอนด์"},"narrow":{"other":"{0}#"},"perUnit":{"long":"{0} ต่อปอนด์","short":"{0}/ปอนด์","narrow":"{0}/ปอนด์"}},"ounce":{"long":{"other":"{0} ออนซ์"},"short":{"other":"{0} ออนซ์"},"narrow":{"other":"{0}ออนซ์"},"perUnit":{"long":"{0} ต่อออนซ์","short":"{0}/ออนซ์","narrow":"{0}/ออนซ์"}},"kilometer-per-hour":{"long":{"other":"{0} กิโลเมตรต่อชั่วโมง"},"short":{"other":"{0} กม./ชม."},"narrow":{"other":"{0}กม./ชม."},"perUnit":{}},"meter-per-second":{"long":{"other":"{0} เมตรต่อวินาที"},"short":{"other":"{0} ม./วิ"},"narrow":{"other":"{0}ม./วิ"},"perUnit":{}},"mile-per-hour":{"long":{"other":"{0} ไมล์ต่อชั่วโมง"},"short":{"other":"{0} ไมล์/ชม."},"narrow":{"other":"{0}ไมล์/ชม."},"perUnit":{}},"celsius":{"long":{"other":"{0} องศาเซลเซียส"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0} องศาฟาเรนไฮต์"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0} ลิตร"},"short":{"other":"{0} ล."},"narrow":{"other":"{0}ล."},"perUnit":{"long":"{0} ต่อลิตร","short":"{0}/ล.","narrow":"{0}/ล."}},"milliliter":{"long":{"other":"{0} มิลลิลิตร"},"short":{"other":"{0} มล."},"narrow":{"other":"{0} มล."},"perUnit":{}},"gallon":{"long":{"other":"{0} แกลลอน"},"short":{"other":"{0} แกลลอน"},"narrow":{"other":"{0} แกลลอน"},"perUnit":{"long":"{0} ต่อแกลลอน","short":"{0}/แกลลอน","narrow":"{0}/แกลลอน"}},"fluid-ounce":{"long":{"other":"{0} ฟลูอิดออนซ์"},"short":{"other":"{0} ฟลูอิดออนซ์"},"narrow":{"other":"{0} ฟลูอิดออนซ์"},"perUnit":{}}},"compound":{"per":{"long":"{0}ต่อ{1}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"เปเซตาอันดอร์รา"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"เดอร์แฮมสหรัฐอาหรับเอมิเรตส์"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"อัฟกานีอัฟกานิสถาน (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"อัฟกานิอัฟกานิสถาน"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"ALK"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"เลกแอลเบเนีย"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"แดรมอาร์เมเนีย"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"กิลเดอร์เนเธอร์แลนด์แอนทิลลิส"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"กวานซาแองโกลา"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"กวานซาแองโกลา (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"นิวกวานซาแองโกลา (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"กวานซารีจัสทาโดแองโกลา (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"ออสตรัลอาร์เจนตินา"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"เปโซเลย์อาร์เจนตินา (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"เปโซอาร์เจนตินา (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"เปโซอาร์เจนตินา (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"เปโซอาร์เจนตินา"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"ชิลลิงออสเตรีย"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"ดอลลาร์ออสเตรเลีย"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"ฟลอรินอารูบา"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"มานัตอาเซอร์ไบจาน (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"มานัตอาเซอร์ไบจาน"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"ดีนาร์บอสเนีย-เฮอร์เซโกวีนา"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"มาร์กบอสเนีย-เฮอร์เซโกวีนา"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"ดีนาร์ใหม่บอสเนีย-เฮอร์เซโกวีนา (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"ดอลลาร์บาร์เบโดส"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"ตากาบังกลาเทศ"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"ฟรังก์เบลเยียม (เปลี่ยนแปลงได้)"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"ฟรังก์เบลเยียม"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"ฟรังก์เบลเยียม (การเงิน)"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"ฮาร์ดเลฟบัลแกเรีย"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"โซเชียลลิสต์เลฟบัลแกเรีย"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"เลฟบัลแกเรีย"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"เลฟบัลเกเรีย (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"ดีนาร์บาห์เรน"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"ฟรังก์บุรุนดี"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"ดอลลาร์เบอร์มิวดา"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"ดอลลาร์บรูไน"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"โบลิเวียโนโบลิเวีย"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"โบลิเวียโนโบลิเวีย (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"เปโซโบลิเวีย"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"มฟดอลโบลิเวีย"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"ครูเซโรโนโวบราซิล (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"ครูซาโดบราซิล"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"ครูเซโรบราซิล (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"เรียลบราซิล"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"ครูซาโดโนโวบราซิล"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"ครูเซโรบราซิล"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"ครูเซโรบราซิล (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"ดอลลาร์บาฮามาส"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"เอ็งกุลตรัมภูฏาน"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"จ๊าดพม่า"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"ปูลาบอตสวานา"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"นิวรูเบิลเบลารุส (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"รูเบิลเบลารุส"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"รูเบิลเบลารุส (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"ดอลลาร์เบลีซ"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"ดอลลาร์แคนาดา"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"ฟรังก์คองโก"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"ยูโรดับเบิลยูไออาร์"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"ฟรังก์สวิส"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"ฟรังก์ดับเบิลยูไออาร์"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"เอสคูโดชิลี"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"ฟูเมนโตชิลี"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"เปโซชิลี"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"หยวน"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"CNX"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"หยวนจีน"},"symbol":"CN¥","narrow":"¥"},"COP":{"displayName":{"other":"เปโซโคลอมเบีย"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"วาเลอร์เรียลโคลอมเบีย"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"โกลองคอสตาริกา"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"ดีนาร์เซอร์เบียเก่า"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"ฮาร์ดโครูนาเช็กโกสโลวัก"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"เปโซคิวบา (แปลงสภาพ)"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"เปโซคิวบา"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"เอสคูโดเคปเวิร์ด"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"ปอนด์ไซปรัส"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"โครูนาสาธารณรัฐเช็ก"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"มาร์กเยอรมันตะวันออก"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"มาร์กเยอรมัน"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"ฟรังก์จิบูตี"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"โครนเดนมาร์ก"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"เปโซโดมินิกัน"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"ดีนาร์แอลจีเรีย"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"ซูเกรเอกวาดอร์"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"วาเลอร์คอนสแตนต์เอกวาดอร์"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"ครูนเอสโตเนีย"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"ปอนด์อียิปต์"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"แนกฟาเอริเทรีย"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"เปเซตาสเปน (บัญชีเอ)"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"เปเซตาสเปน (บัญชีที่เปลี่ยนแปลงได้)"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"เปเซตาสเปน"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"เบอรร์เอธิโอเปีย"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"ยูโร"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"มาร์กกาฟินแลนด์"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"ดอลลาร์ฟิจิ"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"ปอนด์หมู่เกาะฟอล์กแลนด์"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"ฟรังก์ฝรั่งเศส"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"ปอนด์สเตอร์ลิง (สหราชอาณาจักร)"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"คูปอนลาริตจอร์เจีย"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"ลารีจอร์เจีย"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"เซดีกานา (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"เซดีกานา"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"ปอนด์ยิบรอลตาร์"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"ดาลาซีแกมเบีย"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"ฟรังก์กินี"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"ไซลีกินี"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"เอ็กเวเลอิเควทอเรียลกินี"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"ดรัชมากรีก"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"เควตซัลกัวเตมาลา"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"เอสคูโดกินีโปรตุเกส"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"เปโซกินี-บิสเซา"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"ดอลลาร์กายอานา"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"ดอลลาร์ฮ่องกง"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"เลมปิราฮอนดูรัส"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"ดีนาร์โครเอเชีย"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"คูนาโครเอเชีย"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"กูร์ดเฮติ"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"ฟอรินต์ฮังการี"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"รูเปียห์อินโดนีเซีย"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"ปอนด์ไอริช"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"ปอนด์อิสราเอล"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"ILR"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"นิวเชเกลอิสราเอล"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"รูปีอินเดีย"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"ดีนาร์อิรัก"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"เรียลอิหร่าน"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"ISJ"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"โครนาไอซ์แลนด์"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"ลีราอิตาลี"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"ดอลลาร์จาเมกา"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"ดีนาร์จอร์แดน"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"เยนญี่ปุ่น"},"symbol":"¥","narrow":"¥"},"KES":{"displayName":{"other":"ชิลลิงเคนยา"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"ซอมคีร์กีซสถาน"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"เรียลกัมพูชา"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"ฟรังก์คอโมโรส"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"วอนเกาหลีเหนือ"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"ฮวานเกาหลีใต้ (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"วอนเกาหลีใต้ (1945–1953)"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"วอนเกาหลีใต้"},"symbol":"₩","narrow":"₩"},"KWD":{"displayName":{"other":"ดีนาร์คูเวต"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"ดอลลาร์หมู่เกาะเคย์แมน"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"เทงเจคาซัคสถาน"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"กีบลาว"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"ปอนด์เลบานอน"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"รูปีศรีลังกา"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"ดอลลาร์ไลบีเรีย"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"โลตีเลโซโท"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"ลีตัสลิทัวเนีย"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"ทาโลนัสลิทัวเนีย"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"คอนเวอร์ทิเบิลฟรังก์ลักเซมเบิร์ก"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"ฟรังก์ลักเซมเบิร์ก"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"ไฟแนลเชียลฟรังก์ลักเซมเบิร์ก"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"ลัตส์ลัตเวีย"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"รูเบิลลัตเวีย"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"ดีนาร์ลิเบีย"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"ดีแรห์มโมร็อกโก"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"ฟรังก์โมร็อกโก"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"ฟรังก์โมนาโก"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"บัตรปันส่วนมอลโดวา"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"ลิวมอลโดวา"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"อาเรียรีมาลากาซี"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"ฟรังก์มาดากัสการ์"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"ดีนาร์มาซิโดเนีย"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"ดีนาร์มาซิโดเนีย (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"ฟรังก์มาลี"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"จ๊าตพม่า"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"ทูกริกมองโกเลีย"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"ปาตากามาเก๊า"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"อูกียามอริเตเนีย (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"อูกียามอริเตเนีย"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"ลีรามอลตา"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"ปอนด์มอลตา"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"รูปีมอริเชียส"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"MVP"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"รูฟิยามัลดีฟส์"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"ควาชามาลาวี"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"เปโซเม็กซิโก"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"เงินเปโซเม็กซิโก (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"ยูนิแดด ดี อินเวอร์ชั่น เม็กซิโก"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"ริงกิตมาเลเซีย"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"เอสคูโดโมซัมบิก"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"เมติคัลโมซัมบิกเก่า"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"เมติคัลโมซัมบิก"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"ดอลลาร์นามิเบีย"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"ไนราไนจีเรีย"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"คอร์โดบานิการากัว"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"กอร์โดบานิการากัว"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"กิลเดอร์เนเธอร์แลนด์"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"โครนนอร์เวย์"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"รูปีเนปาล"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"ดอลลาร์นิวซีแลนด์"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"เรียลโอมาน"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"บัลบัวปานามา"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"อินตีเปรู"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"ซอลเปรู"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"ซอลเปรู(1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"กีนาปาปัวนิวกินี"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"เปโซฟิลิปปินส์"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"รูปีปากีสถาน"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"ซลอตีโปแลนด์"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"ซลอตีโปแลนด์ (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"เอสคูโดโปรตุเกส"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"กวารานีปารากวัย"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"เรียลกาตาร์"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"ดอลลาร์โรดีเซีย"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"ลิวโรมาเนียเก่า"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"ลิวโรมาเนีย"},"symbol":"RON","narrow":"lei"},"RSD":{"displayName":{"other":"ดีนาร์เซอร์เบีย"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"รูเบิลรัสเซีย"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"รูเบิลรัสเซีย (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"ฟรังก์รวันดา"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"ริยัลซาอุดีอาระเบีย"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"ดอลลาร์หมู่เกาะโซโลมอน"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"รูปีเซเชลส์"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"ดีนาร์ซูดานเก่า"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"ปอนด์ซูดาน"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"ปอนด์ซูดานเก่า"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"โครนาสวีเดน"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"ดอลลาร์สิงคโปร์"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"ปอนด์เซนต์เฮเลนา"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"ทอลาร์สโลวีเนีย"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"โครูนาสโลวัก"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"ลีโอนเซียร์ราลีโอน"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"ชิลลิงโซมาเลีย"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"ดอลลาร์ซูรินาเม"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"กิลเดอร์ซูรินาเม"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"ปอนด์ซูดานใต้"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"ดอบราเซาตูเมและปรินซิปี (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"ดอบราเซาตูเมและปรินซิปี"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"รูเบิลโซเวียต"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"โคลอนเอลซัลวาดอร์"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"ปอนด์ซีเรีย"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"ลิลันเจนีสวาซิ"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"บาทไทย"},"symbol":"฿","narrow":"฿"},"TJR":{"displayName":{"other":"รูเบิลทาจิกิสถาน"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"โซโมนิทาจิกิสถาน"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"มานัตเติร์กเมนิสถาน (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"มานัตเติร์กเมนิสถาน"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"ดีนาร์ตูนิเซีย"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"พาแองกาตองกา"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"เอสคูโดติมอร์"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"ลีราตุรกีเก่า"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"ลีราตุรกี"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"ดอลลาร์ตรินิแดดและโตเบโก"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"ดอลลาร์ไต้หวันใหม่"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"ชิลลิงแทนซาเนีย"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"ฮรีฟเนียยูเครน"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"คาร์โบวาเนตซ์ยูเครน"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"ชิลลิงยูกันดา (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"ชิลลิงยูกันดา"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"ดอลลาร์สหรัฐ"},"symbol":"US$","narrow":"$"},"USN":{"displayName":{"other":"ดอลลาร์สหรัฐ (วันถัดไป)"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"ดอลลาร์สหรัฐ (วันเดียวกัน)"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"เปโซเอนยูนิแดดเซสอินเด็กซาแดสอุรุกวัย"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"เปโซอุรุกวัย (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"เปโซอุรุกวัย"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"ซอมอุซเบกิสถาน"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"โบลิวาร์เวเนซุเอลา (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"โบลิวาร์เวเนซุเอลา"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"โบลีวาร์แห่งเวเนซุเอลา"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"ดองเวียดนาม"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"ดองเวียดนาม (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"วาตูวานูอาตู"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"ทาลาซามัว"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"ฟรังก์เซฟาธนาคารรัฐแอฟริกากลาง"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"เงิน"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"ทอง"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"หน่วยคอมโพสิตยุโรป"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"หน่วยโมเนทารียุโรป"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"หน่วยบัญชียุโรป [XBC]"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"หน่วยบัญชียุโรป [XBD]"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"ดอลลาร์แคริบเบียนตะวันออก"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"สิทธิถอนเงินพิเศษ"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"หน่วยสกุลเงินยุโรป"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"ฟรังก์ทองฝรั่งเศส"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"ฟรังก์ยูไอซีฝรั่งเศส"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"ฟรังก์เซฟาธนาคารกลางรัฐแอฟริกาตะวันตก"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"พัลเลเดียม"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"ฟรังก์ซีเอฟพี"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"แพลตินัม"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"กองทุนไรเน็ต"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"ซูเกร"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"รหัสทดสอบสกุลเงิน"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"หน่วยบัญชี เอดีบี"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"(ไม่ทราบชื่อสกุลเงิน)"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"ดีนาร์เยเมน"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"เรียลเยเมน"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"ฮาร์ดดีนาร์ยูโกสลาเวีย"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"โนวิย์ดีนาร์ยูโกสลาเวีย"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"คอนเวอร์ทิเบิลดีนาร์ยูโกสลาเวีย"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"ดีนาร์ปฏิรูปยูโกสลาเวีย (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"แรนด์แอฟริกาใต้ (การเงิน)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"แรนด์แอฟริกาใต้"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"ควาชาแซมเบีย (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"ควาชาแซมเบีย"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"นิวแซร์คองโก"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"แซร์คองโก"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"ดอลลาร์ซิมบับเว"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"ดอลลาร์ซิมบับเว (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"ดอลลาร์ซิมบับเว (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0 พัน"},"10000":{"other":"0 หมื่น"},"100000":{"other":"0 แสน"},"1000000":{"other":"0 ล้าน"},"10000000":{"other":"00 ล้าน"},"100000000":{"other":"000 ล้าน"},"1000000000":{"other":"0 พันล้าน"},"10000000000":{"other":"0 หมื่นล้าน"},"100000000000":{"other":"0 แสนล้าน"},"1000000000000":{"other":"0 ล้านล้าน"},"10000000000000":{"other":"00 ล้านล้าน"},"100000000000000":{"other":"000 ล้านล้าน"}},"short":{"1000":{"other":"0K"},"10000":{"other":"00K"},"100000":{"other":"000K"},"1000000":{"other":"0M"},"10000000":{"other":"00M"},"100000000":{"other":"000M"},"1000000000":{"other":"0B"},"10000000000":{"other":"00B"},"100000000000":{"other":"000B"},"1000000000000":{"other":"0T"},"10000000000000":{"other":"00T"},"100000000000000":{"other":"000T"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0} {1}","short":{"1000":{"other":"¤0K"},"10000":{"other":"¤00K"},"100000":{"other":"¤000K"},"1000000":{"other":"¤0M"},"10000000":{"other":"¤00M"},"100000000":{"other":"¤000M"},"1000000000":{"other":"¤0B"},"10000000000":{"other":"¤00B"},"100000000000":{"other":"¤000B"},"1000000000000":{"other":"¤0T"},"10000000000000":{"other":"¤00T"},"100000000000000":{"other":"¤000T"}}}}},"nu":["latn"]}},"availableLocales":["th"]},
    {"data":{"zh":{"units":{"simple":{"degree":{"long":{"other":"{0}度"},"short":{"other":"{0}°"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0}公顷"},"short":{"other":"{0}公顷"},"narrow":{"other":"{0}ha"},"perUnit":{}},"acre":{"long":{"other":"{0}英亩"},"short":{"other":"{0}英亩"},"narrow":{"other":"{0}ac"},"perUnit":{}},"percent":{"long":{"other":"{0}%"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"每公里{0}升"},"short":{"other":"{0}升/公里"},"narrow":{"other":"{0}升/公里"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"每加仑{0}英里"},"short":{"other":"{0}英里/加仑"},"narrow":{"other":"{0}英里/加仑"},"perUnit":{}},"petabyte":{"long":{"other":"{0}拍字节"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0}太字节"},"short":{"other":"{0}太字节"},"narrow":{"other":"{0}太字节"},"perUnit":{}},"terabit":{"long":{"other":"{0}太比特"},"short":{"other":"{0}太比特"},"narrow":{"other":"{0}太比特"},"perUnit":{}},"gigabyte":{"long":{"other":"{0}吉字节"},"short":{"other":"{0}吉字节"},"narrow":{"other":"{0}吉字节"},"perUnit":{}},"gigabit":{"long":{"other":"{0}吉比特"},"short":{"other":"{0}吉比特"},"narrow":{"other":"{0}吉比特"},"perUnit":{}},"megabyte":{"long":{"other":"{0}兆字节"},"short":{"other":"{0}兆字节"},"narrow":{"other":"{0}兆字节"},"perUnit":{}},"megabit":{"long":{"other":"{0}兆比特"},"short":{"other":"{0}兆比特"},"narrow":{"other":"{0}兆比特"},"perUnit":{}},"kilobyte":{"long":{"other":"{0}千字节"},"short":{"other":"{0}千字节"},"narrow":{"other":"{0}千字节"},"perUnit":{}},"kilobit":{"long":{"other":"{0}千比特"},"short":{"other":"{0}千比特"},"narrow":{"other":"{0}千比特"},"perUnit":{}},"byte":{"long":{"other":"{0}字节"},"short":{"other":"{0}字节"},"narrow":{"other":"{0}字节"},"perUnit":{}},"bit":{"long":{"other":"{0}比特"},"short":{"other":"{0}比特"},"narrow":{"other":"{0}比特"},"perUnit":{}},"year":{"long":{"other":"{0}年"},"short":{"other":"{0}年"},"narrow":{"other":"{0}年"},"perUnit":{"long":"每年{0}","short":"{0}/年","narrow":"{0}/年"}},"month":{"long":{"other":"{0}个月"},"short":{"other":"{0}个月"},"narrow":{"other":"{0}个月"},"perUnit":{"long":"每月{0}","short":"{0}/月","narrow":"{0}/月"}},"week":{"long":{"other":"{0}周"},"short":{"other":"{0}周"},"narrow":{"other":"{0}周"},"perUnit":{"long":"每周{0}","short":"{0}/周","narrow":"{0}/周"}},"day":{"long":{"other":"{0}天"},"short":{"other":"{0}天"},"narrow":{"other":"{0}天"},"perUnit":{"long":"每天{0}","short":"{0}/天","narrow":"{0}/天"}},"hour":{"long":{"other":"{0}小时"},"short":{"other":"{0}小时"},"narrow":{"other":"{0}小时"},"perUnit":{"long":"每小时{0}","short":"{0}/小时","narrow":"{0}/小时"}},"minute":{"long":{"other":"{0}分钟"},"short":{"other":"{0}分钟"},"narrow":{"other":"{0}分钟"},"perUnit":{"long":"每分钟{0}","short":"{0}/分钟","narrow":"{0}/分钟"}},"second":{"long":{"other":"{0}秒钟"},"short":{"other":"{0}秒"},"narrow":{"other":"{0}秒"},"perUnit":{"long":"{0}/秒","short":"{0}/秒","narrow":"{0}/秒"}},"millisecond":{"long":{"other":"{0}毫秒"},"short":{"other":"{0}毫秒"},"narrow":{"other":"{0}毫秒"},"perUnit":{}},"kilometer":{"long":{"other":"{0}公里"},"short":{"other":"{0}公里"},"narrow":{"other":"{0}公里"},"perUnit":{"long":"每公里{0}","short":"{0}/公里","narrow":"{0}/公里"}},"meter":{"long":{"other":"{0}米"},"short":{"other":"{0}米"},"narrow":{"other":"{0}米"},"perUnit":{"long":"每米{0}","short":"{0}/米","narrow":"{0}/米"}},"centimeter":{"long":{"other":"{0}厘米"},"short":{"other":"{0}厘米"},"narrow":{"other":"{0}厘米"},"perUnit":{"long":"每厘米{0}","short":"{0}/厘米","narrow":"{0}/厘米"}},"millimeter":{"long":{"other":"{0}毫米"},"short":{"other":"{0}毫米"},"narrow":{"other":"{0}毫米"},"perUnit":{}},"mile":{"long":{"other":"{0}英里"},"short":{"other":"{0}英里"},"narrow":{"other":"{0}mi"},"perUnit":{}},"yard":{"long":{"other":"{0}码"},"short":{"other":"{0}码"},"narrow":{"other":"{0}yd"},"perUnit":{}},"foot":{"long":{"other":"{0}英尺"},"short":{"other":"{0}英尺"},"narrow":{"other":"{0}′"},"perUnit":{"long":"每英尺{0}","short":"{0}/英尺","narrow":"{0}/英尺"}},"inch":{"long":{"other":"{0}英寸"},"short":{"other":"{0}英寸"},"narrow":{"other":"{0}″"},"perUnit":{"long":"每英寸{0}","short":"{0}/英寸","narrow":"{0}/英寸"}},"mile-scandinavian":{"long":{"other":"{0}斯堪的纳维亚英里"},"short":{"other":"{0}斯堪的纳维亚英里"},"narrow":{"other":"{0}smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0}千克"},"short":{"other":"{0}千克"},"narrow":{"other":"{0}千克"},"perUnit":{"long":"每千克{0}","short":"{0}/千克","narrow":"{0}/千克"}},"gram":{"long":{"other":"{0}克"},"short":{"other":"{0}克"},"narrow":{"other":"{0}克"},"perUnit":{"long":"每克{0}","short":"{0}/克","narrow":"{0}/克"}},"stone":{"long":{"other":"{0}英石"},"short":{"other":"{0}英石"},"narrow":{"other":"{0}英石"},"perUnit":{}},"pound":{"long":{"other":"{0}磅"},"short":{"other":"{0}磅"},"narrow":{"other":"{0}磅"},"perUnit":{"long":"每磅{0}","short":"{0}/磅","narrow":"{0}/磅"}},"ounce":{"long":{"other":"{0}盎司"},"short":{"other":"{0}盎司"},"narrow":{"other":"{0}盎司"},"perUnit":{"long":"每盎司{0}","short":"{0}/盎司","narrow":"{0}/盎司"}},"kilometer-per-hour":{"long":{"other":"每小时{0}公里"},"short":{"other":"每小时{0}公里"},"narrow":{"other":"{0}公里/小时"},"perUnit":{}},"meter-per-second":{"long":{"other":"每秒{0}米"},"short":{"other":"{0}米/秒"},"narrow":{"other":"{0}m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"每小时{0}英里"},"short":{"other":"{0}英里/小时"},"narrow":{"other":"{0}mi/h"},"perUnit":{}},"celsius":{"long":{"other":"{0}摄氏度"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0}华氏度"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0}升"},"short":{"other":"{0}升"},"narrow":{"other":"{0}升"},"perUnit":{"long":"每升{0}","short":"{0}/升","narrow":"{0}/升"}},"milliliter":{"long":{"other":"{0}毫升"},"short":{"other":"{0}毫升"},"narrow":{"other":"{0}毫升"},"perUnit":{}},"gallon":{"long":{"other":"{0}加仑"},"short":{"other":"{0}加仑"},"narrow":{"other":"{0}加仑"},"perUnit":{"long":"每加仑{0}","short":"{0}/加仑","narrow":"{0}/加仑"}},"fluid-ounce":{"long":{"other":"{0}液盎司"},"short":{"other":"{0}液盎司"},"narrow":{"other":"{0}液盎司"},"perUnit":{}}},"compound":{"per":{"long":"每{1}{0}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"安道尔比塞塔"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"阿联酋迪拉姆"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"阿富汗尼 (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"阿富汗尼"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"阿尔巴尼亚列克(1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"阿尔巴尼亚列克"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"亚美尼亚德拉姆"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"荷属安的列斯盾"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"安哥拉宽扎"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"安哥拉宽扎 (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"安哥拉新宽扎 (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"安哥拉重新调整宽扎 (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"阿根廷奥斯特拉尔"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"阿根廷法定比索 (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"阿根廷比索 (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"阿根廷比索 (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"阿根廷比索"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"奥地利先令"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"澳大利亚元"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"阿鲁巴弗罗林"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"阿塞拜疆马纳特 (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"阿塞拜疆马纳特"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"波士尼亚-赫塞哥维纳第纳尔 (1992–1994)"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"波斯尼亚-黑塞哥维那可兑换马克"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"波士尼亚-赫塞哥维纳新第纳尔 (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"巴巴多斯元"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"孟加拉塔卡"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"比利时法郎（可兑换）"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"比利时法郎"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"比利时法郎（金融）"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"保加利亚硬列弗"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"保加利亚社会党列弗"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"保加利亚新列弗"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"保加利亚列弗 (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"巴林第纳尔"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"布隆迪法郎"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"百慕大元"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"文莱元"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"玻利维亚诺"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"玻利维亚诺 (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"玻利维亚比索"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"玻利维亚 Mvdol（资金）"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"巴西新克鲁赛罗 (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"巴西克鲁扎多 (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"巴西克鲁塞罗 (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"巴西雷亚尔"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"巴西新克鲁扎多 (1989–1990)"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"巴西克鲁塞罗 (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"巴西克鲁塞罗 (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"巴哈马元"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"不丹努尔特鲁姆"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"缅元"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"博茨瓦纳普拉"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"白俄罗斯新卢布 (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"白俄罗斯卢布"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"白俄罗斯卢布 (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"伯利兹元"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"加拿大元"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"刚果法郎"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"欧元 (WIR)"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"瑞士法郎"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"法郎 (WIR)"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"智利埃斯库多"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"智利（资金）"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"智利比索"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"人民币（离岸）"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"中国人民银行元"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"人民币"},"symbol":"¥","narrow":"¥"},"COP":{"displayName":{"other":"哥伦比亚比索"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"哥伦比亚币"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"哥斯达黎加科朗"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"旧塞尔维亚第纳尔"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"捷克硬克朗"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"古巴可兑换比索"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"古巴比索"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"佛得角埃斯库多"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"塞浦路斯镑"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"捷克克朗"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"东德奥斯特马克"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"德国马克"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"吉布提法郎"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"丹麦克朗"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"多米尼加比索"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"阿尔及利亚第纳尔"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"厄瓜多尔苏克雷"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"厄瓜多尔 (UVC)"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"爱沙尼亚克朗"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"埃及镑"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"厄立特里亚纳克法"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"西班牙比塞塔（帐户 A）"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"西班牙比塞塔（兑换帐户）"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"西班牙比塞塔"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"埃塞俄比亚比尔"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"欧元"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"芬兰马克"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"斐济元"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"福克兰群岛镑"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"法国法郎"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"英镑"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"乔治亚库蓬拉瑞特"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"格鲁吉亚拉里"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"加纳塞第"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"加纳塞地"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"直布罗陀镑"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"冈比亚达拉西"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"几内亚法郎"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"几内亚西里"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"赤道几内亚埃奎勒"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"希腊德拉克马"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"危地马拉格查尔"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"葡萄牙几内亚埃斯库多"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"几内亚比绍比索"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"圭亚那元"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"港元"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"洪都拉斯伦皮拉"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"克罗地亚第纳尔"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"克罗地亚库纳"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"海地古德"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"匈牙利福林"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"印度尼西亚盾"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"爱尔兰镑"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"以色列镑"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"以色列谢克尔(1980–1985)"},"symbol":"ILS","narrow":"ILS"},"ILS":{"displayName":{"other":"以色列新谢克尔"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"印度卢比"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"伊拉克第纳尔"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"伊朗里亚尔"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"冰岛克朗(1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"冰岛克朗"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"意大利里拉"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"牙买加元"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"约旦第纳尔"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"日元"},"symbol":"JP¥","narrow":"¥"},"KES":{"displayName":{"other":"肯尼亚先令"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"吉尔吉斯斯坦索姆"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"柬埔寨瑞尔"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"科摩罗法郎"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"朝鲜元"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"韩元 (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"韩元 (1945–1953)"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"韩元"},"symbol":"￦","narrow":"₩"},"KWD":{"displayName":{"other":"科威特第纳尔"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"开曼元"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"哈萨克斯坦坚戈"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"老挝基普"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"黎巴嫩镑"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"斯里兰卡卢比"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"利比里亚元"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"莱索托洛蒂"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"立陶宛立特"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"立陶宛塔咯呐司"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"卢森堡可兑换法郎"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"卢森堡法郎"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"卢森堡金融法郎"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"拉脱维亚拉特"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"拉脱维亚卢布"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"利比亚第纳尔"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"摩洛哥迪拉姆"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"摩洛哥法郎"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"摩纳哥法郎"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"摩尔多瓦库邦"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"摩尔多瓦列伊"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"马达加斯加阿里亚里"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"马达加斯加法郎"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"马其顿第纳尔"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"马其顿第纳尔 (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"马里法郎"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"缅甸元"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"蒙古图格里克"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"澳门元"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"毛里塔尼亚乌吉亚 (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"毛里塔尼亚乌吉亚"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"马耳他里拉"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"马耳他镑"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"毛里求斯卢比"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"马尔代夫卢比(1947–1981)"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"马尔代夫卢菲亚"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"马拉维克瓦查"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"墨西哥比索"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"墨西哥银比索 (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"墨西哥（资金）"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"马来西亚林吉特"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"莫桑比克埃斯库多"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"旧莫桑比克美提卡"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"莫桑比克美提卡"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"纳米比亚元"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"尼日利亚奈拉"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"尼加拉瓜科多巴 (1988–1991)"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"尼加拉瓜金科多巴"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"荷兰盾"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"挪威克朗"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"尼泊尔卢比"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"新西兰元"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"阿曼里亚尔"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"巴拿马巴波亚"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"秘鲁印第"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"秘鲁索尔"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"秘鲁索尔 (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"巴布亚新几内亚基那"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"菲律宾比索"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"巴基斯坦卢比"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"波兰兹罗提"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"波兰兹罗提 (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"葡萄牙埃斯库多"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"巴拉圭瓜拉尼"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"卡塔尔里亚尔"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"罗得西亚元"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"旧罗马尼亚列伊"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"罗马尼亚列伊"},"symbol":"RON","narrow":"lei"},"RSD":{"displayName":{"other":"塞尔维亚第纳尔"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"俄罗斯卢布"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"俄国卢布 (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"卢旺达法郎"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"沙特里亚尔"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"所罗门群岛元"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"塞舌尔卢比"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"苏丹第纳尔 (1992–2007)"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"苏丹镑"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"旧苏丹镑"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"瑞典克朗"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"新加坡元"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"圣赫勒拿群岛磅"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"斯洛文尼亚托拉尔"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"斯洛伐克克朗"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"塞拉利昂利昂"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"索马里先令"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"苏里南元"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"苏里南盾"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"南苏丹镑"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"圣多美和普林西比多布拉 (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"圣多美和普林西比多布拉"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"苏联卢布"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"萨尔瓦多科朗"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"叙利亚镑"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"斯威士兰里兰吉尼"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"泰铢"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"塔吉克斯坦卢布"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"塔吉克斯坦索莫尼"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"土库曼斯坦马纳特 (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"土库曼斯坦马纳特"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"突尼斯第纳尔"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"汤加潘加"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"帝汶埃斯库多"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"土耳其里拉 (1922–2005)"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"土耳其里拉"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"特立尼达和多巴哥元"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"新台币"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"坦桑尼亚先令"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"乌克兰格里夫纳"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"乌克兰币"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"乌干达先令 (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"乌干达先令"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"美元"},"symbol":"US$","narrow":"$"},"USN":{"displayName":{"other":"美元（次日）"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"美元（当日）"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"乌拉圭比索（索引单位）"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"乌拉圭比索 (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"乌拉圭比索"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"乌拉圭票面工资指数单位"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"乌兹别克斯坦苏姆"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"委内瑞拉玻利瓦尔 (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"委内瑞拉玻利瓦尔 (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"委内瑞拉玻利瓦尔"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"越南盾"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"越南盾 (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"瓦努阿图瓦图"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"萨摩亚塔拉"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"中非法郎"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"银"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"黄金"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"欧洲复合单位"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"欧洲货币联盟"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"欧洲计算单位 (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"欧洲计算单位 (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"东加勒比元"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"特别提款权"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"欧洲货币单位"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"法国金法郎"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"法国法郎 (UIC)"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"西非法郎"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"钯"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"太平洋法郎"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"铂"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET 基金"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"苏克雷"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"测试货币代码"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"非洲开发银行记账单位"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"（未知货币）"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"也门第纳尔"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"也门里亚尔"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"南斯拉夫硬第纳尔 (1966–1990)"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"南斯拉夫新第纳尔 (1994–2002)"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"南斯拉夫可兑换第纳尔 (1990–1992)"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"南斯拉夫改良第纳尔 (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"南非兰特 (金融)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"南非兰特"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"赞比亚克瓦查 (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"赞比亚克瓦查"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"新扎伊尔 (1993–1998)"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"扎伊尔 (1971–1993)"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"津巴布韦元 (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"津巴布韦元 (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"津巴布韦元 (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0亿"},"1000000000":{"other":"00亿"},"10000000000":{"other":"000亿"},"100000000000":{"other":"0000亿"},"1000000000000":{"other":"0万亿"},"10000000000000":{"other":"00万亿"},"100000000000000":{"other":"000万亿"}},"short":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0亿"},"1000000000":{"other":"00亿"},"10000000000":{"other":"000亿"},"100000000000":{"other":"0000亿"},"1000000000000":{"other":"0万亿"},"10000000000000":{"other":"00万亿"},"100000000000000":{"other":"000万亿"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0}{1}","short":{"1000":{"other":"0"},"10000":{"other":"¤0万"},"100000":{"other":"¤00万"},"1000000":{"other":"¤000万"},"10000000":{"other":"¤0000万"},"100000000":{"other":"¤0亿"},"1000000000":{"other":"¤00亿"},"10000000000":{"other":"¤000亿"},"100000000000":{"other":"¤0000亿"},"1000000000000":{"other":"¤0万亿"},"10000000000000":{"other":"¤00万亿"},"100000000000000":{"other":"¤000万亿"}}}}},"nu":["latn"]}},"availableLocales":["zh"]},
    {"data":{"zh-Hant":{"units":{"simple":{"degree":{"long":{"other":"{0} 度"},"short":{"other":"{0} 度"},"narrow":{"other":"{0}度"},"perUnit":{}},"hectare":{"long":{"other":"{0} 公頃"},"short":{"other":"{0} 公頃"},"narrow":{"other":"{0}公頃"},"perUnit":{}},"acre":{"long":{"other":"{0} 英畝"},"short":{"other":"{0} 英畝"},"narrow":{"other":"{0}英畝"},"perUnit":{}},"percent":{"long":{"other":"{0}%"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"每公里 {0} 公升"},"short":{"other":"{0} 升/公里"},"narrow":{"other":"{0}L/km"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"每加侖 {0} 英里"},"short":{"other":"{0} 英里/加侖"},"narrow":{"other":"{0}mpg"},"perUnit":{}},"petabyte":{"long":{"other":"{0} PB"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0} TB"},"short":{"other":"{0} TB"},"narrow":{"other":"{0}TB"},"perUnit":{}},"terabit":{"long":{"other":"{0} Tb"},"short":{"other":"{0} Tb"},"narrow":{"other":"{0}Tb"},"perUnit":{}},"gigabyte":{"long":{"other":"{0} GB"},"short":{"other":"{0} GB"},"narrow":{"other":"{0}GB"},"perUnit":{}},"gigabit":{"long":{"other":"{0} Gb"},"short":{"other":"{0} Gb"},"narrow":{"other":"{0}Gb"},"perUnit":{}},"megabyte":{"long":{"other":"{0} MB"},"short":{"other":"{0} MB"},"narrow":{"other":"{0}MB"},"perUnit":{}},"megabit":{"long":{"other":"{0} Mb"},"short":{"other":"{0} Mb"},"narrow":{"other":"{0}Mb"},"perUnit":{}},"kilobyte":{"long":{"other":"{0} kB"},"short":{"other":"{0} kB"},"narrow":{"other":"{0}kB"},"perUnit":{}},"kilobit":{"long":{"other":"{0} kb"},"short":{"other":"{0} kb"},"narrow":{"other":"{0}kb"},"perUnit":{}},"byte":{"long":{"other":"{0} byte"},"short":{"other":"{0} byte"},"narrow":{"other":"{0}byte"},"perUnit":{}},"bit":{"long":{"other":"{0} bit"},"short":{"other":"{0} bit"},"narrow":{"other":"{0}bit"},"perUnit":{}},"year":{"long":{"other":"{0} 年"},"short":{"other":"{0} 年"},"narrow":{"other":"{0} 年"},"perUnit":{"long":"每年 {0}","short":"{0}/年","narrow":"{0}/年"}},"month":{"long":{"other":"{0} 個月"},"short":{"other":"{0} 個月"},"narrow":{"other":"{0} 個月"},"perUnit":{"long":"每月 {0}","short":"{0}/月","narrow":"{0}/月"}},"week":{"long":{"other":"{0} 週"},"short":{"other":"{0} 週"},"narrow":{"other":"{0} 週"},"perUnit":{"long":"每週 {0}","short":"{0}/週","narrow":"{0}/週"}},"day":{"long":{"other":"{0} 天"},"short":{"other":"{0} 天"},"narrow":{"other":"{0} 天"},"perUnit":{"long":"每天 {0}","short":"{0}/天","narrow":"{0}/天"}},"hour":{"long":{"other":"{0} 小時"},"short":{"other":"{0} 小時"},"narrow":{"other":"{0} 小時"},"perUnit":{"long":"每小時 {0}","short":"{0}/小時","narrow":"{0}/小時"}},"minute":{"long":{"other":"{0} 分鐘"},"short":{"other":"{0} 分鐘"},"narrow":{"other":"{0} 分鐘"},"perUnit":{"long":"每分鐘 {0}","short":"{0}/分鐘","narrow":"{0}/分鐘"}},"second":{"long":{"other":"{0} 秒"},"short":{"other":"{0} 秒"},"narrow":{"other":"{0} 秒"},"perUnit":{"long":"每秒 {0}","short":"{0}/秒","narrow":"{0}/秒"}},"millisecond":{"long":{"other":"{0} 毫秒"},"short":{"other":"{0} 毫秒"},"narrow":{"other":"{0} 毫秒"},"perUnit":{}},"kilometer":{"long":{"other":"{0} 公里"},"short":{"other":"{0} 公里"},"narrow":{"other":"{0}公里"},"perUnit":{"long":"每公里 {0}","short":"{0}/公里","narrow":"{0}/公里"}},"meter":{"long":{"other":"{0} 公尺"},"short":{"other":"{0} 公尺"},"narrow":{"other":"{0}公尺"},"perUnit":{"long":"每公尺 {0}","short":"{0}/公尺","narrow":"{0}/公尺"}},"centimeter":{"long":{"other":"{0} 公分"},"short":{"other":"{0} 公分"},"narrow":{"other":"{0}公分"},"perUnit":{"long":"每公分 {0}","short":"{0}/公分","narrow":"{0}/公分"}},"millimeter":{"long":{"other":"{0} 公釐"},"short":{"other":"{0} 公釐"},"narrow":{"other":"{0}公釐"},"perUnit":{}},"mile":{"long":{"other":"{0} 英里"},"short":{"other":"{0} 英里"},"narrow":{"other":"{0}英里"},"perUnit":{}},"yard":{"long":{"other":"{0} 碼"},"short":{"other":"{0} 碼"},"narrow":{"other":"{0}碼"},"perUnit":{}},"foot":{"long":{"other":"{0} 英尺"},"short":{"other":"{0} 呎"},"narrow":{"other":"{0}呎"},"perUnit":{"long":"每英尺 {0}","short":"{0}/呎","narrow":"{0}/呎"}},"inch":{"long":{"other":"{0} 英寸"},"short":{"other":"{0} 吋"},"narrow":{"other":"{0}吋"},"perUnit":{"long":"每英寸 {0}","short":"{0}/吋","narrow":"{0}/吋"}},"mile-scandinavian":{"long":{"other":"{0} 斯堪地那維亞英里"},"short":{"other":"{0} 斯堪地那維亞英里"},"narrow":{"other":"{0} 斯堪地那維亞英里"},"perUnit":{}},"kilogram":{"long":{"other":"{0} 公斤"},"short":{"other":"{0} 公斤"},"narrow":{"other":"{0} 公斤"},"perUnit":{"long":"每公斤 {0}","short":"{0}/公斤","narrow":"{0}/公斤"}},"gram":{"long":{"other":"{0} 克"},"short":{"other":"{0} 克"},"narrow":{"other":"{0}克"},"perUnit":{"long":"每克 {0}","short":"{0}/克","narrow":"{0}/克"}},"stone":{"long":{"other":"{0} 英石"},"short":{"other":"{0} 英石"},"narrow":{"other":"{0}st"},"perUnit":{}},"pound":{"long":{"other":"{0} 磅"},"short":{"other":"{0} 磅"},"narrow":{"other":"{0}磅"},"perUnit":{"long":"每磅 {0}","short":"{0}/磅","narrow":"{0}/磅"}},"ounce":{"long":{"other":"{0} 盎司"},"short":{"other":"{0} 盎司"},"narrow":{"other":"{0}盎司"},"perUnit":{"long":"每盎司 {0}","short":"{0}/盎司","narrow":"{0}/盎司"}},"kilometer-per-hour":{"long":{"other":"每小時 {0} 公里"},"short":{"other":"{0} 公里/小時"},"narrow":{"other":"{0}公里/小時"},"perUnit":{}},"meter-per-second":{"long":{"other":"每秒 {0} 公尺"},"short":{"other":"{0} 公尺/秒"},"narrow":{"other":"{0}公尺/秒"},"perUnit":{}},"mile-per-hour":{"long":{"other":"每小時 {0} 英里"},"short":{"other":"{0} 英里/小時"},"narrow":{"other":"{0}英里/小時"},"perUnit":{}},"celsius":{"long":{"other":"攝氏 {0} 度"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"華氏 {0} 度"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0} 公升"},"short":{"other":"{0} 升"},"narrow":{"other":"{0}升"},"perUnit":{"long":"每公升 {0}","short":"{0}/升","narrow":"{0}/L"}},"milliliter":{"long":{"other":"{0} 毫升"},"short":{"other":"{0} 毫升"},"narrow":{"other":"{0}mL"},"perUnit":{}},"gallon":{"long":{"other":"{0} 加侖"},"short":{"other":"{0} 加侖"},"narrow":{"other":"{0}gal"},"perUnit":{"long":"每加侖 {0}","short":"{0}/加侖","narrow":"{0}/gal"}},"fluid-ounce":{"long":{"other":"{0} 液盎司"},"short":{"other":"{0} 液盎司"},"narrow":{"other":"{0}fl-oz"},"perUnit":{}}},"compound":{"per":{"long":"每{1} {0}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"安道爾陪士特"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"阿拉伯聯合大公國迪爾汗"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"阿富汗尼 (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"阿富汗尼"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"阿爾巴尼亞列克 (1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"阿爾巴尼亞列克"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"亞美尼亞德拉姆"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"荷屬安地列斯盾"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"安哥拉寬扎"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"安哥拉寬扎 (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"安哥拉新寬扎 (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"安哥拉新調寬扎 (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"阿根廷奧斯特納爾"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"阿根廷披索 (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"阿根廷披索 (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"阿根廷披索 (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"阿根廷披索"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"奧地利先令"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"澳幣"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"阿路巴盾"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"亞塞拜然馬納特 (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"亞塞拜然馬納特"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"波士尼亞-赫塞哥維納第納爾"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"波士尼亞-赫塞哥維納可轉換馬克"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"波士尼亞-赫塞哥維納新第納爾"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"巴貝多元"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"孟加拉塔卡"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"比利時法郎（可轉換）"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"比利時法郎"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"比利時法郎（金融）"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"保加利亞硬列弗"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"保加利亞社會黨列弗"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"保加利亞新列弗"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"保加利亞列弗 (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"巴林第納爾"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"蒲隆地法郎"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"百慕達幣"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"汶萊元"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"玻利維亞諾"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"玻利維亞玻利維亞諾 (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"玻利維亞披索"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"玻利維亞幕多"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"巴西克魯薩多農瓦 (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"巴西克魯賽羅 (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"巴西克魯賽羅 (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"巴西里拉"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"巴西克如爾達農瓦"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"巴西克魯賽羅 (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"巴西克魯賽羅 (1942 –1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"巴哈馬元"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"不丹那特倫"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"緬甸基雅特"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"波札那普拉"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"白俄羅斯新盧布 (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"白俄羅斯盧布"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"白俄羅斯盧布 (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"貝里斯元"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"加幣"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"剛果法郎"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"歐元 (WIR)"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"瑞士法郎"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"法郎 (WIR)"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"智利埃斯庫多"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"卡林油達佛曼跎"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"智利披索"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"人民幣（離岸）"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"CNX"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"人民幣"},"symbol":"CN¥","narrow":"¥"},"COP":{"displayName":{"other":"哥倫比亞披索"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"哥倫比亞幣 (COU)"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"哥斯大黎加科朗"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"舊塞爾維亞第納爾"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"捷克斯洛伐克硬克朗"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"古巴可轉換披索"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"古巴披索"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"維德角埃斯庫多"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"賽普勒斯鎊"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"捷克克朗"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"東德奧斯特馬克"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"德國馬克"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"吉布地法郎"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"丹麥克朗"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"多明尼加披索"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"阿爾及利亞第納爾"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"厄瓜多蘇克雷"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"厄瓜多爾由里達瓦康斯坦 (UVC)"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"愛沙尼亞克朗"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"埃及鎊"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"厄立特里亞納克法"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"西班牙比塞塔（會計單位）"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"西班牙比塞塔（可轉換會計單位）"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"西班牙陪士特"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"衣索比亞比爾"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"歐元"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"芬蘭馬克"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"斐濟元"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"福克蘭群島鎊"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"法國法郎"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"英鎊"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"喬治亞庫旁拉里"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"喬治亞拉里"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"迦納賽地 (1979–2007)"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"迦納塞地"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"直布羅陀鎊"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"甘比亞達拉西"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"幾內亞法郎"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"幾內亞西里"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"赤道幾內亞埃奎勒"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"希臘德拉克馬"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"瓜地馬拉格查爾"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"葡屬幾內亞埃斯庫多"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"幾內亞比索披索"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"圭亞那元"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"港幣"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"洪都拉斯倫皮拉"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"克羅埃西亞第納爾"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"克羅埃西亞庫納"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"海地古德"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"匈牙利福林"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"印尼盾"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"愛爾蘭鎊"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"以色列鎊"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"以色列謝克爾 (1980–1985)"},"symbol":"ILR","narrow":"ILR"},"ILS":{"displayName":{"other":"以色列新謝克爾"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"印度盧比"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"伊拉克第納爾"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"伊朗里亞爾"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"冰島克朗 (1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"冰島克朗"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"義大利里拉"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"牙買加元"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"約旦第納爾"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"日圓"},"symbol":"¥","narrow":"¥"},"KES":{"displayName":{"other":"肯尼亞先令"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"吉爾吉斯索姆"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"柬埔寨瑞爾"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"科摩羅法郎"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"北韓元"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"南韓圜"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"南韓圓"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"韓元"},"symbol":"￦","narrow":"₩"},"KWD":{"displayName":{"other":"科威特第納爾"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"開曼群島元"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"哈薩克堅戈"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"寮國基普"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"黎巴嫩鎊"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"斯里蘭卡盧比"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"賴比瑞亞元"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"賴索托洛蒂"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"立陶宛立特"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"立陶宛特羅"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"盧森堡可兌換法郎"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"盧森堡法郎"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"盧森堡金融法郎"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"拉脫維亞拉特銀幣"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"拉脫維亞盧布"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"利比亞第納爾"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"摩洛哥迪拉姆"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"摩洛哥法郎"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"摩納哥法郎"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"摩爾多瓦券"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"摩杜雲列伊"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"馬達加斯加阿里亞里"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"馬達加斯加法郎"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"馬其頓第納爾"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"馬其頓第納爾 (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"馬里法郎"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"緬甸元"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"蒙古圖格里克"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"澳門元"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"茅利塔尼亞烏吉亞 (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"茅利塔尼亞烏吉亞"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"馬爾他里拉"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"馬爾他鎊"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"模里西斯盧比"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"馬爾地夫盧比"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"馬爾地夫盧非亞"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"馬拉維克瓦查"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"墨西哥披索"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"墨西哥銀披索 (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"墨西哥轉換單位 (UDI)"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"馬來西亞令吉"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"莫三比克埃斯庫多"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"莫三比克梅蒂卡爾 (1980–2006)"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"莫三比克梅蒂卡爾"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"納米比亞元"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"奈及利亞奈拉"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"尼加拉瓜科多巴"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"尼加拉瓜金科多巴"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"荷蘭盾"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"挪威克朗"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"尼泊爾盧比"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"紐西蘭幣"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"阿曼里亞爾"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"巴拿馬巴波亞"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"祕魯因蒂"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"秘魯太陽幣"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"秘魯太陽幣 (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"巴布亞紐幾內亞基那"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"菲律賓披索"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"巴基斯坦盧比"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"波蘭茲羅提"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"波蘭茲羅提 (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"葡萄牙埃斯庫多"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"巴拉圭瓜拉尼"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"卡達里亞爾"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"羅德西亞元"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"舊羅馬尼亞列伊"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"羅馬尼亞列伊"},"symbol":"RON","narrow":"L"},"RSD":{"displayName":{"other":"塞爾維亞戴納"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"俄羅斯盧布"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"俄羅斯盧布 (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"盧安達法郎"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"沙烏地里亞爾"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"索羅門群島元"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"塞席爾盧比"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"蘇丹第納爾"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"蘇丹鎊"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"舊蘇丹鎊"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"瑞典克朗"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"新加坡幣"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"聖赫勒拿鎊"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"斯洛維尼亞托勒"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"斯洛伐克克朗"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"獅子山利昂"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"索馬利亞先令"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"蘇利南元"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"蘇利南基爾"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"南蘇丹鎊"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"聖多美島和普林西比島多布拉 (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"聖多美島和普林西比島多布拉"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"蘇聯盧布"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"薩爾瓦多科郎"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"敘利亞鎊"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"史瓦濟蘭里朗吉尼"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"泰銖"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"塔吉克盧布"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"塔吉克索莫尼"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"土庫曼馬納特 (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"土庫曼馬納特"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"突尼西亞第納爾"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"東加潘加"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"帝汶埃斯庫多"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"土耳其里拉"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"新土耳其里拉"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"千里達及托巴哥元"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"新台幣"},"symbol":"$","narrow":"$"},"TZS":{"displayName":{"other":"坦尚尼亞先令"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"烏克蘭格里夫納"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"烏克蘭卡本瓦那茲"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"烏干達先令 (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"烏干達先令"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"美元"},"symbol":"US$","narrow":"$"},"USN":{"displayName":{"other":"美元（次日）"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"美元（當日）"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"烏拉圭披索（指數單位）"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"烏拉圭披索 (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"烏拉圭披索"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"UYW"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"烏茲別克索姆"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"委內瑞拉玻利瓦 (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"委內瑞拉玻利瓦 (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"委內瑞拉玻利瓦"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"越南盾"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"越南盾 (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"萬那杜瓦圖"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"西薩摩亞塔拉"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"法郎 (CFA–BEAC)"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"白銀"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"黃金"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"歐洲綜合單位"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"歐洲貨幣單位 (XBB)"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"歐洲會計單位 (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"歐洲會計單位 (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"格瑞那達元"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"特殊提款權"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"歐洲貨幣單位 (XEU)"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"法國金法郎"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"法國法郎 (UIC)"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"法郎 (CFA–BCEAO)"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"帕拉狄昂"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"法郎 (CFP)"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"白金"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET 基金"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"蘇克雷貨幣"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"測試用貨幣代碼"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"亞洲開發銀行計價單位"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"（未知貨幣）"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"葉門第納爾"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"葉門里亞爾"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"南斯拉夫第納爾硬幣"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"南斯拉夫挪威亞第納爾"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"南斯拉夫可轉換第納爾"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"南斯拉夫改革第納爾 (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"南非蘭特（金融）"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"南非蘭特"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"尚比亞克瓦查 (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"尚比亞克瓦查"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"薩伊新扎伊爾"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"薩伊扎伊爾"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"辛巴威元 (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"辛巴威元 (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"辛巴威元 (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"非數值","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0"},"10000":{"other":"0萬"},"100000":{"other":"00萬"},"1000000":{"other":"000萬"},"10000000":{"other":"0000萬"},"100000000":{"other":"0億"},"1000000000":{"other":"00億"},"10000000000":{"other":"000億"},"100000000000":{"other":"0000億"},"1000000000000":{"other":"0兆"},"10000000000000":{"other":"00兆"},"100000000000000":{"other":"000兆"}},"short":{"1000":{"other":"0"},"10000":{"other":"0萬"},"100000":{"other":"00萬"},"1000000":{"other":"000萬"},"10000000":{"other":"0000萬"},"100000000":{"other":"0億"},"1000000000":{"other":"00億"},"10000000000":{"other":"000億"},"100000000000":{"other":"0000億"},"1000000000000":{"other":"0兆"},"10000000000000":{"other":"00兆"},"100000000000000":{"other":"000兆"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0} {1}","short":{"1000":{"other":"0"},"10000":{"other":"¤0萬"},"100000":{"other":"¤00萬"},"1000000":{"other":"¤000萬"},"10000000":{"other":"¤0000萬"},"100000000":{"other":"¤0億"},"1000000000":{"other":"¤00億"},"10000000000":{"other":"¤000億"},"100000000000":{"other":"¤0000億"},"1000000000000":{"other":"¤0兆"},"10000000000000":{"other":"¤00兆"},"100000000000000":{"other":"¤000兆"}}}}},"nu":["latn"]}},"availableLocales":["zh-Hant"]},
    {"data":{"zh-Hans":{"units":{"simple":{"degree":{"long":{"other":"{0}度"},"short":{"other":"{0}°"},"narrow":{"other":"{0}°"},"perUnit":{}},"hectare":{"long":{"other":"{0}公顷"},"short":{"other":"{0}公顷"},"narrow":{"other":"{0}ha"},"perUnit":{}},"acre":{"long":{"other":"{0}英亩"},"short":{"other":"{0}英亩"},"narrow":{"other":"{0}ac"},"perUnit":{}},"percent":{"long":{"other":"{0}%"},"short":{"other":"{0}%"},"narrow":{"other":"{0}%"},"perUnit":{}},"liter-per-kilometer":{"long":{"other":"每公里{0}升"},"short":{"other":"{0}升/公里"},"narrow":{"other":"{0}升/公里"},"perUnit":{}},"mile-per-gallon":{"long":{"other":"每加仑{0}英里"},"short":{"other":"{0}英里/加仑"},"narrow":{"other":"{0}英里/加仑"},"perUnit":{}},"petabyte":{"long":{"other":"{0}拍字节"},"short":{"other":"{0} PB"},"narrow":{"other":"{0} PB"},"perUnit":{}},"terabyte":{"long":{"other":"{0}太字节"},"short":{"other":"{0}太字节"},"narrow":{"other":"{0}太字节"},"perUnit":{}},"terabit":{"long":{"other":"{0}太比特"},"short":{"other":"{0}太比特"},"narrow":{"other":"{0}太比特"},"perUnit":{}},"gigabyte":{"long":{"other":"{0}吉字节"},"short":{"other":"{0}吉字节"},"narrow":{"other":"{0}吉字节"},"perUnit":{}},"gigabit":{"long":{"other":"{0}吉比特"},"short":{"other":"{0}吉比特"},"narrow":{"other":"{0}吉比特"},"perUnit":{}},"megabyte":{"long":{"other":"{0}兆字节"},"short":{"other":"{0}兆字节"},"narrow":{"other":"{0}兆字节"},"perUnit":{}},"megabit":{"long":{"other":"{0}兆比特"},"short":{"other":"{0}兆比特"},"narrow":{"other":"{0}兆比特"},"perUnit":{}},"kilobyte":{"long":{"other":"{0}千字节"},"short":{"other":"{0}千字节"},"narrow":{"other":"{0}千字节"},"perUnit":{}},"kilobit":{"long":{"other":"{0}千比特"},"short":{"other":"{0}千比特"},"narrow":{"other":"{0}千比特"},"perUnit":{}},"byte":{"long":{"other":"{0}字节"},"short":{"other":"{0}字节"},"narrow":{"other":"{0}字节"},"perUnit":{}},"bit":{"long":{"other":"{0}比特"},"short":{"other":"{0}比特"},"narrow":{"other":"{0}比特"},"perUnit":{}},"year":{"long":{"other":"{0}年"},"short":{"other":"{0}年"},"narrow":{"other":"{0}年"},"perUnit":{"long":"每年{0}","short":"{0}/年","narrow":"{0}/年"}},"month":{"long":{"other":"{0}个月"},"short":{"other":"{0}个月"},"narrow":{"other":"{0}个月"},"perUnit":{"long":"每月{0}","short":"{0}/月","narrow":"{0}/月"}},"week":{"long":{"other":"{0}周"},"short":{"other":"{0}周"},"narrow":{"other":"{0}周"},"perUnit":{"long":"每周{0}","short":"{0}/周","narrow":"{0}/周"}},"day":{"long":{"other":"{0}天"},"short":{"other":"{0}天"},"narrow":{"other":"{0}天"},"perUnit":{"long":"每天{0}","short":"{0}/天","narrow":"{0}/天"}},"hour":{"long":{"other":"{0}小时"},"short":{"other":"{0}小时"},"narrow":{"other":"{0}小时"},"perUnit":{"long":"每小时{0}","short":"{0}/小时","narrow":"{0}/小时"}},"minute":{"long":{"other":"{0}分钟"},"short":{"other":"{0}分钟"},"narrow":{"other":"{0}分钟"},"perUnit":{"long":"每分钟{0}","short":"{0}/分钟","narrow":"{0}/分钟"}},"second":{"long":{"other":"{0}秒钟"},"short":{"other":"{0}秒"},"narrow":{"other":"{0}秒"},"perUnit":{"long":"{0}/秒","short":"{0}/秒","narrow":"{0}/秒"}},"millisecond":{"long":{"other":"{0}毫秒"},"short":{"other":"{0}毫秒"},"narrow":{"other":"{0}毫秒"},"perUnit":{}},"kilometer":{"long":{"other":"{0}公里"},"short":{"other":"{0}公里"},"narrow":{"other":"{0}公里"},"perUnit":{"long":"每公里{0}","short":"{0}/公里","narrow":"{0}/公里"}},"meter":{"long":{"other":"{0}米"},"short":{"other":"{0}米"},"narrow":{"other":"{0}米"},"perUnit":{"long":"每米{0}","short":"{0}/米","narrow":"{0}/米"}},"centimeter":{"long":{"other":"{0}厘米"},"short":{"other":"{0}厘米"},"narrow":{"other":"{0}厘米"},"perUnit":{"long":"每厘米{0}","short":"{0}/厘米","narrow":"{0}/厘米"}},"millimeter":{"long":{"other":"{0}毫米"},"short":{"other":"{0}毫米"},"narrow":{"other":"{0}毫米"},"perUnit":{}},"mile":{"long":{"other":"{0}英里"},"short":{"other":"{0}英里"},"narrow":{"other":"{0}mi"},"perUnit":{}},"yard":{"long":{"other":"{0}码"},"short":{"other":"{0}码"},"narrow":{"other":"{0}yd"},"perUnit":{}},"foot":{"long":{"other":"{0}英尺"},"short":{"other":"{0}英尺"},"narrow":{"other":"{0}′"},"perUnit":{"long":"每英尺{0}","short":"{0}/英尺","narrow":"{0}/英尺"}},"inch":{"long":{"other":"{0}英寸"},"short":{"other":"{0}英寸"},"narrow":{"other":"{0}″"},"perUnit":{"long":"每英寸{0}","short":"{0}/英寸","narrow":"{0}/英寸"}},"mile-scandinavian":{"long":{"other":"{0}斯堪的纳维亚英里"},"short":{"other":"{0}斯堪的纳维亚英里"},"narrow":{"other":"{0}smi"},"perUnit":{}},"kilogram":{"long":{"other":"{0}千克"},"short":{"other":"{0}千克"},"narrow":{"other":"{0}千克"},"perUnit":{"long":"每千克{0}","short":"{0}/千克","narrow":"{0}/千克"}},"gram":{"long":{"other":"{0}克"},"short":{"other":"{0}克"},"narrow":{"other":"{0}克"},"perUnit":{"long":"每克{0}","short":"{0}/克","narrow":"{0}/克"}},"stone":{"long":{"other":"{0}英石"},"short":{"other":"{0}英石"},"narrow":{"other":"{0}英石"},"perUnit":{}},"pound":{"long":{"other":"{0}磅"},"short":{"other":"{0}磅"},"narrow":{"other":"{0}磅"},"perUnit":{"long":"每磅{0}","short":"{0}/磅","narrow":"{0}/磅"}},"ounce":{"long":{"other":"{0}盎司"},"short":{"other":"{0}盎司"},"narrow":{"other":"{0}盎司"},"perUnit":{"long":"每盎司{0}","short":"{0}/盎司","narrow":"{0}/盎司"}},"kilometer-per-hour":{"long":{"other":"每小时{0}公里"},"short":{"other":"每小时{0}公里"},"narrow":{"other":"{0}公里/小时"},"perUnit":{}},"meter-per-second":{"long":{"other":"每秒{0}米"},"short":{"other":"{0}米/秒"},"narrow":{"other":"{0}m/s"},"perUnit":{}},"mile-per-hour":{"long":{"other":"每小时{0}英里"},"short":{"other":"{0}英里/小时"},"narrow":{"other":"{0}mi/h"},"perUnit":{}},"celsius":{"long":{"other":"{0}摄氏度"},"short":{"other":"{0}°C"},"narrow":{"other":"{0}°C"},"perUnit":{}},"fahrenheit":{"long":{"other":"{0}华氏度"},"short":{"other":"{0}°F"},"narrow":{"other":"{0}°F"},"perUnit":{}},"liter":{"long":{"other":"{0}升"},"short":{"other":"{0}升"},"narrow":{"other":"{0}升"},"perUnit":{"long":"每升{0}","short":"{0}/升","narrow":"{0}/升"}},"milliliter":{"long":{"other":"{0}毫升"},"short":{"other":"{0}毫升"},"narrow":{"other":"{0}毫升"},"perUnit":{}},"gallon":{"long":{"other":"{0}加仑"},"short":{"other":"{0}加仑"},"narrow":{"other":"{0}加仑"},"perUnit":{"long":"每加仑{0}","short":"{0}/加仑","narrow":"{0}/加仑"}},"fluid-ounce":{"long":{"other":"{0}液盎司"},"short":{"other":"{0}液盎司"},"narrow":{"other":"{0}液盎司"},"perUnit":{}}},"compound":{"per":{"long":"每{1}{0}","short":"{0}/{1}","narrow":"{0}/{1}"}}},"currencies":{"ADP":{"displayName":{"other":"安道尔比塞塔"},"symbol":"ADP","narrow":"ADP"},"AED":{"displayName":{"other":"阿联酋迪拉姆"},"symbol":"AED","narrow":"AED"},"AFA":{"displayName":{"other":"阿富汗尼 (1927–2002)"},"symbol":"AFA","narrow":"AFA"},"AFN":{"displayName":{"other":"阿富汗尼"},"symbol":"AFN","narrow":"AFN"},"ALK":{"displayName":{"other":"阿尔巴尼亚列克(1946–1965)"},"symbol":"ALK","narrow":"ALK"},"ALL":{"displayName":{"other":"阿尔巴尼亚列克"},"symbol":"ALL","narrow":"ALL"},"AMD":{"displayName":{"other":"亚美尼亚德拉姆"},"symbol":"AMD","narrow":"AMD"},"ANG":{"displayName":{"other":"荷属安的列斯盾"},"symbol":"ANG","narrow":"ANG"},"AOA":{"displayName":{"other":"安哥拉宽扎"},"symbol":"AOA","narrow":"Kz"},"AOK":{"displayName":{"other":"安哥拉宽扎 (1977–1990)"},"symbol":"AOK","narrow":"AOK"},"AON":{"displayName":{"other":"安哥拉新宽扎 (1990–2000)"},"symbol":"AON","narrow":"AON"},"AOR":{"displayName":{"other":"安哥拉重新调整宽扎 (1995–1999)"},"symbol":"AOR","narrow":"AOR"},"ARA":{"displayName":{"other":"阿根廷奥斯特拉尔"},"symbol":"ARA","narrow":"ARA"},"ARL":{"displayName":{"other":"阿根廷法定比索 (1970–1983)"},"symbol":"ARL","narrow":"ARL"},"ARM":{"displayName":{"other":"阿根廷比索 (1881–1970)"},"symbol":"ARM","narrow":"ARM"},"ARP":{"displayName":{"other":"阿根廷比索 (1983–1985)"},"symbol":"ARP","narrow":"ARP"},"ARS":{"displayName":{"other":"阿根廷比索"},"symbol":"ARS","narrow":"$"},"ATS":{"displayName":{"other":"奥地利先令"},"symbol":"ATS","narrow":"ATS"},"AUD":{"displayName":{"other":"澳大利亚元"},"symbol":"AU$","narrow":"$"},"AWG":{"displayName":{"other":"阿鲁巴弗罗林"},"symbol":"AWG","narrow":"AWG"},"AZM":{"displayName":{"other":"阿塞拜疆马纳特 (1993–2006)"},"symbol":"AZM","narrow":"AZM"},"AZN":{"displayName":{"other":"阿塞拜疆马纳特"},"symbol":"AZN","narrow":"AZN"},"BAD":{"displayName":{"other":"波士尼亚-赫塞哥维纳第纳尔 (1992–1994)"},"symbol":"BAD","narrow":"BAD"},"BAM":{"displayName":{"other":"波斯尼亚-黑塞哥维那可兑换马克"},"symbol":"BAM","narrow":"KM"},"BAN":{"displayName":{"other":"波士尼亚-赫塞哥维纳新第纳尔 (1994–1997)"},"symbol":"BAN","narrow":"BAN"},"BBD":{"displayName":{"other":"巴巴多斯元"},"symbol":"BBD","narrow":"$"},"BDT":{"displayName":{"other":"孟加拉塔卡"},"symbol":"BDT","narrow":"৳"},"BEC":{"displayName":{"other":"比利时法郎（可兑换）"},"symbol":"BEC","narrow":"BEC"},"BEF":{"displayName":{"other":"比利时法郎"},"symbol":"BEF","narrow":"BEF"},"BEL":{"displayName":{"other":"比利时法郎（金融）"},"symbol":"BEL","narrow":"BEL"},"BGL":{"displayName":{"other":"保加利亚硬列弗"},"symbol":"BGL","narrow":"BGL"},"BGM":{"displayName":{"other":"保加利亚社会党列弗"},"symbol":"BGM","narrow":"BGM"},"BGN":{"displayName":{"other":"保加利亚新列弗"},"symbol":"BGN","narrow":"BGN"},"BGO":{"displayName":{"other":"保加利亚列弗 (1879–1952)"},"symbol":"BGO","narrow":"BGO"},"BHD":{"displayName":{"other":"巴林第纳尔"},"symbol":"BHD","narrow":"BHD"},"BIF":{"displayName":{"other":"布隆迪法郎"},"symbol":"BIF","narrow":"BIF"},"BMD":{"displayName":{"other":"百慕大元"},"symbol":"BMD","narrow":"$"},"BND":{"displayName":{"other":"文莱元"},"symbol":"BND","narrow":"$"},"BOB":{"displayName":{"other":"玻利维亚诺"},"symbol":"BOB","narrow":"Bs"},"BOL":{"displayName":{"other":"玻利维亚诺 (1863–1963)"},"symbol":"BOL","narrow":"BOL"},"BOP":{"displayName":{"other":"玻利维亚比索"},"symbol":"BOP","narrow":"BOP"},"BOV":{"displayName":{"other":"玻利维亚 Mvdol（资金）"},"symbol":"BOV","narrow":"BOV"},"BRB":{"displayName":{"other":"巴西新克鲁赛罗 (1967–1986)"},"symbol":"BRB","narrow":"BRB"},"BRC":{"displayName":{"other":"巴西克鲁扎多 (1986–1989)"},"symbol":"BRC","narrow":"BRC"},"BRE":{"displayName":{"other":"巴西克鲁塞罗 (1990–1993)"},"symbol":"BRE","narrow":"BRE"},"BRL":{"displayName":{"other":"巴西雷亚尔"},"symbol":"R$","narrow":"R$"},"BRN":{"displayName":{"other":"巴西新克鲁扎多 (1989–1990)"},"symbol":"BRN","narrow":"BRN"},"BRR":{"displayName":{"other":"巴西克鲁塞罗 (1993–1994)"},"symbol":"BRR","narrow":"BRR"},"BRZ":{"displayName":{"other":"巴西克鲁塞罗 (1942–1967)"},"symbol":"BRZ","narrow":"BRZ"},"BSD":{"displayName":{"other":"巴哈马元"},"symbol":"BSD","narrow":"$"},"BTN":{"displayName":{"other":"不丹努尔特鲁姆"},"symbol":"BTN","narrow":"BTN"},"BUK":{"displayName":{"other":"缅元"},"symbol":"BUK","narrow":"BUK"},"BWP":{"displayName":{"other":"博茨瓦纳普拉"},"symbol":"BWP","narrow":"P"},"BYB":{"displayName":{"other":"白俄罗斯新卢布 (1994–1999)"},"symbol":"BYB","narrow":"BYB"},"BYN":{"displayName":{"other":"白俄罗斯卢布"},"symbol":"BYN","narrow":"р."},"BYR":{"displayName":{"other":"白俄罗斯卢布 (2000–2016)"},"symbol":"BYR","narrow":"BYR"},"BZD":{"displayName":{"other":"伯利兹元"},"symbol":"BZD","narrow":"$"},"CAD":{"displayName":{"other":"加拿大元"},"symbol":"CA$","narrow":"$"},"CDF":{"displayName":{"other":"刚果法郎"},"symbol":"CDF","narrow":"CDF"},"CHE":{"displayName":{"other":"欧元 (WIR)"},"symbol":"CHE","narrow":"CHE"},"CHF":{"displayName":{"other":"瑞士法郎"},"symbol":"CHF","narrow":"CHF"},"CHW":{"displayName":{"other":"法郎 (WIR)"},"symbol":"CHW","narrow":"CHW"},"CLE":{"displayName":{"other":"智利埃斯库多"},"symbol":"CLE","narrow":"CLE"},"CLF":{"displayName":{"other":"智利（资金）"},"symbol":"CLF","narrow":"CLF"},"CLP":{"displayName":{"other":"智利比索"},"symbol":"CLP","narrow":"$"},"CNH":{"displayName":{"other":"人民币（离岸）"},"symbol":"CNH","narrow":"CNH"},"CNX":{"displayName":{"other":"中国人民银行元"},"symbol":"CNX","narrow":"CNX"},"CNY":{"displayName":{"other":"人民币"},"symbol":"¥","narrow":"¥"},"COP":{"displayName":{"other":"哥伦比亚比索"},"symbol":"COP","narrow":"$"},"COU":{"displayName":{"other":"哥伦比亚币"},"symbol":"COU","narrow":"COU"},"CRC":{"displayName":{"other":"哥斯达黎加科朗"},"symbol":"CRC","narrow":"₡"},"CSD":{"displayName":{"other":"旧塞尔维亚第纳尔"},"symbol":"CSD","narrow":"CSD"},"CSK":{"displayName":{"other":"捷克硬克朗"},"symbol":"CSK","narrow":"CSK"},"CUC":{"displayName":{"other":"古巴可兑换比索"},"symbol":"CUC","narrow":"$"},"CUP":{"displayName":{"other":"古巴比索"},"symbol":"CUP","narrow":"$"},"CVE":{"displayName":{"other":"佛得角埃斯库多"},"symbol":"CVE","narrow":"CVE"},"CYP":{"displayName":{"other":"塞浦路斯镑"},"symbol":"CYP","narrow":"CYP"},"CZK":{"displayName":{"other":"捷克克朗"},"symbol":"CZK","narrow":"Kč"},"DDM":{"displayName":{"other":"东德奥斯特马克"},"symbol":"DDM","narrow":"DDM"},"DEM":{"displayName":{"other":"德国马克"},"symbol":"DEM","narrow":"DEM"},"DJF":{"displayName":{"other":"吉布提法郎"},"symbol":"DJF","narrow":"DJF"},"DKK":{"displayName":{"other":"丹麦克朗"},"symbol":"DKK","narrow":"kr"},"DOP":{"displayName":{"other":"多米尼加比索"},"symbol":"DOP","narrow":"$"},"DZD":{"displayName":{"other":"阿尔及利亚第纳尔"},"symbol":"DZD","narrow":"DZD"},"ECS":{"displayName":{"other":"厄瓜多尔苏克雷"},"symbol":"ECS","narrow":"ECS"},"ECV":{"displayName":{"other":"厄瓜多尔 (UVC)"},"symbol":"ECV","narrow":"ECV"},"EEK":{"displayName":{"other":"爱沙尼亚克朗"},"symbol":"EEK","narrow":"EEK"},"EGP":{"displayName":{"other":"埃及镑"},"symbol":"EGP","narrow":"E£"},"ERN":{"displayName":{"other":"厄立特里亚纳克法"},"symbol":"ERN","narrow":"ERN"},"ESA":{"displayName":{"other":"西班牙比塞塔（帐户 A）"},"symbol":"ESA","narrow":"ESA"},"ESB":{"displayName":{"other":"西班牙比塞塔（兑换帐户）"},"symbol":"ESB","narrow":"ESB"},"ESP":{"displayName":{"other":"西班牙比塞塔"},"symbol":"ESP","narrow":"₧"},"ETB":{"displayName":{"other":"埃塞俄比亚比尔"},"symbol":"ETB","narrow":"ETB"},"EUR":{"displayName":{"other":"欧元"},"symbol":"€","narrow":"€"},"FIM":{"displayName":{"other":"芬兰马克"},"symbol":"FIM","narrow":"FIM"},"FJD":{"displayName":{"other":"斐济元"},"symbol":"FJD","narrow":"$"},"FKP":{"displayName":{"other":"福克兰群岛镑"},"symbol":"FKP","narrow":"£"},"FRF":{"displayName":{"other":"法国法郎"},"symbol":"FRF","narrow":"FRF"},"GBP":{"displayName":{"other":"英镑"},"symbol":"£","narrow":"£"},"GEK":{"displayName":{"other":"乔治亚库蓬拉瑞特"},"symbol":"GEK","narrow":"GEK"},"GEL":{"displayName":{"other":"格鲁吉亚拉里"},"symbol":"GEL","narrow":"₾"},"GHC":{"displayName":{"other":"加纳塞第"},"symbol":"GHC","narrow":"GHC"},"GHS":{"displayName":{"other":"加纳塞地"},"symbol":"GHS","narrow":"GHS"},"GIP":{"displayName":{"other":"直布罗陀镑"},"symbol":"GIP","narrow":"£"},"GMD":{"displayName":{"other":"冈比亚达拉西"},"symbol":"GMD","narrow":"GMD"},"GNF":{"displayName":{"other":"几内亚法郎"},"symbol":"GNF","narrow":"FG"},"GNS":{"displayName":{"other":"几内亚西里"},"symbol":"GNS","narrow":"GNS"},"GQE":{"displayName":{"other":"赤道几内亚埃奎勒"},"symbol":"GQE","narrow":"GQE"},"GRD":{"displayName":{"other":"希腊德拉克马"},"symbol":"GRD","narrow":"GRD"},"GTQ":{"displayName":{"other":"危地马拉格查尔"},"symbol":"GTQ","narrow":"Q"},"GWE":{"displayName":{"other":"葡萄牙几内亚埃斯库多"},"symbol":"GWE","narrow":"GWE"},"GWP":{"displayName":{"other":"几内亚比绍比索"},"symbol":"GWP","narrow":"GWP"},"GYD":{"displayName":{"other":"圭亚那元"},"symbol":"GYD","narrow":"$"},"HKD":{"displayName":{"other":"港元"},"symbol":"HK$","narrow":"$"},"HNL":{"displayName":{"other":"洪都拉斯伦皮拉"},"symbol":"HNL","narrow":"L"},"HRD":{"displayName":{"other":"克罗地亚第纳尔"},"symbol":"HRD","narrow":"HRD"},"HRK":{"displayName":{"other":"克罗地亚库纳"},"symbol":"HRK","narrow":"kn"},"HTG":{"displayName":{"other":"海地古德"},"symbol":"HTG","narrow":"HTG"},"HUF":{"displayName":{"other":"匈牙利福林"},"symbol":"HUF","narrow":"Ft"},"IDR":{"displayName":{"other":"印度尼西亚盾"},"symbol":"IDR","narrow":"Rp"},"IEP":{"displayName":{"other":"爱尔兰镑"},"symbol":"IEP","narrow":"IEP"},"ILP":{"displayName":{"other":"以色列镑"},"symbol":"ILP","narrow":"ILP"},"ILR":{"displayName":{"other":"以色列谢克尔(1980–1985)"},"symbol":"ILS","narrow":"ILS"},"ILS":{"displayName":{"other":"以色列新谢克尔"},"symbol":"₪","narrow":"₪"},"INR":{"displayName":{"other":"印度卢比"},"symbol":"₹","narrow":"₹"},"IQD":{"displayName":{"other":"伊拉克第纳尔"},"symbol":"IQD","narrow":"IQD"},"IRR":{"displayName":{"other":"伊朗里亚尔"},"symbol":"IRR","narrow":"IRR"},"ISJ":{"displayName":{"other":"冰岛克朗(1918–1981)"},"symbol":"ISJ","narrow":"ISJ"},"ISK":{"displayName":{"other":"冰岛克朗"},"symbol":"ISK","narrow":"kr"},"ITL":{"displayName":{"other":"意大利里拉"},"symbol":"ITL","narrow":"ITL"},"JMD":{"displayName":{"other":"牙买加元"},"symbol":"JMD","narrow":"$"},"JOD":{"displayName":{"other":"约旦第纳尔"},"symbol":"JOD","narrow":"JOD"},"JPY":{"displayName":{"other":"日元"},"symbol":"JP¥","narrow":"¥"},"KES":{"displayName":{"other":"肯尼亚先令"},"symbol":"KES","narrow":"KES"},"KGS":{"displayName":{"other":"吉尔吉斯斯坦索姆"},"symbol":"KGS","narrow":"KGS"},"KHR":{"displayName":{"other":"柬埔寨瑞尔"},"symbol":"KHR","narrow":"៛"},"KMF":{"displayName":{"other":"科摩罗法郎"},"symbol":"KMF","narrow":"CF"},"KPW":{"displayName":{"other":"朝鲜元"},"symbol":"KPW","narrow":"₩"},"KRH":{"displayName":{"other":"韩元 (1953–1962)"},"symbol":"KRH","narrow":"KRH"},"KRO":{"displayName":{"other":"韩元 (1945–1953)"},"symbol":"KRO","narrow":"KRO"},"KRW":{"displayName":{"other":"韩元"},"symbol":"￦","narrow":"₩"},"KWD":{"displayName":{"other":"科威特第纳尔"},"symbol":"KWD","narrow":"KWD"},"KYD":{"displayName":{"other":"开曼元"},"symbol":"KYD","narrow":"$"},"KZT":{"displayName":{"other":"哈萨克斯坦坚戈"},"symbol":"KZT","narrow":"₸"},"LAK":{"displayName":{"other":"老挝基普"},"symbol":"LAK","narrow":"₭"},"LBP":{"displayName":{"other":"黎巴嫩镑"},"symbol":"LBP","narrow":"L£"},"LKR":{"displayName":{"other":"斯里兰卡卢比"},"symbol":"LKR","narrow":"Rs"},"LRD":{"displayName":{"other":"利比里亚元"},"symbol":"LRD","narrow":"$"},"LSL":{"displayName":{"other":"莱索托洛蒂"},"symbol":"LSL","narrow":"LSL"},"LTL":{"displayName":{"other":"立陶宛立特"},"symbol":"LTL","narrow":"Lt"},"LTT":{"displayName":{"other":"立陶宛塔咯呐司"},"symbol":"LTT","narrow":"LTT"},"LUC":{"displayName":{"other":"卢森堡可兑换法郎"},"symbol":"LUC","narrow":"LUC"},"LUF":{"displayName":{"other":"卢森堡法郎"},"symbol":"LUF","narrow":"LUF"},"LUL":{"displayName":{"other":"卢森堡金融法郎"},"symbol":"LUL","narrow":"LUL"},"LVL":{"displayName":{"other":"拉脱维亚拉特"},"symbol":"LVL","narrow":"Ls"},"LVR":{"displayName":{"other":"拉脱维亚卢布"},"symbol":"LVR","narrow":"LVR"},"LYD":{"displayName":{"other":"利比亚第纳尔"},"symbol":"LYD","narrow":"LYD"},"MAD":{"displayName":{"other":"摩洛哥迪拉姆"},"symbol":"MAD","narrow":"MAD"},"MAF":{"displayName":{"other":"摩洛哥法郎"},"symbol":"MAF","narrow":"MAF"},"MCF":{"displayName":{"other":"摩纳哥法郎"},"symbol":"MCF","narrow":"MCF"},"MDC":{"displayName":{"other":"摩尔多瓦库邦"},"symbol":"MDC","narrow":"MDC"},"MDL":{"displayName":{"other":"摩尔多瓦列伊"},"symbol":"MDL","narrow":"MDL"},"MGA":{"displayName":{"other":"马达加斯加阿里亚里"},"symbol":"MGA","narrow":"Ar"},"MGF":{"displayName":{"other":"马达加斯加法郎"},"symbol":"MGF","narrow":"MGF"},"MKD":{"displayName":{"other":"马其顿第纳尔"},"symbol":"MKD","narrow":"MKD"},"MKN":{"displayName":{"other":"马其顿第纳尔 (1992–1993)"},"symbol":"MKN","narrow":"MKN"},"MLF":{"displayName":{"other":"马里法郎"},"symbol":"MLF","narrow":"MLF"},"MMK":{"displayName":{"other":"缅甸元"},"symbol":"MMK","narrow":"K"},"MNT":{"displayName":{"other":"蒙古图格里克"},"symbol":"MNT","narrow":"₮"},"MOP":{"displayName":{"other":"澳门元"},"symbol":"MOP","narrow":"MOP"},"MRO":{"displayName":{"other":"毛里塔尼亚乌吉亚 (1973–2017)"},"symbol":"MRO","narrow":"MRO"},"MRU":{"displayName":{"other":"毛里塔尼亚乌吉亚"},"symbol":"MRU","narrow":"MRU"},"MTL":{"displayName":{"other":"马耳他里拉"},"symbol":"MTL","narrow":"MTL"},"MTP":{"displayName":{"other":"马耳他镑"},"symbol":"MTP","narrow":"MTP"},"MUR":{"displayName":{"other":"毛里求斯卢比"},"symbol":"MUR","narrow":"Rs"},"MVP":{"displayName":{"other":"马尔代夫卢比(1947–1981)"},"symbol":"MVP","narrow":"MVP"},"MVR":{"displayName":{"other":"马尔代夫卢菲亚"},"symbol":"MVR","narrow":"MVR"},"MWK":{"displayName":{"other":"马拉维克瓦查"},"symbol":"MWK","narrow":"MWK"},"MXN":{"displayName":{"other":"墨西哥比索"},"symbol":"MX$","narrow":"$"},"MXP":{"displayName":{"other":"墨西哥银比索 (1861–1992)"},"symbol":"MXP","narrow":"MXP"},"MXV":{"displayName":{"other":"墨西哥（资金）"},"symbol":"MXV","narrow":"MXV"},"MYR":{"displayName":{"other":"马来西亚林吉特"},"symbol":"MYR","narrow":"RM"},"MZE":{"displayName":{"other":"莫桑比克埃斯库多"},"symbol":"MZE","narrow":"MZE"},"MZM":{"displayName":{"other":"旧莫桑比克美提卡"},"symbol":"MZM","narrow":"MZM"},"MZN":{"displayName":{"other":"莫桑比克美提卡"},"symbol":"MZN","narrow":"MZN"},"NAD":{"displayName":{"other":"纳米比亚元"},"symbol":"NAD","narrow":"$"},"NGN":{"displayName":{"other":"尼日利亚奈拉"},"symbol":"NGN","narrow":"₦"},"NIC":{"displayName":{"other":"尼加拉瓜科多巴 (1988–1991)"},"symbol":"NIC","narrow":"NIC"},"NIO":{"displayName":{"other":"尼加拉瓜金科多巴"},"symbol":"NIO","narrow":"C$"},"NLG":{"displayName":{"other":"荷兰盾"},"symbol":"NLG","narrow":"NLG"},"NOK":{"displayName":{"other":"挪威克朗"},"symbol":"NOK","narrow":"kr"},"NPR":{"displayName":{"other":"尼泊尔卢比"},"symbol":"NPR","narrow":"Rs"},"NZD":{"displayName":{"other":"新西兰元"},"symbol":"NZ$","narrow":"$"},"OMR":{"displayName":{"other":"阿曼里亚尔"},"symbol":"OMR","narrow":"OMR"},"PAB":{"displayName":{"other":"巴拿马巴波亚"},"symbol":"PAB","narrow":"PAB"},"PEI":{"displayName":{"other":"秘鲁印第"},"symbol":"PEI","narrow":"PEI"},"PEN":{"displayName":{"other":"秘鲁索尔"},"symbol":"PEN","narrow":"PEN"},"PES":{"displayName":{"other":"秘鲁索尔 (1863–1965)"},"symbol":"PES","narrow":"PES"},"PGK":{"displayName":{"other":"巴布亚新几内亚基那"},"symbol":"PGK","narrow":"PGK"},"PHP":{"displayName":{"other":"菲律宾比索"},"symbol":"PHP","narrow":"₱"},"PKR":{"displayName":{"other":"巴基斯坦卢比"},"symbol":"PKR","narrow":"Rs"},"PLN":{"displayName":{"other":"波兰兹罗提"},"symbol":"PLN","narrow":"zł"},"PLZ":{"displayName":{"other":"波兰兹罗提 (1950–1995)"},"symbol":"PLZ","narrow":"PLZ"},"PTE":{"displayName":{"other":"葡萄牙埃斯库多"},"symbol":"PTE","narrow":"PTE"},"PYG":{"displayName":{"other":"巴拉圭瓜拉尼"},"symbol":"PYG","narrow":"₲"},"QAR":{"displayName":{"other":"卡塔尔里亚尔"},"symbol":"QAR","narrow":"QAR"},"RHD":{"displayName":{"other":"罗得西亚元"},"symbol":"RHD","narrow":"RHD"},"ROL":{"displayName":{"other":"旧罗马尼亚列伊"},"symbol":"ROL","narrow":"ROL"},"RON":{"displayName":{"other":"罗马尼亚列伊"},"symbol":"RON","narrow":"lei"},"RSD":{"displayName":{"other":"塞尔维亚第纳尔"},"symbol":"RSD","narrow":"RSD"},"RUB":{"displayName":{"other":"俄罗斯卢布"},"symbol":"RUB","narrow":"₽"},"RUR":{"displayName":{"other":"俄国卢布 (1991–1998)"},"symbol":"RUR","narrow":"р."},"RWF":{"displayName":{"other":"卢旺达法郎"},"symbol":"RWF","narrow":"RF"},"SAR":{"displayName":{"other":"沙特里亚尔"},"symbol":"SAR","narrow":"SAR"},"SBD":{"displayName":{"other":"所罗门群岛元"},"symbol":"SBD","narrow":"$"},"SCR":{"displayName":{"other":"塞舌尔卢比"},"symbol":"SCR","narrow":"SCR"},"SDD":{"displayName":{"other":"苏丹第纳尔 (1992–2007)"},"symbol":"SDD","narrow":"SDD"},"SDG":{"displayName":{"other":"苏丹镑"},"symbol":"SDG","narrow":"SDG"},"SDP":{"displayName":{"other":"旧苏丹镑"},"symbol":"SDP","narrow":"SDP"},"SEK":{"displayName":{"other":"瑞典克朗"},"symbol":"SEK","narrow":"kr"},"SGD":{"displayName":{"other":"新加坡元"},"symbol":"SGD","narrow":"$"},"SHP":{"displayName":{"other":"圣赫勒拿群岛磅"},"symbol":"SHP","narrow":"£"},"SIT":{"displayName":{"other":"斯洛文尼亚托拉尔"},"symbol":"SIT","narrow":"SIT"},"SKK":{"displayName":{"other":"斯洛伐克克朗"},"symbol":"SKK","narrow":"SKK"},"SLL":{"displayName":{"other":"塞拉利昂利昂"},"symbol":"SLL","narrow":"SLL"},"SOS":{"displayName":{"other":"索马里先令"},"symbol":"SOS","narrow":"SOS"},"SRD":{"displayName":{"other":"苏里南元"},"symbol":"SRD","narrow":"$"},"SRG":{"displayName":{"other":"苏里南盾"},"symbol":"SRG","narrow":"SRG"},"SSP":{"displayName":{"other":"南苏丹镑"},"symbol":"SSP","narrow":"£"},"STD":{"displayName":{"other":"圣多美和普林西比多布拉 (1977–2017)"},"symbol":"STD","narrow":"STD"},"STN":{"displayName":{"other":"圣多美和普林西比多布拉"},"symbol":"STN","narrow":"Db"},"SUR":{"displayName":{"other":"苏联卢布"},"symbol":"SUR","narrow":"SUR"},"SVC":{"displayName":{"other":"萨尔瓦多科朗"},"symbol":"SVC","narrow":"SVC"},"SYP":{"displayName":{"other":"叙利亚镑"},"symbol":"SYP","narrow":"£"},"SZL":{"displayName":{"other":"斯威士兰里兰吉尼"},"symbol":"SZL","narrow":"SZL"},"THB":{"displayName":{"other":"泰铢"},"symbol":"THB","narrow":"฿"},"TJR":{"displayName":{"other":"塔吉克斯坦卢布"},"symbol":"TJR","narrow":"TJR"},"TJS":{"displayName":{"other":"塔吉克斯坦索莫尼"},"symbol":"TJS","narrow":"TJS"},"TMM":{"displayName":{"other":"土库曼斯坦马纳特 (1993–2009)"},"symbol":"TMM","narrow":"TMM"},"TMT":{"displayName":{"other":"土库曼斯坦马纳特"},"symbol":"TMT","narrow":"TMT"},"TND":{"displayName":{"other":"突尼斯第纳尔"},"symbol":"TND","narrow":"TND"},"TOP":{"displayName":{"other":"汤加潘加"},"symbol":"TOP","narrow":"T$"},"TPE":{"displayName":{"other":"帝汶埃斯库多"},"symbol":"TPE","narrow":"TPE"},"TRL":{"displayName":{"other":"土耳其里拉 (1922–2005)"},"symbol":"TRL","narrow":"TRL"},"TRY":{"displayName":{"other":"土耳其里拉"},"symbol":"TRY","narrow":"₺"},"TTD":{"displayName":{"other":"特立尼达和多巴哥元"},"symbol":"TTD","narrow":"$"},"TWD":{"displayName":{"other":"新台币"},"symbol":"NT$","narrow":"NT$"},"TZS":{"displayName":{"other":"坦桑尼亚先令"},"symbol":"TZS","narrow":"TZS"},"UAH":{"displayName":{"other":"乌克兰格里夫纳"},"symbol":"UAH","narrow":"₴"},"UAK":{"displayName":{"other":"乌克兰币"},"symbol":"UAK","narrow":"UAK"},"UGS":{"displayName":{"other":"乌干达先令 (1966–1987)"},"symbol":"UGS","narrow":"UGS"},"UGX":{"displayName":{"other":"乌干达先令"},"symbol":"UGX","narrow":"UGX"},"USD":{"displayName":{"other":"美元"},"symbol":"US$","narrow":"$"},"USN":{"displayName":{"other":"美元（次日）"},"symbol":"USN","narrow":"USN"},"USS":{"displayName":{"other":"美元（当日）"},"symbol":"USS","narrow":"USS"},"UYI":{"displayName":{"other":"乌拉圭比索（索引单位）"},"symbol":"UYI","narrow":"UYI"},"UYP":{"displayName":{"other":"乌拉圭比索 (1975–1993)"},"symbol":"UYP","narrow":"UYP"},"UYU":{"displayName":{"other":"乌拉圭比索"},"symbol":"UYU","narrow":"$"},"UYW":{"displayName":{"other":"乌拉圭票面工资指数单位"},"symbol":"UYW","narrow":"UYW"},"UZS":{"displayName":{"other":"乌兹别克斯坦苏姆"},"symbol":"UZS","narrow":"UZS"},"VEB":{"displayName":{"other":"委内瑞拉玻利瓦尔 (1871–2008)"},"symbol":"VEB","narrow":"VEB"},"VEF":{"displayName":{"other":"委内瑞拉玻利瓦尔 (2008–2018)"},"symbol":"VEF","narrow":"Bs"},"VES":{"displayName":{"other":"委内瑞拉玻利瓦尔"},"symbol":"VES","narrow":"VES"},"VND":{"displayName":{"other":"越南盾"},"symbol":"₫","narrow":"₫"},"VNN":{"displayName":{"other":"越南盾 (1978–1985)"},"symbol":"VNN","narrow":"VNN"},"VUV":{"displayName":{"other":"瓦努阿图瓦图"},"symbol":"VUV","narrow":"VUV"},"WST":{"displayName":{"other":"萨摩亚塔拉"},"symbol":"WST","narrow":"WST"},"XAF":{"displayName":{"other":"中非法郎"},"symbol":"FCFA","narrow":"FCFA"},"XAG":{"displayName":{"other":"银"},"symbol":"XAG","narrow":"XAG"},"XAU":{"displayName":{"other":"黄金"},"symbol":"XAU","narrow":"XAU"},"XBA":{"displayName":{"other":"欧洲复合单位"},"symbol":"XBA","narrow":"XBA"},"XBB":{"displayName":{"other":"欧洲货币联盟"},"symbol":"XBB","narrow":"XBB"},"XBC":{"displayName":{"other":"欧洲计算单位 (XBC)"},"symbol":"XBC","narrow":"XBC"},"XBD":{"displayName":{"other":"欧洲计算单位 (XBD)"},"symbol":"XBD","narrow":"XBD"},"XCD":{"displayName":{"other":"东加勒比元"},"symbol":"EC$","narrow":"$"},"XDR":{"displayName":{"other":"特别提款权"},"symbol":"XDR","narrow":"XDR"},"XEU":{"displayName":{"other":"欧洲货币单位"},"symbol":"XEU","narrow":"XEU"},"XFO":{"displayName":{"other":"法国金法郎"},"symbol":"XFO","narrow":"XFO"},"XFU":{"displayName":{"other":"法国法郎 (UIC)"},"symbol":"XFU","narrow":"XFU"},"XOF":{"displayName":{"other":"西非法郎"},"symbol":"CFA","narrow":"CFA"},"XPD":{"displayName":{"other":"钯"},"symbol":"XPD","narrow":"XPD"},"XPF":{"displayName":{"other":"太平洋法郎"},"symbol":"CFPF","narrow":"CFPF"},"XPT":{"displayName":{"other":"铂"},"symbol":"XPT","narrow":"XPT"},"XRE":{"displayName":{"other":"RINET 基金"},"symbol":"XRE","narrow":"XRE"},"XSU":{"displayName":{"other":"苏克雷"},"symbol":"XSU","narrow":"XSU"},"XTS":{"displayName":{"other":"测试货币代码"},"symbol":"XTS","narrow":"XTS"},"XUA":{"displayName":{"other":"非洲开发银行记账单位"},"symbol":"XUA","narrow":"XUA"},"XXX":{"displayName":{"other":"（未知货币）"},"symbol":"XXX","narrow":"XXX"},"YDD":{"displayName":{"other":"也门第纳尔"},"symbol":"YDD","narrow":"YDD"},"YER":{"displayName":{"other":"也门里亚尔"},"symbol":"YER","narrow":"YER"},"YUD":{"displayName":{"other":"南斯拉夫硬第纳尔 (1966–1990)"},"symbol":"YUD","narrow":"YUD"},"YUM":{"displayName":{"other":"南斯拉夫新第纳尔 (1994–2002)"},"symbol":"YUM","narrow":"YUM"},"YUN":{"displayName":{"other":"南斯拉夫可兑换第纳尔 (1990–1992)"},"symbol":"YUN","narrow":"YUN"},"YUR":{"displayName":{"other":"南斯拉夫改良第纳尔 (1992–1993)"},"symbol":"YUR","narrow":"YUR"},"ZAL":{"displayName":{"other":"南非兰特 (金融)"},"symbol":"ZAL","narrow":"ZAL"},"ZAR":{"displayName":{"other":"南非兰特"},"symbol":"ZAR","narrow":"R"},"ZMK":{"displayName":{"other":"赞比亚克瓦查 (1968–2012)"},"symbol":"ZMK","narrow":"ZMK"},"ZMW":{"displayName":{"other":"赞比亚克瓦查"},"symbol":"ZMW","narrow":"ZK"},"ZRN":{"displayName":{"other":"新扎伊尔 (1993–1998)"},"symbol":"ZRN","narrow":"ZRN"},"ZRZ":{"displayName":{"other":"扎伊尔 (1971–1993)"},"symbol":"ZRZ","narrow":"ZRZ"},"ZWD":{"displayName":{"other":"津巴布韦元 (1980–2008)"},"symbol":"ZWD","narrow":"ZWD"},"ZWL":{"displayName":{"other":"津巴布韦元 (2009)"},"symbol":"ZWL","narrow":"ZWL"},"ZWR":{"displayName":{"other":"津巴布韦元 (2008)"},"symbol":"ZWR","narrow":"ZWR"}},"numbers":{"nu":["latn"],"symbols":{"latn":{"decimal":".","group":",","list":";","percentSign":"%","plusSign":"+","minusSign":"-","exponential":"E","superscriptingExponent":"×","perMille":"‰","infinity":"∞","nan":"NaN","timeSeparator":":"}},"percent":{"latn":"#,##0%"},"decimal":{"latn":{"long":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0亿"},"1000000000":{"other":"00亿"},"10000000000":{"other":"000亿"},"100000000000":{"other":"0000亿"},"1000000000000":{"other":"0万亿"},"10000000000000":{"other":"00万亿"},"100000000000000":{"other":"000万亿"}},"short":{"1000":{"other":"0"},"10000":{"other":"0万"},"100000":{"other":"00万"},"1000000":{"other":"000万"},"10000000":{"other":"0000万"},"100000000":{"other":"0亿"},"1000000000":{"other":"00亿"},"10000000000":{"other":"000亿"},"100000000000":{"other":"0000亿"},"1000000000000":{"other":"0万亿"},"10000000000000":{"other":"00万亿"},"100000000000000":{"other":"000万亿"}}}},"currency":{"latn":{"currencySpacing":{"beforeInsertBetween":" ","afterInsertBetween":" "},"standard":"¤#,##0.00","accounting":"¤#,##0.00;(¤#,##0.00)","unitPattern":"{0}{1}","short":{"1000":{"other":"0"},"10000":{"other":"¤0万"},"100000":{"other":"¤00万"},"1000000":{"other":"¤000万"},"10000000":{"other":"¤0000万"},"100000000":{"other":"¤0亿"},"1000000000":{"other":"¤00亿"},"10000000000":{"other":"¤000亿"},"100000000000":{"other":"¤0000亿"},"1000000000000":{"other":"¤0万亿"},"10000000000000":{"other":"¤00万亿"},"100000000000000":{"other":"¤000万亿"}}}}},"nu":["latn"]}},"availableLocales":["zh-Hans"]});
    }

})));
//# sourceMappingURL=polyfill-with-locales-for-test262.min.js.map
